/**
 *	Copyright (c) 1997-98 
 *	NorthEast Parallel Architectures Center, Syracuse University. 
 * All Rights Reserved. Permission to use, copy, modify, and distribute this
 * software and its documentation for educational, research and non-profit
 * purposes, without fee, and without a written agreement is hereby granted,
 * provided that the above copyright notice and this paragraph appear in all
 * copies. Permission to incorporate this software into commercial products may
 * be obtained by contacting the NorthEast Parallel Architectures Center. 
 *
 * The software program and documentation are supplied "as is",
 * without any accompanying services from NPAC. NPAC does not
 * warrant that the operation of the program will be uninterrupted or
 * error-free. The end-user understands that the program was developed for
 * research purposes and is advised not to rely exclusively on the program for
 * any reason.
 *
 */

package edu.syr.npac.webflow.impl.connection;

import java.util.*;
import java.beans.*;
import java.lang.reflect.*;
import edu.syr.npac.webflow.exception.*;

/**
 * connectionObjectImpl - 
 *
 * @version     0.6  1 Oct 1998
 * @author      Shrideep Pallickara
 *
 */

public class connectionObjectImpl 
extends edu.syr.npac.webflow.connection._connectionObjectImplBase {
  
  private ComputeQueue queue = new ComputeQueue();
  private connectionParser parser = new connectionParser();
  private boolean persistent=false; 
  private String connectionInfo=null;

  /** Construct a persistently named object. */
  public connectionObjectImpl(java.lang.String name) {
    super(name);
  }
  
  /** Construct a transient object. */
  public connectionObjectImpl() {
    super();
  }

  public void addItem(edu.syr.npac.webflow.module.Module _module) {
    queue.addItem(_module);
  }
  
  public void addItemToTop(edu.syr.npac.webflow.module.Module _module) {
    queue.addItemToTop(_module);
  }

  public edu.syr.npac.webflow.module.Module nextItem() {
    return (edu.syr.npac.webflow.module.Module)queue.nextItem();
  }

  public edu.syr.npac.webflow.module.Module
  removeItem(edu.syr.npac.webflow.module.Module _module) {
    return ((edu.syr.npac.webflow.module.Module)queue.removeItem(_module));  
  }
  
  public edu.syr.npac.webflow.module.Module
  insertItemBefore (edu.syr.npac.webflow.module.Module _insert,
		    edu.syr.npac.webflow.module.Module _before) {
    return (edu.syr.npac.webflow.module.Module)queue.insertItem(_insert, _before);
  }
  
  public void setPersistentConnection(boolean _persistence) {
    persistent = _persistence;
  }
  
  public boolean isPersistentConnection() {
    return persistent;
  }
  
  public void setConnectionInfo(String info) {
    connectionInfo = info;
  }


  public void addNode(String nodeName, 
		      edu.syr.npac.webflow.module.Module _module) {
    parser.addNode(nodeName, _module);
  }
  
  public void removeNode(String nodeName, 
			 edu.syr.npac.webflow.module.Module _module)  {
    parser.removeNode(nodeName, _module);
  }
  
  public edu.syr.npac.webflow.module.Module getNode(String nodeName) {
    return (parser.getNode(nodeName));
  }

  public Object invoke(String nodeName, String propName) {

    return (parser.invoke(nodeName, propName));
  }
  
  public void compute() throws webflowException{
    if (connectionInfo != null) {
      try {
	parser.parseAndInvoke(connectionInfo);
      }	catch (propertyException e) {
	throw new webflowException(e);
      }
      return;
    }
    
    int comp = queue.queueSize()-1;
    for (int i=0; i < comp ; i++) {
      ((edu.syr.npac.webflow.module.Module)queue.peekItem()).run();
      propertyConnector(queue.nextItem(), queue.peekItem());
    }
    ((edu.syr.npac.webflow.module.Module)queue.nextItem()).run(); 
  }
  
  private void propertyConnector(Object source, Object target) 
       throws propertyException
  {
    
    PropertyDescriptor sourceProperties[], targetProperties[];
    sourceProperties = getProperties(source);
    targetProperties = getProperties(target);
    methodHookup(source, target, sourceProperties, targetProperties);
    
  }
  
  private PropertyDescriptor[] getProperties(Object anyObject) {
    
    try {
      return ( Introspector.getBeanInfo(anyObject.getClass()).getPropertyDescriptors());
    } catch (Exception e) {
      System.out.println("Error listing the properties "); 
      return null;
    }
  }

  private void methodHookup(Object sourceObject, Object targetObject,
			   PropertyDescriptor sourceProp[],
			   PropertyDescriptor targetProp[]) 
       throws propertyException  
  {
    
    for (int i = 0; i < sourceProp.length-1; i++) {
      PropertyDescriptor sourceProperty = (PropertyDescriptor)sourceProp[i];
      PropertyDescriptor targetProperty= (PropertyDescriptor)targetProp[i];
      for (int j=0; j < targetProp.length -1; j++) { 
	if ( equivalentProperties(sourceProperty, targetProp[j]) ) {
	  targetProperty =  (PropertyDescriptor)targetProp[j];
	  break;
	}  else {targetProperty = null;}
      }
      if (sourceProperty.getReadMethod() != null && targetProperty != null) {
	Method getter = sourceProperty.getReadMethod();
	Method setter = targetProperty.getWriteMethod();
	
	if ( equivalentProperties(sourceProperty,targetProperty) ) {
	  try {
	    Object args[] = {};
	    Object value = getter.invoke(sourceObject, args);
	    /* Array Constants can be used only in initializers, hence
	       the args[] is not being re-used here, though there is an 
	       overhead in  object creation */
	    Object _args[] = {value};
	    setter.invoke(targetObject, _args);
	  } catch (Exception ex) {
	    ex.printStackTrace();
	    propertyException e = 
	      new propertyException(propertyException.PROPERTY_TYPE_MISMATCH);
	      e.setAdditionalInfo("Source" + sourceObject.toString() + 
				  "Target" + targetObject.toString());
	    throw e;
	  }
	}
      }
    }
  }
    
  private boolean equivalentProperties(PropertyDescriptor _sourceProperty,
				      PropertyDescriptor _targetProperty) {
    if ( (_sourceProperty.getDisplayName()).equals(_targetProperty.getDisplayName()) )
      return true;
    return false;
  }

  

}






