/**
 *	Copyright (c) 1997-98 
 *	NorthEast Parallel Architectures Center, Syracuse University. 
 * All Rights Reserved. Permission to use, copy, modify, and distribute this
 * software and its documentation for educational, research and non-profit
 * purposes, without fee, and without a written agreement is hereby granted,
 * provided that the above copyright notice and this paragraph appear in all
 * copies. Permission to incorporate this software into commercial products may
 * be obtained by contacting the NorthEast Parallel Architectures Center. 
 *
 * The software program and documentation are supplied "as is",
 * without any accompanying services from NPAC. NPAC does not
 * warrant that the operation of the program will be uninterrupted or
 * error-free. The end-user understands that the program was developed for
 * research purposes and is advised not to rely exclusively on the program for
 * any reason.
 *
 */

package edu.syr.npac.webflow.impl.connection;

import java.beans.*;
import java.lang.reflect.*;
import edu.syr.npac.webflow.exception.*;
/**
 * propertyLinker - links properties in different objects. 
 *
 *
 * @version     0.1  6 Oct 1998
 * @author      Shrideep Pallickara
 *
 */

public class propertyLinker {

  public void hookPropertiesTogether (Object sourceObject,
				      String sourcePropName,
				      Object targetObject,
				      String targetPropName) 
       throws propertyException 
  {
    PropertyDescriptor sourceProperties[], targetProperties[];
    sourceProperties = getProperties(sourceObject);
    targetProperties = getProperties(targetObject);
    
    PropertyDescriptor sourceProp, targetProp;
    sourceProp = getDescriptor(sourceProperties, sourcePropName);
    targetProp = getDescriptor(targetProperties, targetPropName);
    
    if (targetProp==null) handleNoSuchProperty(targetObject, targetPropName);
    if (sourceProp==null) handleNoSuchProperty(sourceObject, sourcePropName);
    
    Method getter = sourceProp.getReadMethod();
    if (getter == null) handleNoReadMethod(sourceObject, sourcePropName);
    Method setter = targetProp.getWriteMethod();
    if (setter == null) handleNoWriteMethod(targetObject, targetPropName);

     try {
       Object args[] = {};
       Object value = getter.invoke(sourceObject, args);
       /* Array Constants can be used only in initializers, hence
	  the args[] is not being re-used here, though there is an 
	  overhead in  object creation */
       Object _args[] = {value};
       setter.invoke(targetObject, _args);
     } catch (Exception ex) {
       handlePropertyMisMatch(targetObject, targetPropName, setter,
			      sourceObject, sourcePropName, getter);
     }
  }

  private PropertyDescriptor getDescriptor(PropertyDescriptor prop[],
					   String propName) {
    for (int i=0; i < prop.length -1; i++) {
      if ((prop[i].getDisplayName()).equals(propName))
	return prop[i];
    }
    return null;
  }

  /**
   * Prepares the exception to be thrown, with the complete information
   */
  
  private void handleNoSuchProperty(Object someObject, 
				    String someProperty) 
       throws propertyException 
  {
    propertyException e = 
      new propertyException( propertyException.NO_SUCH_PROPERTY);
    e.setAdditionalInfo( " => " + someProperty+ ": in Object  " + 
			 ((org.omg.CORBA.Object)someObject)._object_name()); 
    throw e;
  }
  
  private void handleNoWriteMethod(Object someObject, 
				   String someProperty) 
       throws propertyException 
  {
    propertyException e = 
      new propertyException( propertyException.NO_WRITE_METHOD);
    e.setAdditionalInfo( " => " + someProperty+ ": in Object  " + 
			 ((org.omg.CORBA.Object)someObject)._object_name()); 
    throw e;
    
  }
  
  private void handleNoReadMethod(Object someObject, 
				  String someProperty) 
       throws propertyException 
  {
    propertyException e = 
      new propertyException( propertyException.NO_READ_METHOD);
    e.setAdditionalInfo( " => " + someProperty+ ": in Object  " + 
			 ((org.omg.CORBA.Object)someObject)._object_name()); 
    throw e;
  }
  
  private void handlePropertyMisMatch( Object _targetObj, 
					String _targetProp, Method _setter,
					Object _sourceObj, 
					String _sourceProp, Method _getter) 
       throws propertyException 
  {
    String _setInfo, _getInfo;
    Class _args[] = _setter.getParameterTypes();
    _setInfo = getTypeName(_args[0]);
    _getInfo = getTypeName(_getter.getReturnType());
    propertyException e = 
      new propertyException( propertyException.PROPERTY_TYPE_MISMATCH);
    e.setAdditionalInfo( "\n" + "Property => "+_targetProp+ 
			 " of Type (" + _setInfo +
			 ") : in Object: " + 
			 ((org.omg.CORBA.Object)_targetObj)._object_name() +
			 " &"+
			 "\n" +  "Property => "+_sourceProp+
			 " of Type (" + _getInfo +
			 "): in Object: " + 
			 ((org.omg.CORBA.Object)_sourceObj)._object_name()); 
    throw e;
  }

  private PropertyDescriptor[] getProperties(Object anyObject) {
    
    try {
      return (Introspector.getBeanInfo(anyObject.getClass()).getPropertyDescriptors());
    } catch (Exception e) {
      System.out.println("Error listing the properties "); 
      return null;
    }
  }
  
  private static String getTypeName(Class type) {
    if (type.isArray()) {
      try {
	Class cl = type;
	int dimensions = 0;
	while (cl.isArray()) {
	  dimensions++;
	  cl = cl.getComponentType();
	}
	StringBuffer sb = new StringBuffer();
	sb.append(cl.getName());
	for (int i = 0; i < dimensions; i++) {
	  sb.append("[]");
	}
	return sb.toString();
      } catch (Throwable e) { /*FALLTHRU*/ }
    }
    return type.getName();
  }
}








