import java.sql.*;
import oracle.jdbc.driver.*;

class AdminDB {
	
    private Connection conn;
    private String username_;

    protected void
    write_log(String username, String userid, String operation, String time,
              String zone) {
        try {
            PreparedStatement pstmt =
                conn.prepareStatement("INSERT INTO admin_log VALUES " +
                      "(?, ?, ?, TO_DATE(?, 'MON DD HH24:MI:SS YYYY'), ?)");
            pstmt.setString(1, username);
            pstmt.setString(2, userid);
            pstmt.setString(3, operation);
            pstmt.setString(4, time);
            pstmt.setString(5, zone);
            pstmt.execute();
            pstmt.close();
        } catch (Exception ex) {
            System.err.println("AdminDB-log: Exception->" + ex);
        }
    }

    public void 
    open(String username, String password, String time, String zone) {
        username_ = username;

        try {
            Class.forName ("oracle.jdbc.driver.OracleDriver");
            conn = DriverManager.getConnection ("jdbc:oracle:oci7:", 
                                  "cwou", "Jessica");
            conn.setAutoCommit(false);
            write_log(username, username, "OPEN", time, zone);
            conn.commit();
        } catch (Exception ex) {
            System.err.println("AdminDB: Exception->" + ex);
        }
    }

    public boolean
    addUser(String userid, String lastName, String firstName, String address, 
            String hPhone, String wPhone, String status, float checking, 
            float saving, String time, String zone) {
        try {
            PreparedStatement pstmt1 =
                conn.prepareStatement("INSERT INTO customer VALUES (?, ?, ?, ?, ?, ?, ?)");
            pstmt1.setString(1, userid);
            pstmt1.setString(2, lastName);
            pstmt1.setString(3, firstName);
            pstmt1.setString(4, address);
            pstmt1.setString(5, hPhone);
            pstmt1.setString(6, wPhone);
            pstmt1.setString(7, status);
            pstmt1.execute();
            pstmt1.close();

            PreparedStatement pstmt2 =
                conn.prepareStatement("INSERT INTO checking_account VALUES (?, ?)");
            pstmt2.setString(1, userid);
            pstmt2.setFloat(2, checking);
            pstmt2.execute();
            pstmt2.close();

            PreparedStatement pstmt3 =
                conn.prepareStatement("INSERT INTO saving_account VALUES (?, ?)");
            pstmt3.setString(1, userid);
            pstmt3.setFloat(2, saving);
            pstmt3.execute();
            pstmt3.close();

            write_log(username_, userid, "ADD", time, zone);

            conn.commit();
            return true;
        } catch (Exception ex) {
            System.err.println("AdminDB: Exception->" + ex);
            return false;
        }
    }	

    public boolean
    deleteUser(String userid, String time, String zone) {
        try {
            PreparedStatement pstmt1 = 
                 conn.prepareStatement("DELETE FROM customer WHERE userid = ?");
            pstmt1.setString(1, userid);
            pstmt1.execute ();

            PreparedStatement pstmt2 =
                 conn.prepareStatement("DELETE FROM checking_account WHERE userid = ?");
            pstmt2.setString(1, userid);
            pstmt2.execute ();

            PreparedStatement pstmt3 =
                 conn.prepareStatement("DELETE FROM saving_account WHERE userid = ?");
            pstmt3.setString(1, userid);
            pstmt3.execute ();

            write_log(username_, userid, "DELETE", time, zone);

            conn.commit();
            return true;
        } catch (Exception ex) {
            System.err.println("AdminDB: Exception->" + ex);
            return false;
        }
    }

    public String
    searchUser(String userid, String lastName, String firstName, String hPhone) {
        StringBuffer sb = new StringBuffer();;
        try {
            CallableStatement cstmt =
              conn.prepareCall("{ ? = call admin_ref.searchUser( ?, ?, ?, ?)}");
            cstmt.registerOutParameter (1, OracleTypes.CURSOR);
            cstmt.setString(2, userid);
            cstmt.setString(3, lastName);
            cstmt.setString(4, firstName);
            cstmt.setString(5, hPhone);
            cstmt.execute ();
            ResultSet result = (ResultSet)cstmt.getObject(1);
            while (result.next()) {
                for (int i=1; i < 7 ; i++)
                    sb.append(result.getString(i) + "+");
                sb.append(result.getString(7) + "|");
            }
            cstmt.close();
            return sb.toString();
        } catch (Exception ex) {
            System.err.println("AdminDB: Exception->" + ex);
            return null;
        }
    }

    public String
    showBalance(String userid) {
        try {
            String checking = "0";
            String saving = "0";

            PreparedStatement pstmt1 =
                 conn.prepareStatement("SELECT amount FROM checking_account WHERE userid = ?");
            pstmt1.setString(1, userid);
            ResultSet resultc = pstmt1.executeQuery();
            resultc.next();
            checking = Float.toString(resultc.getFloat(1));

            PreparedStatement pstmt2 =
                 conn.prepareStatement("SELECT amount FROM saving_account WHERE userid = ?");
            pstmt2.setString(1, userid);
            ResultSet results = pstmt2.executeQuery();
            results.next();
            saving = Float.toString(results.getFloat(1));

            return checking + "+" + saving;
        } catch (Exception ex) {
            System.err.println("AdminDB: Exception->" + ex);
            return "0+0";
        }
    }

    public boolean
    updateUser(String userid, String lastName, String firstName, String address,
               String hPhone, String wPhone, String status, float checking, 
               float saving, String time, String zone) {
        try {
            PreparedStatement pstmt1 =
                 conn.prepareStatement("UPDATE customer SET lastname = ?, firstname = ?, " +
                                       "address = ?, phoneno1 = ?, phoneno2 = ?, " +
                                       "status = ? WHERE userid = ?");
                                                          
            pstmt1.setString(1, lastName);
            pstmt1.setString(2, firstName);
            pstmt1.setString(3, address);
            pstmt1.setString(4, hPhone);
            pstmt1.setString(5, wPhone);
            pstmt1.setString(6, status);
            pstmt1.setString(7, userid);
            pstmt1.execute();
            pstmt1.close();

            PreparedStatement pstmt2 =
                 conn.prepareStatement("UPDATE checking_account SET amount = ? WHERE userid = ?");

            pstmt2.setFloat(1, checking);
            pstmt2.setString(2, userid);
            pstmt2.execute();
            pstmt2.close();

            PreparedStatement pstmt3 =
                 conn.prepareStatement("UPDATE saving_account SET amount = ? WHERE userid = ?");

            pstmt3.setFloat(1, saving);
            pstmt3.setString(2, userid);
            pstmt3.execute();
            pstmt3.close();

            String oper = "UPDATE: " + userid + "/" + lastName + "/" + firstName + "/" +
                          address + "/" + hPhone + "/" + wPhone + "/" + status + "/" +
                          Float.toString(checking) + "/" + Float.toString(saving);
            write_log(username_, userid, oper, time, zone);

            conn.commit();
            return true;
        } catch (Exception ex) {
            System.err.println("AdminDB: Exception->" + ex);
            return false;
        }
    }

    public void 
    close(String time, String zone) {
        try {
            write_log(username_, username_, "CLOSE", time, zone);
            conn.commit();

            conn.close();
        } catch (Exception ex) {
            System.err.println("AdminDB: Exception->" + ex);
        }
    }
/*
    public static void
    main(String[] argv) {
        AdminDB adb = new AdminDB();
        adb.open("CWOU", "Jessica", "Mar 24 09:54:02 1998", "EST");
        // adb.addUser("EYU", "YU", "EDMOND", "CST1-011", "3154434455", "3154434455",
                    // "A", (float)50000.0, (float)100000.0, "Mar 24 09:58:10 1998",
                    // "EST");
        // System.out.println(adb.searchUser("EYU", "", "", ""));
        // adb.updateUser("EYU", "YU", "EDMOND1", "CST1-011", "3154434455", "3154434455",
                    // "A", (float)50000.0, (float)100000.0, "Mar 24 11:58:10 1998",
                    // "EST");
        // System.out.println(adb.searchUser("EYU", "", "", ""));
        // adb.deleteUser("EYU", "Mar 24 13:18:10 1998", "EST");
        // System.out.println(adb.searchUser("EYU", "", "", ""));
        // System.out.println(adb.showBalance("EYU"));
        adb.close("Mar 24 10:10:20 1998", "EST");
    }
*/
}
