import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.io.*;
import java.util.Properties;
import java.util.*;

import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextPackage.*;
import org.omg.CORBA.*;
import com.sun.CORBA.iiop.*;

// import Client-related packages which generated by idltojava
import ClientCoordinator.*;
import ClientServer.*;

class ClientBeepingThread extends Thread {

    private ClientServer.Transaction transaction_;
    private boolean stopFlag_;
    private long beeps = 1;

    public ClientBeepingThread(ClientServer.Transaction tr) {
        transaction_ = tr;
    }

    public void setStopFlag(boolean flag) {
        stopFlag_ = flag;
    }

    public void run() {
        try {
            while (true) {
                this.sleep(22500);
                if (transaction_.beeping(++beeps)) {
                    if (stopFlag_) {
                        return;
                    }
                } else {
                    System.out.println("Client: Transaction connection failed\n");
                }
            }
        } catch (Exception ex) {
            System.err.println("Client: Thread Exception-> " + ex) ;
        }
    }
}

public
class ClientApplet extends Applet implements ActionListener {
    /** ORB
    **/
    private static ClientCoordinator.Registration registration_;
    private static ClientCoordinator.Dispenser dispenser_;
    private static ClientServer.AccountManager accountManager_;
    private static ClientServer.Transaction transaction_;
    
    private Insets mInsets                 = new Insets(3,3,3,3);

    private Label       registerLabel;
    private Label       registerUserLabel;
    private Label       registerPasswordLabel;
    private TextField   registerUserField;
    private TextField   registerPasswordField;
    private Button      registerButton;

    private Button      queryButton;
    private Button      transferButton;

    private Button      queryCButton;
    private Button      querySButton;

    private Label       transferLabel;
    private TextField   transferField;
    private Button      transferCSButton;
    private Button      transferSCButton;
    
    private Button      disconnectButton;
    private Button      printButton;

    private Label       messageLabel;
    private TextArea    messageArea;

    private String      serverName;
    private String      portNo;
    private String      objectName;
    private long        transactionNo;
    private boolean     isConnectToCoordinator = false;

    private long startTime;
    private long stopTime;

    private ClientBeepingThread beepingThread;

    public
    void init() {
      super.init();
      setSize(320, 350);
      createGui();
    }

    /** Initialize ORB connection **/
   
    protected
    void setORBConnectionToCoordinator() {
      Properties props = new Properties();
      props.put("org.omg.CORBA.ORBClass", "com.sun.CORBA.iiop.ORB");
      props.put("org.omg.CORBA.ORBInitialHost", "osprey8.npac.syr.edu");
      props.put("org.omg.CORBA.ORBInitialPort", "9966");

      try {
        // create and initialize the ORB
        org.omg.CORBA.ORB theCoordinatorOrb_ =  org.omg.CORBA.ORB.init(this, props);

        // get the root naming context
        org.omg.CORBA.Object objRefCoordinator = 
                          theCoordinatorOrb_.resolve_initial_references("NameService");
        NamingContext ncRefCoordinator = NamingContextHelper.narrow(objRefCoordinator);

        // resolve the Object Reference in Naming
        NameComponent nc = new NameComponent("NPAC_Coordinator", "");
        NameComponent pathCoordinator[] = {nc};
        dispenser_ = ClientCoordinator.DispenserHelper.narrow(ncRefCoordinator.resolve(pathCoordinator));
      }
      catch (Exception ex) {
        System.out.println("Client: ORB-Coordinator Exception-> " + ex) ;
      }
    }

    protected
    void setORBConnectionToServer(String serverName, String portNo, 
                                  String objectName) {
      Properties props = new Properties();
      props.put("org.omg.CORBA.ORBInitialHost", serverName);
      props.put("org.omg.CORBA.ORBInitialPort", portNo);

      try {
        org.omg.CORBA.ORB theServerOrb_ =  org.omg.CORBA.ORB.init(this, props);

        // get the root naming context
        org.omg.CORBA.Object objRef =
                                theServerOrb_.resolve_initial_references("NameService");
        NamingContext ncRef = NamingContextHelper.narrow(objRef);

        // resolve the Object Reference in Naming
        NameComponent nc = new NameComponent(objectName, "");
        NameComponent path[] = {nc};
        accountManager_ = ClientServer.AccountManagerHelper.narrow(ncRef.resolve(path));
      }
      catch (Exception ex) {
        System.err.println("Client: ORB-Server Exception-> " + ex) ;
      }
    }

    protected 
    Button makeButton(String name,
                      int row,
                      GridBagLayout gridbag,
                      GridBagConstraints c) {

        c.gridy = row;
        Button button = new Button(name);
        button.addActionListener(this);
        gridbag.setConstraints(button, c);
        add(button);

        return button;
    }

    protected 
    Label makeLabel(String name,
                    int row,
                    int size,
                    GridBagLayout gridbag,
                    GridBagConstraints c) {

        c.gridy = row;
        Label label = new Label(name, Label.CENTER);
        label.setFont(new Font("Helvetica", Font.BOLD, size));
        gridbag.setConstraints(label, c);
        add(label);
  
        return label;
    }

    protected 
    TextField  makeTextField(String name,
                             int row,
                             boolean passwd,
                             GridBagLayout gridbag,
                             GridBagConstraints c) {

        c.gridy = row;
        TextField field = new TextField(15);
        if (passwd) {
            field.setEchoChar('*');
        }
        field.addActionListener(this);
        gridbag.setConstraints(field, c);
        add(field);

        return field;
    }

    protected 
    TextArea makeTextArea(String name,
                          int row,
                          GridBagLayout gridbag,
                          GridBagConstraints c) {

        c.gridy = row;
        TextArea area = new TextArea(name, 6, 60);
        gridbag.setConstraints(area, c);
        add(area);

        return area;
    }

    public
    void createGui() {
      setBackground(Color.lightGray);

      GridBagLayout gb = new GridBagLayout();
      GridBagConstraints gbc = new GridBagConstraints();
      setLayout(gb);

      int row = 0;

      gbc.anchor      = GridBagConstraints.NORTH;
      gbc.fill        = GridBagConstraints.BOTH;
      gbc.gridx       = 0;
      gbc.gridwidth   = 2;
      gbc.gridheight  = 2;
      gbc.weightx     = 1;
      gbc.weighty     = 1;
      registerLabel = makeLabel("NPAC Client", row, 16, gb, gbc);

      row += 3;

      gbc.anchor      = GridBagConstraints.CENTER;
      gbc.fill        = GridBagConstraints.NONE;
      gbc.gridwidth   = 1;
      gbc.gridheight  = 1;
      registerUserLabel = makeLabel("Account: ", row, 12, gb, gbc);

      gbc.gridx       = 1;
      registerUserField = makeTextField("", row, false, gb, gbc);

      row += 2;

      gbc.gridx       = 0;
      registerPasswordLabel = makeLabel("Password: ", row, 12, gb, gbc);

      gbc.gridx       = 1;
      registerPasswordField = makeTextField("", row, true, gb, gbc);

      row += 2;

      gbc.gridx       = 0;
      gbc.gridwidth   = 2;
      gbc.gridheight  = 2;
      registerButton = makeButton("Connect", row, gb, gbc);

      row += 2;

      gbc.gridwidth   = 1;
      gbc.gridheight  = 1;
      queryButton = makeButton("Query", row, gb, gbc);
      queryButton.setEnabled(false);

      gbc.gridx       = 1;
      transferButton = makeButton("Transfer", row, gb, gbc);
      transferButton.setEnabled(false);

      row += 5;

      gbc.gridx       = 0;
      querySButton = makeButton("Saving Account", row, gb, gbc);
      querySButton.setEnabled(false);

      gbc.gridx       = 1;
      queryCButton = makeButton("Checking Account", row, gb, gbc);
      queryCButton.setEnabled(false);

      row += 5;

      gbc.gridx       = 0;
      transferLabel = makeLabel("Transfer Amount: ", row, 12, gb, gbc);
      transferLabel.setEnabled(false);

      gbc.gridx       = 1;
      transferField = makeTextField("0", row, false, gb, gbc);
      transferField.setEnabled(false);

      row += 2;

      gbc.gridx       = 0;
      transferSCButton = makeButton("Saving ==> Checking", row, gb, gbc);
      transferSCButton.setEnabled(false);

      gbc.gridx       = 1;
      transferCSButton = makeButton("Checking ==> Saving", row, gb, gbc);
      transferCSButton.setEnabled(false);

      row += 5;

      gbc.gridx       = 0;
      printButton = makeButton("Print", row, gb, gbc);
      printButton.setEnabled(false);

      gbc.gridx       = 1;
      disconnectButton = makeButton("Exit", row, gb, gbc);
      disconnectButton.setEnabled(false);

      row += 5;

      gbc.anchor      = GridBagConstraints.SOUTH;
      gbc.fill        = GridBagConstraints.NONE;
      gbc.gridx       = 0;
      gbc.gridwidth   = 2;
      messageLabel = makeLabel("Transaction Log", row, 16, gb, gbc);

      row += 2;

      messageArea = makeTextArea("", row, gb, gbc);
      messageArea.setEditable(false);
   }

   /**
    *  Handle action events from various controls
    *  Overrides Component.action().
    */
   public
   void actionPerformed(ActionEvent pe) {
     // User selected month from the choice
     String actionLabel = pe.getActionCommand();

     if (actionLabel.equals("Connect")) {
         messageArea.replaceRange("", 0, messageArea.getText().length());
         String userid = registerUserField.getText();
         String passwd = registerPasswordField.getText();
         if (userid.length() == 0) {
             messageArea.replaceRange("Null username!  Try again!\n", 0, 
                                      messageArea.getText().length());
             return;
         }
         if (passwd.length() == 0) {
             messageArea.replaceRange("Null password!  Try again!\n", 0,
                                      messageArea.getText().length());
             return;
         }
         getAppletContext().showStatus("Connecting to NPAC Coordinator");
         if (!isConnectToCoordinator) {
             setORBConnectionToCoordinator();
             isConnectToICoordinator = true;
         }
         if ((registration_ = dispenser_.reserveRegistrationObject()) == null) {
             messageArea.replaceRange("Try again later.", 0, 
                                      messageArea.getText().length());
             return;
         }
         if (registration_.register(userid, passwd)) {
             startTime = System.currentTimeMillis();
             messageArea.append("Connected to NPAC Coordinator\n");

             StringTokenizer serviceInfo = 
                               new StringTokenizer(registration_.getServiceInfo());

             serverName = serviceInfo.nextToken();
             portNo = serviceInfo.nextToken();
             objectName = serviceInfo.nextToken();
             transactionNo = Long.parseLong(serviceInfo.nextToken());
             messageArea.append(serverName + "|" + portNo + "|" +
                              Long.toString(transactionNo) + "\n");
             dispenser_.releaseRegistrationObject(transactionNo);

             Date time = new Date();
             messageArea.append(time.toString() + "\n");
             // Connect to Server
             // URL url = new URL("http://" + serverName + ":1965/NPACSystem/client1.html")
             // getAppletContext().showDocument(url);
             setORBConnectionToServer(serverName, portNo, objectName);
             if ((transaction_ = accountManager_.connect(userid, passwd, 
                                    transactionNo, time.toString())) == null) {
                 messageArea.append("Null service object return\n");
                 messageArea.append("Try again\n");
                 return;
             }

             // Start beeping thread to infrom Server
             beepingThread = new ClientBeepingThread(transaction_);
             beepingThread.setPriority(java.lang.Thread.NORM_PRIORITY+1);
             beepingThread.setStopFlag(false);
             beepingThread.start();

             // Change the applet display
             registerButton.setEnabled(false);
             registerUserLabel.setEnabled(false);
             registerUserField.setText("");
             registerUserField.setEnabled(false);
             registerPasswordLabel.setEnabled(false);
             registerPasswordField.setText("");
             registerPasswordField.setEnabled(false);
             queryButton.setEnabled(true);
             transferButton.setEnabled(true);
             printButton.setEnabled(true);
             disconnectButton.setEnabled(true);
         }
         else {
             messageArea.append("Connection failed!\nTry again!\n");
         }
     }
     if (actionLabel.equals("Query")) {
         getAppletContext().showStatus("Query");
         querySButton.setEnabled(true);
         queryCButton.setEnabled(true);
         transferLabel.setEnabled(false);
         transferCSButton.setEnabled(false);
         transferSCButton.setEnabled(false);
         transferField.setEnabled(false);
         messageArea.append("Query\n");
     }
     if (actionLabel.equals("Transfer")) {
         getAppletContext().showStatus("Transfer");
         querySButton.setEnabled(false);
         queryCButton.setEnabled(false);
         transferCSButton.setEnabled(true);
         transferSCButton.setEnabled(true);
         transferField.setEnabled(true);
         transferField.requestFocus();
         transferLabel.setEnabled(true);
         messageArea.append("Transfer\n");
     }
     if (actionLabel.equals("Saving Account")) {
         getAppletContext().showStatus("Querying Saving Account");
         messageArea.append(transaction_.querySavingAccount(transactionNo));
     }
     if (actionLabel.equals("Checking Account")) {
         getAppletContext().showStatus("Querying Checking Account");
         messageArea.append(transaction_.queryCheckingAccount(transactionNo));
     }
     if (actionLabel.equals("Checking ==> Saving")) {
         String amount = transferField.getText();
         getAppletContext().showStatus("Transfering " + amount +  
                                                 " from Checking to Saving");
         float amountValue = Float.valueOf(amount).floatValue();
         messageArea.append(transaction_.transferFromCheckingToSaving(transactionNo, amountValue));
         transferField.setText("");
     }
     if (actionLabel.equals("Saving ==> Checking")) {
         String amount = transferField.getText();
         getAppletContext().showStatus("Transfering " + amount + 
                                                 " from Saving to Checking");
         float amountValue = Float.valueOf(amount).floatValue();
         messageArea.append(transaction_.transferFromSavingToChecking(transactionNo, amountValue));
         transferField.setText("");
     }
     if (actionLabel.equals("Print")) {
         getAppletContext().showStatus("Printing Trasaction Log");
         try {
             PrintWriter out = 
               new PrintWriter(new FileWriter(Long.toString(transactionNo) + ".log"));
             out.print(messageArea.getText());
             out.close();
         } catch (Exception ex) {
             System.err.println("Client: I/O Exception -> " + ex);
         }
         getAppletContext().showStatus("Done Printing");
     }
     if (actionLabel.equals("Exit")) {
         accountManager_.disconnect(transactionNo);
         beepingThread.stop(); // beepingThread.destroy();
         beepingThread.setStopFlag(true);
         stopTime = System.currentTimeMillis();
         messageArea.replaceRange("\nTransaction Elapsed Time: " + 
                                  Float.toString((float)(stopTime-startTime)/1000) + 
                                  "seconds\n", 0, messageArea.getText().length());
         registerButton.setEnabled(true);
         registerUserLabel.setEnabled(true);
         registerUserField.setEnabled(true);
         registerUserField.setText("");
         registerUserField.requestFocus();
         registerPasswordLabel.setEnabled(true);
         registerPasswordField.setEnabled(true);
         registerPasswordField.setText("");
         queryButton.setEnabled(false);
         transferButton.setEnabled(false);
         printButton.setEnabled(false);
         disconnectButton.setEnabled(false);
         queryCButton.setEnabled(false);
         querySButton.setEnabled(false);
         transferCSButton.setEnabled(false);
         transferSCButton.setEnabled(false);
         transferLabel.setEnabled(false);
         transferField.setEnabled(false);
         transferField.setText("");
     }
   }

    public void destroy() {
        accountManager_.disconnect(transactionNo);
        beepingThread.stop(); // this.beepingThread.destroy();
    }

    /**
     *  Paint nice raized 3D background.
     *  Overrides Component.paint().
     */
    public void
    paint(Graphics pg)
    {
        pg.setColor(Color.black);
        pg.drawLine(1, 0, getBounds().width-2, 0);
        pg.drawLine(0, 1, 0, getBounds().height-2);
        pg.drawLine(1, getBounds().height-1, getBounds().width - 2, getBounds().height - 1);
        pg.drawLine(getBounds().width - 1, 1, getBounds().width - 1, getBounds().height - 2);

        pg.setColor(getBackground());
        //pg.draw3DRect(1, 1, getBounds().width - 3, getBounds().height - 3, true);
        pg.fill3DRect(1, 1, getBounds().width - 2, getBounds().height - 2, true);
    }

    /**
     *  For better looks, let Layout Manager leave some margin.
     */
	public
	Insets
	getInsets()
	{
	    return mInsets;
	}
}
