import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.io.*;

import java.util.Properties;
import java.util.*;

import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextPackage.*;
import org.omg.CORBA.*;
import com.sun.CORBA.iiop.*;

import ConsoleCoordinator.*;

class
RefreshThread extends Thread {
    ConsoleApplet  ca;

    public RefreshThread(ConsoleApplet a) {
       ca = a;
    }

    public void run() {
       try {
           while (true) {
               ca.refreshServerStatus();
               ca.getAppletContext().showStatus("Refreshed");
               this.sleep(10000);
           }
       } catch (Exception ex) {
           System.err.println("Console: Refresh Thread Exception->" + ex);
       }
    }
}

public
class ConsoleApplet extends Applet implements ActionListener {
    /** ORB
    **/

    private Insets mInsets                 = new Insets(3,3,3,3);

    private Label       consoleLabel;
    private Label       consoleUserLabel;
    private Label       consolePasswordLabel;
    private TextField   consoleUserField;
    private TextField   consolePasswordField;
    private Button      consoleButton;

    private Button      startupButton;
    private Button      shutdownButton;
    private Button      exitButton;
    private Button      refreshButton;

    private int row = 0;
    private GridBagLayout gb = new GridBagLayout();
    private GridBagConstraints gbc = new GridBagConstraints();

    private Label       statusLabel;
    private int maxNumServers = 4;
    private int numServers_ = 0;
    private TextArea[] statusArea = new TextArea[maxNumServers];
    private Button[]    startButton = new Button[maxNumServers];
    private Button[]    stopButton = new Button[maxNumServers];
    private ConsoleCoordinator.CoordinatorManager coordinatorManager_;
    private ConsoleCoordinator.Register[] register_ =
                      new ConsoleCoordinator.Register[maxNumServers];
    private RefreshThread thread;

    public
    void init() {
      super.init();
      createGui();
      setSize(600, 350);
      thread = new RefreshThread(this);
    }

    /** Initialize ORB connection **/

    public
    void setORBConnectionToCoordinator() {
      try {
        Properties props = new Properties();
        props.put("org.omg.CORBA.ORBClass", "com.sun.CORBA.iiop.ORB");
        props.put("org.omg.CORBA.ORBInitialHost", "osprey8.npac.syr.edu");
        props.put("org.omg.CORBA.ORBInitialPort", "9966");

        // create and initialize the ORB
        org.omg.CORBA.ORB theOrb_ =  org.omg.CORBA.ORB.init(this, props);

        // get the root naming context
        org.omg.CORBA.Object objRef = 
                          theOrb_.resolve_initial_references("NameService");
        NamingContext ncRef = NamingContextHelper.narrow(objRef);

        // resolve the Object Reference in Naming
        NameComponent nc = new NameComponent("Coordinator", "");
        NameComponent path[] = {nc};
        coordinatorManager_ = ConsoleCoordinator.CoordinatorManagerHelper.narrow(
                                     ncRef.resolve(path));
      }
      catch (Exception ex) {
        ex.printStackTrace();
        System.out.println("Console: ORB Exception-> " + ex) ;
      }
    }

    protected 
    Button makeButton(String name) {

        gbc.gridy = row;
        Button button = new Button(name);
        button.addActionListener(this);
        gb.setConstraints(button, gbc);
        add(button);

        return button;
    }

    protected 
    Label makeLabel(String name,
                    int size,
                    int position) {

        gbc.gridy = row;
        Label label;
        if (position == 0) {
            label = new Label(name, Label.CENTER);
        } else {
            label = new Label(name, Label.RIGHT);
        }
         
        label.setFont(new Font("Helvetica", Font.BOLD, size));
        gb.setConstraints(label, gbc);
        add(label);
  
        return label;
    }

    protected 
    TextField  makeTextField(String name,
                             boolean passwd) {

        gbc.gridy = row;
        TextField field = new TextField(12);
        if (passwd) {
            field.setEchoChar('*');
        }
        field.addActionListener(this);
        gb.setConstraints(field, gbc);
        add(field);

        return field;
    }

    protected void addServerStatus(int id) {
        gbc.gridx       = 0;
        gbc.gridy       = row;
        gbc.gridwidth   = 2;
        statusArea[id] = new TextArea("", 1, 30, java.awt.TextArea.SCROLLBARS_NONE);
        statusArea[id].setEditable(false);
        gb.setConstraints(statusArea[id], gbc);
        add(statusArea[id]);
        statusArea[id].setEnabled(true);
        gbc.gridx       = 2;
        gbc.gridwidth   = 1;
        startButton[id] = new Button("Start" + Integer.toString(id));
        startButton[id].addActionListener(this);
        gb.setConstraints(startButton[id], gbc);
        add(startButton[id]);
        startButton[id].setEnabled(false);
        gbc.gridx       = 3;
        stopButton[id] = new Button("Stop" + Integer.toString(id));
        stopButton[id].addActionListener(this);
        gb.setConstraints(stopButton[id], gbc);
        add(stopButton[id]);
        stopButton[id].setEnabled(false);
        row += 2;
    }
    
    public
    void createGui() {
      setBackground(Color.lightGray);
      setLayout(gb);

      gbc.anchor      = GridBagConstraints.NORTH;
      gbc.fill        = GridBagConstraints.BOTH;
      gbc.gridx       = 0;
      gbc.gridwidth   = GridBagConstraints.REMAINDER;
      gbc.gridheight  = 2;
      gbc.weightx     = 1;
      gbc.weighty     = 1;
      consoleLabel = makeLabel("NPAC Console", 16, 0);

      row += 3;

      gbc.anchor      = GridBagConstraints.CENTER;
      gbc.gridx       = 1;
      gbc.gridwidth   = 1;
      gbc.gridheight  = 1;
      consoleUserLabel = makeLabel("Account: ", 12, 1);

      gbc.gridx       = 2;
      consoleUserField = makeTextField("", false);

      row += 2;

      gbc.gridx       = 1;
      consolePasswordLabel = makeLabel("Password: ", 12, 1);

      gbc.gridx       = 2;
      consolePasswordField = makeTextField("", true);

      row += 2;

      gbc.gridx       = 1;
      gbc.gridwidth   = 2;
      consoleButton = makeButton("Connect");

      row += 10;

      gbc.gridx       = 0;
      gbc.gridwidth   = GridBagConstraints.REMAINDER;
      gbc.gridheight  = 2;
      statusLabel = makeLabel("Server Status", 16, 0);

      row += 5;

      gbc.gridx       = 0;
      gbc.gridwidth   = 1;
      gbc.gridheight  = 1;
      startupButton = makeButton("Startup");
      startupButton.setEnabled(false);

      gbc.gridx       = 1;
      shutdownButton = makeButton("Shutdown");
      shutdownButton.setEnabled(false);

      gbc.gridx       = 2;
      refreshButton = makeButton("Refresh");
      refreshButton.setEnabled(false);

      gbc.gridx       = 3;
      exitButton = makeButton("Exit");
      exitButton.setEnabled(false);

      row += 10;

      gbc.weightx     = 1.0;
      gbc.gridwidth   = 1;
      gbc.gridheight  = 1;
      addServerStatus(0);
      addServerStatus(1);
      addServerStatus(2);
      addServerStatus(3);
   }

   public void refreshServerStatus() {
     StringTokenizer st = new StringTokenizer(coordinatorManager_.refresh(), "+");
     for (int i = 0; i < numServers_; i++) {
         String s = st.nextToken();
         statusArea[i].replaceRange(s, 0, statusArea[i].getText().length());
     }
   }

   /**
    *  Handle action events from various controls
    *  Overrides Component.action().
    */
   public void actionPerformed(ActionEvent pe) {
     // User selected month from the choice
     String actionLabel = pe.getActionCommand();
     if (actionLabel.equals("Connect")) {
         getAppletContext().showStatus("Connecting to NPAC Coordinator");
         setORBConnectionToCoordinator();
         String userid = consoleUserField.getText();
         String passwd = consolePasswordField.getText();

         if (!thread.isAlive()) {
             thread.start();
         }
         numServers_ = coordinatorManager_.numServers();

         if (coordinatorManager_.authentication(userid, passwd)) {
             consoleButton.setEnabled(false);
             consoleUserLabel.setEnabled(false);
             consoleUserField.setText("");
             consoleUserField.setEnabled(false);
             consolePasswordLabel.setEnabled(false);
             consolePasswordField.setText("");
             consolePasswordField.setEnabled(false);

             if (coordinatorManager_.checkServerStatus()) {
                 shutdownButton.setEnabled(true);
                 refreshButton.setEnabled(true);
                 exitButton.setEnabled(true);
                 for (int i = 0; i < numServers_; i++) {
                     register_[i] = coordinatorManager_.nextRegister();
                     startButton[i].setEnabled(true);
                     stopButton[i].setEnabled(true);
                 }
             } else {
                 startupButton.setEnabled(true);
                 exitButton.setEnabled(true);
             }
            getAppletContext().showStatus("Login successful");
         } else {
            getAppletContext().showStatus("Login failed");
         }
         refreshServerStatus();
     } else if (actionLabel.equals("Startup")) {
         getAppletContext().showStatus("Starting up NPAC services");
         coordinatorManager_.startup();
         shutdownButton.setEnabled(true);
         refreshButton.setEnabled(true);
         for (int i = 0; i < numServers_; i++) {
             register_[i] = coordinatorManager_.nextRegister();
             startButton[i].setEnabled(true);
             stopButton[i].setEnabled(true);
         }
         refreshServerStatus();
         startupButton.setEnabled(false);
         getAppletContext().showStatus("NPAC services are up");
     } else if (actionLabel.equals("Shutdown")) {
         getAppletContext().showStatus("Shutting down NPAC services");
         if (coordinatorManager_.shutdown()) {
             shutdownButton.setEnabled(false);
             refreshButton.setEnabled(false);
             for (int i = 0; i < maxNumServers; i++) {
                 startButton[i].setEnabled(false);
                 stopButton[i].setEnabled(false);
             }
             startupButton.setEnabled(true);
             refreshServerStatus();
             getAppletContext().showStatus("NPAC services are down");
         } else {
             getAppletContext().showStatus("Error in shutting down NPAC services");
         }
     } else if (actionLabel.equals("Refresh")) {
         getAppletContext().showStatus("Refreshing");
         refreshServerStatus();
         getAppletContext().showStatus("Refreshed");
     } else if (actionLabel.equals("Exit")) {
         thread.stop(); // thread.destroy();
         getAppletContext().showStatus("Exiting");
         consoleButton.setEnabled(true);
         consoleUserLabel.setEnabled(true);
         consoleUserField.setEnabled(true);
         consoleUserField.setText("");
         consoleUserField.requestFocus();
         consolePasswordLabel.setEnabled(true);
         consolePasswordField.setEnabled(true);
         consolePasswordField.setText("");
         exitButton.setEnabled(false);
         startupButton.setEnabled(false);
         shutdownButton.setEnabled(false);
         refreshButton.setEnabled(false);
         for (int i = 0; i < numServers_; i++) {
             startButton[i].setEnabled(false);
             stopButton[i].setEnabled(false);
             statusArea[i].setText("");
             statusArea[i].setEnabled(false);
         }
         getAppletContext().showStatus("Exited");
     } else {
         int serverNo = 
            Character.digit(actionLabel.charAt(actionLabel.length()-1), 10);
         if (actionLabel.length() == 5) {
             getAppletContext().showStatus("Stop Server" + Integer.toString(serverNo));
             register_[serverNo].stop();
         } else {
             getAppletContext().showStatus("Start Server" + Integer.toString(serverNo));
             register_[serverNo].start();
         }
     }
   }

    public void destroy() {
        thread.stop(); // thread.destroy();
    }

    /**
     *  Paint nice raized 3D background.
     *  Overrides Component.paint().
     */
    public void
    paint(Graphics pg)
    {
        pg.setColor(Color.black);
        pg.drawLine(1, 0, getBounds().width-2, 0);
        pg.drawLine(0, 1, 0, getBounds().height-2);
        pg.drawLine(1, getBounds().height-1, getBounds().width - 2, getBounds().height - 1);
        pg.drawLine(getBounds().width - 1, 1, getBounds().width - 1, getBounds().height - 2);

        pg.setColor(getBackground());
        //pg.draw3DRect(1, 1, getBounds().width - 3, getBounds().height - 3, true);
        pg.fill3DRect(1, 1, getBounds().width - 2, getBounds().height - 2, true);
    }

    /**
     *  For better looks, let Layout Manager leave some margin.
     */
	public
	Insets
	getInsets()
	{
	    return mInsets;
	}
}
