import java.io.*;
import java.net.*;
import java.awt.*;
import java.util.Properties;

import org.omg.CORBA.*;
import com.sun.CORBA.iiop.*;
import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextPackage.*;

import ClientCoordinator.*;
import ConsoleCoordinator.*;
// import CoordinatorConsole.*;
import CoordinatorServer.*;
// import ServerCoordinator.*;

class
RegistrationServantImpl extends ClientCoordinator._RegistrationImplBase {

    private String serverName_;
    private String portNo_;
    private String objectName_;
    private long trNo_;
    private String clientTrNo;
    private String id_;
    private  CoordinatorServer.Acknowledge acknowledge_;
    private String[] args_;

    public RegistrationServantImpl(String[] args, String id, 
                                   String serverName, String portNo) {
                                   
        super();
        id_ = id;
        serverName_ = serverName;
        portNo_ = portNo; 
        args_ = args;
    
        try {
            Properties props = new Properties();
            props.put("org.omg.CORBA.ORBInitialHost", serverName_);
            props.put("org.omg.CORBA.ORBInitialPort", portNo_);

            org.omg.CORBA.ORB theOrb =  org.omg.CORBA.ORB.init(args_, props);
            org.omg.CORBA.Object objRef =
                                    theOrb.resolve_initial_references("NameService");
            NamingContext ncRef = NamingContextHelper.narrow(objRef);

            NameComponent nc = new NameComponent("Server_" + portNo_, "");  // need id
            NameComponent path[] = {nc};
            acknowledge_ =
                  CoordinatorServer.AcknowledgeHelper.narrow(ncRef.resolve(path));
        } catch (Exception ex) {
            System.err.println("Registration-" + portNo_ + ": Exception-> " + ex) ;
        }
    }

    public synchronized void setTranNo(long no) {
        trNo_ = no;
    }

    public String getServiceInfo() {
        return serverName_ + " " + portNo_ + " " + objectName_ + " " + clientTrNo;
    }
                           
    public synchronized boolean register(String userName, String password) {
        clientTrNo = Long.toString(trNo_);
        objectName_ = "AccountManager_" + portNo_;  // need to add id

        // Connect to Server to setup service objects
        if (acknowledge_.announceClientInfo(userName, password, trNo_)) {
            return true;
        }
        System.out.println(userName + "|" + password + " obtains " + objectName_);
        return false;
    }

    public boolean checkServer() {
        return acknowledge_.checkServer();
    }

    public int getNumActiveClients() {
        return acknowledge_.numActiveClients();
    }

    public int getNumMaxClients() {
        return acknowledge_.numMaxClients();
    }
}

class RegistrationStatus {
    String serverName;
    String portNo;
    RegistrationServantImpl registrationServant;
    boolean inUse;
    boolean isAvailable;
    boolean isReady;
    long transactionNo;

    RegistrationStatus(String serverName_, String portNo_) {
        serverName = serverName_;
        portNo = portNo_;
        registrationServant = null;
        inUse = false;
        transactionNo = 0;
        isAvailable = false;
        isReady = false;
    }
}

class 
RegisterServantImpl extends ConsoleCoordinator._RegisterImplBase {
    public String serverName;
    public String portNo;
    private RegistrationStatus reg;

    RegisterServantImpl(RegistrationStatus reg_) {
        serverName = reg_.serverName;
        portNo = reg_.portNo;
        reg = reg_;
    }

    public synchronized void start() {
        if (reg.isReady) {
            reg.isAvailable = true;
        } else {
            reg.isAvailable = false;
        }
    }

    public synchronized void stop() {
        if (reg.isReady) {
            reg.isAvailable = false;
        }
    }
}

class
DispenserServantImpl extends ClientCoordinator._DispenserImplBase {

    private String[] serverList = {"osprey1.npac.syr.edu",
                                   "osprey2.npac.syr.edu",
                                   "osprey3.npac.syr.edu",
                                   "osprey4.npac.syr.edu"};
    private String[] portList = {"9966", "9966", "9966", "9966"};
    private int numServiceObjects;
    RegistrationStatus[] registrationStatus;
    private long trNo;
    private int queuePtr;
    private int queueHead;
    private int queueTail;

    public DispenserServantImpl(String[] args, org.omg.CORBA.ORB orb, int servers) {
        numServiceObjects = servers;
        trNo = 1;
        queuePtr = 0;
        queueHead = 0;
        queueTail = numServiceObjects;
        registrationStatus = new RegistrationStatus[numServiceObjects];

        try {
            for (int i = 0 ;i < numServiceObjects; i++) {
                registrationStatus[i] =
                                 new RegistrationStatus(serverList[i],
                                                        portList[i]);
                registrationStatus[i].registrationServant =
                     new RegistrationServantImpl(args,
                                                 Integer.toString(i),
                                                 registrationStatus[i].serverName,
                                                 registrationStatus[i].portNo);
                orb.connect(registrationStatus[i].registrationServant);
            }
        }
        catch (Exception ex) {
            System.err.println("Coordinator: Exception-> " + ex);
        }
    }

    public synchronized ClientCoordinator.Registration reserveRegistrationObject() {
        while (queuePtr < queueTail) {
            if (registrationStatus[queuePtr].isAvailable) {
                if (!registrationStatus[queuePtr].inUse) {
                    if (registrationStatus[queuePtr].registrationServant.checkServer()) {
                        registrationStatus[queuePtr].inUse = true;
                        registrationStatus[queuePtr].transactionNo = trNo;
                        registrationStatus[queuePtr].registrationServant.setTranNo(trNo++);
                        queueHead = queuePtr + 1;
                        return registrationStatus[queuePtr++].registrationServant;
                    }
                    System.out.println("checkServer Error");
                }
            }
            queuePtr++;
        }
        queuePtr = 0;
        while (queuePtr < queueHead) {
            if (registrationStatus[queuePtr].isAvailable) {
                if (!registrationStatus[queuePtr].inUse) {
                    if (registrationStatus[queuePtr].registrationServant.checkServer()) {
                        registrationStatus[queuePtr].inUse = true;
                        registrationStatus[queuePtr].transactionNo = trNo;
                        registrationStatus[queuePtr].registrationServant.setTranNo(trNo++);
                        System.out.println("ServiceObject" + queuePtr + " reserved.");
                        queueHead = queuePtr + 1;
                        return registrationStatus[queuePtr++].registrationServant;
                    }
                    System.out.println("checkServer Error");
                }
            }
            queuePtr++;
        }

        System.out.println(queuePtr + " " + queueHead + " " + queueTail + 
                           " " + numServiceObjects);
        return null;
    }

    public synchronized void releaseRegistrationObject(long transactionNo_) {
        for (int i = 0; i < numServiceObjects; i++) {
            if (registrationStatus[i].transactionNo == transactionNo_) {
                registrationStatus[i].inUse = false;
                return;
            }
        }
        System.out.println("Reserved ServiceObject not found.");
        return;
    }
}

class
CoordinatorManagerServantImpl extends ConsoleCoordinator._CoordinatorManagerImplBase {
    private RegisterServantImpl[] register_;
    private int numServers_;
    private int currentRegister = -1;
    private boolean startupFlag = false;
    private DispenserServantImpl dispenser_;

    public CoordinatorManagerServantImpl(DispenserServantImpl disp, int servers) {
        numServers_ = servers;
        dispenser_ = disp;
        register_ = new RegisterServantImpl[numServers_];
        for (int i = 0; i < numServers_; i++) {
            register_[i] = new RegisterServantImpl(disp.registrationStatus[i]);
        }
    }

    public synchronized ConsoleCoordinator.Register nextRegister() {
        if (++currentRegister >= numServers_) {
           currentRegister = 0;
        }
        return register_[currentRegister];
    }

    public int numServers() {
        return this.numServers_;
    }

    public String refresh() {
        StringBuffer serverInfo = new StringBuffer();
        for (int i = 0; i < numServers_ ; i++) {
            serverInfo.append(register_[i].serverName + ":");
            serverInfo.append(register_[i].portNo + "   ");
            int numActC = 
              dispenser_.registrationStatus[i].registrationServant.getNumActiveClients();
            int numMaxC =
              dispenser_.registrationStatus[i].registrationServant.getNumMaxClients();
            if (dispenser_.registrationStatus[i].registrationServant.checkServer()) {
                if (dispenser_.registrationStatus[i].isReady) {
                    if (dispenser_.registrationStatus[i].isAvailable) {
                        if (dispenser_.registrationStatus[i].inUse) {
                            serverInfo.append("Wait   ");
                            serverInfo.append(Integer.toString(numActC));
                            serverInfo.append("/");
                            serverInfo.append(Integer.toString(numMaxC));
                        } else {
                            serverInfo.append("Ready   ");
                            serverInfo.append(Integer.toString(numActC));
                            serverInfo.append("/");
                            serverInfo.append(Integer.toString(numMaxC));
                        }
                    } else {
                        serverInfo.append("Halt   ");
                        serverInfo.append("0/");
                        serverInfo.append(Integer.toString(numMaxC));
                    }
                } else {
                  serverInfo.append("Off");
                }
            } else {
                dispenser_.registrationStatus[i].isAvailable = false;
                serverInfo.append("Busy   ");
                serverInfo.append(Integer.toString(numActC));
                serverInfo.append("/");
                serverInfo.append(Integer.toString(numMaxC));
            }
            if (i < (numServers_-1)) {
                serverInfo.append("+");
            }
        }
        return serverInfo.toString();
    }

    public synchronized boolean startup() {
        startupFlag = true;
        for (int i = 0; i < numServers_; i++) {
            dispenser_.registrationStatus[i].inUse = false;
            dispenser_.registrationStatus[i].isAvailable = true;
            dispenser_.registrationStatus[i].isReady = true;
        }
        return true;
    }

    public synchronized boolean shutdown() {
        for (int i = 0; i < numServers_; i++) {
            dispenser_.registrationStatus[i].inUse = false;
            dispenser_.registrationStatus[i].isAvailable = false;
            dispenser_.registrationStatus[i].isReady = false;
        }
        startupFlag = false;
        return true;
    }

    public synchronized boolean authentication(String username, String password) {
        if (username.equals("cwou")) {
            if (password.equals("Jessica")) {
                return true;
            }
        }
        System.out.println("Username: " + username + "/" + password);
        return false;
       
    }

    public boolean checkServerStatus() {
        return(this.startupFlag);
    }
}

public class Coordinator {

  public static org.omg.CORBA.ORB theOrb_;
  private static int numServers = 2;  // UPDATE SERVERS

  public static void main(String[] args) {
    try {
      // create and initialize ORB
      Properties props = new Properties();
      props.put("org.omg.CORBA.ORBInitialPort", "9966");
      theOrb_ =  org.omg.CORBA.ORB.init(args, props);

      org.omg.CORBA.Object objRef = theOrb_.resolve_initial_references("NameService");
      NamingContext ncRef = NamingContextHelper.narrow(objRef);

      DispenserServantImpl dispenserServant =
                         new DispenserServantImpl(args, theOrb_, numServers);
      theOrb_.connect(dispenserServant);

      NameComponent nc = new NameComponent("NPAC_Coordinator", "");
      NameComponent path[] = {nc};
      ncRef.rebind(path, dispenserServant);

      CoordinatorManagerServantImpl coordinatorManager =
                   new CoordinatorManagerServantImpl(dispenserServant, numServers);
      theOrb_.connect(coordinatorManager);

      NameComponent nc1 = new NameComponent("Coordinator", "");
      NameComponent path1[] = {nc1};
      ncRef.rebind(path1, coordinatorManager);


      System.out.println("Coordinator Started\n");

      // wait for invocations from clients
      java.lang.Object sync = new java.lang.Object();
      synchronized (sync) {
         sync.wait();
      }
    }
    catch (Exception ex) {
      ex.printStackTrace();
      System.err.println("NPAC Coordinator: Exception-> " + ex);
    }
  }
}
