
import org.omg.CORBA.*;
import com.sun.CORBA.iiop.*;
import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextPackage.*;   

import ServerDB.*;

class DBServiceServantImpl extends ServerDB._DBServiceImplBase {
    private BankDB db;

    public DBServiceServantImpl() {
        super();
        db = new BankDB();
    }

    public boolean open(String username, String password, long trNo,
            String time, String zone) {
        try {
            db.open(username, password, trNo, time, zone);
        } catch (Exception ex) {
            System.err.println("DBServer Exception ->" + ex);
            return false;
        }
        return true;
    }

    public String querySaving()  {
        String msg = null;
        try {
           msg = db.querySaving();
        } catch (Exception ex) {
            System.err.println("DBServer Exception ->" + ex);
        }
        return msg;
    }	

    public String queryChecking() {
        String msg = null;
        try {
            msg = db.queryChecking();
        } catch (Exception ex) {
            System.err.println("DBServer Exception ->" + ex);
        }
        return msg;
    }

    public String transferSC(float trAmount) {
        String msg = null;
        try {
            msg = db.transferSC(trAmount);
        } catch (Exception ex) {
            System.err.println("DBServer Exception ->" + ex);
        }
        return msg;
    }

    public String transferCS(float trAmount) {
        String msg = null;
        try {
            msg = db.transferCS(trAmount);
        } catch (Exception ex) {
            System.err.println("DBServer Exception ->" + ex);
        }
        return msg;
    }

    public void close() {
        try {
            db.close();
        } catch (Exception ex) {
            System.err.println("DBServer Exception ->" + ex);
        }
    }
}

class DBIServantImpl extends ServerDB._DBIImplBase {
    private int numServiceObjects = 5;
    DBServiceServantImpl[] dbs = new
                              DBServiceServantImpl[numServiceObjects];

    public DBIServantImpl(org.omg.CORBA.ORB orb) {
        for (int i = 0 ;i < numServiceObjects; i++) {
            dbs[i] = new DBServiceServantImpl();
            orb.connect(dbs[i]);
        } 
    }

    public synchronized ServerDB.DBService reserveService(int id){
        return dbs[id];
    }

    public synchronized void releaseService(int id) {
    }
}

public class DBServer {
    public static void main(String[] args) {
        int i = 0;
        while (!args[i++].equals("-ID")) {};
        String id = args[i];
        System.out.println("DBServer " + id);

        try {
            // create and initialize ORB
            org.omg.CORBA.ORB theOrb_ =  org.omg.CORBA.ORB.init(args, null);
            org.omg.CORBA.Object objRef =
                         theOrb_.resolve_initial_references("NameService");
            NamingContext ncRef = NamingContextHelper.narrow(objRef);
            DBIServantImpl dbi = new DBIServantImpl(theOrb_);
            theOrb_.connect(dbi);
            NameComponent nc = new NameComponent("DBServer_" + id, "");
            NameComponent path[] = {nc};
            ncRef.rebind(path, dbi);
            System.out.println("DBServer_" + id + " Started");

            // wait for invocations from clients
            java.lang.Object sync = new java.lang.Object();
            synchronized (sync) {
               sync.wait();
            }
        } catch (Exception ex) {
            System.err.println("DBServer_" + id + " Exception -> " + ex);
        }
    }
}

