import java.io.*;
import java.net.*;
import java.awt.*;
import java.util.Properties;
import java.util.*;

import org.omg.CORBA.*;
import com.sun.CORBA.iiop.*;
import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextPackage.*;

import AdministratorICoordinator.*;
import ICoordinatorDB.*;

class
AdministrationServantImpl extends AdministratorICoordinator._AdministrationImplBase {
    boolean inUse;
    String username;
    String password;
    String time;
    String zone;
    private ICoordinatorDB.AdminDBService adb_;
    private Hashtable matchMap;

    public 
    AdministrationServantImpl (ICoordinatorDB.AdminDBI adbi, int id) {
        inUse = false;
        username = "";
        password = "";
 
        matchMap = new Hashtable();
        adb_ = adbi.reserve(id);
    }

    protected void 
    setTime(String timein) {
        StringTokenizer st = new StringTokenizer(timein);
        st.nextToken();
        String tmptime = st.nextToken() + " " + st.nextToken() + " " + 
                         st.nextToken();
        zone = st.nextToken();
        time = tmptime + " " + st.nextToken();
    }

    public synchronized boolean
    addUser(String userid, String lastName, String firstName, String address, 
            String hPhone, String wPhone, String status, float checking, 
            float saving, String timein) {
        setTime(timein);
        return adb_.addUser(userid, lastName, firstName, address, hPhone, wPhone, 
                     status, checking, saving, time, zone);
    }

    public synchronized boolean
    deleteUser(String userid, String timein) {
        setTime(timein);
        return adb_.deleteUser(userid, time, zone);
    }

    public synchronized String 
    searchUser(String userid, String lastName, String firstName, String hPhone) {
        matchMap.clear();
        String s = adb_.searchUser(userid, lastName, firstName, hPhone);
        StringTokenizer st = new StringTokenizer(s, "|");
        StringBuffer returnBuffer = new StringBuffer();
        while (st.hasMoreTokens()) {
            String record = st.nextToken();
            StringTokenizer rst = new StringTokenizer(record, "+");
            String uid = rst.nextToken();
            matchMap.put(uid, record);
            returnBuffer.append(uid + "+" + rst.nextToken() +
                                ", " + rst.nextToken() + "|");
        }
        return returnBuffer.toString();
    }

    public synchronized String
    getUserData(String userid) {
        return((String)matchMap.get(userid));
    }

    public synchronized String 
    showBalance(String userid) {
        return(adb_.showBalance(userid));
    }

    public synchronized boolean
    updateUser(String userid, String lastName, String firstName, String address,
               String hPhone, String wPhone, String status, float checking, 
               float saving, String timein) {
        setTime(timein);
        return adb_.updateUser(userid, lastName, firstName, address, hPhone, wPhone,
                     status, checking, saving, time, zone);
    }

    protected synchronized boolean
    open(String timein) {
        setTime(timein);
        return adb_.open(username, password, time, zone);
    }

    protected synchronized void
    close(String timein) {
        setTime(timein);
        adb_.close(time, zone);
    }
}

class 
AdministrationManagerServantImpl extends 
        AdministratorICoordinator._AdministrationManagerImplBase {
    private int numServiceObjects = 5;
    private int numServiceObjectsInUse = 0;
    private String serverInfo;
    AdministrationServantImpl[] admin = 
                          new AdministrationServantImpl[numServiceObjects];

    public 
    AdministrationManagerServantImpl(org.omg.CORBA.ORB orb) {
        String[] argv = new String[4];
        argv[0] = new String("-ORBInitialHost");
        argv[1] = new String("osprey8.npac.syr.edu");
        argv[2] = new String("-ORBInitialPort");
        argv[3] = new String("9966");
        ICoordinatorDB.AdminDBI dbi_;

      try {
        // create and initialize the ORB
        org.omg.CORBA.ORB theOrb_ =  org.omg.CORBA.ORB.init(argv, null);

        // get the root naming context
        org.omg.CORBA.Object objRef =
                         theOrb_.resolve_initial_references("NameService");
        NamingContext ncRef = NamingContextHelper.narrow(objRef);

        // resolve the Object Reference in Naming
        NameComponent nc = new NameComponent("AdminDBServer", "");
        NameComponent path[] = {nc};
        dbi_ = ICoordinatorDB.AdminDBIHelper.narrow(ncRef.resolve(path));

        for (int i = 0 ;i < numServiceObjects; i++) {
            admin[i] = new AdministrationServantImpl(dbi_, i);
            orb.connect(admin[i]);
        }
      }
      catch (Exception ex) {
        System.err.println("AdministrationManager: Exception-> " + ex);
      }
    }

    public synchronized AdministratorICoordinator.Administration
    connect(String username, String password, String timein) {

        for (int i = 0; i < numServiceObjects; i++) {
            if (!admin[i].inUse) {
                admin[i].username = username;
                admin[i].password = password;
                if (admin[i].open(timein)) {
                    admin[i].inUse = true;
                    numServiceObjectsInUse++;
                    return admin[i];
                } else {
                    System.out.println("ICoordinator: DB Error");
                    return null;
                }
            }
        }
        System.out.println("ICoordinator: Busy");
        return null;
    }

    public synchronized void 
    disconnect(AdministratorICoordinator.Administration adm, String timein) {
        for (int i = 0; i < numServiceObjects; i++) {
            if (admin[i] == adm) {
                releaseServiceObject(i, timein);
                return;
            }
        }
        System.out.println("Inter Coordinator: reserved admin object not found");
    }

    public void 
    releaseServiceObject(int i, String timein) {
        admin[i].inUse = false;
        admin[i].username = null;
        admin[i].password = null;
        numServiceObjectsInUse--;
        admin[i].close(timein);
    }
}

public class ICoordinator {

    public static org.omg.CORBA.ORB theOrb_;

    public static void main(String[] args) {
        try {
            // create and initialize ORB
            theOrb_ =  org.omg.CORBA.ORB.init(args, null);
            
            // create Servant and register it with ORB
            AdministrationManagerServantImpl adminManager =
                new AdministrationManagerServantImpl(theOrb_);
            theOrb_.connect(adminManager);

            // get the root naming context
            org.omg.CORBA.Object objRef = 
                         theOrb_.resolve_initial_references("NameService");
            NamingContext ncRef = NamingContextHelper.narrow(objRef);

            // bind the Object Reference in Naming
            NameComponent nc = new NameComponent("InternalCoordinator", "");
            NameComponent path[] = {nc};
            ncRef.rebind(path, adminManager);

            System.out.println("ICoordinator started\n");

            // wait for invocations from clients
            java.lang.Object sync = new java.lang.Object();
            synchronized (sync) {
               sync.wait();
            }
        } catch (Exception ex) {
            System.err.println("ICoordinator: Exception-> " + ex);
        }
    }
}
