import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.io.*;

import java.util.Properties;
import java.util.*;

import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextPackage.*;
import org.omg.CORBA.*;
import com.sun.CORBA.iiop.*;

import MonitorServer.*;

class
MonitorThread extends Thread {
    private MonitorApplet a;
    private MonitorServer.Monitor m;
    private MonitorPanel mp;

    public MonitorThread(MonitorApplet a_) {
        a = a_;
        m = a.getMonitor();
        mp = a.getMonitorPanel();
    }

    public void run() {
        try {
            while (true) {
                mp.refreshStatus(m.getServerStatus());
                this.sleep(5000);
            }
        } catch (Exception ex) {
            System.err.println("Monitor: Thread Exception->" + ex);
        }
    }
}

class MonitorPanel extends Panel {
    private Insets mInsets                 = new Insets(3,3,3,3);
    private Button      exitButton;
    private Button      refreshButton;
    private static final int maxNumServices = 10;
    private Label       statusLabel;
    private TextArea    statusArea;
    private GridBagLayout gb = new GridBagLayout();
    private GridBagConstraints gbc = new GridBagConstraints();
    private int row = 0;
    private MonitorApplet parent;
    private ActionListener al;

    public MonitorPanel (MonitorApplet a) {
        al = (ActionListener)a;

        setBackground(Color.lightGray);
        setLayout(gb);

        gbc.anchor      = GridBagConstraints.NORTH;
        gbc.fill        = GridBagConstraints.BOTH;
        gbc.gridx       = 0;
        gbc.gridwidth   = GridBagConstraints.REMAINDER;
        gbc.gridheight  = 2;
        gbc.weightx     = 1;
        gbc.weighty     = 1;
        statusLabel = makeLabel("Server Status", 16, 0);

        gbc.anchor      = GridBagConstraints.CENTER;
        gbc.fill        = GridBagConstraints.NONE;

        row += 5;

        gbc.gridx       = 0;
        gbc.gridwidth   = 1;
        gbc.gridheight  = 1;
        refreshButton = makeButton("Refresh");

        gbc.gridx       = 1;
        exitButton = makeButton("Exit");

        gbc.gridwidth   = GridBagConstraints.REMAINDER;

        row += 10;

        gbc.gridx       = 0;
        gbc.gridwidth   = 2;
        statusArea = makeTextArea("");
    }

    protected
    Button makeButton(String name) {

        gbc.gridy = row;
        Button button = new Button(name);
        button.addActionListener(al);
        gb.setConstraints(button, gbc);
        add(button);

        return button;
    }

    protected
    Label makeLabel(String name,
                    int size,
                    int position) {

        gbc.gridy = row;
        Label label;
        if (position == 0) {
            label = new Label(name, Label.CENTER);
        } else {
            label = new Label(name, Label.RIGHT);
        }

        label.setFont(new Font("Helvetica", Font.BOLD, size));
        gb.setConstraints(label, gbc);
        add(label);

        return label;
    }

    protected
    TextArea makeTextArea(String name) {

        gbc.gridy = row;
        TextArea area = new TextArea(name, maxNumServices, 50,
                                     java.awt.TextArea.SCROLLBARS_NONE);
        area.setEditable(false);
        gb.setConstraints(area, gbc);
        add(area);

        return area;
    }

    public void refreshStatus(String msg) {
        statusArea.replaceRange(msg, 0,
                                statusArea.getText().length());
    }

    public Insets
    getInsets()
    {
        return mInsets;
    }
}

public
class MonitorApplet extends Applet implements ActionListener {

    private ConnectPanel cp;
    private MonitorPanel mp;
    private HeaderPanel  hp;

    private MonitorServer.Monitor monitor_;
    private static final int maxNumServices = 10;
    private String      serverInfo;
    private String      portInfo;
    private MonitorThread  thread;

    public void init() {
      portInfo = getParameter("org.omg.CORBA.ORBInitialPort");
      serverInfo = getParameter("org.omg.CORBA.ORBInitialHost") + ":" + portInfo;
        setSize(375, 280);
        setBackground(Color.lightGray);
        add(hp = new HeaderPanel(this, "NPAC Monitor"));
        add(cp = new ConnectPanel(this));
        mp = new MonitorPanel(this);
    }

    /** Initialize ORB connection **/

    public
    void setORBConnectionToServer() {
      try {
        Properties props = new Properties();
        props.put("org.omg.CORBA.ORBClass", "com.sun.CORBA.iiop.ORB");

        // create and initialize the ORB
        org.omg.CORBA.ORB theOrb_ =  org.omg.CORBA.ORB.init(this, props);

        // get the root naming context
        org.omg.CORBA.Object objRef = 
                          theOrb_.resolve_initial_references("NameService");
        NamingContext ncRef = NamingContextHelper.narrow(objRef);

        // resolve the Object Reference in Naming
        NameComponent nc = new NameComponent("Monitor_" + portInfo, "");
        NameComponent path[] = {nc};
        monitor_ = MonitorServer.MonitorHelper.narrow(ncRef.resolve(path));
      }
      catch (Exception ex) {
        System.out.println("Monitor: ORB Exception-> " + ex) ;
      }
    }

    public void showConnectPanel() {
        // mp.setVisible(false);
        // cp.setVisible(true);
        remove(mp);
        add(cp);
        validate();
    }

    public void showMonitorPanel() {
        // mp.setVisible(true);
        // cp.setVisible(false);
        remove(cp);
        add(mp);
        validate();
    }

   /**
    *  Handle action events from various controls
    *  Overrides Component.action().
    */
   public void actionPerformed(ActionEvent pe) {
     // User selected month from the choice
     String actionLabel = pe.getActionCommand();
     if (actionLabel.equals("Connect")) {
         getAppletContext().showStatus("Connecting to NPAC Server");
         setORBConnectionToServer();
         String userid = cp.connectUserField.getText();
         String passwd = cp.connectPasswordField.getText();
         if (monitor_.connect(userid, passwd)) {
             showMonitorPanel();
             thread = new MonitorThread(this);
             thread.start();
             getAppletContext().showStatus("Verification succeed");
         } else {
             getAppletContext().showStatus("Verification fail");
         }
     } else if (actionLabel.equals("Refresh")) {
         getAppletContext().showStatus("Refreshing");
         mp.refreshStatus(monitor_.getServerStatus());
         getAppletContext().showStatus("Refreshed");
     } else if (actionLabel.equals("Exit")) {
         getAppletContext().showStatus("Exiting");
         showConnectPanel();
         getAppletContext().showStatus("Exited");
     }
   }

    public MonitorServer.Monitor getMonitor() {
        return this.monitor_;
    }

    public MonitorPanel getMonitorPanel() {
        return this.mp;
    }

    public void destroy() {
        this.thread.stop(); // this.thread.destroy();
    }

    /**
     *  Paint nice raized 3D background.
     *  Overrides Component.paint().
     */
    public void
    paint(Graphics pg)
    {
        pg.setColor(Color.black);
        pg.drawLine(1, 0, getBounds().width-2, 0);
        pg.drawLine(0, 1, 0, getBounds().height-2);
        pg.drawLine(1, getBounds().height-1, getBounds().width - 2, getBounds().height - 1);
        pg.drawLine(getBounds().width - 1, 1, getBounds().width - 1, getBounds().height - 2);

        pg.setColor(getBackground());
        //pg.draw3DRect(1, 1, getBounds().width - 3, getBounds().height - 3, true);
        pg.fill3DRect(1, 1, getBounds().width - 2, getBounds().height - 2, true);
    }
}
