import java.io.*;
import java.net.*;
import java.awt.*;
import java.util.Properties;
import java.util.*;

import org.omg.CORBA.*;
import com.sun.CORBA.iiop.*;
import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextPackage.*;

import CoordinatorServer.*;
import ClientServer.*;
import MonitorServer.*;
import ServerDB.*;

class
MonitorServantImpl extends MonitorServer._MonitorImplBase {
    private AccountManagerServantImpl acc;

    public MonitorServantImpl(AccountManagerServantImpl acc_) {
        acc = acc_;
    }

    public boolean connect(String username, String password) {
        return true;
    }

    public String getServerStatus() {
        StringBuffer st = new StringBuffer();
        for (int i = 0; i < acc.getNumServiceObjects(); i++) {
            if (acc.transaction[i].inUse) {
                st.append("Service[" + Integer.toString(i) + "]  >>>  ");
                st.append(acc.transaction[i].username + "/");
                st.append(Long.toString(acc.transaction[i].trNo));
                st.append("\n");
            }
        }
       return st.toString();
    }
}

//
//  Checking the client is still online by receiving 
//  clients' beepings 
//
class 
ClientCheckingThread extends Thread {

    private AccountManagerServantImpl acc;
    private long lastBeeps = 0;
    private long currentBeeps = 0;

    public ClientCheckingThread(AccountManagerServantImpl a) {
        acc = a;
    }

    public void run() {
        int i;
        int numServiceObjects = acc.getNumServiceObjects();
        try {
            while (true) {
                for (i = 0; i < numServiceObjects; i++) {
                  if (acc.transaction[i].inUse) {
                    if (acc.transaction[i].checkBeeps()) {
                         acc.releaseServiceObject(i);
                    }
                  }
                }
                this.sleep(60000);
            }
        } catch (Exception ex) {
            System.err.println("Server: clientCheckingThread Exception-> " + ex) ;
        }
    }
}

class
TransactionServantImpl extends ClientServer._TransactionImplBase {
    long trNo;
    boolean inUse;
    boolean isAvailable;
    String username;
    String password;
    String time;
    String zone;
    private long currentBeeps;
    private long lastBeeps;
    private ServerDB.DBService dbs_;

    public TransactionServantImpl (ServerDB.DBI dbi_, int id) {
        trNo = 0;
        inUse = false;
        isAvailable = false;
        username = "";
        password = "";
        currentBeeps = 0;
        lastBeeps = 0;

        dbs_ = dbi_.reserveService(id);
    }

    protected boolean open() {
        if (!dbs_.open(this.username, this.password, this.trNo, this.time,
                  this.zone)) {
            System.err.println("Transaction Error: DB connection failed");
            return false;
        }
        return true;
    }

    protected void close() {
        dbs_.close();
    }

    public void setTime(String timein) {
        StringTokenizer st = new StringTokenizer(timein);
        st.nextToken();
        String tmptime = st.nextToken() + " " + st.nextToken() + " " + 
                         st.nextToken();
        zone = st.nextToken();
        time = tmptime + " " + st.nextToken();
    }

    public synchronized String queryCheckingAccount(long tr) {
        if (this.trNo == tr) {
            return dbs_.queryChecking();
        } else {
            return "Connection lost\n";
        }
    }

    public synchronized String querySavingAccount(long tr) {
        if (this.trNo == tr) {
            return dbs_.querySaving();
        } else {
            return "Connection lost\n";
        }
    }

    public synchronized 
        String transferFromCheckingToSaving(long tr, float amount) {
        if (this.trNo == tr) {
            return dbs_.transferCS(amount);
        } else {
            return "Connection lost\n";
        }
    }

    public synchronized 
        String transferFromSavingToChecking(long tr, float amount) {
        if (this.trNo == tr) {
            String msg = dbs_.transferSC(amount);
            return msg;
        } else {
            return "Connection lost\n";
        }
    }

    public boolean beeping(long beeps) {
        currentBeeps = beeps;
        return true;
    }

    public boolean checkBeeps() {
        if (currentBeeps == lastBeeps) {
            if (lastBeeps != 0) {
                currentBeeps = 0;
                lastBeeps = 0;
                return true;
            }
        } else {
            lastBeeps = currentBeeps;
        }
        return false;
    }
}

class 
AccountManagerServantImpl extends ClientServer._AccountManagerImplBase {
    private int numServiceObjects = 5;
    private int numServiceObjectsInUse = 0;
    private String serverInfo;
    TransactionServantImpl[] transaction = 
                          new TransactionServantImpl[numServiceObjects];
    private ClientCheckingThread thread;

    public AccountManagerServantImpl(org.omg.CORBA.ORB orb,
                                     String server_,
                                     String id) {
        String[] argv = new String[4];
        argv[0] = new String("-ORBInitialHost");
        argv[1] = new String("osprey8.npac.syr.edu");
        argv[2] = new String("-ORBInitialPort");
        argv[3] = new String("9966");

        ServerDB.DBI dbi_;
        serverInfo = server_;
        thread = new ClientCheckingThread(this);
        thread.setPriority(java.lang.Thread.NORM_PRIORITY);

      try {
        // create and initialize the ORB
        org.omg.CORBA.ORB theOrb_ =  org.omg.CORBA.ORB.init(argv, null);

        // get the root naming context
        org.omg.CORBA.Object objRef =
                         theOrb_.resolve_initial_references("NameService");
        NamingContext ncRef = NamingContextHelper.narrow(objRef);

        // resolve the Object Reference in Naming
        NameComponent nc =
                 new NameComponent("DBServer_" + id, "");
        NameComponent path[] = {nc};
        dbi_ = ServerDB.DBIHelper.narrow(ncRef.resolve(path));

        for (int i = 0 ;i < numServiceObjects; i++) {
            transaction[i] = new TransactionServantImpl(dbi_, i);
            orb.connect(transaction[i]);
        }
        thread.start();
      }
      catch (Exception ex) {
        System.out.println("DB connection for Server_" + id + " failed");
        System.err.println("AccountManager: Exception-> " + ex);
      }
    }

    public synchronized ClientServer.Transaction
              connect(String username, String password, long transactionNo,
                      String timein) {

        for (int i = 0; i < numServiceObjects; i++) {
            if (transaction[i].isAvailable) {
                if (transaction[i].username.equals(username)) {
                    if (transaction[i].password.equals(password)) {
                        if (transaction[i].trNo == transactionNo) {
                            transaction[i].setTime(timein);
                            if (transaction[i].open()) {
                                transaction[i].inUse = true;
                                numServiceObjectsInUse++;
                                return transaction[i];
                            } else {
                                return null;
                            }
                        }
                        System.out.println("Server[" + serverInfo + "] <" + i + "> " +
                                           transaction[i].trNo + "/" + transactionNo);
                    }
                    System.out.println("Server[" + serverInfo + "] <" + i + "> " +
                                       transaction[i].password + "/" + password);
                }
                System.out.println("Server[" + serverInfo + "] <" + i + "> " +
                                   transaction[i].username + "/" + username);
            }
            System.out.println("Server[" + serverInfo + "] <" + i + "> " +
                               "isAvailable = false");
        }
        System.out.println("Server-" + serverInfo +": " + username + "|" +
                           password + "|" + transactionNo + "|" +
                           numServiceObjectsInUse + "/" + numServiceObjects);
        return null;
    }

    public synchronized void disconnect(long transactionNo) {
        System.out.println("Server: " + transactionNo);
        for (int i = 0; i < numServiceObjects; i++) {
            if (transaction[i].trNo == transactionNo) {
                transaction[i].beeping(0);
                releaseServiceObject(i);
                return;
            }
        }
        System.out.println("Server-" + serverInfo + 
                           ": reserved transaction object not found");
    }

    public void releaseServiceObject(int i) {
        transaction[i].inUse = false;
        transaction[i].isAvailable = false;
        transaction[i].trNo = 0;
        transaction[i].username = "";
        transaction[i].password = "";
        numServiceObjectsInUse--;
        transaction[i].close();
    }

    public int getNumServiceObjects() {
        return this.numServiceObjects;
    }

    public int getNumServiceObjectsInUse() {
        return this.numServiceObjectsInUse;
    }
}

class
AcknowledgeServantImpl extends CoordinatorServer._AcknowledgeImplBase {

    public String clientUserName;
    public String clientPassword;
    public long   clientTrNo;
    private String serverInfo;
    private boolean isReady = false;
    private AccountManagerServantImpl accountManager;

    public AcknowledgeServantImpl(String server_, AccountManagerServantImpl a) {
        super();
        serverInfo = server_;
        accountManager = a;
    }

    public synchronized boolean announceClientInfo(String username, String password, 
                                                long trNo) {

        for (int i = 0; i < numMaxClients(); i++) {
            if (!accountManager.transaction[i].isAvailable) {
                accountManager.transaction[i].username = username;
                accountManager.transaction[i].password = password;
                accountManager.transaction[i].trNo = trNo;
                accountManager.transaction[i].isAvailable = true;
                return true;
            }
        }
        return false;
    }

    public int numActiveClients() {
        return accountManager.getNumServiceObjectsInUse();
    }

    public int numMaxClients() {
        return accountManager.getNumServiceObjects();
    }

    public synchronized boolean checkServer() {
        if (numActiveClients() >= numMaxClients()) {
           return false;
        }
        return true;
    }
}

public class Server {

    public static org.omg.CORBA.ORB theOrb_;

    public static void main(String[] args) {
        int i = 0;
        while (!args[i++].equals("-ID")) {};
        String id = args[i];
        System.out.println("Server " + id);

        try {
            // create and initialize ORB
            theOrb_ =  org.omg.CORBA.ORB.init(args, null);
            
            // create Servant and register it with ORB
            AccountManagerServantImpl accountManager =
                new AccountManagerServantImpl(theOrb_, args[1], id);
            theOrb_.connect(accountManager);

            // get the root naming context
            org.omg.CORBA.Object objRef = 
                         theOrb_.resolve_initial_references("NameService");
            NamingContext ncRef = NamingContextHelper.narrow(objRef);

            // bind the Object Reference in Naming
            NameComponent nc1 = new NameComponent("AccountManager_" + args[1], "");
            NameComponent path1[] = {nc1};
            ncRef.rebind(path1, accountManager);

            // Second ORB reference
            AcknowledgeServantImpl acknowledgeServant =
                               new AcknowledgeServantImpl(args[1], accountManager);
            theOrb_.connect(acknowledgeServant);
            NameComponent nc2 = new NameComponent("Server_" + args[1], "");
            NameComponent path2[] = {nc2};
            ncRef.rebind(path2, acknowledgeServant);

            // Third ORB reference
            MonitorServantImpl monitor = 
                new MonitorServantImpl(accountManager);
            NameComponent nc3 = new NameComponent("Monitor_" + args[1], "");
            NameComponent path3[] = {nc3};
            ncRef.rebind(path3, monitor);

            System.out.println("Server_" + args[1] + " Started\n");

            // wait for invocations from clients
            java.lang.Object sync = new java.lang.Object();
            synchronized (sync) {
               sync.wait();
            }
        } catch (Exception ex) {
            System.err.println("Server-" + args[1] + ": Exception-> " + ex);
        }
    }
}
