
CREATE OR REPLACE PACKAGE banking AS

    TYPE person_rec_type IS RECORD              -- person-info record
       (userid          VARCHAR2(10),
        lastname        VARCHAR2(20),
        firstname       VARCHAR2(30),
        address         VARCHAR2(80),
        phoneno1        VARCHAR2(20),
        phoneno2        VARCHAR2(20),
        status          CHAR(1));

    FUNCTION transfer_s_c(uid      IN  VARCHAR2,
                          trno     IN  NUMBER,
                          trans    IN  REAL,
                          time     IN  VARCHAR2,
                          zone     IN  VARCHAR2,
                          saving   OUT REAL,
                          checking OUT REAL) RETURN NUMBER;

    FUNCTION transfer_c_s(uid      IN  VARCHAR2,
                          trno     IN  NUMBER,
                          trans    IN  REAL,
                          time     IN  VARCHAR2,
                          zone     IN  VARCHAR2,
                          saving   OUT REAL,
                          checking OUT REAL) RETURN NUMBER;

    FUNCTION query_c(uid      IN  VARCHAR2,
                     trno     IN  NUMBER,
                     time     IN  VARCHAR2,
                     zone     IN  VARCHAR2) RETURN REAL;

    FUNCTION query_s(uid      IN  VARCHAR2,
                     trno     IN  NUMBER,
                     time     IN  VARCHAR2,
                     zone     IN  VARCHAR2) RETURN REAL;
                      
END;
.
/
show errors

CREATE OR REPLACE PACKAGE BODY banking AS

    FUNCTION transfer_s_c(uid      IN  VARCHAR2,
                          trno     IN  NUMBER,
                          trans    IN  REAL,
                          time     IN  VARCHAR2,
                          zone     IN  VARCHAR2,
                          saving   OUT REAL,
                          checking OUT REAL) RETURN NUMBER IS

        CURSOR saving_c (cuid IN VARCHAR2) IS
            SELECT amount FROM saving_account
            WHERE cuid = userid;

        CURSOR checking_c (cuid IN VARCHAR2) IS
            SELECT amount FROM checking_account
            WHERE cuid = userid;

        local_amount       REAL;
        tr_status          NUMBER;
    BEGIN

        IF NOT saving_c%ISOPEN THEN
            OPEN saving_c(uid);
        END IF;
        FETCH saving_c INTO local_amount;
        CLOSE saving_c;

        IF local_amount >= trans THEN
            UPDATE checking_account 
                SET amount = amount + trans
                WHERE uid = userid;

            UPDATE saving_account 
                SET amount = amount - trans
                WHERE uid = userid;

            tr_status := 1;
        ELSE
            tr_status := 0;
        END IF;
 
        INSERT INTO transaction_log VALUES(trno, uid, 2, trans, tr_status,
            TO_DATE(time, 'MON DD HH24:MI:SS YYYY'), zone);

        COMMIT;

        IF NOT saving_c%ISOPEN THEN
            OPEN saving_c(uid);
        END IF;
        FETCH saving_c INTO saving;
        CLOSE saving_c;

        IF NOT checking_c%ISOPEN THEN
            OPEN checking_c(uid);
        END IF;
        FETCH checking_c INTO checking;
        CLOSE checking_c;

        RETURN tr_status;
    END;


    FUNCTION transfer_c_s(uid      IN  VARCHAR2,
                          trno     IN  NUMBER,
                          trans    IN  REAL,
                          time     IN  VARCHAR2,
                          zone     IN  VARCHAR2,
                          saving   OUT REAL,
                          checking OUT REAL) RETURN NUMBER IS

        CURSOR checking_c (cuid IN VARCHAR2) IS
            SELECT amount FROM checking_account
            WHERE cuid = userid;

        CURSOR saving_c (cuid IN VARCHAR2) IS
            SELECT amount FROM saving_account
            WHERE cuid = userid;

        local_amount       REAL;
        tr_status          NUMBER;
    BEGIN

        IF NOT checking_c%ISOPEN THEN
            OPEN checking_c(uid);
        END IF;
        FETCH checking_c INTO local_amount;
        CLOSE checking_c;

        IF local_amount >= trans THEN
            UPDATE saving_account 
                SET amount = amount + trans
                WHERE uid = userid;
    
            UPDATE checking_account 
                SET amount = amount - trans
                WHERE uid = userid;

            tr_status := 1;
        ELSE
            tr_status := 0;
        END IF;
 
        INSERT INTO transaction_log VALUES (trno, uid, 3, trans, tr_status,
            TO_DATE(time, 'MON DD HH24:MI:SS YYYY'), zone);

        COMMIT;

        IF NOT saving_c%ISOPEN THEN
            OPEN saving_c(uid);
        END IF;
        FETCH saving_c INTO saving;
        CLOSE saving_c;

        IF NOT checking_c%ISOPEN THEN
            OPEN checking_c(uid);
        END IF;
        FETCH checking_c INTO checking;
        CLOSE checking_c;

        RETURN tr_status;
    END;

    FUNCTION query_s(uid      IN  VARCHAR2,
                     trno     IN  NUMBER,
                     time     IN  VARCHAR2,
                     zone     IN  VARCHAR2) RETURN REAL IS

        CURSOR saving_c (uid IN VARCHAR2) IS
            SELECT * FROM saving_account
            WHERE uid = userid;

        CURSOR queryc(qid    IN VARCHAR2) IS
            SELECT status FROM customer
            WHERE qid = userid;

        local_uid          VARCHAR2(10);
        local_status       VARCHAR2(10);
        local_amount       REAL;
        tr_status          NUMBER;
    BEGIN

        IF NOT queryc%ISOPEN THEN
            OPEN queryc(uid);
        END IF;
        FETCH queryc INTO local_status;
        CLOSE queryc;

        IF NOT saving_c%ISOPEN THEN
            OPEN saving_c(uid);
        END IF;
        FETCH saving_c INTO local_uid, local_amount;
        CLOSE saving_c;
        tr_status := 1;

        INSERT INTO transaction_log VALUES (trno, local_uid, 0, local_amount, tr_status,
            TO_DATE(time, 'MON DD HH24:MI:SS YYYY'), zone);

        COMMIT;
        RETURN local_amount;
    END;

    FUNCTION query_c(uid      IN  VARCHAR2,
                     trno     IN  NUMBER,
                     time     IN  VARCHAR2,
                     zone     IN  VARCHAR2) RETURN REAL IS

        CURSOR checking_c (uid IN VARCHAR2) IS
            SELECT * FROM checking_account
            WHERE uid = userid;

        CURSOR queryc(qid    IN VARCHAR2) IS
            SELECT status FROM customer
            WHERE qid = userid;

        local_uid          VARCHAR2(10);
        local_status       VARCHAR2(10);
        local_amount       REAL;
        tr_status          NUMBER;
    BEGIN

        IF NOT queryc%ISOPEN THEN
	    OPEN queryc(uid);
	END IF;
        FETCH queryc INTO local_status;
	CLOSE queryc;

        IF NOT checking_c%ISOPEN THEN
            OPEN checking_c(uid);
        END IF;
        FETCH checking_c INTO local_uid, local_amount;
        CLOSE checking_c;
        tr_status := 1;
 
        INSERT INTO transaction_log VALUES(trno, local_uid, 1, local_amount, tr_status,
            TO_DATE(time, 'MON DD HH24:MI:SS YYYY'), zone);

        COMMIT;
        RETURN local_amount;

    END;

END;
.
/
show errors

