import java.io.*;
import java.security.*;
import javax.crypto.*;

public class CipherControl {
    private MacMessage   mm;
    private Cipher       cipher;
    private SealedObject so;

    // Default algorithm for crypto cipher
    private String     cipherAlgorithm = "DES/ECB/PKCS5Padding";

    public CipherControl() {
        try {
            cipher = Cipher.getInstance(cipherAlgorithm);
        } catch (Exception ex) {
            System.err.println("Cipher Initialization Exception-> " + ex);
        }
    }

    public CipherControl(MacMessage m) {
        mm = m;
        try {
            cipher = Cipher.getInstance(cipherAlgorithm);
        } catch (Exception ex) {
            System.err.println("Cipher Initialization Exception-> " + ex);
        }
    }

    public void init(SecretKey sKey, int option) {
        try {
            if (option == 0) {
                cipher.init(Cipher.ENCRYPT_MODE, sKey);
            } else {
                cipher.init(Cipher.DECRYPT_MODE, sKey);
            }
        } catch (Exception ex) {
            System.err.println("Cipher Initialization Exception-> " + ex);
        }
    }

    public SealedObject getSealedObject() {
        try {
            so = new SealedObject(mm, cipher);
            return this.so;
        } catch (Exception ex) {
            System.err.println("CipherControl Exception-> " + ex);
            return null;
        }
    }

    public MacMessage getMacMessage() {
        try {
            mm = (MacMessage)so.getObject(cipher);
            return this.mm;
        } catch (Exception ex) {
            System.err.println("CipherControl Exception-> " + ex);
            return null;
        }
    }

    public String getAlgorithm() {
        return this.cipherAlgorithm;
    }

    public void setMacMessage(MacMessage m) {
        mm = m;
    }

    public void setSealedObject(SealedObject o) {
        so = o;
    }

    public void setAlgorithm(String s) {
        cipherAlgorithm = s;
    }
}

