import java.io.*;
import java.security.*;
import java.security.spec.*;
import javax.crypto.*;
import javax.crypto.spec.*;
import javax.crypto.interfaces.*;
import com.sun.crypto.provider.SunJCE;

public class ClientDHKeyAgreement {

    private ObjectInputStream   ois;
    private ObjectOutputStream  oos;

    public ClientDHKeyAgreement(ObjectInputStream in, ObjectOutputStream out) {
        ois = in;
        oos = out;

        try {
            Provider sunJCE = new com.sun.crypto.provider.SunJCE();
            Security.addProvider(sunJCE);

            // Add JCE to the list of providers
            SunJCE jce = new SunJCE();
            Security.addProvider(jce);
        } catch (Exception e) {
            System.err.println("Error: " + e);
        }
    }

    public SecretKey agree() throws Exception {
 

        // Receive Server's public key
        byte[] serverPubKeyEnc = (byte[])ois.readObject();

        // String st = toHexString(serverPubKeyEnc);
        // System.err.println("DH serverPubKeyEnc: " + st);

        /*
         * Let's turn over to Client. Client has received Server's public key
         * in encoded format.
         * He instantiates a DH public key from the encoded key material.
         */
        KeyFactory clientKeyFac = KeyFactory.getInstance("DH");
        X509EncodedKeySpec x509KeySpec = new X509EncodedKeySpec(serverPubKeyEnc);
        PublicKey serverPubKey = clientKeyFac.generatePublic(x509KeySpec);

        /*
         * Client gets the DH parameters associated with Server's public key. 
         * He must use the same parameters when he generates its own key
         * pair.
         */
        DHParameterSpec dhParamSpec = ((DHPublicKey)serverPubKey).getParams();

        // Client creates its own DH key pair
        System.err.println("CLIENT: Generate DH keypair ...");
        KeyPairGenerator clientKpairGen = KeyPairGenerator.getInstance("DH");
        clientKpairGen.initialize(dhParamSpec);
        KeyPair clientKpair = clientKpairGen.generateKeyPair();

        // Client executes Phase1 of its version of the DH protocol
        System.err.println("CLIENT: Execute PHASE1 ...");
        KeyAgreement clientKeyAgree = KeyAgreement.getInstance("DH");
        clientKeyAgree.init(clientKpair.getPrivate());

        // Client encodes its public key, and sends it over to Server.
        byte[] clientPubKeyEnc = clientKpair.getPublic().getEncoded();

        // st = toHexString(clientPubKeyEnc);
        // System.err.println("DH clientPubKeyEnc: " + st);

        oos.writeObject(clientPubKeyEnc);
        oos.flush();

        /*
         * Client uses Server's public key for Phase2 of its version of the DH
         * protocol.
         */
        System.err.println("CLIENT: Execute PHASE2 ...");
        clientKeyAgree.doPhase(serverPubKey, true);
            
        /*
         * At this stage, both Server and Client have completed the DH key
         * agreement protocol.
         * Each generates the (same) shared secret.
         */

        SecretKey skey = clientKeyAgree.generateSecret("DES");
        return skey;
    }

    public void byte2hex(byte b, StringBuffer buf) {
        char hexChars[] = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'};

        int low = (b & 0x0f);
        int high = (b & 0xf0) >> 4;

        buf.append(hexChars[high]);
        buf.append(hexChars[low]);
    }

    /*
     * Converts a byte array to hex string
     */
    public String toHexString(byte[] block) {
        StringBuffer buf = new StringBuffer();

        int len = block.length;

        for (int i = 0; i < len; i++) {
             byte2hex(block[i], buf);
             if (i < len-1) {
                 buf.append(":");
             }
        } 
        return buf.toString();
    }
}
