import java.io.*;
import java.security.*;
import java.security.spec.*;
import java.security.interfaces.*;
import javax.crypto.*;

public class ClientHandShaking {

    private ObjectInputStream   ois;
    private ObjectOutputStream  oos;
    private PublicKey           clientPublicKey;
    private PublicKey           serverPublicKey;
    private PrivateKey          clientPrivateKey;
    private SecretKey           secretKey;

    public ClientHandShaking(ObjectInputStream in, ObjectOutputStream out) {
        ois = in;
        oos = out;
    }

    public boolean init() {
        try {
            // Initialize the DH protocol and exchange the secret key for cipher
            ClientDHKeyAgreement dhKey = new ClientDHKeyAgreement(ois, oos);
            secretKey = dhKey.agree();

            // Client's public/private keys generator
            // This will be replaced by reading keys from keys repository
            KeyPairGenerator clientKeyGen = KeyPairGenerator.getInstance("DSA");
            clientKeyGen.initialize(1024, new SecureRandom());
            KeyPair kPair = clientKeyGen.generateKeyPair();
            clientPublicKey = kPair.getPublic();
            clientPrivateKey = kPair.getPrivate();
            byte[] clientEncodedPublicKey = clientPublicKey.getEncoded();

            // String st = toHexString(clientEncodedPublicKey);
            // System.err.println("Client's Encoded PublicKey: " + st);            

            // obtain Server's public key
            // This will be replaced by reading from Server's certificate

            byte[] serverEncodedPublicKey = (byte[])ois.readObject();

            // st = toHexString(serverEncodedPublicKey);
            // System.err.println("Server's Encoded PublicKey: " + st);

            KeyFactory clientKeyFac = KeyFactory.getInstance("DSA");
            X509EncodedKeySpec x509KeySpec = new X509EncodedKeySpec(serverEncodedPublicKey);
            serverPublicKey = clientKeyFac.generatePublic(x509KeySpec);


            // send Client's public key to Client
            oos.writeObject(clientEncodedPublicKey);
            oos.flush();

            return true;
        } catch (Exception ex) {
            System.err.println("ClientHandShaking Exception-> " + ex);
            return false;
        }
    }

    public PublicKey getClientPublicKey() {
        return this.clientPublicKey;
    }

    public PublicKey getServerPublicKey() {
        return this.serverPublicKey;
    }

    public PrivateKey getClientPrivateKey() {
        return this.clientPrivateKey;
    }

    public SecretKey getSecretKey() {
        return this.secretKey;
    }

    public void byte2hex(byte b, StringBuffer buf) {
        char hexChars[] = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'};

        int low = (b & 0x0f);
        int high = (b & 0xf0) >> 4;

        buf.append(hexChars[high]);
        buf.append(hexChars[low]);
    }

    /*
     * Converts a byte array to hex string
     */
    public String toHexString(byte[] block) {
        StringBuffer buf = new StringBuffer();

        int len = block.length;

        for (int i = 0; i < len; i++) {
             byte2hex(block[i], buf);
             if (i < len-1) {
                 buf.append(":");
             }
        }
        return buf.toString();
    }
}
