import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.io.*;

import java.util.Properties;
import java.util.*;

import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextPackage.*;
import org.omg.CORBA.*;
import com.sun.CORBA.iiop.*;

import ConsoleCoordinator.*;

class
RefreshThread extends Thread {
    ConsoleApplet  ca;

    public RefreshThread(ConsoleApplet a) {
       ca = a;
    }

    public void run() {
       try {
           while (true) {
               ca.refreshServerStatus();
               ca.getAppletContext().showStatus("Refreshed");
               this.sleep(30000);
           }
       } catch (Exception ex) {
           System.err.println("Console: Refresh Thread Exception->" + ex);
       }
    }
}

class
ConsoleControlPanel extends Panel {
    private ActionListener al;

    private Button      exitButton;
    private Button      startButton;
    private Button      stopButton;
    private Button      startupButton;
    private Button      shutdownButton;
    private Button      refreshButton;

    public ConsoleControlPanel(EventListener a) {
        al = (ActionListener)a;

        setLayout(new GridLayout(2, 3));

        startupButton = makeButton("Startup");
        shutdownButton = makeButton("Shutdown");
        refreshButton = makeButton("Refresh");
        startButton = makeButton("Start");
        stopButton = makeButton("Stop");
        exitButton = makeButton("Exit");
    }

    protected Button makeButton(String name) {
        Button button = new Button(name);
        button.addActionListener(al);
        add(button);
        return button;
    }

    public void enablePanelExceptExit(boolean b) {
        startupButton.setEnabled(b);
        shutdownButton.setEnabled(b);
        refreshButton.setEnabled(b);
        startButton.setEnabled(b);
        stopButton.setEnabled(b);
    }
}

class
StatusDisplayPanel extends Panel {
    public  java.awt.List  list;
    private ItemListener   il;

    public StatusDisplayPanel(EventListener a) {
        il = (ItemListener)a;

        setLayout(new BorderLayout());
        Label label = new Label("Server Status", Label.CENTER);
        label.setFont(new Font("Helvetica", Font.BOLD, 16));
        add(label, "North");
        label = new Label("   Server-ID   Host                                      " +
                                "Port   Max   Free  Status   ");
        add(label, "Center");
        list = new java.awt.List(10);
        list.addItemListener(il);
        add(list, "South");
    }

    public void addItem(String name) {
        list.add(name);
    }

    public void removeAllItems() {
        list.removeAll();
    }
}

public
class ConsoleApplet extends Applet implements ActionListener, ItemListener {
    
    private static int            maxNumServers = 10;
    private Insets                mInsets  = new Insets(3,3,3,3);

    private HeaderPanel           hp;
    private ConnectPanel          cp;
    private ConsoleControlPanel   ccp;
    private StatusDisplayPanel    sdp;

    private String[]              serverList = new String[maxNumServers];
    private String                selectedSID = "";
    private boolean               isAdmin;

    private ConsoleCoordinator.CoordinatorManager coordinatorManager_;

    private RefreshThread thread;

    public void init() {
      setSize(640, 340);
      createGui();
      thread = new RefreshThread(this);
    }

    /** Initialize ORB connection **/

    public
    void setORBConnectionToCoordinator() {
      try {
        Properties props = new Properties();
        props.put("org.omg.CORBA.ORBClass", "com.sun.CORBA.iiop.ORB");
        props.put("org.omg.CORBA.ORBInitialHost", "osprey6.npac.syr.edu");
        props.put("org.omg.CORBA.ORBInitialPort", "9966");

        // create and initialize the ORB
        org.omg.CORBA.ORB theOrb_ =  org.omg.CORBA.ORB.init(this, props);

        // get the root naming context
        org.omg.CORBA.Object objRef = 
                          theOrb_.resolve_initial_references("NameService");
        NamingContext ncRef = NamingContextHelper.narrow(objRef);

        // resolve the Object Reference in Naming
        NameComponent nc = new NameComponent("Coordinator", "");
        NameComponent path[] = {nc};
        coordinatorManager_ = ConsoleCoordinator.CoordinatorManagerHelper.narrow(
                                     ncRef.resolve(path));
      }
      catch (Exception ex) {
        ex.printStackTrace();
        System.out.println("Console: ORB Exception-> " + ex) ;
      }
    }


    public
    void createGui() {
        setLayout(new BorderLayout());

        add(hp = new HeaderPanel(this, "NPAC Console"), "North");

        add(cp = new ConnectPanel(this), "Center");

        ccp = new ConsoleControlPanel(this);

        sdp = new StatusDisplayPanel(this); }

    protected String adjustText(String s, int n, boolean b) {
        StringBuffer sb = new StringBuffer();
        if (s.length() < n) {
            if (b) {
                sb.append(s);
                for (int i = 0; i < n - s.length(); i++) {
                    sb.append(" ");
                }
            } else {
               for (int i = 0; i < n - s.length(); i++) {
                    sb.append(" ");
                }
                sb.append(s);
            }
        } else {
            sb.append(s.substring(0, n));
        }

        // append delimiter
        for (int i = 0; i < 2; i++) {
            sb.append(" ");
        }

        return sb.toString();
    }
            
    protected String updateStatus(String s) {
        StringBuffer sb = new StringBuffer();
        StringTokenizer st = new StringTokenizer(s);
        String sid = st.nextToken();
        if (selectedSID.equals(sid)) {
            sb.append(adjustText("*", 1, true));
        } else {
            sb.append(adjustText(" ", 1, true));
        }
        sb.append(adjustText(sid, 10, false));
        sb.append(adjustText(st.nextToken(), 40, true));
        sb.append(adjustText(st.nextToken(), 5, false));
        sb.append(adjustText(st.nextToken(), 4, false));
        sb.append(adjustText(st.nextToken(), 4, false));
        sb.append(adjustText(st.nextToken(), 10, true));
        return sb.toString();
    }

    public void refreshServerStatus() {
        sdp.removeAllItems();
        String[] st = coordinatorManager_.refresh();
        for (int i = 0; i < st.length; i++) {
            sdp.addItem(serverList[i] = updateStatus(st[i]));
        }
    }

    /**
     *  Handle action events from various controls
     *  Overrides Component.action().
     */
    public void actionPerformed(ActionEvent pe) {
        // User selected month from the choice
        String actionLabel = pe.getActionCommand();
        if (actionLabel.equals("Connect")) {
            getAppletContext().showStatus("Connecting to NPAC Coordinator");
            setORBConnectionToCoordinator();
            String userid = cp.connectUserField.getText().toUpperCase();
            String passwd = cp.connectPasswordField.getText();
            cp.connectUserField.setText("");
            cp.connectPasswordField.setText("");

            if (!thread.isAlive()) {
                thread.start();
            }

            if (coordinatorManager_.authentication(userid, passwd)) {
                isAdmin = true;
                getAppletContext().showStatus("Login successful");
            } else {
                isAdmin = false;
                ccp.enablePanelExceptExit(false);
                getAppletContext().showStatus("Login failed");
            }
            remove(cp);
            add(ccp, "Center");
            add(sdp, "South");
            refreshServerStatus();
        } else if (actionLabel.equals("Startup")) {
            getAppletContext().showStatus("Starting up all servers");
            if (!coordinatorManager_.startup()) {
                getAppletContext().showStatus("Some servers are already started");
            }
            refreshServerStatus();
            getAppletContext().showStatus("All servers are up");
        } else if (actionLabel.equals("Shutdown")) {
            getAppletContext().showStatus("Shutting down all servers");
            if (!coordinatorManager_.shutdown()) {
                getAppletContext().showStatus("Some servers are already stopped");
            }
            refreshServerStatus();
            getAppletContext().showStatus("All servers are down");
        } else if (actionLabel.equals("Refresh")) {
            getAppletContext().showStatus("Refreshing");
            refreshServerStatus();
            getAppletContext().showStatus("Refreshed");
        } else if (actionLabel.equals("Exit")) {
            thread.stop(); // thread.destroy();
            getAppletContext().showStatus("Exiting");
            if (!isAdmin) {
                ccp.enablePanelExceptExit(true);
            } 
            remove(ccp);
            remove(sdp);
            add(cp, "Center");
            getAppletContext().showStatus("Exited");
        } else if (actionLabel.equals("Start")) {
            if (!selectedSID.equals("")) {
                getAppletContext().showStatus("Starting Server " + selectedSID);
                coordinatorManager_.start(Integer.parseInt(selectedSID));
                getAppletContext().showStatus("Server " + selectedSID + " started");
                selectedSID = "";
                refreshServerStatus();
            } else {
                getAppletContext().showStatus("Please select a server");
            }
        } else if (actionLabel.equals("Stop")) {
            if (!selectedSID.equals("")) {
                getAppletContext().showStatus("Stopping Server " + selectedSID);
                coordinatorManager_.stop(Integer.parseInt(selectedSID));
                getAppletContext().showStatus("Server " + selectedSID + " stopped");
                selectedSID = "";
                refreshServerStatus();
            } else {
                getAppletContext().showStatus("Please select a server");
            }
        }
        validate();
    }

    public void
    itemStateChanged(ItemEvent ie) {
        String actionLabel = sdp.list.getSelectedItem();
        StringTokenizer st = new StringTokenizer(actionLabel);
        if ((selectedSID = st.nextToken()).equals("*")) {
            selectedSID = st.nextToken();
        }
        getAppletContext().showStatus(selectedSID + " is selected");
        refreshServerStatus();
    }

    public void destroy() {
        thread.stop(); // thread.destroy();
    }

    /**
     *  Paint nice raized 3D background.
     *  Overrides Component.paint().
     */
    public void
    paint(Graphics pg)
    {
        pg.setColor(Color.black);
        pg.drawLine(1, 0, getBounds().width-2, 0);
        pg.drawLine(0, 1, 0, getBounds().height-2);
        pg.drawLine(1, getBounds().height-1, getBounds().width - 2, getBounds().height - 1);
        pg.drawLine(getBounds().width - 1, 1, getBounds().width - 1, getBounds().height - 2);

        pg.setColor(getBackground());
        //pg.draw3DRect(1, 1, getBounds().width - 3, getBounds().height - 3, true);
        pg.fill3DRect(1, 1, getBounds().width - 2, getBounds().height - 2, true);
    }

    /**
     *  For better looks, let Layout Manager leave some margin.
     */
	public
	Insets
	getInsets()
	{
	    return mInsets;
	}
}
