import java.io.*;
import java.util.*;

public class CoordinatorServerInfo {
    private Hashtable           serverHash;
    private Hashtable           infoHash;

    public CoordinatorServerInfo() {
        serverHash = new Hashtable();
        infoHash = new Hashtable();
    }

    public synchronized void putServer(int id, String host, int port) {
        CoordinatorServer cs = new CoordinatorServer();
        cs.id = id;
        cs.host = host;
        cs.port = port;
 
        serverHash.put(new Integer(cs.id), cs);
    }

    public synchronized void removeServer(int id) {
        Integer i = new Integer(id);
        serverHash.remove(i);
        infoHash.remove(i);
    }

    public synchronized void removeServer(Integer id) {
        serverHash.remove(id);
        infoHash.remove(id);
    }

    public synchronized CoordinatorServer getServer(int id) {
        return (CoordinatorServer)serverHash.get(new Integer(id));
    }

    public synchronized CoordinatorServer getServer(Integer id) {
        return (CoordinatorServer)serverHash.get(id);
    }

    public synchronized int numServers() {
        return this.serverHash.size();
    }

    public synchronized void putServerStatus(int id, int total) {
        CoordinatorServerStatus css = new CoordinatorServerStatus();
        css.id = id;
        css.total = total;
        css.available = 0;
        css.status = false;

        infoHash.put(new Integer(css.id), css);
    }

    public synchronized void updateServerStatus(int id, int total, int available) {

        CoordinatorServerStatus css = getServerStatus(id);

        infoHash.remove(new Integer(id));

        css.id = id;
        css.total = total;
        css.available = available;

        infoHash.put(new Integer(css.id), css);
    }

    public synchronized void updateServerStatus(int id, boolean b) {
        CoordinatorServerStatus css = getServerStatus(id);
        
        infoHash.remove(new Integer(id));

        css.status = b;

        infoHash.put(new Integer(css.id), css);
    }

    public synchronized CoordinatorServerStatus getServerStatus(int id) {
        return (CoordinatorServerStatus)infoHash.get(new Integer(id));
    }

    public synchronized CoordinatorServerStatus getServerStatus(Integer id) {
        return (CoordinatorServerStatus)infoHash.get(id);
    }

    public synchronized Enumeration getAllServerIDs() {
        return serverHash.keys();
    }

    public synchronized CoordinatorServer getAvailableServer() {

        Integer[] serverIDs = new Integer[numServers()];
        int i = 0;
        for (Enumeration e = getAllServerIDs(); e.hasMoreElements();) {
            serverIDs[i++]  = (Integer)e.nextElement();
        }

        int length = i; 
        int nReady = length;

        boolean done = false;
        int avail = -1;
        while (!done) {
            i = 0;
            float vacancy = (float)0;

            while (i < length) {
                CoordinatorServerStatus css = getServerStatus(serverIDs[i]);
                if (css.status) {
                    if (css.total > 0) {
                        float tmp = (float)css.available/(float)css.total;
                        if (vacancy < tmp) {
                            vacancy = tmp;
                            avail = i;
                        }
                    }
                } else {
                    nReady--;
                }
                i++;
            }
            if (nReady == 0) done = true;
            if ((avail >= 0) && (vacancy > 0)) done = true;
        }
        if (avail < 0) {
            CoordinatorServer cs = new CoordinatorServer();
            cs.port = 0;
            cs.host = "No server available";
            cs.id = -1;
            return cs;
        } else {
            return getServer(serverIDs[avail]);
        }
    }
}
