import java.io.*;
import java.net.*;
import java.security.*;
import javax.crypto.*;

public class CoordinatorThreadSocketHandler extends Thread {

    private long                   trNo;
    private Socket                 socket;
    private CoordinatorSecuredComm csc;
    private CoordinatorServerInfo  csi;

    public CoordinatorThreadSocketHandler() {}
    
    public CoordinatorThreadSocketHandler(Socket s, long i, CoordinatorServerInfo c,
                         CoordinatorKeyPair ckp) {
        socket = s;
        trNo = i;
        csi = c;
        csc = new CoordinatorSecuredComm(socket, ckp);
    }

    public void setSocket(Socket s) {
       socket = s;
    }

    public void setTrNo(long no) {
        trNo = no;
    }

    public void run() {
        try {

            // initialize the socket connection and all message control objects
            int i = 0;
            if (!csc.init()) {
                System.err.println("CoordinatorSecuredComm.init(): failed " + i + " times.");
                csc.done();
                return;
            }

            // obtain available server from local server pool
            CoordinatorServer cs = csi.getAvailableServer();

            // send an available server information to Client
            csc.setupMessage(cs.port, cs.host);
            csc.send();

            // check if there exist an available server
            if (cs.id < 0) {
                System.err.println("No server available.");
                return;
            }

            // send transaction no to Client
            csc.setupMessage(2, Long.toString(trNo));
            csc.send();
/*
            if (csc.receive() && (csc.getMessageType() == 2)) {
                ReturnMessage rm = (ReturnMessage)csc.getMessage();
                if (rm.type == 5) {
                    if (!rm.msg.equals("Ackonwledge")) {
                        System.err.println("Coordinator error: failed to recognize client's reponse");
                    }
                } else {
                    System.err.println("Coordinator error: failed to verify client's reponse");
                }
            } else {
                System.err.println("Coordinator error: failed to authenticate message type");
            }
*/
        } catch (Exception ex) {
            System.err.println("Coordinator Exception-> " + ex);
        }
        csc.done();
    }
}
