import java.io.*;
import java.security.*;
import javax.crypto.*;

public class MacControl {
    private MacMessage mm;
    private Mac        macSHA1;
    private String     macAlgorithm = "HmacSHA1";

    public MacControl() {}

    public MacControl(MacMessage m) {
        mm = m;
    }

    public void init(SecretKey sKey) {
        try {
            macSHA1 = Mac.getInstance(macAlgorithm);
            macSHA1.init(sKey);
        } catch (Exception ex) {
            System.err.println("MAC Initialization Exception-> " + ex);
        }
    }

    public MacMessage getMacMessage() {
        return this.mm;
    }

    public int getMessageType() {
        return this.mm.type;
    }

    public SerializedMessage getMessage() {
        return this.mm.msg;
    }

    public void setMacMessage(MacMessage m) {
        mm = m;
    }

    public void setMessageType(int i) {
        mm.type = i;
    }

    public void setMessage(SerializedMessage o) {
        mm.msg = o;
    }

    public void calc() {
        mm.mac = calcMac();
    }

    public void setAlgorithm(String s) {
        macAlgorithm = s;
    }

    private byte[] calcMac() {
        try {
            macSHA1.reset();
            switch(mm.type) {
                case 0: UserInfo ui = (UserInfo)mm.msg;
                        macSHA1.update(ui.username.getBytes());
                        macSHA1.update(ui.password.getBytes());
                        macSHA1.update(Long.toString(ui.trNo).getBytes());
                        break;
                case 1: TransactionMessage tm = (TransactionMessage)mm.msg;
                        macSHA1.update(Integer.toString(tm.transactionType).getBytes());
                        macSHA1.update(Float.toString(tm.amount).getBytes());
                        break;
                case 2: ReturnMessage rm = (ReturnMessage)mm.msg;
                        macSHA1.update(Integer.toString(rm.type).getBytes());
                        macSHA1.update(rm.msg.getBytes());
                        break;
                default: System.err.println("MacControl Error: Wrong Message Type");
                         break;
            }
            return macSHA1.doFinal();
        } catch (Exception ex) {
            System.err.println("MAC Calculation Exception-> " + ex);
            return null;
        }
    }

    public boolean verify() {
        byte[] inMac = calcMac();

        if (inMac.length != mm.mac.length) {
            return false;
        }

        for (int i = 0; i < inMac.length; i++) {
            if (inMac[i] != mm.mac[i]) {
                return false;
            }
        }
        return true;
    }
}

