import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.io.*;

import java.util.Properties;
import java.util.*;

import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextPackage.*;
import org.omg.CORBA.*;
import com.sun.CORBA.iiop.*;

import MonitorServer.*;

class
MonitorThread extends Thread {
    private MonitorApplet         a;
    private long                  sleepSec;
    private int                   id;

    public MonitorThread(MonitorApplet a_, long t, int i) {
        a = a_;
        sleepSec = t;
        id = i;
    }

    public void run() {
        try {
            while (true) {
                a.refreshStatus(id);
                this.sleep(sleepSec*1000);
            }
        } catch (Exception ex) {
            System.err.println("Monitor: Thread Exception->" + ex);
        }
    }
}

class ServiceDisplayPanel extends Panel {
    public  java.awt.List  list;
    private ItemListener   il;

    public ServiceDisplayPanel(EventListener a, String title, String head) {
        il = (ItemListener)a;

        setLayout(new BorderLayout());
        Label label = new Label(title, Label.CENTER);
        label.setFont(new Font("Helvetica", Font.BOLD, 16));
        add(label, "North");
        label = new Label(head);
        add(label, "Center");
        list = new java.awt.List(8);
        list.addItemListener(il);
        add(list, "South");
    }

    public void addItem(String name) {
        list.add(name);
    }

    public void removeItem(String name) {
        list.remove(name);
    }

    public void removeAllItems() {
        list.removeAll();
    }
}

class MonitorPanel extends Panel {
    private Insets               mInsets = new Insets(3,3,3,3);
    private Button               exitButton;
    private Button               refreshButton;
    private Button               upButton;
    private Button               downButton;
    private Button               cancelButton;
    private Label                statusLabel;
    private Label                activeLabel;
    private Label                idleLabel;
    private GridBagLayout        gb = new GridBagLayout();
    private GridBagConstraints   gbc = new GridBagConstraints();
    private int                  row = 0;
    private MonitorApplet        parent;
    private ActionListener       al;
    public  ServiceDisplayPanel  sdpActive;
    public  ServiceDisplayPanel  sdpIdle;

    public MonitorPanel (MonitorApplet a) {
        al = (ActionListener)a;

        setBackground(Color.lightGray);
        setLayout(gb);

        gbc.anchor      = GridBagConstraints.NORTH;
        gbc.fill        = GridBagConstraints.NONE;
        gbc.gridx       = 0;
        gbc.gridwidth   = GridBagConstraints.REMAINDER;
        gbc.gridheight  = 2;
        gbc.weightx     = 1;
        gbc.weighty     = 1;
        statusLabel = makeLabel(a.getHost() + ":" + a.getPort(), 14, 0);

        gbc.anchor      = GridBagConstraints.CENTER;

        row += 10;
        gbc.gridy = row;
        sdpActive = new ServiceDisplayPanel(a, "Active Service List",
                        "   Transaction-No  User        " +
                        "Host                              " + 
                        "Connect Time               " + "Operation ");
        gb.setConstraints(sdpActive, gbc);
        add(sdpActive);

        row += 5;

        gbc.fill        = GridBagConstraints.BOTH;
        gbc.gridwidth   = 1;
        upButton = makeButton("  Up  ");

        gbc.gridx       = 1;
        downButton = makeButton(" Down ");

        gbc.gridx       = 2;
        refreshButton = makeButton("Refresh");

        gbc.gridx       = 3;
        cancelButton = makeButton("Cancel");

        gbc.gridx       = 4;
        exitButton = makeButton(" Exit ");

        row += 5;
        gbc.fill        = GridBagConstraints.NONE;
        gbc.gridwidth   = GridBagConstraints.REMAINDER;
        gbc.gridx       = 0;
        gbc.gridy = row;
        sdpIdle = new ServiceDisplayPanel(a, "Idle Service List",
                        "   Transaction-No  User        " +
                        "Host                              " +
                        "Connect Time               " + "Operation ");
        gb.setConstraints(sdpIdle, gbc);
        add(sdpIdle);

        setButtonEnabled(false, false, true, false, true);
    }

    protected
    Button makeButton(String name) {

        gbc.gridy = row;
        Button button = new Button(name);
        button.addActionListener(al);
        gb.setConstraints(button, gbc);
        add(button);

        return button;
    }

    protected
    Label makeLabel(String name,
                    int size,
                    int position) {

        gbc.gridy = row;
        Label label;
        if (position == 0) {
            label = new Label(name, Label.CENTER);
        } else {
            label = new Label(name, Label.RIGHT);
        }

        label.setFont(new Font("Helvetica", Font.BOLD, size));
        gb.setConstraints(label, gbc);
        add(label);

        return label;
    }

    public void setButtonEnabled(boolean b1, boolean b2, boolean b3,
                                 boolean b4, boolean b5) {
        upButton.setEnabled(b1);
        downButton.setEnabled(b2);
        refreshButton.setEnabled(b3);
        cancelButton.setEnabled(b4);
        exitButton.setEnabled(b5);
    }

    public Insets
    getInsets()
    {
        return mInsets;
    }
}

public
class MonitorApplet extends Applet implements ActionListener, ItemListener {

    private ConnectPanel          cp;
    private MonitorPanel          mp;
    private HeaderPanel           hp;

    private MonitorServer.Monitor monitor_;
    private String                serverInfo;
    private String                portInfo;
    private MonitorThread         activeThread;
    private MonitorThread         idleThread;
    private int                   selectedSID;

    public void init() {
        portInfo = getParameter("org.omg.CORBA.ORBInitialPort");
        serverInfo = getParameter("org.omg.CORBA.ORBInitialHost");
        setSize(720, 520);
        setBackground(Color.lightGray);
        setLayout(new BorderLayout());
        add(hp = new HeaderPanel(this, "NPAC Monitor"), "North");
        add(cp = new ConnectPanel(this), "Center");
        mp = new MonitorPanel(this);
    }

    /** Initialize ORB connection **/

    public String getHost() {
        return this.serverInfo;
    }

    public String getPort() {
        return this.portInfo;
    }

    public
    void setORBConnectionToServer() {
      try {
        Properties props = new Properties();
        props.put("org.omg.CORBA.ORBClass", "com.sun.CORBA.iiop.ORB");

        // create and initialize the ORB
        org.omg.CORBA.ORB theOrb_ =  org.omg.CORBA.ORB.init(this, props);

        // get the root naming context
        org.omg.CORBA.Object objRef = 
                          theOrb_.resolve_initial_references("NameService");
        NamingContext ncRef = NamingContextHelper.narrow(objRef);

        // resolve the Object Reference in Naming
        NameComponent nc = new NameComponent("Server", "");
        NameComponent path[] = {nc};
        monitor_ = MonitorServer.MonitorHelper.narrow(ncRef.resolve(path));
      }
      catch (Exception ex) {
        System.out.println("Monitor: ORB Exception-> " + ex) ;
      }
    }

    public void showConnectPanel() {
        // mp.setVisible(false);
        // cp.setVisible(true);
        remove(mp);
        add(cp, "Center");
    }

    public void showMonitorPanel() {
        // mp.setVisible(true);
        // cp.setVisible(false);
        remove(cp);
        add(mp, "Center");
    }

    protected String adjustText(String s, int n, boolean b) {
        StringBuffer sb = new StringBuffer();
        if (s.length() <= n) {
            if (b) {
                sb.append(s);
                for (int i = 0; i < n - s.length(); i++) {
                    sb.append(" ");
                }
            } else {
                for (int i = 0; i < n - s.length(); i++) {
                    sb.append(" ");
                }
                sb.append(s);
            }
        } else {
            sb.append(s.substring(0, n));
        }

        // append delimiter
        for (int i = 0; i < 2; i++) {
            sb.append(" ");
        }

        return sb.toString();
    }

    protected String betterString(String s) {
        StringBuffer sb = new StringBuffer();
        StringTokenizer st = new StringTokenizer(s);
        String sid = st.nextToken();
        if (Integer.parseInt(sid) == selectedSID) {
            sb.append(adjustText("*", 1, true));
        } else {
            sb.append(adjustText(" ", 1, true));
        }
        sb.append(adjustText(sid, 14, false));
        sb.append(adjustText(st.nextToken(), 10, true));
        sb.append(adjustText(st.nextToken(), 32, true));
        st.nextToken();
        String times = st.nextToken() + " " + st.nextToken() + " " + 
                   st.nextToken() + " " + st.nextToken() + 
                   " " + st.nextToken();
        sb.append(adjustText(times, 25, true));
        sb.append(adjustText(st.nextToken(), 8, true));
        return sb.toString();
    }

    protected void displayServiceStatus(ServiceDisplayPanel sdp, String[] s) {
        sdp.removeAllItems();
        for (int i = 0; i < s.length; i++) {
            sdp.addItem(betterString(s[i]));
        }
    }

    public void refreshStatus(int id) {
        switch (id) {
            case 1:  String[] s1 = monitor_.getActiveServiceStatus();
                     displayServiceStatus(mp.sdpActive, s1);
                     break;
            case 2:  String[] s2 = monitor_.getIdleServiceStatus();
                     displayServiceStatus(mp.sdpIdle, s2);
                     break;
            default: String[] sa = monitor_.getActiveServiceStatus();
                     displayServiceStatus(mp.sdpActive, sa);
                     String[] si = monitor_.getIdleServiceStatus();
                     displayServiceStatus(mp.sdpIdle, si);
                     break;
        }
    }

   /**
    *  Handle action events from various controls
    *  Overrides Component.action().
    */
    public void actionPerformed(ActionEvent pe) {
        // User selected month from the choice
        String actionLabel = pe.getActionCommand();
        if (actionLabel.equals("Connect")) {
            getAppletContext().showStatus("Connecting to NPAC Server");
            setORBConnectionToServer();
            String userid = cp.connectUserField.getText();
            String passwd = cp.connectPasswordField.getText();
            if (monitor_.connect(userid, passwd)) {
                showMonitorPanel();
                activeThread = new MonitorThread(this, 10, 1);
                idleThread = new MonitorThread(this, 30, 2);
                activeThread.start();
                idleThread.start();
                getAppletContext().showStatus("Verification succeed");
            } else {
                getAppletContext().showStatus("Verification fail");
            }
        } else if (actionLabel.equals("Refresh")) {
            getAppletContext().showStatus("Refreshing");
            refreshStatus(0);
            getAppletContext().showStatus("Refreshed");
        } else if (actionLabel.equals(" Exit ")) {
            getAppletContext().showStatus("Exiting");
            showConnectPanel();
            getAppletContext().showStatus("Exited");
        } else if (actionLabel.equals("  Up  ")) {
            getAppletContext().showStatus("Moving " + Integer.toString(selectedSID) + " up");
            monitor_.moveUp(selectedSID);
            getAppletContext().showStatus(Integer.toString(selectedSID) + " has been moved up.");
            mp.setButtonEnabled(false, false, true, false, true);
            selectedSID = -1;
            refreshStatus(0);
        } else if (actionLabel.equals(" Down ")) {
            getAppletContext().showStatus("Moving " + Integer.toString(selectedSID) + " down");
            monitor_.moveDown(selectedSID);
            getAppletContext().showStatus(Integer.toString(selectedSID) + " has been moved down.");
            mp.setButtonEnabled(false, false, true, false, true);
            selectedSID = -1;
            refreshStatus(0);
        } else if (actionLabel.equals("Cancel")) {
            getAppletContext().showStatus(" ");
            selectedSID = -1;
            mp.setButtonEnabled(false, false, true, false, true);
        }
        validate();
    }

    public void itemStateChanged(ItemEvent ie) {
        String actionLabel = mp.sdpActive.list.getSelectedItem();
        boolean actionSelection = true;
        if (actionLabel != null) {
            mp.setButtonEnabled(false, true, false, true, false);
        } else {
            actionLabel = mp.sdpIdle.list.getSelectedItem();
            mp.setButtonEnabled(true, true, false, true, false);
            actionSelection = false;
        }
        StringTokenizer st = new StringTokenizer(actionLabel);
        String sid = "";
        if ((sid = st.nextToken()).equals("*")) {
            sid = st.nextToken();
        }
        selectedSID = Integer.parseInt(sid);
        getAppletContext().showStatus(selectedSID + " is selected");
        if (actionSelection) refreshStatus(1);
        else                 refreshStatus(2);
    }

    public void destroy() {
        this.activeThread.stop(); // this.activeThread.destroy();
        this.idleThread.stop(); // this.idleThread.destroy();
    }

    /**
     *  Paint nice raized 3D background.
     *  Overrides Component.paint().
     */
    public void
    paint(Graphics pg)
    {
        pg.setColor(Color.black);
        pg.drawLine(1, 0, getBounds().width-2, 0);
        pg.drawLine(0, 1, 0, getBounds().height-2);
        pg.drawLine(1, getBounds().height-1, getBounds().width - 2, getBounds().height - 1);
        pg.drawLine(getBounds().width - 1, 1, getBounds().width - 1, getBounds().height - 2);

        pg.setColor(getBackground());
        //pg.draw3DRect(1, 1, getBounds().width - 3, getBounds().height - 3, true);
        pg.fill3DRect(1, 1, getBounds().width - 2, getBounds().height - 2, true);
    }
}
