import java.io.*;
import java.net.*;


public class Server {

    private ServerInfo        si;
    private ServerKeyPair     skp;
    private ThreadGroup       tg;
    private String            serverID;
    private ServerServiceInfo ssi;

    protected Server() {};

    protected void init(String id) {
        System.out.println("Generating key pair....");
        skp = new ServerKeyPair();
        tg = new ThreadGroup("Service_" + id);
        ssi = new ServerServiceInfo();
    }

    protected void run(String id, int port) {
        int i = 1;

        try {

            new ServerManagerThread(ssi, id).start();
            new ServerServiceThread(ssi, tg).start();

            ServerSocket ss = new ServerSocket(port, 5);

            // Server-Coordinator communication socket
            Socket s = new Socket("osprey6.npac.syr.edu", 2831);
            si = new ServerInfo(s, skp, id, ss);
            si.init();
            System.out.println("Server " + id + " is ready");

            for (;;) {
                Socket incoming = ss.accept();
                if (si.increase()) {
                    System.out.println("Spawning Service Thread " + i);
                    incoming.setSoTimeout(60000);
                    new ServerThreadSocketHandler(i, incoming, si, skp, tg, ssi).start();
                    i++;
                }
            }
        } catch (Exception ex) {
            System.err.println("Server Exception->" + ex);
        }
    }

    public static void main(String[] args) {
        int i = 0;
        while (!args[i++].equals("-ID")) {};
        String id = args[i];

        i = 0;
        while (!args[i++].equals("-Port")) {};
        int port = Integer.parseInt(args[i]);

        System.out.println("Server " + id + " is being started at " + port);

        Server s = new Server();
        s.init(id);
        s.run(id, port);
    }
}
