import java.io.*;
import java.math.BigInteger;
import java.security.*;
import java.security.spec.*;
import javax.crypto.*;
import javax.crypto.spec.*;
import javax.crypto.interfaces.*;
import com.sun.crypto.provider.SunJCE;

public class ServerDHKeyAgreement {

    private ObjectInputStream   ois;
    private ObjectOutputStream  oos;

    public  ServerDHKeyAgreement(ObjectInputStream in, ObjectOutputStream out) {
        ois = in;
        oos = out;

        try {
            Provider sunJCE = new com.sun.crypto.provider.SunJCE();
            Security.addProvider(sunJCE);

            // Add JCE to the list of providers
            SunJCE jce = new SunJCE();
            Security.addProvider(jce);
        } catch (Exception e) {
            System.err.println("Error: " + e);
        }
    }

    public SecretKey agree() throws Exception {

        DHParameterSpec dhSkipParamSpec;

        // use some pre-generated, default DH parameters
        System.err.println("Using SKIP Diffie-Hellman parameters");
        dhSkipParamSpec = new DHParameterSpec(skip1024Modulus, skip1024Base);

        /*
         * Server creates its own DH key pair, using the DH parameters from
         * above
         */
        System.out.println("Server: Generate DH keypair ...");
        KeyPairGenerator serverKpairGen = KeyPairGenerator.getInstance("DH");
        serverKpairGen.initialize(dhSkipParamSpec);
        KeyPair serverKpair = serverKpairGen.generateKeyPair();

        // Server executes Phase1 of its version of the DH protocol
        System.err.println("Server: Execute PHASE1 ...");
        KeyAgreement serverKeyAgree = KeyAgreement.getInstance("DH");
        serverKeyAgree.init(serverKpair.getPrivate());

        // Server encodes its public key, and sends it over to Client.
        byte[] serverPubKeyEnc = serverKpair.getPublic().getEncoded();

        // String st = toHexString(serverPubKeyEnc);
        // System.err.println("DH serverPubKeyEnc: " + st);

        // Server send its public key to Client
        oos.writeObject(serverPubKeyEnc);
        oos.flush();

        // Server receive Client's encoded public key
        byte[] clientPubKeyEnc = (byte[])ois.readObject();

        // st = toHexString(clientPubKeyEnc);
        // System.err.println("DH clientPubKeyEnc: " + st);

        /*
         * Server uses Client's public key for Phase2 of its version of the DH
         * protocol.
         * Before she can do so, she has to instanticate a DH public key
         * from Client's encoded key material.
         */
        KeyFactory serverKeyFac = KeyFactory.getInstance("DH");
        X509EncodedKeySpec x509KeySpec = new X509EncodedKeySpec(clientPubKeyEnc);
        PublicKey clientPubKey = serverKeyFac.generatePublic(x509KeySpec);
        System.err.println("Server: Execute PHASE2 ...");
        serverKeyAgree.doPhase(clientPubKey, true);

        /*
         * At this stage, both Server and Client have completed the DH key
         * agreement protocol.
         * Each generates the (same) shared secret.
         */

        SecretKey skey = serverKeyAgree.generateSecret("DES");
        return skey;
    }

    public void byte2hex(byte b, StringBuffer buf) {
        char hexChars[] = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
                           'A', 'B', 'C', 'D', 'E', 'F'};

        int low = (b & 0x0f);
        int high = (b & 0xf0) >> 4;

        buf.append(hexChars[high]);
        buf.append(hexChars[low]);
    }

    /*
     * Converts a byte array to hex string
     */
    public String toHexString(byte[] block) {
        StringBuffer buf = new StringBuffer();

        int len = block.length;

        for (int i = 0; i < len; i++) {
             byte2hex(block[i], buf);
             if (i < len-1) {
                 buf.append(":");
             }
        } 
        return buf.toString();
    }


    // The 1024 bit Diffie-Hellman modulus values used by SKIP
    private static final byte skip1024ModulusBytes[] = {
        (byte)0xF4, (byte)0x88, (byte)0xFD, (byte)0x58,
        (byte)0x4E, (byte)0x49, (byte)0xDB, (byte)0xCD,
        (byte)0x20, (byte)0xB4, (byte)0x9D, (byte)0xE4,
        (byte)0x91, (byte)0x07, (byte)0x36, (byte)0x6B,
        (byte)0x33, (byte)0x6C, (byte)0x38, (byte)0x0D,
        (byte)0x45, (byte)0x1D, (byte)0x0F, (byte)0x7C,
        (byte)0x88, (byte)0xB3, (byte)0x1C, (byte)0x7C,
        (byte)0x5B, (byte)0x2D, (byte)0x8E, (byte)0xF6,
        (byte)0xF3, (byte)0xC9, (byte)0x23, (byte)0xC0,
        (byte)0x43, (byte)0xF0, (byte)0xA5, (byte)0x5B,
        (byte)0x18, (byte)0x8D, (byte)0x8E, (byte)0xBB,
        (byte)0x55, (byte)0x8C, (byte)0xB8, (byte)0x5D,
        (byte)0x38, (byte)0xD3, (byte)0x34, (byte)0xFD,
        (byte)0x7C, (byte)0x17, (byte)0x57, (byte)0x43,
        (byte)0xA3, (byte)0x1D, (byte)0x18, (byte)0x6C,
        (byte)0xDE, (byte)0x33, (byte)0x21, (byte)0x2C,
        (byte)0xB5, (byte)0x2A, (byte)0xFF, (byte)0x3C,
        (byte)0xE1, (byte)0xB1, (byte)0x29, (byte)0x40,
        (byte)0x18, (byte)0x11, (byte)0x8D, (byte)0x7C,
        (byte)0x84, (byte)0xA7, (byte)0x0A, (byte)0x72,
        (byte)0xD6, (byte)0x86, (byte)0xC4, (byte)0x03,
        (byte)0x19, (byte)0xC8, (byte)0x07, (byte)0x29,
        (byte)0x7A, (byte)0xCA, (byte)0x95, (byte)0x0C,
        (byte)0xD9, (byte)0x96, (byte)0x9F, (byte)0xAB,
        (byte)0xD0, (byte)0x0A, (byte)0x50, (byte)0x9B,
        (byte)0x02, (byte)0x46, (byte)0xD3, (byte)0x08,
        (byte)0x3D, (byte)0x66, (byte)0xA4, (byte)0x5D,
        (byte)0x41, (byte)0x9F, (byte)0x9C, (byte)0x7C,
        (byte)0xBD, (byte)0x89, (byte)0x4B, (byte)0x22,
        (byte)0x19, (byte)0x26, (byte)0xBA, (byte)0xAB,
        (byte)0xA2, (byte)0x5E, (byte)0xC3, (byte)0x55,
        (byte)0xE9, (byte)0x2F, (byte)0x78, (byte)0xC7
    };

    // The SKIP 1024 bit modulus
    private static final BigInteger skip1024Modulus
    = new BigInteger(1, skip1024ModulusBytes);

    // The base used with the SKIP 1024 bit modulus
    private static final BigInteger skip1024Base = BigInteger.valueOf(2);
}
