import java.io.*;
import java.security.*;
import java.security.spec.*;
import java.security.interfaces.*;
import javax.crypto.*;

public class ServerHandShaking {

    private ObjectInputStream   ois;
    private ObjectOutputStream  oos;
    private PublicKey           serverPublicKey;
    private PublicKey           clientPublicKey;
    private PrivateKey          serverPrivateKey;
    private SecretKey           secretKey;
    private ServerKeyPair       skp;

    public ServerHandShaking(ObjectInputStream in, ObjectOutputStream out,
                             ServerKeyPair sk) {
        ois = in;
        oos = out;
        skp = sk;
    }

    public boolean init() {
        try {
            // Initialize the DH protocol and exchange the secret key for cipher
            ServerDHKeyAgreement dhKey = new ServerDHKeyAgreement(ois, oos);
            secretKey = dhKey.agree();

            serverPublicKey = skp.getPublicKey();
            serverPrivateKey = skp.getPrivateKey();
            byte[] serverEncodedPublicKey = skp.getEncodedPublicKey();

            // send Server's public key to Client
            oos.writeObject(serverEncodedPublicKey);
            oos.flush();
        
            // obtain Client's public key
            // This will be replaced by reading from client's certificate

            byte[] clientEncodedPublicKey = (byte[])ois.readObject();

            // st = toHexString(clientEncodedPublicKey);
            // System.err.println("Client's Encoded PublicKey: " + st);

            KeyFactory serverKeyFac = KeyFactory.getInstance("DSA");
            X509EncodedKeySpec x509KeySpec = new X509EncodedKeySpec(clientEncodedPublicKey);
            clientPublicKey = serverKeyFac.generatePublic(x509KeySpec);

            return true;
        } catch (Exception ex) {
            System.err.println("ServerHandShaking Exception-> " + ex);
            return false;
        }
    }

    public boolean init_() {
        try {

            // Initialize the DH protocol and exchange the secret key for cipher
            ServerDHKeyAgreement dhKey = new ServerDHKeyAgreement(ois, oos);
            secretKey = dhKey.agree();

            serverPublicKey = skp.getPublicKey();
            serverPrivateKey = skp.getPrivateKey();
            byte[] serverEncodedPublicKey = skp.getEncodedPublicKey();

            // obtain Client's public key
            // This will be replaced by reading from client's certificate

            byte[] clientEncodedPublicKey = (byte[])ois.readObject();

            // send Server's public key to Client
            oos.writeObject(serverEncodedPublicKey);
            oos.flush();

            // st = toHexString(clientEncodedPublicKey);
            // System.err.println("Client's Encoded PublicKey: " + st);

            KeyFactory serverKeyFac = KeyFactory.getInstance("DSA");
            X509EncodedKeySpec x509KeySpec = new X509EncodedKeySpec(clientEncodedPublicKey);
            clientPublicKey = serverKeyFac.generatePublic(x509KeySpec);

            return true;
        } catch (Exception ex) {
            System.err.println("ServerHandShaking Exception-> " + ex);
            return false;
        }
    }

    public PublicKey getServerPublicKey() {
        return this.serverPublicKey;
    }

    public PublicKey getClientPublicKey() {
        return this.clientPublicKey;
    }

    public PrivateKey getServerPrivateKey() {
        return this.serverPrivateKey;
    }

    public SecretKey getSecretKey() {
        return this.secretKey;
    }

    public void byte2hex(byte b, StringBuffer buf) {
        char hexChars[] = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'};

        int low = (b & 0x0f);
        int high = (b & 0xf0) >> 4;

        buf.append(hexChars[high]);
        buf.append(hexChars[low]);
    }

    /*
     * Converts a byte array to hex string
     */
    public String toHexString(byte[] block) {
        StringBuffer buf = new StringBuffer();

        int len = block.length;

        for (int i = 0; i < len; i++) {
             byte2hex(block[i], buf);
             if (i < len-1) {
                 buf.append(":");
             }
        }
        return buf.toString();
    }
}
