import java.io.*;
import java.net.*;
import java.security.*;
import javax.crypto.*;


public class ServerThreadSocketHandler extends Thread {

    private long                   id;
    private long                   trNo;
    private TransactionServantImpl ti;
    private boolean                action;
    private Socket                 socket;
    private ServerSecuredComm      ssc;
    private ServerInfo             si;
    private ServerServiceInfo      ssi;

    public ServerThreadSocketHandler() {}
    
    public ServerThreadSocketHandler(Socket s) {
        socket = s;
    }

    public ServerThreadSocketHandler(int i, Socket s, ServerInfo inf, 
                ServerKeyPair skp, ThreadGroup g, ServerServiceInfo ss) {
        super(g, Integer.toString(i));
        socket = s;
        si = inf;
        ssi = ss;
        action = false;
        ssc = new ServerSecuredComm(socket, skp);
        ti = new TransactionServantImpl();
    }

    public void setID(long l) {
        id = l;
        this.setName(Long.toString(id));
    }

    public void setTrNo(long no) {
        trNo = no;
    }

    public boolean isActive() {
        return action;
    }

    public void run() {
        try {

            action = true;

            // initialize the socket connection and all message control objects
            int i = 0;
            if (!ssc.init()) {
                System.err.println("ServerSecuredComm.init(): failed");
                ssc.done();
            }

            ssc.setupMessage(2, "Welcome to NPAC Server!");
            ssc.send();

            // temporary parameters
            UserInfo ui;
            TransactionMessage tm;
   
            // check the message type 
            boolean verification = false;
            i = 0;
            while ((!verification) && (i++ < 5)) {
                if (ssc.receive()) {
                    if (ssc.getMessageType() == 0) {
                        // verify user information
                        ui = (UserInfo)ssc.getMessage();

                        // receive time information from client
                        if (ssc.receive()) {
                            if (ssc.getMessageType() == 2) {
                                ReturnMessage rm = (ReturnMessage)ssc.getMessage();
                                if (rm.type == 10) {
                                    // initialize the transaction for client
                                    if (ti.init(ui.username, ui.password, ui.trNo, rm.msg)) {
                                        verification = true;
                                        action = true;
                                        System.err.println("User verification succeeded.");
                                        ssc.setupMessage((int)1, "User verification succeeded.");
                                        ssc.send();
                                        ti.open();
                                        setID(ui.trNo);
                                        ssi.putService(id, ui.username, 
                                                   (socket.getInetAddress()).getHostName(),
                                                   rm.msg, socket);
                                    } else {
                                        System.err.println("User verification failed. Try again.");
                                        ssc.setupMessage((int)0, "User verification failed. Try again.");
                                        ssc.send();
                                    }
                                } else {
                                    System.err.println("Server: Wrong type of return message");
                                }
                            } else {
                                System.err.println("Server: Wrong type of message.");
                            }
                        }
                    } else {
                        System.err.println("Message authentication failed. Try again.");
                        ssc.setupMessage((int)0, "Message authentication failed. Try again.");
                        ssc.send();
                    }
                } else {
                    System.err.println("Message authentication failed. Try again.");
                    ssc.setupMessage((int)0, "Message authentication failed. Try again.");
                    ssc.send();
                }
            }

            if (i >= 5) {
                System.err.println("Failed after 5 trials");
                ssc.setupMessage((int)0, "Message authentication failed. Try again later.");
                ssc.send();
                ssc.done();
                si.decrease();
                ssi.removeService(id);
                return;
            }

            boolean done = false;
 
            while (!done) {
                i = 0;
                boolean received = false;
                while (!received && (i++ < 5)) {
                    if (ssc.receive()) {
                        if (ssc.getMessageType() == 1) {
                            tm = (TransactionMessage)ssc.getMessage();
                            received = true;
                            switch(tm.transactionType) {
                            case 0: ssi.updateServiceTransaction(id, 0);
                                    ti.close();
                                    action = false;
                                    ssc.done();
                                    si.decrease();
                                    done = true;
                                    break;
                            case 1: ssi.updateServiceTransaction(id, 1);
                                    ssc.setupMessage((int)2, ti.querySavingAccount());
                                    ssc.send();
                                    break;
                            case 2: ssi.updateServiceTransaction(id, 2);
                                    ssc.setupMessage((int)2, ti.queryCheckingAccount());
                                    ssc.send();
                                    break;
                            case 3: ssi.updateServiceTransaction(id, 3);
                                    ssc.setupMessage((int)2, 
                                            ti.transferFromCheckingToSaving(tm.amount));
                                    ssc.send();
                                    break;
                            case 4: ssi.updateServiceTransaction(id, 4);
                                    ssc.setupMessage((int)2,
                                            ti.transferFromSavingToChecking(tm.amount));
                                    ssc.send();
                                    break;
                            default: System.err.println("No such transaction.");
                                     ssc.setupMessage((int)2,
                                             "Transaction missed. Please click [OK] again.");
                                     ssc.send();
                                     break;
                            }
                        } else {
                            ssc.setupMessage((int)3, "Transaction authentication failed.");
                            ssc.send();
                        }
                    } else {
                        ssc.setupMessage((int)3, "Transaction authentication failed.");
                        ssc.send();
                    }

                    try {
                        yield();
                    } catch (Exception e) {
                        System.err.println("ServerThread Exception-> " + e);
                    }
                } 

                int status = ssi.getServiceStatus(id).status;
                if (i >= 5) {
                    if (status > 0) {
                        ssi.updateServiceStatus(id, 0);
                        System.err.println("Data receiving Failed after 5 trials");
                    } else {
                        ssc.setupMessage((int)0, "Transaction authentication failed. Try again later.");
                        ssc.send();
                        ssc.done();
                        si.decrease();
                        ssi.removeService(id);
                        return;
                    }
                } else if (status < 1) {
                    ssi.updateServiceStatus(id, 1);
                }
            }
        } catch (Exception ex) {
            System.err.println("ServerThread Exception-> " + ex);
            ssc.done();
            si.decrease();
        }
        ssi.removeService(id);
    }
}
