import java.io.*;
import java.security.*;
import javax.crypto.*;

public class SignedControl {
    private SealedObject    seo;
    private SignedObject    sio;
    private String          signatureAlgorithm = "SHA/DSA";


    public SignedControl() {}

    public SignedControl(SealedObject o) {
        seo = o;
    }

    public SignedControl(SignedObject o) {
        sio = o;
    }

    public void initSign(PrivateKey key) {
        try {
            // sign the object with PrivateKey
            Signature sig = Signature.getInstance(signatureAlgorithm);
            sio = new SignedObject(seo, key, sig);
        } catch (Exception ex) {
            System.err.println("SignedControl.initSign()  Exception-> " + ex);
        }
    }

    public boolean verifySign(PublicKey key) {
        try {
            // verify the signature
            Signature sig = Signature.getInstance(signatureAlgorithm);
            if (!sio.verify(key, sig)) {
                System.err.println("Signature verification failed.");
                return false;
            }
        
            seo = (SealedObject)sio.getObject();
        } catch (Exception ex) {
            System.err.println("SignedControl.verifySign() Exception-> " + ex);
            return false;
        }
        return true;
    }

    public SealedObject getSealedObject() {
        return this.seo;
    }

    public SignedObject getSignedObject() {
        return sio;
    }

    public String getSignatureAlgorithm() {
        return this.signatureAlgorithm;
    }

    public void setSealedObject(SealedObject o) {
        seo = o;
    }

    public void setSignedObject(SignedObject o) {
        sio = o;
    }

    public void setSignatureAlgorithm(String s) {
        signatureAlgorithm = s;
    }
}
