import java.io.*;
import java.security.*;
import javax.crypto.*;

public class SignedMDControl {
    private MDMessage       mdMsg;
    private SealedObject    seo;
    private SignedObject    sio;
    private String          MDAlgorithm = "DSA";
    private String          signatureAlgorithm = "SHA/DSA";


    public SignedMDControl() {}

    public SignedMDControl(SealedObject o) {
        seo = o;
    }

    public SignedMDControl(SignedObject o) {
        sio = o;
    }

    public void initSign(PrivateKey key) {
        try {
            // convert SealedObject to byte[]
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            ObjectOutputStream oos = new ObjectOutputStream(baos);
            oos.writeObject(seo);
            oos.flush();
            oos.close();

            // compute MessageDigest
            MessageDigest md = MessageDigest.getInstance(MDAlgorithm);
            md.update(baos.toByteArray());

            // create a MDMessage of MessageDigest and SealedObject
            mdMsg = new MDMessage(seo, md.digest());

            // sign the object with PrivateKey
            Signature sig = Signature.getInstance(signatureAlgorithm);
            sio = new SignedObject(mdMsg, key, sig);
        } catch (Exception ex) {
            System.err.println("Signing Object Exception-> " + ex);
        }
    }

    public boolean verifySign(PublicKey key) {
        try {
            // verify the signature
            Signature sig = Signature.getInstance(signatureAlgorithm);
            if (!sio.verify(key, sig)) {
                System.err.println("Signature verification failed.");
                return false;
            }
        
            mdMsg = (MDMessage)sio.getObject();

            // convert SealedObject to byte[]
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            ObjectOutputStream oos = new ObjectOutputStream(baos);
            oos.writeObject(mdMsg.so);
            oos.flush();

            // verify MessageDigest
            MessageDigest md = MessageDigest.getInstance(MDAlgorithm);
            md.update(baos.toByteArray());
            if (!this.verify(mdMsg.md, md.digest())) {
                System.err.println("MessageDigest verification failes.");
                return false;
            }
            seo = mdMsg.so;
            oos.close();
        } catch (Exception ex) {
            System.err.println("Signature Verification Exception-> " + ex);
            return false;
        }
        return true;
    }

    public SealedObject getSealedObject() {
        return this.seo;
    }

    public SignedObject getSignedObject() {
        return sio;
    }

    public String getMDAlgorithm() {
        return this.MDAlgorithm;
    }

    public String getSignatureAlgorithm() {
        return this.signatureAlgorithm;
    }

    public void setSealedObject(SealedObject o) {
        seo = o;
    }

    public void setSignedObject(SignedObject o) {
        sio = o;
    }

    public void setMDAlgorithm(String s) {
        MDAlgorithm = s;
    }

    public void setSignatureAlgorithm(String s) {
        signatureAlgorithm = s;
    }

    private boolean verify(byte[] a, byte[] b) {
        if (a.length != b.length) return false;
        for (int i = 0; i < a.length; i++) {
            if (a[i] != b[i]) return false;
        }
        return true;
    }
}
