import java.awt.*;
import JSci.awt.*;
import JSci.maths.*;

/**
* RadialWavefunction.
* Calculates the radial wavefunctions of the hydrogen atom.
* @author Mark Hale
* @version 1.0
*/
public final class RadialWavefunction extends Frame {
        private final int N=100;
        private final double dr=0.05;
        private DoubleVector eVector[]=new DoubleVector[N];
        private double eValue[];
        private Label title,usage;
        private LineGraph graph;
        private List nList;
        public static void main(String argv[]) {
                new RadialWavefunction();
        }
        /**
        * Construct and Perform calculations.
        */
        public RadialWavefunction() {
                super("Hydrogen Radial Wavefunctions");
                DoubleTridiagonalMatrix m=new DoubleTridiagonalMatrix(N);
                double r=dr;
                int z=1;        // Nuclear Charge
// Create Hamiltonian matrix
                m.setElement(0,0,2.0-2.0*dr*dr*z/r);
                m.setElement(0,1,-1.0);
                for(int i=1;i<N-1;i++) {
                        r+=dr;
                        m.setElement(i,i-1,-1.0);
                        m.setElement(i,i,2.0-2.0*dr*dr*z/r);
                        m.setElement(i,i+1,-1.0);
                }
                r+=dr;
                m.setElement(N-1,N-2,-1.0);
                m.setElement(N-1,N-1,2.0-2.0*dr*dr*z/r);
// Calculate eigenvectors and eigenvalues
                try {
                        eValue=MatrixMath.eigenSolveSymmetric(m,eVector);
                } catch(MaximumIterationsExceededException e) {}
// Setup the display
                GridBagLayout gb=new GridBagLayout();
                GridBagConstraints gbc=new GridBagConstraints();
                setLayout(gb);
                title=new Label("n=1 energy level",Label.CENTER);
                graph=new LineGraph(getResults(0));
                usage=new Label("Double click to select an energy level:",Label.CENTER);
                nList=new List(5,false);
                for(int i=0;i<N;)
                        nList.addItem(String.valueOf(++i));
                nList.select(0);
                gbc.gridwidth=GridBagConstraints.REMAINDER;
                gbc.fill=GridBagConstraints.NONE;
                gbc.weightx=0.7;
                gbc.weighty=0.7;
                gbc.insets=new Insets(0,30,0,30);
                gb.setConstraints(title,gbc);
                add(title);
                gbc.fill=GridBagConstraints.BOTH;
                gb.setConstraints(graph,gbc);
                add(graph);
                gbc.fill=GridBagConstraints.NONE;
                gb.setConstraints(usage,gbc);
                add(usage);
                gb.setConstraints(nList,gbc);
                add(nList);
                resize(450,450);
                show();
        }
        /**
        * Get wavefunction from eigenvectors.
        * @param n (n+1)th energy level
        * @return an array of data points
        */
        private double[][] getResults(int n) {
                double results[][]=new double[2][N];
                double r=dr;
                for(int i=0;i<N;i++) {
                        results[0][i]=r;
                        results[1][i]=eVector[n].getComponent(i)/r;
                        r+=dr;
                }
                return results;
        }
        public boolean action(Event evt,Object obj) {
                if(evt.id==Event.ACTION_EVENT) {
                        int n=nList.getSelectedIndex();
                        title.setText("n="+(n+1)+" energy level");
                        graph.setData(getResults(n));
                        return true;
                } else
                        return super.action(evt,obj);
        }
        public boolean handleEvent(Event evt) {
                if(evt.id==Event.WINDOW_DESTROY) {
                        dispose();
                        System.exit(0);
                        return true;
                } else
                        return super.handleEvent(evt);
        }
}

