package JSci.maths;

/**
* The Complex class provides an object for encapsulating complex numbers.
* @version 1.6
* @author Mark Hale
*/
public final class Complex extends Number {
        static {System.loadLibrary("JSci");}
        private double re;
        private double im;
        /**
        * The complex number 0+1i.
        */
        public final static Complex I=new Complex(0.0,1.0);
        /**
        * The complex number 1+0i.
        */
        public final static Complex ONE=new Complex(1.0,0.0);
        /**
        * The complex number 0+0i.
        */
        public final static Complex ZERO=new Complex(0.0,0.0);
        /**
        * The complex number -1+0i.
        */
        public final static Complex MINUS_ONE=new Complex(-1.0,0.0);
        /**
        * The complex number 0-1i.
        */
        public final static Complex MINUS_I=new Complex(0.0,-1.0);
        /**
        * Constructs the complex number x+yi.
        * @param x the real value of a complex number
        * @param y the imaginary value of a complex number
        */
        public Complex(double x,double y) {
                re=x;
                im=y;
        }
        /**
        * Creates a complex number with the given modulus and argument.
        * @param mod the modulus of a complex number
        * @param arg the argument of a complex number
        */
        public native static Complex polar(double mod, double arg);
        /**
        * Compares two complex numbers for equality.
        * @param z a complex number
        */
        public boolean equals(Object z) {
                return (z!=null) && (z instanceof Complex) &&
                        (re==((Complex)z).real()) &&
                        (im==((Complex)z).imag());
        }
        /**
        * Creates a clone of this complex number.
        * @return a copy of this complex number.
        */
        public Object clone() {
                return new Complex(re,im);
        }
        /**
        * Returns a string representing the value of this complex number.
        */
        public String toString() {
                StringBuffer buf=new StringBuffer();
                buf.append(re);
                if(im>=0.0) {
                        buf.append("+");
                        buf.append(im);
                } else {
                        buf.append(im);
                }
                buf.append("i");
                return buf.toString();
        }
        /**
        * Returns a hashcode for this complex number.
        */
        public int hashCode() {
                return (int)Math.exp(mod());
        }
        /**
        * Maps the complex number to a real number using the scalar product.
        */
        public byte byteValue() {
                return (byte)(intValue());
        }
        /**
        * Maps the complex number to a real number using the scalar product.
        */
        public short shortValue() {
                return (short)(intValue());
        }
        /**
        * Maps the complex number to a real number using the scalar product.
        */
        public int intValue() {
                return Math.round(floatValue());
        }
        /**
        * Maps the complex number to a real number using the scalar product.
        */
        public long longValue() {
                return Math.round(doubleValue());
        }
        /**
        * Maps the complex number to a real number using the scalar product.
        */
        public float floatValue() {
                return (float)(mod());
        }
        /**
        * Maps the complex number to a real number using the scalar product.
        */
        public double doubleValue() {
                return mod();
        }
        /**
        * Returns true if either the real or imaginary part is NaN.
        */
        public boolean isNaN() {
                return (re!=re) || (im!=im);
        }
        /**
        * Returns true if either the real or imaginary part is infinite.
        */
        public boolean isInfinite() {
                return (re==Double.POSITIVE_INFINITY) || (re==Double.NEGATIVE_INFINITY)
                        || (im==Double.POSITIVE_INFINITY) || (im==Double.NEGATIVE_INFINITY);
        }
        /**
        * Returns the real part of this complex number.
        */
        public double real() {
                return re;
        }
        /**
        * Returns the imaginary part of this complex number.
        */
        public double imag() {
                return im;
        }
        /**
        * Returns the modulus of this complex number.
        */
        public native double mod();
        /**
        * Returns the argument of this complex number.
        */
        public native double arg();
}

