
#include "JSci.hpp"
#include "JSci_maths_ComplexMath.h"


JNIEXPORT jobject JNICALL Java_JSci_maths_ComplexMath_negate(JNIEnv *env, jclass self, jobject z)
{
   jclass cls=env->GetObjectClass(z);
   jmethodID midReal=env->GetMethodID(cls,"real","()D");
   jmethodID midImag=env->GetMethodID(cls,"imag","()D");
   jmethodID midCon=env->GetMethodID(cls,"<init>","(DD)V");
   jdouble re=env->CallDoubleMethod(z,midReal);
   jdouble im=env->CallDoubleMethod(z,midImag);
   return env->NewObject(cls,midCon,-re,-im);
}

JNIEXPORT jobject JNICALL Java_JSci_maths_ComplexMath_conjugate(JNIEnv *env, jclass self, jobject z)
{
   jclass cls=env->GetObjectClass(z);
   jmethodID midReal=env->GetMethodID(cls,"real","()D");
   jmethodID midImag=env->GetMethodID(cls,"imag","()D");
   jmethodID midCon=env->GetMethodID(cls,"<init>","(DD)V");
   jdouble re=env->CallDoubleMethod(z,midReal);
   jdouble im=env->CallDoubleMethod(z,midImag);
   return env->NewObject(cls,midCon,re,-im);
}

JNIEXPORT jobject JNICALL Java_JSci_maths_ComplexMath_add(JNIEnv *env, jclass self, jobject z1, jobject z2)
{
   jclass cls=env->GetObjectClass(z1);
   jmethodID midReal=env->GetMethodID(cls,"real","()D");
   jmethodID midImag=env->GetMethodID(cls,"imag","()D");
   jmethodID midCon=env->GetMethodID(cls,"<init>","(DD)V");
   jdouble z1Re=env->CallDoubleMethod(z1,midReal);
   jdouble z1Im=env->CallDoubleMethod(z1,midImag);
   jdouble z2Re=env->CallDoubleMethod(z2,midReal);
   jdouble z2Im=env->CallDoubleMethod(z2,midImag);
   return env->NewObject(cls,midCon,z1Re+z2Re,z1Im+z2Im);
}

JNIEXPORT jobject JNICALL Java_JSci_maths_ComplexMath_subtract(JNIEnv *env, jclass self, jobject z1, jobject z2)
{
   jclass cls=env->GetObjectClass(z1);
   jmethodID midReal=env->GetMethodID(cls,"real","()D");
   jmethodID midImag=env->GetMethodID(cls,"imag","()D");
   jmethodID midCon=env->GetMethodID(cls,"<init>","(DD)V");
   jdouble z1Re=env->CallDoubleMethod(z1,midReal);
   jdouble z1Im=env->CallDoubleMethod(z1,midImag);
   jdouble z2Re=env->CallDoubleMethod(z2,midReal);
   jdouble z2Im=env->CallDoubleMethod(z2,midImag);
   return env->NewObject(cls,midCon,z1Re-z2Re,z1Im-z2Im);
}

JNIEXPORT jobject JNICALL Java_JSci_maths_ComplexMath_multiply(JNIEnv *env, jclass self, jobject z1, jobject z2)
{
   jclass cls=env->GetObjectClass(z1);
   jmethodID midReal=env->GetMethodID(cls,"real","()D");
   jmethodID midImag=env->GetMethodID(cls,"imag","()D");
   jmethodID midCon=env->GetMethodID(cls,"<init>","(DD)V");
   jdouble z1Re=env->CallDoubleMethod(z1,midReal);
   jdouble z1Im=env->CallDoubleMethod(z1,midImag);
   jdouble z2Re=env->CallDoubleMethod(z2,midReal);
   jdouble z2Im=env->CallDoubleMethod(z2,midImag);
   return env->NewObject(cls,midCon,z1Re*z2Re-z1Im*z2Im,z1Re*z2Im+z1Im*z2Re);
}

JNIEXPORT jobject JNICALL Java_JSci_maths_ComplexMath_divide(JNIEnv *env, jclass self, jobject z1, jobject z2)
{
   jclass cls=env->GetObjectClass(z1);
   jmethodID midReal=env->GetMethodID(cls,"real","()D");
   jmethodID midImag=env->GetMethodID(cls,"imag","()D");
   jmethodID midCon=env->GetMethodID(cls,"<init>","(DD)V");
   jdouble z1Re=env->CallDoubleMethod(z1,midReal);
   jdouble z1Im=env->CallDoubleMethod(z1,midImag);
   jdouble z2Re=env->CallDoubleMethod(z2,midReal);
   jdouble z2Im=env->CallDoubleMethod(z2,midImag);
   jdouble denominator,re,im,a;
   if(fabs(z2Re)<fabs(z2Im)) {
      a=z2Re/z2Im;
      denominator=z2Re*a+z2Im;
      re=z1Re*a+z1Im;
      im=z1Im*a-z1Re;
   } else {
      a=z2Im/z2Re;
      denominator=z2Re+z2Im*a;
      re=z1Re+z1Im*a;
      im=z1Im-z1Re*a;
   }
   return env->NewObject(cls,midCon,re/denominator,im/denominator);
}

JNIEXPORT jobject JNICALL Java_JSci_maths_ComplexMath_pow(JNIEnv *env, jclass self, jobject z1, jobject z2)
{
   jclass cls=env->GetObjectClass(z1);
   jmethodID midPolar=env->GetMethodID(cls,"polar","(DD)LJSci/maths/Complex;");
   jmethodID midReal=env->GetMethodID(cls,"real","()D");
   jmethodID midImag=env->GetMethodID(cls,"imag","()D");
   jmethodID midMod=env->GetMethodID(cls,"mod","()D");
   jmethodID midArg=env->GetMethodID(cls,"arg","()D");
   jdouble z1Mod=env->CallDoubleMethod(z1,midMod);
   jdouble z1Arg=env->CallDoubleMethod(z1,midArg);
   jdouble z2Re=env->CallDoubleMethod(z2,midReal);
   jdouble z2Im=env->CallDoubleMethod(z2,midImag);
   jdouble r=pow(z1Mod,z2Re)/exp(z1Arg*z2Im);
   jdouble a=z1Arg*z2Re+log(z1Mod)*z2Im;
   return env->CallStaticObjectMethod(cls,midPolar,r,a);
}

JNIEXPORT jobject JNICALL Java_JSci_maths_ComplexMath_exp(JNIEnv *env, jclass self, jobject z)
{
   jclass cls=env->GetObjectClass(z);
   jmethodID midReal=env->GetMethodID(cls,"real","()D");
   jmethodID midImag=env->GetMethodID(cls,"imag","()D");
   jmethodID midCon=env->GetMethodID(cls,"<init>","(DD)V");
   jdouble zRe=env->CallDoubleMethod(z,midReal);
   jdouble zIm=env->CallDoubleMethod(z,midImag);
   return env->NewObject(cls,midCon,exp(zRe)*cos(zIm),exp(zRe)*sin(zIm));
}

JNIEXPORT jobject JNICALL Java_JSci_maths_ComplexMath_log(JNIEnv *env, jclass self, jobject z)
{
   jclass cls=env->GetObjectClass(z);
   jmethodID midMod=env->GetMethodID(cls,"mod","()D");
   jmethodID midArg=env->GetMethodID(cls,"arg","()D");
   jmethodID midCon=env->GetMethodID(cls,"<init>","(DD)V");
   jdouble zMod=env->CallDoubleMethod(z,midMod);
   jdouble zArg=env->CallDoubleMethod(z,midArg);
   return env->NewObject(cls,midCon,log(zMod),zArg);
}

JNIEXPORT jobject JNICALL Java_JSci_maths_ComplexMath_sin(JNIEnv *env, jclass self, jobject z)
{
   jclass cls=env->GetObjectClass(z);
   jmethodID midReal=env->GetMethodID(cls,"real","()D");
   jmethodID midImag=env->GetMethodID(cls,"imag","()D");
   jmethodID midCon=env->GetMethodID(cls,"<init>","(DD)V");
   jdouble zRe=env->CallDoubleMethod(z,midReal);
   jdouble zIm=env->CallDoubleMethod(z,midImag);
   return env->NewObject(cls,midCon,sin(zRe)*cosh(zIm),cos(zRe)*sinh(zIm));
}

JNIEXPORT jobject JNICALL Java_JSci_maths_ComplexMath_cos(JNIEnv *env, jclass self, jobject z)
{
   jclass cls=env->GetObjectClass(z);
   jmethodID midReal=env->GetMethodID(cls,"real","()D");
   jmethodID midImag=env->GetMethodID(cls,"imag","()D");
   jmethodID midCon=env->GetMethodID(cls,"<init>","(DD)V");
   jdouble zRe=env->CallDoubleMethod(z,midReal);
   jdouble zIm=env->CallDoubleMethod(z,midImag);
   return env->NewObject(cls,midCon,cos(zRe)*cosh(zIm),-sin(zRe)*sinh(zIm));
}

JNIEXPORT jobject JNICALL Java_JSci_maths_ComplexMath_tan(JNIEnv *env, jclass self, jobject z)
{
   return Java_JSci_maths_ComplexMath_divide(env,self,
      Java_JSci_maths_ComplexMath_sin(env,self,z),
      Java_JSci_maths_ComplexMath_cos(env,self,z));
}

JNIEXPORT jobject JNICALL Java_JSci_maths_ComplexMath_sinh(JNIEnv *env, jclass self, jobject z)
{
   jclass cls=env->GetObjectClass(z);
   jmethodID midReal=env->GetMethodID(cls,"real","()D");
   jmethodID midImag=env->GetMethodID(cls,"imag","()D");
   jmethodID midCon=env->GetMethodID(cls,"<init>","(DD)V");
   jdouble zRe=env->CallDoubleMethod(z,midReal);
   jdouble zIm=env->CallDoubleMethod(z,midImag);
   return env->NewObject(cls,midCon,sinh(zRe)*cos(zIm),cosh(zRe)*sin(zIm));
}

JNIEXPORT jobject JNICALL Java_JSci_maths_ComplexMath_cosh(JNIEnv *env, jclass self, jobject z)
{
   jclass cls=env->GetObjectClass(z);
   jmethodID midReal=env->GetMethodID(cls,"real","()D");
   jmethodID midImag=env->GetMethodID(cls,"imag","()D");
   jmethodID midCon=env->GetMethodID(cls,"<init>","(DD)V");
   jdouble zRe=env->CallDoubleMethod(z,midReal);
   jdouble zIm=env->CallDoubleMethod(z,midImag);
   return env->NewObject(cls,midCon,cosh(zRe)*cos(zIm),sinh(zRe)*sin(zIm));
}

JNIEXPORT jobject JNICALL Java_JSci_maths_ComplexMath_tanh(JNIEnv *env, jclass self, jobject z)
{
   return Java_JSci_maths_ComplexMath_divide(env,self,
      Java_JSci_maths_ComplexMath_sinh(env,self,z),
      Java_JSci_maths_ComplexMath_cosh(env,self,z));
}


