
#include "JSci.hpp"
#include "JSci_maths_MatrixMath.h"


JNIEXPORT void JNICALL Java_JSci_maths_MatrixMath_eigenvalueSolveSymmetricTridiagonalMatrix(JNIEnv *env, jclass self, jdoubleArray diagj, jdoubleArray offdiagj)
{
   jint m,l,iteration,i,k,n=env->GetArrayLength(diagj),nm1=n-1;
   jdouble s,r,p,g,f,dd,c,b;
   jdouble *diag=env->GetDoubleArrayElements(diagj,0);
   jdouble *offdiag=env->GetDoubleArrayElements(offdiagj,0);
   for(l=0;l<n;l++) {
      iteration=0;
      do {
         for(m=l;m<nm1;m++) {
            dd=fabs(diag[m])+fabs(diag[m+1]);
            if(fabs(offdiag[m])+dd==dd)
               break;
         }
         if(m!=l) {
            if(iteration++==50) {
               jclass excCls=env->FindClass("JSci/maths/MaximumIterationsExceededException");
               env->ThrowNew(excCls,"No convergence after 50 iterations.");
            }
            g=(diag[l+1]-diag[l])/(2.0*offdiag[l]);
            r=sqrt(g*g+1.0);
            g=diag[m]-diag[l]+offdiag[l]/(g+(g<0?-fabs(r):fabs(r)));
            s=c=1.0;
            p=0.0;
            for(i=m-1;i>=l;i--) {
               f=s*offdiag[i];
               b=c*offdiag[i];
               if(fabs(f)>=fabs(g)) {
                  c=g/f;
                  r=sqrt(c*c+1.0);
                  offdiag[i+1]=f*r;
                  s=1/r;
                  c*=s;
               } else {
                  s=f/g;
                  r=sqrt(s*s+1.0);
                  offdiag[i+1]=g*r;
                  c=1/r;
                  s*=c;
               }
               g=diag[i+1]-p;
               r=(diag[i]-g)*s+2.0*c*b;
               p=s*r;
               diag[i+1]=g+p;
               g=c*r-b;
            }
            diag[l]=diag[l]-p;
            offdiag[l]=g;
            offdiag[m]=0.0;
         }
      } while(m!=l);
   }
   env->ReleaseDoubleArrayElements(diagj,diag,0);
   env->ReleaseDoubleArrayElements(offdiagj,offdiag,0);
}

JNIEXPORT void JNICALL Java_JSci_maths_MatrixMath_reduceSymmetric1_1SquareToTridiagonal(JNIEnv *env, jclass self, jobjectArray matrixj, jdoubleArray diagj, jdoubleArray offdiagj)
{
   jint i,j,k,l,n=env->GetArrayLength(diagj);
   jdouble f,g,h,hh,scale;
   jdouble *diag=env->GetDoubleArrayElements(diagj,0);
   jdouble *offdiag=env->GetDoubleArrayElements(offdiagj,0);
   jdouble **matrix=new jdouble*[n];
   jdoubleArray *arrayj=new jdoubleArray[n];
   for(i=0;i<n;i++) {
      arrayj[i]=(jdoubleArray)env->GetObjectArrayElement(matrixj,i);
      matrix[i]=env->GetDoubleArrayElements(arrayj[i],0);
   }
   for(i=n-1;i>0;i--) {
      l=i-1;
      h=scale=0.0;
      if(l>0) {
         for(k=0;k<=l;k++)
            scale+=fabs(matrix[i][k]);
         if(scale==0.0)
            offdiag[i]=matrix[i][l];
         else {
            for(k=0;k<=l;k++) {
               matrix[i][k]/=scale;
               h+=matrix[i][k]*matrix[i][k];
            }
            f=matrix[i][l];
            g=(f>0.0?-sqrt(h):sqrt(h));
            offdiag[i]=scale*g;
            h-=f*g;
            matrix[i][l]=f-g;
            f=0.0;
            for(j=0;j<=l;j++) {
               g=0.0;
               for(k=0;k<j;k++)
                  g+=matrix[j][k]*matrix[i][k];
               for(k=j;k<=l;k++)
                  g+=matrix[k][j]*matrix[i][k];
               offdiag[j]=g/h;
               f+=offdiag[j]*matrix[i][j];
            }
            hh=f/(h+h);
            for(j=0;j<=l;j++) {
               f=matrix[i][j];
               offdiag[j]=g=offdiag[j]-hh*f;
               for(k=0;k<j;k++)
                  matrix[j][k]-=f*offdiag[k]+g*matrix[i][k];
            }
         }
      } else
         offdiag[i]=matrix[i][l];
      diag[i]=h;
   }
   offdiag[0]=0.0;
   for(i=0;i<n;i++)
      diag[i]=matrix[i][i];
   for(i=0;i<n;i++) {
      env->ReleaseDoubleArrayElements(arrayj[i],matrix[i],0);
      env->SetObjectArrayElement(matrixj,i,arrayj[i]);
   }
   env->ReleaseDoubleArrayElements(diagj,diag,0);
   env->ReleaseDoubleArrayElements(offdiagj,offdiag,0);
}

JNIEXPORT void JNICALL Java_JSci_maths_MatrixMath_eigenSolveSymmetricTridiagonalMatrix(JNIEnv *env, jclass self, jdoubleArray diagj, jdoubleArray offdiagj, jobjectArray transfj)
{
   jint m,l,iteration,i,k,n=env->GetArrayLength(diagj),nm1=n-1;
   jdouble s,r,p,g,f,dd,c,b;
   jdouble *diag=env->GetDoubleArrayElements(diagj,0);
   jdouble *offdiag=env->GetDoubleArrayElements(offdiagj,0);
   jdouble **transf=new jdouble*[n];
   jdoubleArray *arrayj=new jdoubleArray[n];
   for(i=0;i<n;i++) {
      arrayj[i]=(jdoubleArray)env->GetObjectArrayElement(transfj,i);
      transf[i]=env->GetDoubleArrayElements(arrayj[i],0);
   }
   for(l=0;l<n;l++) {
      iteration=0;
      do {
         for(m=l;m<nm1;m++) {
            dd=fabs(diag[m])+fabs(diag[m+1]);
            if(fabs(offdiag[m])+dd==dd)
               break;
         }
         if(m!=l) {
            if(iteration++==50) {
               jclass excCls=env->FindClass("JSci/maths/MaximumIterationsExceededException");
               env->ThrowNew(excCls,"No convergence after 50 iterations.");
            }
            g=(diag[l+1]-diag[l])/(2.0*offdiag[l]);
            r=sqrt(g*g+1.0);
            g=diag[m]-diag[l]+offdiag[l]/(g+(g<0?-fabs(r):fabs(r)));
            s=c=1.0;
            p=0.0;
            for(i=m-1;i>=l;i--) {
               f=s*offdiag[i];
               b=c*offdiag[i];
               if(fabs(f)>=fabs(g)) {
                  c=g/f;
                  r=sqrt(c*c+1.0);
                  offdiag[i+1]=f*r;
                  s=1/r;
                  c*=s;
               } else {
                  s=f/g;
                  r=sqrt(s*s+1.0);
                  offdiag[i+1]=g*r;
                  c=1/r;
                  s*=c;
               }
               g=diag[i+1]-p;
               r=(diag[i]-g)*s+2.0*c*b;
               p=s*r;
               diag[i+1]=g+p;
               g=c*r-b;
               for(k=0;k<n;k++) {
                  f=transf[k][i+1];
                  transf[k][i+1]=s*transf[k][i]+c*f;
                  transf[k][i]=c*transf[k][i]-s*f;
               }
            }
            diag[l]=diag[l]-p;
            offdiag[l]=g;
            offdiag[m]=0.0;
         }
      } while(m!=l);
   }
   for(i=0;i<n;i++) {
      env->ReleaseDoubleArrayElements(arrayj[i],transf[i],0);
      env->SetObjectArrayElement(transfj,i,arrayj[i]);
   }
   env->ReleaseDoubleArrayElements(diagj,diag,0);
   env->ReleaseDoubleArrayElements(offdiagj,offdiag,0);
}

JNIEXPORT void JNICALL Java_JSci_maths_MatrixMath_reduceSymmetric2_1SquareToTridiagonal(JNIEnv *env, jclass self, jobjectArray matrixj, jdoubleArray diagj, jdoubleArray offdiagj)
{
   jint i,j,k,l,n=env->GetArrayLength(diagj);
   jdouble f,g,h,hh,scale;
   jdouble *diag=env->GetDoubleArrayElements(diagj,0);
   jdouble *offdiag=env->GetDoubleArrayElements(offdiagj,0);
   jdouble **matrix=new jdouble*[n];
   jdoubleArray *arrayj=new jdoubleArray[n];
   for(i=0;i<n;i++) {
      arrayj[i]=(jdoubleArray)env->GetObjectArrayElement(matrixj,i);
      matrix[i]=env->GetDoubleArrayElements(arrayj[i],0);
   }
   for(i=n-1;i>0;i--) {
      l=i-1;
      h=scale=0.0;
      if(l>0) {
         for(k=0;k<=l;k++)
            scale+=fabs(matrix[i][k]);
         if(scale==0.0)
            offdiag[i]=matrix[i][l];
         else {
            for(k=0;k<=l;k++) {
               matrix[i][k]/=scale;
               h+=matrix[i][k]*matrix[i][k];
            }
            f=matrix[i][l];
            g=(f>0.0?-sqrt(h):sqrt(h));
            offdiag[i]=scale*g;
            h-=f*g;
            matrix[i][l]=f-g;
            f=0.0;
            for(j=0;j<=l;j++) {
               matrix[j][i]=matrix[i][j]/h;
               g=0.0;
               for(k=0;k<j;k++)
                  g+=matrix[j][k]*matrix[i][k];
               for(k=j;k<=l;k++)
                  g+=matrix[k][j]*matrix[i][k];
               offdiag[j]=g/h;
               f+=offdiag[j]*matrix[i][j];
            }
            hh=f/(h+h);
            for(j=0;j<=l;j++) {
               f=matrix[i][j];
               offdiag[j]=g=offdiag[j]-hh*f;
               for(k=0;k<j;k++)
                  matrix[j][k]-=f*offdiag[k]+g*matrix[i][k];
            }
         }
      } else
         offdiag[i]=matrix[i][l];
      diag[i]=h;
   }
   diag[0]=offdiag[0]=0.0;
   for(i=0;i<n;i++) {
      l=i-1;
      if(diag[i]!=0.0) {
         for(j=0;j<=l;j++) {
            g=0.0;
            for(k=0;k<=l;k++)
               g+=matrix[i][k]*matrix[k][j];
            for(k=0;k<=l;k++)
               matrix[k][j]-=g*matrix[k][i];
         }
      }
      diag[i]=matrix[i][i];
      matrix[i][i]=1.0;
      for(j=0;j<=l;j++)
         matrix[j][i]=matrix[i][j]=0.0;
   }
   for(i=0;i<n;i++) {
      env->ReleaseDoubleArrayElements(arrayj[i],matrix[i],0);
      env->SetObjectArrayElement(matrixj,i,arrayj[i]);
   }
   env->ReleaseDoubleArrayElements(diagj,diag,0);
   env->ReleaseDoubleArrayElements(offdiagj,offdiag,0);
}

