package JSci.io;

import java.io.*;
import JSci.maths.*;

/**
* Text file output stream, writes data text files.
* @version 1.0
* @author Mark Hale
*/
public final class TextFileOutputStream extends FileOutputStream {
        private char delimiter;
        /**
        * Creates a text output file with the specified system dependent file name.
        * @param name the system dependent file name
        * @param ch the character that delimits data columns
        * @exception IOException If the file is not found.
        */
        public TextFileOutputStream(String name,char ch) throws IOException {
                super(name);
                delimiter=ch;
        }
        /**
        * Creates a text output file with the specified File object.
        * @param file the file to be opened for writing
        * @param ch the character that delimits data columns
        * @exception IOException If the file is not found.
        */
        public TextFileOutputStream(File file,char ch) throws IOException {
                super(file);
                delimiter=ch;
        }
        /**
        * Writes an array of data.
        * @param data the data to be written
        */
        public void writeData(double data[][]) throws IOException {
                DataOutputStream os=new DataOutputStream(this);
                for(int i=0;i<data.length;i++) {
                        for(int j=0;j<data[0].length-1;j++)
				os.writeBytes(data[i][j]+" "+delimiter+" ");
                        os.writeBytes(data[i][data[0].length-1]+"\r\n");
                }
                os.close();
        }
        /**
        * Writes a matrix.
        * @param matrix the matrix to be written
        */
        public void writeMatrix(DoubleMatrix matrix) throws IOException {
                int mRow=matrix.rows(),mCol=matrix.columns();
                DataOutputStream os=new DataOutputStream(this);
                for(int i=0;i<mRow;i++) {
                        for(int j=0;j<mCol-1;j++)
				os.writeBytes(matrix.getElement(i,j)+" "+delimiter+" ");
                        os.writeBytes(matrix.getElement(i,mCol-1)+"\r\n");
                }
                os.close();
        }

}



