package JSci.maths;

/**
* The ComplexSquareMatrix class provides an object for encapsulating square matrices containing complex numbers.
* @version 1.5
* @author Mark Hale
*/
public class ComplexSquareMatrix extends ComplexMatrix {
        /**
        * Constructs a matrix.
        */
        protected ComplexSquareMatrix() {
                super();
        }
        /**
        * Constructs an empty matrix.
        * @param size the number of rows/columns
        */
        public ComplexSquareMatrix(int size) {
                super(size,size);
        }
        /**
        * Constructs a matrix by wrapping an array.
        * @param array an assigned value
        * @exception MatrixDimensionException If the array is not square.
        */
        public ComplexSquareMatrix(Complex array[][]) {
                super(array);
                if(array.length!=array[0].length) {
                        matrix=null;
                        throw new MatrixDimensionException("The array is not square.");
                }
        }
        /**
        * Creates a clone of this matrix.
        * @return a copy of this matrix.
        */
        public Object clone() {
                Complex array[][]=new Complex[matrix.length][matrix[0].length];
                for(int j,i=0;i<matrix.length;i++) {
                        for(j=0;j<matrix[0].length;j++)
                                array[i][j]=(Complex)matrix[i][j].clone();
                }
                return new ComplexSquareMatrix(array);
        }
        /**
        * Returns true if this matrix is unitary.
        */
        public boolean isUnitary() {
                return ComplexDiagonalMatrix.identity(matrix[0].length).equals(MatrixMath.multiply(this,MatrixMath.hermitianAdjoint(this)));
        }
        /**
        * Returns the determinant.
        */
        public Complex determinant() {
                ComplexSquareMatrix lu[]=MatrixMath.LUDecomposition(this);
                Complex det=lu[1].getElement(0,0);
                for(int i=1;i<matrix.length;i++)
                        det=ComplexMath.multiply(det,lu[1].getElement(i,i));
                return det;
        }
        /**
        * Returns the trace.
        * @author Taber Smith
        */
        public Complex trace() {
                Complex result=matrix[0][0];
                for (int i=1;i<matrix.length;i++)
                        result=ComplexMath.add(result,matrix[i][i]);
                return result;
        }
}

