package JSci.maths;

/**
* The DoubleSquareMatrix class provides an object for encapsulating square matrices containing doubles.
* @version 1.5
* @author Mark Hale
*/
public class DoubleSquareMatrix extends DoubleMatrix {
        /**
        * Constructs a matrix.
        */
        protected DoubleSquareMatrix() {
                super();
        }
        /**
        * Constructs an empty matrix.
        * @param size the number of rows/columns
        */
        public DoubleSquareMatrix(int size) {
                super(size,size);
        }
        /**
        * Constructs a matrix by wrapping an array.
        * @param array an assigned value
        * @exception MatrixDimensionException If the array is not square.
        */
        public DoubleSquareMatrix(double array[][]) {
                super(array);
                if(array.length!=array[0].length) {
                        matrix=null;
                        throw new MatrixDimensionException("The array is not square.");
                }
        }
        /**
        * Creates a clone of this matrix.
        * @return a copy of this matrix.
        */
        public Object clone() {
                double array[][]=new double[matrix.length][matrix[0].length];
                for(int j,i=0;i<matrix.length;i++) {
                        for(j=0;j<matrix[0].length;j++)
                                array[i][j]=matrix[i][j];
                }
                return new DoubleSquareMatrix(array);
        }
        /**
        * Returns true if this matrix is unitary.
        */
        public boolean isUnitary() {
                return DoubleDiagonalMatrix.identity(matrix[0].length).equals(MatrixMath.multiply(this,MatrixMath.transpose(this)));
        }
        /**
        * Returns the determinant.
        */
        public double determinant() {
                DoubleSquareMatrix lu[]=MatrixMath.LUDecomposition(this);
                double det=lu[1].getElement(0,0);
                for(int i=1;i<matrix.length;i++)
                        det*=lu[1].getElement(i,i);
                return det;
        }
        /**
        * Returns the trace.
        * @author Taber Smith
        */
        public double trace() {
                double result=matrix[0][0];
                for(int i=1;i<matrix.length;i++)
                        result+=matrix[i][i];
                return result;
        }
}

