package JSci.maths;

/**
* The Group class provides an abstract encapsulation for groups.
* @version 1.1
* @author Mark Hale
*/
public abstract class Group extends Set {
        /**
        * Constructs a group.
        */
        public Group() {
        }
        /**
        * The group composition law.
        * @param a a group element
        * @param b a group element
        */
        public abstract Object compose(Object a, Object b);
        /**
        * Returns the identity element.
        */
        public abstract Object identity();
        /**
        * Returns the inverse element.
        */
        public abstract Object inverse(Object a);
        /**
        * Returns true if the object is a member of this group.
        * @param a an object
        */
        public abstract boolean isMember(Object a);
        /**
        * Returns true if the element is the identity element of this group.
        * @param a a group element
        */
        public abstract boolean isIdentity(Object a);
        /**
        * Returns true if one element is the inverse of the other.
        * @param a a group element
        * @param b a group element
        */
        public final boolean isInverse(Object a, Object b) {
                return isIdentity(compose(a,b));
        }
}

