package JSci.maths;

/**
* The IntegerSquareMatrix class provides an object for encapsulating square matrices containing integers.
* @version 1.5
* @author Mark Hale
*/
public class IntegerSquareMatrix extends IntegerMatrix {
        /**
        * Constructs a matrix.
        */
        protected IntegerSquareMatrix() {
                super();
        }
        /**
        * Constructs an empty matrix.
        * @param size the number of rows/columns
        */
        public IntegerSquareMatrix(int size) {
                super(size,size);
        }
        /**
        * Constructs a matrix by wrapping an array.
        * @param array an assigned value
        * @exception MatrixDimensionException If the array is not square.
        */
        public IntegerSquareMatrix(int array[][]) {
                super(array);
                if(array.length!=array[0].length) {
                        matrix=null;
                        throw new MatrixDimensionException("The array is not square.");
                }
        }
        /**
        * Creates a clone of this matrix.
        * @return a copy of this matrix.
        */
        public Object clone() {
                int array[][]=new int[matrix.length][matrix[0].length];
                for(int j,i=0;i<matrix.length;i++) {
                        for(j=0;j<matrix[0].length;j++)
                                array[i][j]=matrix[i][j];
                }
                return new IntegerSquareMatrix(array);
        }
        /**
        * Returns true if this matrix is unitary.
        */
        public boolean isUnitary() {
                return IntegerDiagonalMatrix.identity(matrix[0].length).equals(MatrixMath.multiply(this,MatrixMath.transpose(this)));
        }
        /**
        * Returns the determinant.
        */
        public int determinant() {
                DoubleSquareMatrix lu[]=MatrixMath.LUDecomposition(this);
                double det=lu[1].getElement(0,0);
                for(int i=1;i<matrix.length;i++)
                        det*=lu[1].getElement(i,i);
                return (int)(det);
        }
        /**
        * Returns the trace.
        * @author Taber Smith
        */
        public int trace() {
                int result=matrix[0][0];
                for (int i=1;i<matrix.length;i++)
                        result+=matrix[i][i];
                return result;
        }
}

