package JSci.maths.groups;

import JSci.maths.*;

/**
* The CyclicGroup class represents the nth order cyclic group.
* Elements are stored as strings.
* @version 1.0
* @author Mark Hale
*/
public final class CyclicGroup extends Group {
        /**
        * Constructs a cyclic group.
        * @param n order of group
        */
        public CyclicGroup(int n) {
                elements=new Object[n];
                elements[0]=new String("e");
                for(int i=1;i<n;i++) {
                        StringBuffer buf=new StringBuffer(i);
                        for(int j=0;j<i;j++)
                                buf.append('c');
                        elements[i]=buf.toString();
                }
        }
        /**
        * The group composition law.
        * @param a a group element
        * @param b a group element
        */
        public Object compose(Object a, Object b) {
                if(isIdentity(a))
                        return b;
                else if(isIdentity(b))
                        return a;
                else {
                        String c=a.toString()+b.toString();
                        if(c.length()==elements.length)
                                return identity();
                        else if(c.length()<elements.length)
                                return c;
                        else
                                return c.substring(elements.length);
                }
        }
        /**
        * Returns the identity element.
        */
        public Object identity() {
                return elements[0];
        }
        /**
        * Returns the inverse element.
        */
        public Object inverse(Object a) {
                if(isIdentity(a))
                        return identity();
                else
                        return (elements[elements.length-1].toString()).substring((a.toString()).length()-1);
        }
        /**
        * Returns true if the object is a member of this group.
        * @param a an object
        */
        public boolean isMember(Object a) {
                for(int i=0;i<elements.length;i++) {
                        if((a.toString()).equals(elements[i].toString()))
                                return true;
                }
                return false;
        }
        /**
        * Returns true if the element is the identity element of this group.
        * @param a a group element
        */
        public boolean isIdentity(Object a) {
                return (a.toString()).equals(identity().toString());
        }
}

