package JSci.maths.groups;

import JSci.maths.*;

/**
* The DihedralGroup class represents the nth order dihedral group.
* Elements are stored as strings.
* @version 1.0
* @author Mark Hale
*/
public final class DihedralGroup extends Group {
        /**
        * Constructs a dihedral group.
        * @param n order of group
        */
        public DihedralGroup(int n) {
                elements=new Object[2*n];
                elements[0]=new String("e");
                elements[1]=new String("b");
                for(int i=1;i<n;i++) {
                        StringBuffer buf=new StringBuffer(i);
                        for(int j=0;j<i;j++)
                                buf.append('c');
                        elements[i+1]=buf.toString();
                        buf.insert(0,'b');
                        elements[i+n]=buf.toString();
                }
        }
        /**
        * The group composition law.
        * @param a a group element
        * @param b a group element
        */
        public Object compose(Object a, Object b) {
                String aStr=a.toString(),bStr=b.toString();
                if(isIdentity(a))
                        return b;
                else if(isIdentity(b))
                        return a;
                else if(aStr.equals("b")) {
                        if(bStr.equals("b"))
                                return identity();
                        else if(bStr.charAt(0)=='b')
                                return bStr.substring(1);
                        else
                                return aStr+bStr;
                } else if(bStr.charAt(0)=='b') {
                        if(aStr.charAt(0)=='b') {
                                int nc=bStr.length()-aStr.length();
                                if(nc==0)
                                        return identity();
                                else if(nc<0)
                                        nc+=elements.length/2;
                                return elements[1+nc];
                        } else {
                                int nc=bStr.length()-1-aStr.length();
                                if(nc==0)
                                        return elements[1];
                                else if(nc<0)
                                        nc+=elements.length/2;
                                return elements[elements.length/2+nc];
                        }
                } else {
                        if(aStr.charAt(0)=='b') {
                                String c=aStr+bStr;
                                if(c.length()>elements.length/2)
                                        return c.substring(0,c.length()-elements.length/2);
                                else
                                        return c;
                        } else {
                                String c=aStr+bStr;
                                if(c.length()==elements.length/2)
                                        return identity();
                                if(c.length()<elements.length/2)
                                        return c;
                                else
                                        return c.substring(0,c.length()-elements.length/2);
                        }
                }
        }
        /**
        * Returns the identity element.
        */
        public Object identity() {
                return elements[0];
        }
        /**
        * Returns the inverse element.
        */
        public Object inverse(Object a) {
                if(isIdentity(a))
                        return identity();
                else if(a.toString().equals("b"))
                        return elements[1];
                else {
                        for(int i=2;i<elements.length;i++) {
                                if(isIdentity(compose(a,elements[i])))
                                        return elements[i];
                        }
                        return null;
                }
        }
        /**
        * Returns true if the object is a member of this group.
        * @param a an object
        */
        public boolean isMember(Object a) {
                for(int i=0;i<elements.length;i++) {
                        if((a.toString()).equals(elements[i].toString()))
                                return true;
                } 
                return false;
        }
        /**
        * Returns true if the element is the identity element of this group.
        * @param a a group element
        */
        public boolean isIdentity(Object a) {
                return (a.toString()).equals(identity().toString());
        }
}

