package JSci.physics;

/**
* The physical constants library.
* All values expressed in SI units.
* @version 1.0
* @author Mark Hale
*/
public final class Constants extends Object {
        private Constants() {}

        /**
        * Planck's constant.
        */
        public final static double planck=6.6260755E-34;
        /**
        * Planck's constant divided by 2*PI.
        */
        public final static double planckBar=planck/(2*Math.PI);
        /**
        * Speed of light in vacuo (exact).
        */
        public final static double lightSpeed=2.99792458E+8;
        /**
        * Permeability constant (exact).
        */
        public final static double permeability=Math.PI*4E-7;
        /**
        * Permittivity constant (defined).
        */
        public final static double permittivity=1/(permeability*lightSpeed*lightSpeed);
        /**
        * Gravitational constant.
        */
        public final static double gravitation=6.67259E-11;
        /**
        * Electronic charge.
        */
        public final static double eCharge=1.60217733E-19;
        /**
        * Electron rest mass.
        */
        public final static double eMass=9.1093897E-31;
        /**
        * Proton rest mass.
        */
        public final static double pMass=1.6726231E-27;
        /**
        * Neutron rest mass.
        */
        public final static double nMass=1.6749286E-27;
        /**
        * Avogadro's constant.
        */
        public final static double avogadro=6.0221367E+23;
        /**
        * Molar gas constant.
        */
        public final static double molarGas=8.314510;
        /**
        * Boltzmann's constant.
        */
        public final static double boltzmann=molarGas/avogadro;
        /**
        * Stefan-Boltzmann constant.
        */
        public final static double stefanBoltzmann=5.67051E-8;
        /**
        * Rydberg constant.
        */
        public final static double rydberg=lightSpeed*permeability*lightSpeed*permeability*lightSpeed*eMass*eCharge*eCharge/planck*eCharge/planck*eCharge/planck/8;
        /**
        * Fine structure constant.
        */
        public final static double fineStructure=permeability*lightSpeed*eCharge/planck*eCharge/2;
        /**
        * Faraday constant.
        */
        public final static double faraday=96485.309;
        /**
        * Magnetic flux quantum.
        */
        public final static double fluxQuantum=planck/(2*eCharge);
        /**
        * Bohr magneton.
        */
        public final static double bohrMagneton=eCharge/eMass*planckBar/2;
        /**
        * Magnetic moment of electron.
        */
        public final static double eMagnetic=9.2847701E-24;
        /**
        * Magnetic moment of proton.
        */
        public final static double pMagnetic=1.41060761E-26;
}

