package JSci.physics.particles;

import JSci.physics.quantum.QuantumParticle;

/**
* A class representing positrons.
* @version 1.5
* @author Mark Hale
*/
public final class Positron extends AntiLepton {
        /**
        * Constructs a positron.
        */
        public Positron() {}
        /**
        * Returns the rest mass (MeV).
        * @return 0.51099907
        */
        public double restMass() {return 0.51099907;}
        /**
        * Returns the electric charge.
        * @return 1
        */
        public int charge() {return 1;}
        /**
        * Returns the electron lepton number.
        * @return -1
        */
        public int eLeptonQN() {return -1;}
        /**
        * Returns the muon lepton number.
        * @return 0
        */
        public int muLeptonQN() {return 0;}
        /**
        * Returns the tau lepton number.
        * @return 0
        */
        public int tauLeptonQN() {return 0;}
        /**
        * Returns the antiparticle of this particle.
        */
        public QuantumParticle anti() {
                return new Electron();
        }
        /**
        * Returns true if qp is the antiparticle.
        */
        public boolean isAnti(QuantumParticle qp) {
                return (qp!=null) && (qp instanceof Electron);
        }
        /**
        * Returns a string representing this class.
        */
        public String toString() {
                return new String("Positron");
        }
}

