package JSci.physics.quantum;

import JSci.maths.*;

/**
* The BraVector class provides an object for encapsulating Dirac bra vectors.
* @version 1.5
* @author Mark Hale
*/
public final class BraVector extends Vector {
        private ComplexVector representation;

        /**
        * Constructs a bra vector given a vector representation.
        * @param rep a vector representation
        */
        public BraVector(ComplexVector rep) {
                representation=rep;
        }
        /**
        * Compares two bra vectors for equality.
        * @param a a bra vector
        */
        public boolean equals(Object a) {
                return representation.equals(((BraVector)a).getRepresentation());
        }
        /**
        * Creates a clone of this bra vector.
        * @return a copy of this bra vector.
        */
        public Object clone() {
                return new BraVector((ComplexVector)representation.clone());
        }
        /**
        * Returns a comma delimited string representing the value of this bra vector.
        */
        public String toString() {
                return representation.toString();
        }
        /**
        * Returns a hashcode for this bra vector.
        */
        public int hashCode() {
                return representation.hashCode();
        }
        /**
        * Map this bra vector to a ket vector.
        */
        public KetVector toKetVector() {
                return new KetVector(VectorMath.conjugate(representation));
        }
        /**
        * Returns the representation.
        */
        public ComplexVector getRepresentation() {
                return representation;
        }
        /**
        * Returns the dimension.
        */
        public int dimension() {
                return representation.dimension();
        }
        /**
        * Returns the norm.
        */
        public double norm() {
                return representation.norm();
        }
}

