package JSci.physics.quantum;

import JSci.maths.*;

/**
* The DensityMatrix class provides an object for encapsulating density matrices.
* @version 1.5
* @author Mark Hale
*/
public final class DensityMatrix extends Operator {
        private static ComplexSquareMatrix constructor(KetVector kets[],double probs[]) {
                ComplexSquareMatrix rep=MatrixMath.scalarMultiply(new Complex(probs[0],0.0),(new Projector(kets[0])).getRepresentation());
                for(int i=1;i<kets.length;i++)
                        rep=MatrixMath.add(rep,MatrixMath.scalarMultiply(new Complex(probs[i],0.0),(new Projector(kets[i])).getRepresentation()));
                return rep;
        }
        private static ComplexSquareMatrix constructor(Projector projs[],double probs[]) {
                ComplexSquareMatrix rep=MatrixMath.scalarMultiply(new Complex(probs[0],0.0),projs[0].getRepresentation());
                for(int i=1;i<projs.length;i++)
                        rep=MatrixMath.add(rep,MatrixMath.scalarMultiply(new Complex(probs[i],0.0),projs[i].getRepresentation()));
                return rep;
        }

        /**
        * Constructs a density matrix.
        * @param kets an array of ket vectors
        * @param probs the probabilities of being in the ket vector states.
        */
        public DensityMatrix(KetVector kets[],double probs[]) {
                super(constructor(kets,probs));
        }
        /**
        * Constructs a density matrix.
        * @param projs an array of projectors
        * @param probs the probabilities of being in the projector states.
        */
        public DensityMatrix(Projector projs[],double probs[]) {
                super(constructor(projs,probs));
        }
        /**
        * Returns true if this density matrix is a pure state.
        */
        public boolean isPureState() {
                ComplexSquareMatrix rep=getRepresentation();
                return rep.equals(MatrixMath.multiply(rep,rep));
        }
}

