package JSci.physics.quantum;

import JSci.maths.*;

/**
* The KetVector class provides an object for encapsulating Dirac ket vectors.
* @version 1.5
* @author Mark Hale
*/
public final class KetVector extends Vector {
        private ComplexVector representation;

        /**
        * Constructs a ket vector given a vector representation.
        * @param rep a vector representation
        */
        public KetVector(ComplexVector rep) {
                representation=rep;
        }
        /**
        * Compares two ket vectors for equality.
        * @param a a ket vector
        */
        public boolean equals(Object a) {
                return representation.equals(((KetVector)a).getRepresentation());
        }
        /**
        * Creates a clone of this ket vector.
        * @return a copy of this ket vector.
        */
        public Object clone() {
                return new KetVector((ComplexVector)representation.clone());
        }
        /**
        * Returns a comma delimited string representing the value of this ket vector.
        */
        public String toString() {
                return representation.toString();
        }
        /**
        * Returns a hashcode for this ket vector.
        */
        public int hashCode() {
                return representation.hashCode();
        }
        /**
        * Map this ket vector to a bra vector.
        */
        public BraVector toBraVector() {
                return new BraVector(VectorMath.conjugate(representation));
        }
        /**
        * Returns the representation.
        */
        public ComplexVector getRepresentation() {
                return representation;
        }
        /**
        * Returns the dimension.
        */
        public int dimension() {
                return representation.dimension();
        }
        /**
        * Returns the norm.
        */
        public double norm() {
                return representation.norm();
        }
}

