package JSci.physics.quantum;

import JSci.maths.*;

/**
* The SpinOperator class provides an object for encapsulating spin operators.
* @version 1.5
* @author Mark Hale
*/
public abstract class SpinOperator extends Operator {
        /**
        * Constructs a spin operator.
        */
        protected SpinOperator(Complex spinMatrix[][]) {
                super(new ComplexSquareMatrix(spinMatrix));
        }
        /**
        * Returns true if this operator is self-adjoint.
        */
        public boolean isSelfAdjoint() {
                return true;
        }
        /**
        * Returns true if this operator is unitary.
        */
        public boolean isUnitary() {
                return true;
        }
        /**
        * Returns the trace.
        */
        public Complex trace() {
                return Complex.ZERO;
        }
}

