package JSci.physics.relativity;

import JSci.physics.quantum.QuantumParticle;
import JSci.physics.*;

/**
* The relativity library.
* This class cannot be subclassed or instantiated because all methods are static.
* @version 1.0
* @author Mark Hale
*/
public final class RelativityMath extends Object {
        private RelativityMath() {}

        /**
        * Performs the Lorentz Transformation on a particle.
        * @param p particle to be transformed
        * @param relframevel the relative frame velocity
        */
        public static void lorentzTransform(Particle p,double relframevel) {
                double cc=Constants.lightSpeed*Constants.lightSpeed;
                double gamma=1/Math.sqrt(1-relframevel*relframevel/cc);
                double e=p.momentum.getComponent(0);
                double px=p.momentum.getComponent(1);
                double t=p.position.getComponent(0);
                double rx=p.position.getComponent(1);
                p.momentum=new ContraVector((e-relframevel*px)*gamma,
                        (px-relframevel*e/cc)*gamma,0.0,0.0);
                p.position=new ContraVector((t-relframevel*rx/cc)*gamma,
                        (rx-relframevel*t)*gamma,0.0,0.0);
        }
        /**
        * Performs the Lorentz Transformation on a quantum particle.
        * @param p quantum particle to be transformed
        * @param relframevel the relative frame velocity
        */
        public static void lorentzTransform(QuantumParticle qp,double relframevel) {
                double cc=Constants.lightSpeed*Constants.lightSpeed;
                double gamma=1/Math.sqrt(1-relframevel*relframevel);
                double e=qp.momentum.getComponent(0);
                double px=qp.momentum.getComponent(1);
                double t=qp.position.getComponent(0);
                double rx=qp.position.getComponent(1);
                qp.momentum=new ContraVector((e-relframevel*px)*gamma,
                        (px-relframevel*e)*gamma,0.0,0.0);
                qp.position=new ContraVector((t-relframevel*rx/cc)*gamma,
                        (rx-relframevel*t)*gamma,0.0,0.0);
        }


        /**
        * Returns the multiplication of a covariant vector by a contravariant vector.
        * @param v a contravariant vector
        * @param cv a covariant vector
        */
        public static double multiply(ContraVector v,CoVector cv) {
                return v.getComponent(0)*cv.getComponent(0)
                        +v.getComponent(1)*cv.getComponent(1)
                        +v.getComponent(2)*cv.getComponent(2)
                        +v.getComponent(3)*cv.getComponent(3);
        }
}

