package JSci.maths;

/**
* The matrix math library.
* This class cannot be subclassed or instantiated because all methods are static.
* @version 1.5
* @author Mark Hale
*/
public final class MatrixMath extends Object {
        private MatrixMath() {}

// HERMITIAN ADJOINT

        /**
        * Returns the hermitian adjoint of a complex matrix.
        * @param m a complex matrix
        */
        public static ComplexMatrix hermitianAdjoint(ComplexMatrix m) {
                int mRow=m.rows(),mCol=m.columns();
                Complex array[][]=new Complex[mRow][mCol];
                for(int j,i=0;i<mRow;i++) {
                        for(j=0;j<mCol;j++)
                                array[j][i]=ComplexMath.conjugate(m.getElement(i,j));
                }
                return new ComplexMatrix(array);
        }
        /**
        * Returns the hermitian adjoint of a complex square matrix.
        * @param m a complex square matrix
        */
        public static ComplexSquareMatrix hermitianAdjoint(ComplexSquareMatrix m) {
                int mRow=m.rows();
                Complex array[][]=new Complex[mRow][mRow];
                for(int j,i=0;i<mRow;i++) {
                        for(j=0;j<mRow;j++)
                                array[j][i]=ComplexMath.conjugate(m.getElement(i,j));
                }
                return new ComplexSquareMatrix(array);
        }
        /**
        * Returns the hermitian adjoint of a complex tridiagonal matrix.
        * @param m a complex tridiagonal matrix
        */
        public static ComplexTridiagonalMatrix hermitianAdjoint(ComplexTridiagonalMatrix m) {
                int mRow=m.rows();
                ComplexTridiagonalMatrix ans=new ComplexTridiagonalMatrix(mRow);
                ans.setElement(0,0,ComplexMath.conjugate(m.getElement(0,0)));
                ans.setElement(0,1,ComplexMath.conjugate(m.getElement(1,0)));
                mRow--;
                for(int i=1;i<mRow;i++) {
                        ans.setElement(i,i-1,ComplexMath.conjugate(m.getElement(i-1,i)));
                        ans.setElement(i,i,ComplexMath.conjugate(m.getElement(i,i)));
                        ans.setElement(i,i+1,ComplexMath.conjugate(m.getElement(i+1,i)));
                }
                ans.setElement(mRow,mRow-1,ComplexMath.conjugate(m.getElement(mRow-1,mRow)));
                ans.setElement(mRow,mRow,ComplexMath.conjugate(m.getElement(mRow,mRow)));
                return ans;
        }

// COMPLEX CONJUGATE

        /**
        * Returns the complex conjugate of a complex matrix.
        * @param m a complex matrix
        */
        public static ComplexMatrix conjugate(ComplexMatrix m) {
                int mRow=m.rows(),mCol=m.columns();
                Complex array[][]=new Complex[mRow][mCol];
                for(int j,i=0;i<mRow;i++) {
                        for(j=0;j<mCol;j++)
                                array[i][j]=ComplexMath.conjugate(m.getElement(i,j));
                }
                return new ComplexMatrix(array);
        }
        /**
        * Returns the complex conjugate of a complex square matrix.
        * @param m a complex square matrix
        */
        public static ComplexSquareMatrix conjugate(ComplexSquareMatrix m) {
                int mRow=m.rows();
                Complex array[][]=new Complex[mRow][mRow];
                for(int j,i=0;i<mRow;i++) {
                        for(j=0;j<mRow;j++)
                                array[i][j]=ComplexMath.conjugate(m.getElement(i,j));
                }
                return new ComplexSquareMatrix(array);
        }
        /**
        * Returns the complex conjugate of a complex tridiagonal matrix.
        * @param m a complex tridiagonal matrix
        */
        public static ComplexTridiagonalMatrix conjugate(ComplexTridiagonalMatrix m) {
                int mRow=m.rows();
                ComplexTridiagonalMatrix ans=new ComplexTridiagonalMatrix(mRow);
                ans.setElement(0,0,ComplexMath.conjugate(m.getElement(0,0)));
                ans.setElement(1,0,ComplexMath.conjugate(m.getElement(1,0)));
                mRow--;
                for(int i=1;i<mRow;i++) {
                        ans.setElement(i-1,i,ComplexMath.conjugate(m.getElement(i-1,i)));
                        ans.setElement(i,i,ComplexMath.conjugate(m.getElement(i,i)));
                        ans.setElement(i+1,i,ComplexMath.conjugate(m.getElement(i+1,i)));
                }
                ans.setElement(mRow-1,mRow,ComplexMath.conjugate(m.getElement(mRow-1,mRow)));
                ans.setElement(mRow,mRow,ComplexMath.conjugate(m.getElement(mRow,mRow)));
                return ans;
        }
        /**
        * Returns the complex conjugate of a complex diagonal matrix.
        * @param m a complex diagonal matrix
        */
        public static ComplexDiagonalMatrix conjugate(ComplexDiagonalMatrix m) {
                int mRow=m.rows();
                Complex array[]=new Complex[mRow];
                for(int i=0;i<mRow;i++)
                        array[i]=ComplexMath.conjugate(m.getElement(i,i));
                return new ComplexDiagonalMatrix(array);
        }

// ADDITION
 
        /**
        * Returns the addition of two matrices.
        * @param a an integer matrix
        * @param b an integer matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static IntegerMatrix add(IntegerMatrix a, IntegerMatrix b) throws MatrixDimensionException {
                int aRow=a.rows(),aCol=a.columns();
                if(aRow==b.rows() && aCol==b.columns()) {
                        int array[][]=new int[aRow][aCol];
// Create threads
                        for(int j,i=0;i<aRow;i++) {
                                for(j=0;j<aCol;j++)
                                        (new IntegerMatrixAddThread(a,b,i,j,array)).start();
                        }
                        return new IntegerMatrix(array);
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }
        /**
        * Returns the addition of two square matrices.
        * @param a an integer square matrix
        * @param b an integer square matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static IntegerSquareMatrix add(IntegerSquareMatrix a, IntegerSquareMatrix b) throws MatrixDimensionException {
                int aRow=a.rows();
                if(aRow==b.rows()) {
                        int array[][]=new int[aRow][aRow];
// Create threads
                        for(int j,i=0;i<aRow;i++) {
                                for(j=0;j<aRow;j++)
                                        (new IntegerMatrixAddThread(a,b,i,j,array)).start();
                        }
                        return new IntegerSquareMatrix(array);
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }
        /**
        * Returns the addition of two tridiagonal matrices.
        * @param a an integer tridiagonal matrix
        * @param b an integer tridiagonal matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static IntegerTridiagonalMatrix add(IntegerTridiagonalMatrix a,IntegerTridiagonalMatrix b) throws MatrixDimensionException {
                int aRow=a.rows();
                if(aRow==b.rows()) {
                        IntegerTridiagonalMatrix ans=new IntegerTridiagonalMatrix(aRow);
                        ans.setElement(0,0,a.getElement(0,0)+b.getElement(0,0));
                        ans.setElement(1,0,a.getElement(1,0)+b.getElement(1,0));
                        aRow--;
                        for(int i=1;i<aRow;i++) {
                                ans.setElement(i-1,i,a.getElement(i-1,i)+b.getElement(i-1,i));
                                ans.setElement(i,i,a.getElement(i-1,i)+b.getElement(i,i));
                                ans.setElement(i+1,i,a.getElement(i+1,i)+b.getElement(i+1,i));
                        }
                        ans.setElement(aRow-1,aRow,a.getElement(aRow-1,aRow)+b.getElement(aRow-1,aRow));
                        ans.setElement(aRow,aRow,a.getElement(aRow,aRow)+b.getElement(aRow,aRow));
                        return ans;
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }
        /**
        * Returns the addition of two diagonal matrices.
        * @param a an integer diagonal matrix
        * @param b an integer diagonal matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static IntegerDiagonalMatrix add(IntegerDiagonalMatrix a, IntegerDiagonalMatrix b) throws MatrixDimensionException {
                int aRow=a.rows();
                if(aRow==b.rows()) {
                        int array[]=new int[aRow];
                        for(int i=0;i<aRow;i++)
                                array[i]=a.getElement(i,i)+b.getElement(i,i);
                        return new IntegerDiagonalMatrix(array);
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }

        /**
        * Returns the addition of two matrices.
        * @param a a double matrix
        * @param b a double matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static DoubleMatrix add(DoubleMatrix a, DoubleMatrix b) throws MatrixDimensionException {
                int aRow=a.rows(),aCol=a.columns();
                if(aRow==b.rows() && aCol==b.columns()) {
                        double array[][]=new double[aRow][aCol];
// Create threads
                        for(int j,i=0;i<aRow;i++) {
                                for(j=0;j<aCol;j++)
                                        (new DoubleMatrixAddThread(a,b,i,j,array)).start();
                        }
                        return new DoubleMatrix(array);
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }
        /**
        * Returns the addition of two square matrices.
        * @param a a double square matrix
        * @param b a double square matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static DoubleSquareMatrix add(DoubleSquareMatrix a, DoubleSquareMatrix b) throws MatrixDimensionException {
                int aRow=a.rows();
                if(aRow==b.rows()) {
                        double array[][]=new double[aRow][aRow];
// Create threads
                        for(int j,i=0;i<aRow;i++) {
                                for(j=0;j<aRow;j++)
                                        (new DoubleMatrixAddThread(a,b,i,j,array)).start();
                        }
                        return new DoubleSquareMatrix(array);
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }
        /**
        * Returns the addition of two tridiagonal matrices.
        * @param a a double tridiagonal matrix
        * @param b a double tridiagonal matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static DoubleTridiagonalMatrix add(DoubleTridiagonalMatrix a,DoubleTridiagonalMatrix b) throws MatrixDimensionException {
                int aRow=a.rows();
                if(aRow==b.rows()) {
                        DoubleTridiagonalMatrix ans=new DoubleTridiagonalMatrix(aRow);
                        ans.setElement(0,0,a.getElement(0,0)+b.getElement(0,0));
                        ans.setElement(1,0,a.getElement(1,0)+b.getElement(1,0));
                        aRow--;
                        for(int i=1;i<aRow;i++) {
                                ans.setElement(i-1,i,a.getElement(i-1,i)+b.getElement(i-1,i));
                                ans.setElement(i,i,a.getElement(i-1,i)+b.getElement(i,i));
                                ans.setElement(i+1,i,a.getElement(i+1,i)+b.getElement(i+1,i));
                        }
                        ans.setElement(aRow-1,aRow,a.getElement(aRow-1,aRow)+b.getElement(aRow-1,aRow));
                        ans.setElement(aRow,aRow,a.getElement(aRow,aRow)+b.getElement(aRow,aRow));
                        return ans;
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }
        /**
        * Returns the addition of two diagonal matrices.
        * @param a a double diagonal matrix
        * @param b a double diagonal matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static DoubleDiagonalMatrix add(DoubleDiagonalMatrix a, DoubleDiagonalMatrix b) throws MatrixDimensionException {
                int aRow=a.rows();
                if(aRow==b.rows()) {
                        double array[]=new double[aRow];
                        for(int i=0;i<aRow;i++)
                                array[i]=a.getElement(i,i)+b.getElement(i,i);
                        return new DoubleDiagonalMatrix(array);
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }

        /**
        * Returns the addition of two matrices.
        * @param a a complex matrix
        * @param b a complex matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static ComplexMatrix add(ComplexMatrix a, ComplexMatrix b) throws MatrixDimensionException {
                int aRow=a.rows(),aCol=a.columns();
                if(aRow==b.rows() && aCol==b.columns()) {
                        Complex array[][]=new Complex[aRow][aCol];
// Create threads
                        for(int j,i=0;i<aRow;i++) {
                                for(j=0;j<aCol;j++)
                                        (new ComplexMatrixAddThread(a,b,i,j,array)).start();
                        }
                        return new ComplexMatrix(array);
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }
        /**
        * Returns the addition of two square matrices.
        * @param a a complex square matrix
        * @param b a complex square matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static ComplexSquareMatrix add(ComplexSquareMatrix a, ComplexSquareMatrix b) throws MatrixDimensionException {
                int aRow=a.rows();
                if(aRow==b.rows()) {
                        Complex array[][]=new Complex[aRow][aRow];
// Create threads
                        for(int j,i=0;i<aRow;i++) {
                                for(j=0;j<aRow;j++)
                                        (new ComplexMatrixAddThread(a,b,i,j,array)).start();
                        }
                        return new ComplexSquareMatrix(array);
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }
        /**
        * Returns the addition of two tridiagonal matrices.
        * @param a a complex tridiagonal matrix
        * @param b a complex tridiagonal matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static ComplexTridiagonalMatrix add(ComplexTridiagonalMatrix a,ComplexTridiagonalMatrix b) throws MatrixDimensionException {
                int aRow=a.rows();
                if(aRow==b.rows()) {
                        ComplexTridiagonalMatrix ans=new ComplexTridiagonalMatrix(aRow);
                        ans.setElement(0,0,ComplexMath.add(a.getElement(0,0),b.getElement(0,0)));
                        ans.setElement(1,0,ComplexMath.add(a.getElement(1,0),b.getElement(1,0)));
                        aRow--;
                        for(int i=1;i<aRow;i++) {
                                ans.setElement(i-1,i,ComplexMath.add(a.getElement(i-1,i),b.getElement(i-1,i)));
                                ans.setElement(i,i,ComplexMath.add(a.getElement(i-1,i),b.getElement(i,i)));
                                ans.setElement(i+1,i,ComplexMath.add(a.getElement(i+1,i),b.getElement(i+1,i)));
                        }
                        ans.setElement(aRow-1,aRow,ComplexMath.add(a.getElement(aRow-1,aRow),b.getElement(aRow-1,aRow)));
                        ans.setElement(aRow,aRow,ComplexMath.add(a.getElement(aRow,aRow),b.getElement(aRow,aRow)));
                        return ans;
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }
        /**
        * Returns the addition of two diagonal matrices.
        * @param a a complex diagonal matrix
        * @param b a complex diagonal matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static ComplexDiagonalMatrix add(ComplexDiagonalMatrix a, ComplexDiagonalMatrix b) throws MatrixDimensionException {
                int aRow=a.rows();
                if(aRow==b.rows()) {
                        Complex array[]=new Complex[aRow];
                        for(int i=0;i<aRow;i++)
                                array[i]=ComplexMath.add(a.getElement(i,i),b.getElement(i,i));
                        return new ComplexDiagonalMatrix(array);
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }

// SUBTRACTION

        /**
        * Returns the subtraction of two matrices.
        * @param a an integer matrix
        * @param b an integer matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static IntegerMatrix subtract(IntegerMatrix a, IntegerMatrix b) throws MatrixDimensionException {
                int aRow=a.rows(),aCol=a.columns();
                if(aRow==b.rows() && aCol==b.columns()) {
                        int array[][]=new int[aRow][aCol];
// Create threads
                        for(int j,i=0;i<aRow;i++) {
                                for(j=0;j<aCol;j++)
                                        (new IntegerMatrixSubtractThread(a,b,i,j,array)).start();
                        }
                        return new IntegerMatrix(array);
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }
        /**
        * Returns the subtraction of two square matrices.
        * @param a an integer square matrix
        * @param b an integer square matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static IntegerSquareMatrix subtract(IntegerSquareMatrix a, IntegerSquareMatrix b) throws MatrixDimensionException {
                int aRow=a.rows();
                if(aRow==b.rows()) {
                        int array[][]=new int[aRow][aRow];
// Create threads
                        for(int j,i=0;i<aRow;i++) {
                                for(j=0;j<aRow;j++)
                                        (new IntegerMatrixSubtractThread(a,b,i,j,array)).start();
                        }
                        return new IntegerSquareMatrix(array);
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }
        /**
        * Returns the subtraction of two tridiagonal matrices.
        * @param a an integer tridiagonal matrix
        * @param b an integer tridiagonal matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static IntegerTridiagonalMatrix subtract(IntegerTridiagonalMatrix a,IntegerTridiagonalMatrix b) throws MatrixDimensionException {
                int aRow=a.rows();
                if(aRow==b.rows()) {
                        IntegerTridiagonalMatrix ans=new IntegerTridiagonalMatrix(aRow);
                        ans.setElement(0,0,a.getElement(0,0)-b.getElement(0,0));
                        ans.setElement(1,0,a.getElement(1,0)-b.getElement(1,0));
                        aRow--;
                        for(int i=1;i<aRow;i++) {
                                ans.setElement(i-1,i,a.getElement(i-1,i)-b.getElement(i-1,i));
                                ans.setElement(i,i,a.getElement(i-1,i)-b.getElement(i,i));
                                ans.setElement(i+1,i,a.getElement(i+1,i)-b.getElement(i+1,i));
                        }
                        ans.setElement(aRow-1,aRow,a.getElement(aRow-1,aRow)-b.getElement(aRow-1,aRow));
                        ans.setElement(aRow,aRow,a.getElement(aRow,aRow)-b.getElement(aRow,aRow));
                        return ans;
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }
        /**
        * Returns the subtraction of two diagonal matrices.
        * @param a an integer diagonal matrix
        * @param b an integer diagonal matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static IntegerDiagonalMatrix subtract(IntegerDiagonalMatrix a, IntegerDiagonalMatrix b) throws MatrixDimensionException {
                int aRow=a.rows();
                if(aRow==b.rows()) {
                        int array[]=new int[aRow];
                        for(int i=0;i<aRow;i++)
                                array[i]=a.getElement(i,i)-b.getElement(i,i);
                        return new IntegerDiagonalMatrix(array);
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }

        /**
        * Returns the subtraction of two matrices.
        * @param a a double matrix
        * @param b a double matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static DoubleMatrix subtract(DoubleMatrix a, DoubleMatrix b) throws MatrixDimensionException {
                int aRow=a.rows(),aCol=a.columns();
                if(aRow==b.rows() && aCol==b.columns()) {
                        double array[][]=new double[aRow][aCol];
// Create threads
                        for(int j,i=0;i<aRow;i++) {
                                for(j=0;j<aCol;j++)
                                        (new DoubleMatrixSubtractThread(a,b,i,j,array)).start();
                        }
                        return new DoubleMatrix(array);
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }
        /**
        * Returns the subtraction of two square matrices.
        * @param a a double square matrix
        * @param b a double square matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static DoubleSquareMatrix subtract(DoubleSquareMatrix a, DoubleSquareMatrix b) throws MatrixDimensionException {
                int aRow=a.rows();
                if(aRow==b.rows()) {
                        double array[][]=new double[aRow][aRow];
// Create threads
                        for(int j,i=0;i<aRow;i++) {
                                for(j=0;j<aRow;j++)
                                        (new DoubleMatrixSubtractThread(a,b,i,j,array)).start();
                        }
                        return new DoubleSquareMatrix(array);
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }
        /**
        * Returns the subtraction of two tridiagonal matrices.
        * @param a a double tridiagonal matrix
        * @param b a double tridiagonal matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static DoubleTridiagonalMatrix subtract(DoubleTridiagonalMatrix a,DoubleTridiagonalMatrix b) throws MatrixDimensionException {
                int aRow=a.rows();
                if(aRow==b.rows()) {
                        DoubleTridiagonalMatrix ans=new DoubleTridiagonalMatrix(aRow);
                        ans.setElement(0,0,a.getElement(0,0)-b.getElement(0,0));
                        ans.setElement(1,0,a.getElement(1,0)-b.getElement(1,0));
                        aRow--;
                        for(int i=1;i<aRow;i++) {
                                ans.setElement(i-1,i,a.getElement(i-1,i)-b.getElement(i-1,i));
                                ans.setElement(i,i,a.getElement(i-1,i)-b.getElement(i,i));
                                ans.setElement(i+1,i,a.getElement(i+1,i)-b.getElement(i+1,i));
                        }
                        ans.setElement(aRow-1,aRow,a.getElement(aRow-1,aRow)-b.getElement(aRow-1,aRow));
                        ans.setElement(aRow,aRow,a.getElement(aRow,aRow)-b.getElement(aRow,aRow));
                        return ans;
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }
        /**
        * Returns the subtraction of two diagonal matrices.
        * @param a a double diagonal matrix
        * @param b a double diagonal matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static DoubleDiagonalMatrix subtract(DoubleDiagonalMatrix a, DoubleDiagonalMatrix b) throws MatrixDimensionException {
                int aRow=a.rows();
                if(aRow==b.rows()) {
                        double array[]=new double[aRow];
                        for(int i=0;i<aRow;i++)
                                array[i]=a.getElement(i,i)-b.getElement(i,i);
                        return new DoubleDiagonalMatrix(array);
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }

        /**
        * Returns the subtraction of two matrices.
        * @param a a complex matrix
        * @param b a complex matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static ComplexMatrix subtract(ComplexMatrix a, ComplexMatrix b) throws MatrixDimensionException {
                int aRow=a.rows(),aCol=a.columns();
                if(aRow==b.rows() && aCol==b.columns()) {
                        Complex array[][]=new Complex[aRow][aCol];
// Create threads
                        for(int j,i=0;i<aRow;i++) {
                                for(j=0;j<aCol;j++)
                                        (new ComplexMatrixSubtractThread(a,b,i,j,array)).start();
                        }
                        return new ComplexMatrix(array);
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }
        /**
        * Returns the subtraction of two square matrices.
        * @param a a complex square matrix
        * @param b a complex square matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static ComplexSquareMatrix subtract(ComplexSquareMatrix a, ComplexSquareMatrix b) throws MatrixDimensionException {
                int aRow=a.rows();
                if(aRow==b.rows()) {
                        Complex array[][]=new Complex[aRow][aRow];
// Create threads
                        for(int j,i=0;i<aRow;i++) {
                                for(j=0;j<aRow;j++)
                                        (new ComplexMatrixSubtractThread(a,b,i,j,array)).start();
                        }
                        return new ComplexSquareMatrix(array);
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }
        /**
        * Returns the subtraction of two tridiagonal matrices.
        * @param a a complex tridiagonal matrix
        * @param b a complex tridiagonal matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static ComplexTridiagonalMatrix subtract(ComplexTridiagonalMatrix a,ComplexTridiagonalMatrix b) throws MatrixDimensionException {
                int aRow=a.rows();
                if(aRow==b.rows()) {
                        ComplexTridiagonalMatrix ans=new ComplexTridiagonalMatrix(aRow);
                        ans.setElement(0,0,ComplexMath.subtract(a.getElement(0,0),b.getElement(0,0)));
                        ans.setElement(1,0,ComplexMath.subtract(a.getElement(1,0),b.getElement(1,0)));
                        aRow--;
                        for(int i=1;i<aRow;i++) {
                                ans.setElement(i-1,i,ComplexMath.subtract(a.getElement(i-1,i),b.getElement(i-1,i)));
                                ans.setElement(i,i,ComplexMath.subtract(a.getElement(i-1,i),b.getElement(i,i)));
                                ans.setElement(i+1,i,ComplexMath.subtract(a.getElement(i+1,i),b.getElement(i+1,i)));
                        }
                        ans.setElement(aRow-1,aRow,ComplexMath.subtract(a.getElement(aRow-1,aRow),b.getElement(aRow-1,aRow)));
                        ans.setElement(aRow,aRow,ComplexMath.subtract(a.getElement(aRow,aRow),b.getElement(aRow,aRow)));
                        return ans;
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }
        /**
        * Returns the subtraction of two diagonal matrices.
        * @param a a complex diagonal matrix
        * @param b a complex diagonal matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static ComplexDiagonalMatrix subtract(ComplexDiagonalMatrix a, ComplexDiagonalMatrix b) throws MatrixDimensionException {
                int aRow=a.rows();
                if(aRow==b.rows()) {
                        Complex array[]=new Complex[aRow];
                        for(int i=0;i<aRow;i++)
                                array[i]=ComplexMath.subtract(a.getElement(i,i),b.getElement(i,i));
                        return new ComplexDiagonalMatrix(array);
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }

// MATRIX OPERATION

        /**
        * Returns the multiplication of a vector by a matrix.
        * @param m an integer matrix
        * @param a an integer vector
        * @exception DimensionException If the matrix and vector are incompatible.
        */
        public static IntegerVector multiply(IntegerMatrix m, IntegerVector a) throws DimensionException {
                int mRow=m.rows(),mCol=m.columns();
                if(mCol==a.dimension()) {
                        int array[]=new int[mRow];
// Create threads
                        for(int i=0;i<mRow;i++)
                                (new IntegerMatrixOpThread(m,a,i,array)).start();
                        return new IntegerVector(array);
                } else
                        throw new DimensionException("Matrix and vector are incompatible.");
        }
        /**
        * Returns the multiplication of a vector by a matrix.
        * @param m an integer diagonal matrix
        * @param a an integer vector
        * @exception DimensionException If the matrix and vector are incompatible.
        */
        public static IntegerVector multiply(IntegerDiagonalMatrix m, IntegerVector a) throws DimensionException {
                int mRow=m.rows();
                if(mRow==a.dimension()) {
                        int array[]=new int[mRow];
                        for(int i=0;i<mRow;i++)
                                array[i]=m.getElement(i,i)*a.getComponent(i);
                        return new IntegerVector(array);
                } else
                        throw new DimensionException("Matrix and vector are incompatible.");
        }

        /**
        * Returns the multiplication of a vector by a matrix.
        * @param m a double matrix
        * @param a a double vector
        * @exception DimensionException If the matrix and vector are incompatible.
        */
        public static DoubleVector multiply(DoubleMatrix m, DoubleVector a) throws DimensionException {
                int mRow=m.rows(),mCol=m.columns();
                if(mCol==a.dimension()) {
                        double array[]=new double[mRow];
// Create threads
                        for(int i=0;i<mRow;i++)
                                (new DoubleMatrixOpThread(m,a,i,array)).start();
                        return new DoubleVector(array);
                } else
                        throw new DimensionException("Matrix and vector are incompatible.");
        }
        /**
        * Returns the multiplication of a vector by a matrix.
        * @param m a double diagonal matrix
        * @param a a double vector
        * @exception DimensionException If the matrix and vector are incompatible.
        */
        public static DoubleVector multiply(DoubleDiagonalMatrix m, DoubleVector a) throws DimensionException {
                int mRow=m.rows();
                if(mRow==a.dimension()) {
                        double array[]=new double[mRow];
                        for(int i=0;i<mRow;i++)
                                array[i]=m.getElement(i,i)*a.getComponent(i);
                        return new DoubleVector(array);
                } else
                        throw new DimensionException("Matrix and vector are incompatible.");
        }

        /**
        * Returns the multiplication of a vector by a matrix.
        * @param m a complex matrix
        * @param a a complex vector
        * @exception DimensionException If the matrix and vector are incompatible.
        */
        public static ComplexVector multiply(ComplexMatrix m, ComplexVector a) throws DimensionException {
                int mRow=m.rows(),mCol=m.columns();
                if(mCol==a.dimension()) {
                        Complex array[]=new Complex[mRow];
// Create threads
                        for(int j,i=0;i<mRow;i++)
                                (new ComplexMatrixOpThread(m,a,i,array)).start();
                        return new ComplexVector(array);
                } else
                        throw new DimensionException("Matrix and vector are incompatible.");
        }
        /**
        * Returns the multiplication of a vector by a matrix.
        * @param m a complex diagonal matrix
        * @param a a complex vector
        * @exception DimensionException If the matrix and vector are incompatible.
        */
        public static ComplexVector multiply(ComplexDiagonalMatrix m, ComplexVector a) throws DimensionException {
                int mRow=m.rows();
                if(mRow==a.dimension()) {
                        Complex array[]=new Complex[mRow];
                        for(int i=0;i<mRow;i++)
                                array[i]=ComplexMath.multiply(m.getElement(i,i),a.getComponent(i));
                        return new ComplexVector(array);
                } else
                        throw new DimensionException("Matrix and vector are incompatible.");
        }

// SCALAR MULTIPLICATION

        /**
        * Returns the multiplication of a matrix by a scalar.
        * @param x an integer
        * @param m an integer matrix
        */
        public static IntegerMatrix scalarMultiply(int x, IntegerMatrix m) {
                int row=m.rows(),col=m.columns();
                int array[][]=new int[row][col];
                for(int j,i=0;i<row;i++) {
                        for(j=0;j<col;j++)
                                array[i][j]=x*m.getElement(i,j);
                }
                return new IntegerMatrix(array);
        }
        /**
        * Returns the multiplication of a matrix by a scalar.
        * @param x an integer
        * @param m an integer square matrix
        */
        public static IntegerSquareMatrix scalarMultiply(int x, IntegerSquareMatrix m) {
                int row=m.rows();
                int array[][]=new int[row][row];
                for(int j,i=0;i<row;i++) {
                        for(j=0;j<row;j++)
                                array[i][j]=x*m.getElement(i,j);
                }
                return new IntegerSquareMatrix(array);
        }
        /**
        * Returns the multiplication of a matrix by a scalar.
        * @param x an integer
        * @param m an integer tridiagonal matrix
        */
        public static IntegerTridiagonalMatrix scalarMultiply(int x, IntegerTridiagonalMatrix m) {
                int row=m.rows();
                IntegerTridiagonalMatrix ans=new IntegerTridiagonalMatrix(row);
                ans.setElement(0,0,x*m.getElement(0,0));
                ans.setElement(0,1,x*m.getElement(1,0));
                row--;
                for(int i=1;i<row;i++) {
                        ans.setElement(i,i-1,x*m.getElement(i-1,i));
                        ans.setElement(i,i,x*m.getElement(i,i));
                        ans.setElement(i,i+1,x*m.getElement(i+1,i));
                }
                ans.setElement(row,row-1,x*m.getElement(row-1,row));
                ans.setElement(row,row,x*m.getElement(row,row));
                return ans;
        }
        /**
        * Returns the multiplication of a matrix by a scalar.
        * @param x an integer
        * @param m an integer diagonal matrix
        */
        public static IntegerDiagonalMatrix scalarMultiply(int x, IntegerDiagonalMatrix m) {
                int row=m.rows();
                int array[]=new int[row];
                for(int i=0;i<row;i++) {
                        array[i]=x*m.getElement(i,i);
                }
                return new IntegerDiagonalMatrix(array);
        }

        /**
        * Returns the multiplication of a matrix by a scalar.
        * @param x a double
        * @param m a double matrix
        */
        public static DoubleMatrix scalarMultiply(double x, DoubleMatrix m) {
                int row=m.rows(),col=m.columns();
                double array[][]=new double[row][col];
                for(int j,i=0;i<row;i++) {
                        for(j=0;j<col;j++)
                                array[i][j]=x*m.getElement(i,j);
                }
                return new DoubleMatrix(array);
        }
        /**
        * Returns the multiplication of a matrix by a scalar.
        * @param x a double
        * @param m a double square matrix
        */
        public static DoubleSquareMatrix scalarMultiply(double x, DoubleSquareMatrix m) {
                int row=m.rows();
                double array[][]=new double[row][row];
                for(int j,i=0;i<row;i++) {
                        for(j=0;j<row;j++)
                                array[i][j]=x*m.getElement(i,j);
                }
                return new DoubleSquareMatrix(array);
        }
        /**
        * Returns the multiplication of a matrix by a scalar.
        * @param x a double
        * @param m a double tridiagonal matrix
        */
        public static DoubleTridiagonalMatrix scalarMultiply(double x, DoubleTridiagonalMatrix m) {
                int row=m.rows();
                DoubleTridiagonalMatrix ans=new DoubleTridiagonalMatrix(row);
                ans.setElement(0,0,x*m.getElement(0,0));
                ans.setElement(0,1,x*m.getElement(1,0));
                row--;
                for(int i=1;i<row;i++) {
                        ans.setElement(i,i-1,x*m.getElement(i-1,i));
                        ans.setElement(i,i,x*m.getElement(i,i));
                        ans.setElement(i,i+1,x*m.getElement(i+1,i));
                }
                ans.setElement(row,row-1,x*m.getElement(row-1,row));
                ans.setElement(row,row,x*m.getElement(row,row));
                return ans;
        }
        /**
        * Returns the multiplication of a matrix by a scalar.
        * @param x a double
        * @param m a double diagonal matrix
        */
        public static DoubleDiagonalMatrix scalarMultiply(double x, DoubleDiagonalMatrix m) {
                int row=m.rows();
                double array[]=new double[row];
                for(int i=0;i<row;i++) {
                        array[i]=x*m.getElement(i,i);
                }
                return new DoubleDiagonalMatrix(array);
        }

        /**
        * Returns the multiplication of a matrix by a scalar.
        * @param z a complex number
        * @param m a complex matrix
        */
        public static ComplexMatrix scalarMultiply(Complex z, ComplexMatrix m) {
                int row=m.rows(),col=m.columns();
                Complex array[][]=new Complex[row][col];
                for(int j,i=0;i<row;i++) {
                        for(j=0;j<col;j++)
                                array[i][j]=ComplexMath.multiply(z,m.getElement(i,j));
                }
                return new ComplexMatrix(array);
        }
        /**
        * Returns the multiplication of a matrix by a scalar.
        * @param z a complex number
        * @param m a complex square matrix
        */
        public static ComplexSquareMatrix scalarMultiply(Complex z, ComplexSquareMatrix m) {
                int row=m.rows();
                Complex array[][]=new Complex[row][row];
                for(int j,i=0;i<row;i++) {
                        for(j=0;j<row;j++)
                                array[i][j]=ComplexMath.multiply(z,m.getElement(i,j));
                }
                return new ComplexSquareMatrix(array);
        }
        /**
        * Returns the multiplication of a matrix by a scalar.
        * @param z a complex number
        * @param m a complex tridiagonal matrix
        */
        public static ComplexTridiagonalMatrix scalarMultiply(Complex z, ComplexTridiagonalMatrix m) {
                int row=m.rows();
                ComplexTridiagonalMatrix ans=new ComplexTridiagonalMatrix(row);
                ans.setElement(0,0,ComplexMath.multiply(z,m.getElement(0,0)));
                ans.setElement(0,1,ComplexMath.multiply(z,m.getElement(1,0)));
                row--;
                for(int i=1;i<row;i++) {
                        ans.setElement(i,i-1,ComplexMath.multiply(z,m.getElement(i-1,i)));
                        ans.setElement(i,i,ComplexMath.multiply(z,m.getElement(i,i)));
                        ans.setElement(i,i+1,ComplexMath.multiply(z,m.getElement(i+1,i)));
                }
                ans.setElement(row,row-1,ComplexMath.multiply(z,m.getElement(row-1,row)));
                ans.setElement(row,row,ComplexMath.multiply(z,m.getElement(row,row)));
                return ans;
        }
        /**
        * Returns the multiplication of a matrix by a scalar.
        * @param z a complex number
        * @param m a complex diagonal matrix
        */
        public static ComplexDiagonalMatrix scalarMultiply(Complex z, ComplexDiagonalMatrix m) {
                int row=m.rows();
                Complex array[]=new Complex[row];
                for(int i=0;i<row;i++) {
                        array[i]=ComplexMath.multiply(z,m.getElement(i,i));
                }
                return new ComplexDiagonalMatrix(array);
        }

// MATRIX MULTIPLICATION

        /**
        * Returns the multiplication of two matrices.
        * @param a an integer matrix
        * @param b an integer matrix
        * @exception MatrixDimensionException If the matrices are incompatible.
        */
        public static IntegerMatrix multiply(IntegerMatrix a, IntegerMatrix b) throws MatrixDimensionException {
                int aRow=a.rows(),bRow=b.rows(),bCol=b.columns();
                if(a.columns()==bRow) {
                        int array[][]=new int[aRow][bCol];
// Create threads
                        for(int k,j=0;j<aRow;j++) {
                                for(k=0;k<bCol;k++)
                                        (new IntegerMatrixMultiplyThread(a,b,j,k,array)).start();
                        }
                        return new IntegerMatrix(array);
                } else
                        throw new MatrixDimensionException("Incompatible matrices.");
        }
        /**
        * Returns the multiplication of two square matrices.
        * @param a an integer square matrix
        * @param b an integer square matrix
        * @exception MatrixDimensionException If the matrices are incompatible.
        */
        public static IntegerSquareMatrix multiply(IntegerSquareMatrix a, IntegerSquareMatrix b) throws MatrixDimensionException {
                int aRow=a.rows();
                if(aRow==b.rows()) {
                        int array[][]=new int[aRow][aRow];
// Create threads
                        for(int k,j=0;j<aRow;j++) {
                                for(k=0;k<aRow;k++)
                                        (new IntegerMatrixMultiplyThread(a,b,j,k,array)).start();
                        }
                        return new IntegerSquareMatrix(array);
                } else
                        throw new MatrixDimensionException("Incompatible matrices.");
        }
        /**
        * Returns the multiplication of two diagonal matrices.
        * @param a an integer diagonal matrix
        * @param b an integer diagonal matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static IntegerDiagonalMatrix multiply(IntegerDiagonalMatrix a, IntegerDiagonalMatrix b) throws MatrixDimensionException {
                int aRow=a.rows();
                if(aRow==b.rows()) {
                        int array[]=new int[aRow];
                        for(int i=0;i<aRow;i++)
                                array[i]=a.getElement(i,i)*b.getElement(i,i);
                        return new IntegerDiagonalMatrix(array);
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }

        /**
        * Returns the multiplication of two matrices.
        * @param a a double matrix
        * @param b a double matrix
        * @exception MatrixDimensionException If the matrices are incompatible.
        */
        public static DoubleMatrix multiply(DoubleMatrix a, DoubleMatrix b) throws MatrixDimensionException {
                int aRow=a.rows(),bRow=b.rows(),bCol=b.columns();
                if(a.columns()==bRow) {
                        double array[][]=new double[aRow][bCol];
// Create threads
                        for(int k,j=0;j<aRow;j++) {
                                for(k=0;k<bCol;k++)
                                        (new DoubleMatrixMultiplyThread(a,b,j,k,array)).start();
                        }
                        return new DoubleMatrix(array);
                } else
                        throw new MatrixDimensionException("Incompatible matrices.");
        }
        /**
        * Returns the multiplication of two square matrices.
        * @param a a double square matrix
        * @param b a double square matrix
        * @exception MatrixDimensionException If the matrices are incompatible.
        */
        public static DoubleSquareMatrix multiply(DoubleSquareMatrix a, DoubleSquareMatrix b) throws MatrixDimensionException {
                int aRow=a.rows();
                if(aRow==b.rows()) {
                        double array[][]=new double[aRow][aRow];
// Create threads
                        for(int k,j=0;j<aRow;j++) {
                                for(k=0;k<aRow;k++)
                                        (new DoubleMatrixMultiplyThread(a,b,j,k,array)).start();
                        }
                        return new DoubleSquareMatrix(array);
                } else
                        throw new MatrixDimensionException("Incompatible matrices.");
        }
        /**
        * Returns the multiplication of two diagonal matrices.
        * @param a a double diagonal matrix
        * @param b a double diagonal matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static DoubleDiagonalMatrix multiply(DoubleDiagonalMatrix a, DoubleDiagonalMatrix b) throws MatrixDimensionException {
                int aRow=a.rows();
                if(aRow==b.rows()) {
                        double array[]=new double[aRow];
                        for(int i=0;i<aRow;i++)
                                array[i]=a.getElement(i,i)*b.getElement(i,i);
                        return new DoubleDiagonalMatrix(array);
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }

        /**
        * Returns the multiplication of two matrices.
        * @param a a complex matrix
        * @param b a complex matrix
        * @exception MatrixDimensionException If the matrices are incompatible.
        */
        public static ComplexMatrix multiply(ComplexMatrix a, ComplexMatrix b) throws MatrixDimensionException {
                int aRow=a.rows(),bRow=b.rows(),bCol=b.columns();
                if(a.columns()==bRow) {
                        Complex array[][]=new Complex[aRow][bCol];
// Create threads
                        for(int k,j=0;j<aRow;j++) {
                                for(k=0;k<bCol;k++)
                                        (new ComplexMatrixMultiplyThread(a,b,j,k,array)).start();
                        }
                        return new ComplexMatrix(array);
                } else
                        throw new MatrixDimensionException("Incompatible matrices.");
        }
        /**
        * Returns the multiplication of two square matrices.
        * @param a a complex square matrix
        * @param b a complex square matrix
        * @exception MatrixDimensionException If the matrices are incompatible.
        */
        public static ComplexSquareMatrix multiply(ComplexSquareMatrix a, ComplexSquareMatrix b) throws MatrixDimensionException {
                int aRow=a.rows();
                if(aRow==b.rows()) {
                        Complex array[][]=new Complex[aRow][aRow];
// Create threads
                        for(int k,j=0;j<aRow;j++) {
                                for(k=0;k<aRow;k++)
                                        (new ComplexMatrixMultiplyThread(a,b,j,k,array)).start();
                        }
                        return new ComplexSquareMatrix(array);
                } else
                        throw new MatrixDimensionException("Incompatible matrices.");
        }
        /**
        * Returns the multiplication of two diagonal matrices.
        * @param a a complex diagonal matrix
        * @param b a complex diagonal matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static ComplexDiagonalMatrix multiply(ComplexDiagonalMatrix a, ComplexDiagonalMatrix b) throws MatrixDimensionException {
                int aRow=a.rows();
                if(aRow==b.rows()) {
                        Complex array[]=new Complex[aRow];
                        for(int i=0;i<aRow;i++)
                                array[i]=ComplexMath.multiply(a.getElement(i,i),b.getElement(i,i));
                        return new ComplexDiagonalMatrix(array);
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }

// TRANSPOSE

        /**
        * Returns the transpose of a matrix.
        * @param m an integer matrix
        */
        public static IntegerMatrix transpose(IntegerMatrix m) {
                int mRow=m.rows(),mCol=m.columns();
                int array[][]=new int[mCol][mRow];
                for(int j,i=0;i<mRow;i++) {
                        for(j=0;j<mCol;j++)
                                array[j][i]=m.getElement(i,j);
                }
                return new IntegerMatrix(array);
        }
        /**
        * Returns the transpose of a square matrix.
        * @param m an integer square matrix
        */
        public static IntegerSquareMatrix transpose(IntegerSquareMatrix m) {
                int mRow=m.rows();
                int array[][]=new int[mRow][mRow];
                for(int j,i=0;i<mRow;i++) {
                        for(j=0;j<mRow;j++)
                                array[j][i]=m.getElement(i,j);
                }
                return new IntegerSquareMatrix(array);
        }
        /**
        * Returns the transpose of a tridiagonal matrix.
        * @param m an integer tridiagonal matrix
        */
        public static IntegerTridiagonalMatrix transpose(IntegerTridiagonalMatrix m) {
                int mRow=m.rows();
                IntegerTridiagonalMatrix ans=new IntegerTridiagonalMatrix(mRow);
                ans.setElement(0,0,m.getElement(0,0));
                ans.setElement(0,1,m.getElement(1,0));
                mRow--;
                for(int i=1;i<mRow;i++) {
                        ans.setElement(i,i-1,m.getElement(i-1,i));
                        ans.setElement(i,i,m.getElement(i,i));
                        ans.setElement(i,i+1,m.getElement(i+1,i));
                }
                ans.setElement(mRow,mRow-1,m.getElement(mRow-1,mRow));
                ans.setElement(mRow,mRow,m.getElement(mRow,mRow));
                return ans;
        }

        /**
        * Returns the transpose of a matrix.
        * @param m a double matrix
        */
        public static DoubleMatrix transpose(DoubleMatrix m) {
                int mRow=m.rows(),mCol=m.columns();
                double array[][]=new double[mCol][mRow];
                for(int j,i=0;i<mRow;i++) {
                        for(j=0;j<mCol;j++)
                                array[j][i]=m.getElement(i,j);
                }
                return new DoubleMatrix(array);
        }
        /**
        * Returns the transpose of a square matrix.
        * @param m a double square matrix
        */
        public static DoubleSquareMatrix transpose(DoubleSquareMatrix m) {
                int mRow=m.rows();
                double array[][]=new double[mRow][mRow];
                for(int j,i=0;i<mRow;i++) {
                        for(j=0;j<mRow;j++)
                                array[j][i]=m.getElement(i,j);
                }
                return new DoubleSquareMatrix(array);
        }
        /**
        * Returns the transpose of a tridiagonal matrix.
        * @param m a double tridiagonal matrix
        */
        public static DoubleTridiagonalMatrix transpose(DoubleTridiagonalMatrix m) {
                int mRow=m.rows();
                DoubleTridiagonalMatrix ans=new DoubleTridiagonalMatrix(mRow);
                ans.setElement(0,0,m.getElement(0,0));
                ans.setElement(0,1,m.getElement(1,0));
                mRow--;
                for(int i=1;i<mRow;i++) {
                        ans.setElement(i,i-1,m.getElement(i-1,i));
                        ans.setElement(i,i,m.getElement(i,i));
                        ans.setElement(i,i+1,m.getElement(i+1,i));
                }
                ans.setElement(mRow,mRow-1,m.getElement(mRow-1,mRow));
                ans.setElement(mRow,mRow,m.getElement(mRow,mRow));
                return ans;
        }

        /**
        * Returns the transpose of a matrix.
        * @param m a complex matrix
        */
        public static ComplexMatrix transpose(ComplexMatrix m) {
                int mRow=m.rows(),mCol=m.columns();
                Complex array[][]=new Complex[mCol][mRow];
                for(int j,i=0;i<mRow;i++) {
                        for(j=0;j<mCol;j++)
                                array[j][i]=m.getElement(i,j);
                }
                return new ComplexMatrix(array);
        }
        /**
        * Returns the transpose of a square matrix.
        * @param m a complex square matrix
        */
        public static ComplexSquareMatrix transpose(ComplexSquareMatrix m) {
                int mRow=m.rows();
                Complex array[][]=new Complex[mRow][mRow];
                for(int j,i=0;i<mRow;i++) {
                        for(j=0;j<mRow;j++)
                                array[j][i]=m.getElement(i,j);
                }
                return new ComplexSquareMatrix(array);
        }
        /**
        * Returns the transpose of a tridiagonal matrix.
        * @param m a complex tridiagonal matrix
        */
        public static ComplexTridiagonalMatrix transpose(ComplexTridiagonalMatrix m) {
                int mRow=m.rows();
                ComplexTridiagonalMatrix ans=new ComplexTridiagonalMatrix(mRow);
                ans.setElement(0,0,m.getElement(0,0));
                ans.setElement(0,1,m.getElement(1,0));
                mRow--;
                for(int i=1;i<mRow;i++) {
                        ans.setElement(i,i-1,m.getElement(i-1,i));
                        ans.setElement(i,i,m.getElement(i,i));
                        ans.setElement(i,i+1,m.getElement(i+1,i));
                }
                ans.setElement(mRow,mRow-1,m.getElement(mRow-1,mRow));
                ans.setElement(mRow,mRow,m.getElement(mRow,mRow));
                return ans;
        }

// INVERSE

        /**
        * Returns the inverse of a matrix.
        * @param m an integer square matrix
        */
        public static DoubleSquareMatrix inverse(IntegerSquareMatrix m) {
                int i,j,k,mRow=m.rows();
                double tmpL,tmpU,array[][][]=new double[2][mRow][mRow];
                DoubleSquareMatrix lu[]=LUDecomposition(m);
                for(i=0;i<mRow;i++) {
                        array[0][i][i]=1/lu[0].getElement(i,i);
                        array[1][i][i]=1/lu[1].getElement(i,i);
                }
                for(i=0;i<mRow-1;i++) {
                        for(j=i+1;j<mRow;j++) {
                                tmpL=tmpU=0.0;
                                for(k=i;k<j;k++) {
                                        tmpL-=lu[0].getElement(j,k)*array[0][k][i];
                                        tmpU-=array[1][i][k]*lu[1].getElement(k,j);
                                }
                                array[0][j][i]=tmpL/lu[0].getElement(j,j);
                                array[1][i][j]=tmpU/lu[1].getElement(j,j);
                        }
                }
                return MatrixMath.multiply(new DoubleSquareMatrix(array[1]),new DoubleSquareMatrix(array[0]));
        }
        /**
        * Returns the inverse of a matrix.
        * @param m an integer diagonal matrix
        */
        public static DoubleDiagonalMatrix inverse(IntegerDiagonalMatrix m) {
                int mRow=m.rows();
                double array[]=new double[mRow];
                for(int i=0;i<mRow;i++)
                        array[i]=1/m.getElement(i,i);
                return new DoubleDiagonalMatrix(array);
        }

        /**
        * Returns the inverse of a matrix.
        * @param m a double square matrix
        */
        public static DoubleSquareMatrix inverse(DoubleSquareMatrix m) {
                int i,j,k,mRow=m.rows();
                double tmpL,tmpU,array[][][]=new double[2][mRow][mRow];
                DoubleSquareMatrix lu[]=LUDecomposition(m);
                for(i=0;i<mRow;i++) {
                        array[0][i][i]=1/lu[0].getElement(i,i);
                        array[1][i][i]=1/lu[1].getElement(i,i);
                }
                for(i=0;i<mRow-1;i++) {
                        for(j=i+1;j<mRow;j++) {
                                tmpL=tmpU=0.0;
                                for(k=i;k<j;k++) {
                                        tmpL-=lu[0].getElement(j,k)*array[0][k][i];
                                        tmpU-=array[1][i][k]*lu[1].getElement(k,j);
                                }
                                array[0][j][i]=tmpL/lu[0].getElement(j,j);
                                array[1][i][j]=tmpU/lu[1].getElement(j,j);
                        }
                }
                return MatrixMath.multiply(new DoubleSquareMatrix(array[1]),new DoubleSquareMatrix(array[0]));
        }
        /**
        * Returns the inverse of a matrix.
        * @param m a double diagonal matrix
        */
        public static DoubleDiagonalMatrix inverse(DoubleDiagonalMatrix m) {
                int mRow=m.rows();
                double array[]=new double[mRow];
                for(int i=0;i<mRow;i++)
                        array[i]=1/m.getElement(i,i);
                return new DoubleDiagonalMatrix(array);
        }

        /**
        * Returns the inverse of a matrix.
        * @param m a complex square matrix
        */
        public static ComplexSquareMatrix inverse(ComplexSquareMatrix m) {
                int i,j,k,mRow=m.rows();
                Complex tmpL,tmpU,array[][][]=new Complex[2][mRow][mRow];
                ComplexSquareMatrix lu[]=LUDecomposition(m);
                for(i=0;i<mRow;i++) {
                        array[0][i][i]=ComplexMath.divide(Complex.ONE,lu[0].getElement(i,i));
                        array[1][i][i]=ComplexMath.divide(Complex.ONE,lu[1].getElement(i,i));
                }
                for(i=0;i<mRow-1;i++) {
                        for(j=i+1;j<mRow;j++) {
                                tmpL=tmpU=Complex.ZERO;
                                for(k=i;k<j;k++) {
                                        tmpL=ComplexMath.subtract(tmpL,ComplexMath.multiply(lu[0].getElement(j,k),array[0][k][i]));
                                        tmpU=ComplexMath.subtract(tmpU,ComplexMath.multiply(array[1][i][k],lu[1].getElement(k,j)));
                                }
                                array[0][i][j]=Complex.ZERO;
                                array[0][j][i]=ComplexMath.divide(tmpL,lu[0].getElement(j,j));
                                array[1][j][i]=Complex.ZERO;
                                array[1][i][j]=ComplexMath.divide(tmpU,lu[1].getElement(j,j));
                        }
                }
                return MatrixMath.multiply(new ComplexSquareMatrix(array[1]),new ComplexSquareMatrix(array[0]));
        }
        /**
        * Returns the inverse of a matrix.
        * @param m a complex diagonal matrix
        */
        public static ComplexDiagonalMatrix inverse(ComplexDiagonalMatrix m) {
                int mRow=m.rows();
                Complex array[]=new Complex[mRow];
                for(int i=0;i<mRow;i++)
                        array[i]=ComplexMath.divide(Complex.ONE,m.getElement(i,i));
                return new ComplexDiagonalMatrix(array);
        }

// ELEMENT MAP

        /**
        * Applies a function on all the matrix elements.
        * @param m an integer matrix
        * @param f a user-defined function
        * @return a double matrix
        */
        public static DoubleMatrix mapElements(IntegerMatrix m, Mapping f) {
                int mRow=m.rows(),mCol=m.columns();
                double array[][]=new double[mRow][mCol];
                for(int j,i=0;i<mRow;i++) {
                        for(j=0;j<mCol;j++)
                                array[i][j]=f.map(m.getElement(i,j));
                }
                return new DoubleMatrix(array);
        }
        /**
        * Applies a function on all the matrix elements.
        * @param m an integer square matrix
        * @param f a user-defined function
        * @return a double square matrix
        */
        public static DoubleSquareMatrix mapElements(IntegerSquareMatrix m, Mapping f) {
                int mRow=m.rows();
                double array[][]=new double[mRow][mRow];
                for(int j,i=0;i<mRow;i++) {
                        for(j=0;j<mRow;j++)
                                array[i][j]=f.map(m.getElement(i,j));
                }
                return new DoubleSquareMatrix(array);
        }
        /**
        * Applies a function on all the matrix elements.
        * @param m an integer tridiagonal matrix
        * @param f a user-defined function
        * @return a double tridiagonal matrix
        */
        public static DoubleTridiagonalMatrix mapElements(IntegerTridiagonalMatrix m, Mapping f) {
                int mRow=m.rows();
                DoubleTridiagonalMatrix ans=new DoubleTridiagonalMatrix(mRow);
                ans.setElement(0,0,f.map(m.getElement(0,0)));
                ans.setElement(0,1,f.map(m.getElement(1,0)));
                mRow--;
                for(int i=1;i<mRow;i++) {
                        ans.setElement(i,i-1,f.map(m.getElement(i-1,i)));
                        ans.setElement(i,i,f.map(m.getElement(i,i)));
                        ans.setElement(i,i+1,f.map(m.getElement(i+1,i)));
                }
                ans.setElement(mRow,mRow-1,f.map(m.getElement(mRow-1,mRow)));
                ans.setElement(mRow,mRow,f.map(m.getElement(mRow,mRow)));
                return ans;
        }
        /**
        * Applies a function on all the matrix elements.
        * @param m an integer diagonal matrix
        * @param f a user-defined function
        * @return a double diagonal matrix
        */
        public static DoubleDiagonalMatrix mapElements(IntegerDiagonalMatrix m, Mapping f) {
                int mRow=m.rows();
                double array[]=new double[mRow];
                for(int i=0;i<mRow;i++)
                        array[i]=f.map(m.getElement(i,i));
                return new DoubleDiagonalMatrix(array);
        }

        /**
        * Applies a function on all the matrix elements.
        * @param m a double matrix
        * @param f a user-defined function
        * @return a double matrix
        */
        public static DoubleMatrix mapElements(DoubleMatrix m, Mapping f) {
                int mRow=m.rows(),mCol=m.columns();
                double array[][]=new double[mRow][mCol];
                for(int j,i=0;i<mRow;i++) {
                        for(j=0;j<mCol;j++)
                                array[i][j]=f.map(m.getElement(i,j));
                }
                return new DoubleMatrix(array);
        }
        /**
        * Applies a function on all the matrix elements.
        * @param m a double square matrix
        * @param f a user-defined function
        * @return a double square matrix
        */
        public static DoubleSquareMatrix mapElements(DoubleSquareMatrix m, Mapping f) {
                int mRow=m.rows();
                double array[][]=new double[mRow][mRow];
                for(int j,i=0;i<mRow;i++) {
                        for(j=0;j<mRow;j++)
                                array[i][j]=f.map(m.getElement(i,j));
                }
                return new DoubleSquareMatrix(array);
        }
        /**
        * Applies a function on all the matrix elements.
        * @param m a double tridiagonal matrix
        * @param f a user-defined function
        * @return a double tridiagonal matrix
        */
        public static DoubleTridiagonalMatrix mapElements(DoubleTridiagonalMatrix m, Mapping f) {
                int mRow=m.rows();
                DoubleTridiagonalMatrix ans=new DoubleTridiagonalMatrix(mRow);
                ans.setElement(0,0,f.map(m.getElement(0,0)));
                ans.setElement(0,1,f.map(m.getElement(1,0)));
                mRow--;
                for(int i=1;i<mRow;i++) {
                        ans.setElement(i,i-1,f.map(m.getElement(i-1,i)));
                        ans.setElement(i,i,f.map(m.getElement(i,i)));
                        ans.setElement(i,i+1,f.map(m.getElement(i+1,i)));
                }
                ans.setElement(mRow,mRow-1,f.map(m.getElement(mRow-1,mRow)));
                ans.setElement(mRow,mRow,f.map(m.getElement(mRow,mRow)));
                return ans;
        }
        /**
        * Applies a function on all the matrix elements.
        * @param m a double diagonal matrix
        * @param f a user-defined function
        * @return a double diagonal matrix
        */
        public static DoubleDiagonalMatrix mapElements(DoubleDiagonalMatrix m, Mapping f) {
                int mRow=m.rows();
                double array[]=new double[mRow];
                for(int i=0;i<mRow;i++)
                        array[i]=f.map(m.getElement(i,i));
                return new DoubleDiagonalMatrix(array);
        }

        /**
        * Applies a function on all the matrix elements.
        * @param m a complex matrix
        * @param f a user-defined function
        * @return a complex matrix
        */
        public static ComplexMatrix mapElements(ComplexMatrix m, Mapping f) {
                int mRow=m.rows(),mCol=m.columns();
                Complex array[][]=new Complex[mRow][mCol];
                for(int j,i=0;i<mRow;i++) {
                        for(j=0;j<mCol;j++)
                                array[i][j]=f.map(m.getElement(i,j));
                }
                return new ComplexMatrix(array);
        }
        /**
        * Applies a function on all the matrix elements.
        * @param m a complex square matrix
        * @param f a user-defined function
        * @return a complex square matrix
        */
        public static ComplexSquareMatrix mapElements(ComplexSquareMatrix m, Mapping f) {
                int mRow=m.rows();
                Complex array[][]=new Complex[mRow][mRow];
                for(int j,i=0;i<mRow;i++) {
                        for(j=0;j<mRow;j++)
                                array[i][j]=f.map(m.getElement(i,j));
                }
                return new ComplexSquareMatrix(array);
        }
        /**
        * Applies a function on all the matrix elements.
        * @param m a complex tridiagonal matrix
        * @param f a user-defined function
        * @return a complex tridiagonal matrix
        */
        public static ComplexTridiagonalMatrix mapElements(ComplexTridiagonalMatrix m, Mapping f) {
                int mRow=m.rows();
                ComplexTridiagonalMatrix ans=new ComplexTridiagonalMatrix(mRow);
                ans.setElement(0,0,f.map(m.getElement(0,0)));
                ans.setElement(0,1,f.map(m.getElement(1,0)));
                mRow--;
                for(int i=1;i<mRow;i++) {
                        ans.setElement(i,i-1,f.map(m.getElement(i-1,i)));
                        ans.setElement(i,i,f.map(m.getElement(i,i)));
                        ans.setElement(i,i+1,f.map(m.getElement(i+1,i)));
                }
                ans.setElement(mRow,mRow-1,f.map(m.getElement(mRow-1,mRow)));
                ans.setElement(mRow,mRow,f.map(m.getElement(mRow,mRow)));
                return ans;
        }
        /**
        * Applies a function on all the matrix elements.
        * @param m a complex diagonal matrix
        * @param f a user-defined function
        * @return a complex diagonal matrix
        */
        public static ComplexDiagonalMatrix mapElements(ComplexDiagonalMatrix m, Mapping f) {
                int mRow=m.rows();
                Complex array[]=new Complex[mRow];
                for(int i=0;i<mRow;i++)
                        array[i]=f.map(m.getElement(i,i));
                return new ComplexDiagonalMatrix(array);
        }

// LU DECOMPOSITION

        /**
        * Returns the LU decomposition of a matrix.
        * @param m a double square matrix
        * @return an array with [0] containing the L-matrix and [1] containing the U-matrix.
        */
        public static DoubleSquareMatrix[] LUDecomposition(DoubleSquareMatrix m) {
                int i,j,k,mRow=m.rows();
                double tmp,array[][][]=new double[2][mRow][mRow];
                for(i=0;i<mRow;i++)
                        array[0][i][i]=1.0;
                for(j=0;j<mRow;j++) {
                        for(i=0;i<=j;i++) {
                                tmp=m.getElement(i,j);
                                for(k=0;k<i;k++)
                                        tmp-=array[0][i][k]*array[1][k][j];
                                array[1][i][j]=tmp;
                        }
                        for(i=j+1;i<mRow;i++) {
                                tmp=m.getElement(i,j);
                                for(k=0;k<j;k++)
                                        tmp-=array[0][i][k]*array[1][k][j];
                                array[0][i][j]=tmp/array[1][j][j];
                        }
                }
                DoubleSquareMatrix LU[]=new DoubleSquareMatrix[2];
                LU[0]=new DoubleSquareMatrix(array[0]);
                LU[1]=new DoubleSquareMatrix(array[1]);
                return LU;
        }
        /**
        * Returns the LU decomposition of a matrix.
        * @param m a complex square matrix
        * @return an array with [0] containing the L-matrix and [1] containing the U-matrix.
        */
        public static ComplexSquareMatrix[] LUDecomposition(ComplexSquareMatrix m) {
                int i,j,k,mRow=m.rows();
                Complex tmp,array[][][]=new Complex[2][mRow][mRow];
                for(i=0;i<mRow;i++)
                        array[0][i][i]=Complex.ONE;
                for(j=0;j<mRow;j++) {
                        for(i=0;i<=j;i++) {
                                tmp=m.getElement(i,j);
                                for(k=0;k<i;k++)
                                        tmp=ComplexMath.subtract(tmp,ComplexMath.multiply(array[0][i][k],array[1][k][j]));
                                array[1][j][i]=Complex.ZERO;
                                array[1][i][j]=tmp;
                        }
                        for(i=j+1;i<mRow;i++) {
                                tmp=m.getElement(i,j);
                                for(k=0;k<j;k++)
                                        tmp=ComplexMath.subtract(tmp,ComplexMath.multiply(array[0][i][k],array[1][k][j]));
                                array[0][j][i]=Complex.ZERO;
                                array[0][i][j]=ComplexMath.divide(tmp,array[1][j][j]);
                        }
                }
                ComplexSquareMatrix LU[]=new ComplexSquareMatrix[2];
                LU[0]=new ComplexSquareMatrix(array[0]);
                LU[1]=new ComplexSquareMatrix(array[1]);
                return LU;
        }
        /**
        * Returns the LU decomposition of a matrix.
        * @param m an integer square matrix
        * @return an array with [0] containing the L-matrix and [1] containing the U-matrix.
        */
        public static DoubleSquareMatrix[] LUDecomposition(IntegerSquareMatrix m) {
                int i,j,k,mRow=m.rows(),tmp;
                double array[][][]=new double[2][mRow][mRow];
                for(i=0;i<mRow;i++)
                        array[0][i][i]=1.0;
                for(j=0;j<mRow;j++) {
                        for(i=0;i<=j;i++) {
                                tmp=m.getElement(i,j);
                                for(k=0;k<i;k++)
                                        tmp-=array[0][i][k]*array[1][k][j];
                                array[1][i][j]=tmp;
                        }
                        for(i=j+1;i<mRow;i++) {
                                tmp=m.getElement(i,j);
                                for(k=0;k<j;k++)
                                        tmp-=array[0][i][k]*array[1][k][j];
                                array[0][i][j]=tmp/array[1][j][j];
                        }
                }
                DoubleSquareMatrix LU[]=new DoubleSquareMatrix[2];
                LU[0]=new DoubleSquareMatrix(array[0]);
                LU[1]=new DoubleSquareMatrix(array[1]);
                return LU;
        }

// EIGENVALUES & EIGENVECTORS

        /**
        * This method finds the eigenvalues of a symmetric tridiagonal matrix by the QL method.
        * It is based on the NETLIB algol/fortran procedure tql1 by Bowdler, Martin, Reinsch and Wilkinson.
        * @param matrix a double symmetric tridiagonal matrix
        * @return an array containing the eigenvalues in ascending order
        * @exception MaximumIterationsExceededException If it takes more than 50 iterations to determine an eigenvalue.
        */
        public static double[] eigenvalueSolveSymmetric(DoubleTridiagonalMatrix matrix) throws MaximumIterationsExceededException {
                int i,m,iteration;
                int n=matrix.rows();
                double c,c2,c3=0.0,f=0.0,g,h,p,r,s,s2=0.0,test1=0.0,test2;
                double eigenvalue[]=new double[n];
                double offdiag[]=new double[n];

                for(int x=0;x<n-1;x++) {
                        eigenvalue[x]=matrix.getElement(x,x);
                        offdiag[x]=matrix.getElement(x,x+1);
                }
                eigenvalue[n-1]=matrix.getElement(n-1,n-1);
                offdiag[n-1]=0.0;

                for(int l=0;l<n;l++) {
                        iteration=0;
                        h=Math.abs(eigenvalue[l])+Math.abs(offdiag[l]);
                        if(test1<h)
                                test1=h;
// look for small sub-diagonal element
                        m=l-1;
                        do {
                                m++;
                                test2=test1+Math.abs(offdiag[m]);
                        } while(test2!=test1 && m+1<n);
                        if(m!=l) {
                                do {
                                        if(iteration==50)      // Max iterations reached
                                                throw new MaximumIterationsExceededException("No convergence after 50 iterations.");
                                        iteration++;
// form shift
                                        g=eigenvalue[l];
                                        p=(eigenvalue[l+1]-g)/(2.0*offdiag[l]);
                                        r=Math.sqrt(p*p+1.0);
                                        eigenvalue[l]=offdiag[l]/(p+Math.abs(r)*p/Math.abs(p));
                                        eigenvalue[l+1]=offdiag[l]*(p+Math.abs(r)*p/Math.abs(p));
                                        h=g-eigenvalue[l];
                                        for(i=l+2;i<n;i++)
                                                eigenvalue[i]-=h;
                                        f+=h;
// ql transformation
                                        p=eigenvalue[m];
                                        c2=c=1.0;
                                        s=0.0;
                                        for(i=m-1;i>l-1;i--) {
                                                c3=c2;
                                                c2=c;
                                                s2=s;
                                                g=c*offdiag[i];
                                                h=c*p;
                                                r=Math.sqrt(p*p+offdiag[i]*offdiag[i]);
                                                offdiag[i+1]=s*r;
                                                s=offdiag[i]/r;
                                                c=p/r;
                                                p=c*eigenvalue[i]-s*g;
                                                eigenvalue[i+1]=h+s*(c*g+s*eigenvalue[i]);
                                        }
                                        p=-s*s2*c3*offdiag[l+1]*offdiag[l]/eigenvalue[l+1];
                                        offdiag[l]=s*p;
                                        eigenvalue[l]=c*p;
                                        test2=test1+Math.abs(offdiag[l]);
                                } while(test2>test1);
                        }
                        p=eigenvalue[l]+f;
// order eigenvalues
                        for(i=l;i>0 && p<eigenvalue[i-1];i--)
                                eigenvalue[i]=eigenvalue[i-1];
                        eigenvalue[i]=p;
                }
                return eigenvalue;
        }

        /**
        * This method finds the eigenvalues and eigenvectors of a symmetric tridiagonal matrix by the QL method.
        * It is based on the NETLIB algol/fortran procedure tql2 by Bowdler, Martin, Reinsch and Wilkinson.
        * @param matrix a double symmetric tridiagonal matrix
        * @param eigenvector an empty array of double vectors to hold the eigenvectors
        * @return an array containing the eigenvalues in ascending order
        * @exception MaximumIterationsExceededException If it takes more than 50 iterations to determine an eigenvalue.
        */
        public static double[] eigenSolveSymmetric(DoubleTridiagonalMatrix matrix,DoubleVector eigenvector[]) throws MaximumIterationsExceededException {
                int x,y,n=matrix.rows();
                double eigenvalue[]=new double[n];
                double offdiag[]=new double[n];
                double id[][]=new double[n][n];

                for(x=0;x<n-1;x++) {
                        id[x][x]=1;
                        eigenvalue[x]=matrix.getElement(x,x);
                        offdiag[x]=matrix.getElement(x,x+1);
                }
                id[n-1][n-1]=1;
                eigenvalue[n-1]=matrix.getElement(n-1,n-1);
                offdiag[n-1]=0.0;

                eigenSolveSymmetricTridiagonalMatrix(eigenvalue,offdiag,id);

                for(x=0;x<n;x++) {
                        eigenvector[x]=new DoubleVector(n);
                        for(y=0;y<n;y++)
                                eigenvector[x].setComponent(y,id[y][x]);
                }
                return eigenvalue;
        }

        /**
        * This method finds the eigenvalues and eigenvectors of a symmetric matrix.
        * The matrix is reduced to tridiagonal form and then the QL method is applied.
        * It is based on the NETLIB algol/fortran procedure tred2/tql2 by Bowdler, Martin, Reinsch and Wilkinson.
        * @param matrix a double symmetric matrix
        * @param eigenvector an empty array of double vectors to hold the eigenvectors
        * @return an array containing the eigenvalues in ascending order
        * @exception MaximumIterationsExceededException If it takes more than 50 iterations to determine an eigenvalue.
        */
        public static double[] eigenSolveSymmetric(DoubleSquareMatrix matrix,DoubleVector eigenvector[]) throws MaximumIterationsExceededException {
                int x,y,n=matrix.rows();
                double eigenvalue[]=new double[n];
                double offdiag[]=new double[n];
                double transf[][]=new double[n][n];

                for(x=0;x<n;x++) {
                        for(y=0;y<n;y++)
                                transf[x][y]=matrix.getElement(x,y);
                }

                reduceSymmetricMatrixToSymmetricTridiagonalMatrix(transf,eigenvalue,offdiag);
                eigenSolveSymmetricTridiagonalMatrix(eigenvalue,offdiag,transf);

                for(x=0;x<n;x++) {
                        eigenvector[x]=new DoubleVector(n);
                        for(y=0;y<n;y++)
                                eigenvector[x].setComponent(y,transf[y][x]);
                }
                return eigenvalue;
        }



        /**
        * Internal NETLIB tql2 routine.
        * @param diag output eigenvalues
        * @param transf output eigenvectors
        */
        private static void eigenSolveSymmetricTridiagonalMatrix(double diag[],double offdiag[],double transf[][]) throws MaximumIterationsExceededException {
                int i,k,m,iteration;
                int n=diag.length;
                double c,c2,c3=0,f=0,g,h,p,r,s,s2=0,test1=0,test2;

                for(int l=0;l<n;l++) {
                        iteration=0;
                        h=Math.abs(diag[l])+Math.abs(offdiag[l]);
                        if(test1<h)
                                test1=h;
// look for small sub-diagonal element
                        m=l-1;
                        do {
                                m++;
                                test2=test1+Math.abs(offdiag[m]);
                        } while(test2!=test1 && m+1<n);
                        if(m!=l) {
                                do {
                                        if(iteration==50)      // Max iterations reached
                                                throw new MaximumIterationsExceededException("No convergence after 50 iterations.");
                                        iteration++;
// form shift
                                        g=diag[l];
                                        p=(diag[l+1]-g)/(2.0*offdiag[l]);
                                        r=Math.sqrt(p*p+1.0);
                                        diag[l]=offdiag[l]/(p+Math.abs(r)*p/Math.abs(p));
                                        diag[l+1]=offdiag[l]*(p+Math.abs(r)*p/Math.abs(p));
                                        h=g-diag[l];
                                        for(i=l+2;i<n;i++)
                                                diag[i]-=h;
                                        f+=h;
// ql transformation
                                        p=diag[m];
                                        c2=c=1.0;
                                        s=0.0;
                                        for(i=m-1;i>l-1;i--) {
                                                c3=c2;
                                                c2=c;
                                                s2=s;
                                                g=c*offdiag[i];
                                                h=c*p;
                                                r=Math.sqrt(p*p+offdiag[i]*offdiag[i]);
                                                offdiag[i+1]=s*r;
                                                s=offdiag[i]/r;
                                                c=p/r;
                                                p=c*diag[i]-s*g;
                                                diag[i+1]=h+s*(c*g+s*diag[i]);
// form vector
                                                for(k=0;k<n;k++) {
                                                        h=transf[k][i+1];
                                                        transf[k][i+1]=s*transf[k][i]+c*h;
                                                        transf[k][i]=c*transf[k][i]-s*h;
                                                }
                                        }
                                        p=-s*s2*c3*offdiag[l+1]*offdiag[l]/diag[l+1];
                                        offdiag[l]=s*p;
                                        diag[l]=c*p;
                                        test2=test1+Math.abs(offdiag[l]);
                                } while(test2>test1);
                        }
                        diag[l]+=f;
                }
// order eigenvalues and eigenvectors
                for(i=0;i<n;i++) {
                        k=i;
                        p=diag[i];
                        for(int j=i+1;j<n;j++) {
                                if(diag[j]<p) {
                                        k=j;
                                        p=diag[j];
                                }
                        }
                        if(k!=i) {
                                diag[k]=diag[i];
                                diag[i]=p;
                                for (int j=0;j<n;j++) {
                                        p=transf[j][i];
                                        transf[j][i]=transf[j][k];
                                        transf[j][k]=p;
                                }
                        }
                }
                return;
        }
        /**
        * Internal NETLIB tred2 routine.
        * @param matrix output orthogonal transformations
        */
        private static void reduceSymmetricMatrixToSymmetricTridiagonalMatrix(double matrix[][],double diag[],double offdiag[]) {
                int n=diag.length;
                int i,j,k,l,jp1;
                double f,g,h,hh,scale;

                for(i=0;i<n;i++)
                        diag[i]=matrix[n-1][i];
                if(n!=1) {
                        for(i=n-1;i>0;i--) {
                                l=i-1;
                                h=scale=0.0;
                                if(l>=1) {
// scale row (algol tol then not needed)
                                        for(k=0;k<l;k++)
                                                scale+=Math.abs(diag[k]);
                                }
                                if((scale==0 && l>=1) || l<1) {
                                        offdiag[i]=diag[l];
                                        for(j=0;j<l;j++) {
                                                diag[j]=matrix[l][j];
                                                matrix[i][j]=matrix[j][i]=0.0;
                                        }
                                } else {
                                        for(k=0;k<l;k++) {
                                                        diag[k]/=scale;
                                                        h+=diag[k]*diag[k];
                                        }
                                        f=diag[l];
                                        g=-Math.abs(Math.sqrt(h))*f/Math.abs(f);
                                        offdiag[i]=scale*g;
                                        h=h-f*g;
                                        diag[l]=f-g;
// form a*u
                                        for(j=0;j<l;j++)
                                                offdiag[j]=0.0;
                                        for(j=0;j<l;j++) {
                                                f=diag[j];
                                                matrix[j][i]=f;
                                                g=offdiag[j]+matrix[j][j]*f;
                                                jp1=j+1;
                                                if(l>=jp1) {
                                                        for(k=jp1;k<l;k++) {
                                                                g+=matrix[k][j]*diag[k];
                                                                offdiag[k]+=matrix[k][j]*f;
                                                        }
                                                }
                                                offdiag[j]=g;
                                        }
// form p
                                        f=0.0;
                                        for(j=0;j<l;j++) {
                                                offdiag[j]/=h;
                                                f+=offdiag[j]*diag[j];
                                        }
                                        hh=f/(h+h);
// form q
                                        for(j=0;j<l;j++)
                                                offdiag[j]-=hh*diag[j];
// form reduced a
                                        for(j=0;j<l;j++) {
                                                f=diag[j];
                                                g=offdiag[j];
                                                for(k=j;k<l;k++)
                                                        matrix[k][j]-=f*offdiag[k]+g*diag[k];
                                                diag[j]=matrix[l][j];
                                                matrix[i][j]=0.0;
                                        }
                                }
                                diag[i]=h;
                        }
// accumulation of transformation matrices
                        for(i=1;i<n;i++) {
                                l=i-1;
                                matrix[n-1][l]=matrix[l][l];
                                matrix[l][l]=1.0;
                                h=diag[i];
                                if(h!=0) {
                                        for(k=0;k<l;k++)
                                                diag[k]=matrix[k][i]/h;
                                        for(j=0;j<l;j++) {
                                                g=0.0;
                                                for(k=0;k<l;k++)
                                                        g+=matrix[k][i]*matrix[k][j];
                                                for(k=0;k<l;k++)
                                                        matrix[k][j]-=g*diag[k];
                                        }
                                }
                                for(k=0;k<l;k++)
                                        matrix[k][i]=0.0;
                        }                
                }
                for(i=0;i<n;i++) {
                        diag[i]=matrix[n-1][i];
                        matrix[n-1][i]=0.0;
                }
                matrix[n-1][n-1]=1.0;
                for(i=0;i<n-1;i++)
                        offdiag[i]=offdiag[i+1];
                offdiag[n-1]=0.0;
        }
}

//===================================================================
//                            THREAD CLASSES FOR MatrixMath
//===================================================================

/**
* A thread that performs matrix addition.
* This class should not be used directly.
* @version 1.0
* @author Mark Hale
*/
final class IntegerMatrixAddThread extends Thread {
        private IntegerMatrix matrixA,matrixB;
        private int r,c,answer[][];
        /**
        * Constructs a new thread.
        */
        public IntegerMatrixAddThread(IntegerMatrix a,IntegerMatrix b,int row,int col,int ans[][]) {
                matrixA=a;
                matrixB=b;
                r=row;
                c=col;
                answer=ans;
        }
        /**
        * The body of the thread.
        */
        public void run() {
                answer[r][c]=matrixA.getElement(r,c)+matrixB.getElement(r,c);
        }
}
/**
* A thread that performs matrix addition.
* This class should not be used directly.
* @version 1.0
* @author Mark Hale
*/
final class DoubleMatrixAddThread extends Thread {
        private DoubleMatrix matrixA,matrixB;
        private double answer[][];
        private int r,c;
        /**
        * Constructs a new thread.
        */
        public DoubleMatrixAddThread(DoubleMatrix a,DoubleMatrix b,int row,int col,double ans[][]) {
                matrixA=a;
                matrixB=b;
                r=row;
                c=col;
                answer=ans;
        }
        /**
        * The body of the thread.
        */
        public void run() {
                answer[r][c]=matrixA.getElement(r,c)+matrixB.getElement(r,c);
        }
}
/**
* A thread that performs matrix addition.
* This class should not be used directly.
* @version 1.0
* @author Mark Hale
*/
final class ComplexMatrixAddThread extends Thread {
        private ComplexMatrix matrixA,matrixB;
        private Complex answer[][];
        private int r,c;
        /**
        * Constructs a new thread.
        */
        public ComplexMatrixAddThread(ComplexMatrix a,ComplexMatrix b,int row,int col,Complex ans[][]) {
                matrixA=a;
                matrixB=b;
                r=row;
                c=col;
                answer=ans;
        }
        /**
        * The body of the thread.
        */
        public void run() {
                answer[r][c]=ComplexMath.add(matrixA.getElement(r,c),matrixB.getElement(r,c));
        }
}
/**
* A thread that performs matrix subtraction.
* This class should not be used directly.
* @version 1.0
* @author Mark Hale
*/
final class IntegerMatrixSubtractThread extends Thread {
        private IntegerMatrix matrixA,matrixB;
        private int r,c,answer[][];
        /**
        * Constructs a new thread.
        */
        public IntegerMatrixSubtractThread(IntegerMatrix a,IntegerMatrix b,int row,int col,int ans[][]) {
                matrixA=a;
                matrixB=b;
                r=row;
                c=col;
                answer=ans;
        }
        /**
        * The body of the thread.
        */
        public void run() {
                answer[r][c]=matrixA.getElement(r,c)-matrixB.getElement(r,c);
        }
}
/**
* A thread that performs matrix subtraction.
* This class should not be used directly.
* @version 1.0
* @author Mark Hale
*/
final class DoubleMatrixSubtractThread extends Thread {
        private DoubleMatrix matrixA,matrixB;
        private double answer[][];
        private int r,c;
        /**
        * Constructs a new thread.
        */
        public DoubleMatrixSubtractThread(DoubleMatrix a,DoubleMatrix b,int row,int col,double ans[][]) {
                matrixA=a;
                matrixB=b;
                r=row;
                c=col;
                answer=ans;
        }
        /**
        * The body of the thread.
        */
        public void run() {
                answer[r][c]=matrixA.getElement(r,c)-matrixB.getElement(r,c);
        }
}
/**
* A thread that performs matrix subtraction.
* This class should not be used directly.
* @version 1.0
* @author Mark Hale
*/
final class ComplexMatrixSubtractThread extends Thread {
        private ComplexMatrix matrixA,matrixB;
        private Complex answer[][];
        private int r,c;
        /**
        * Constructs a new thread.
        */
        public ComplexMatrixSubtractThread(ComplexMatrix a,ComplexMatrix b,int row,int col,Complex ans[][]) {
                matrixA=a;
                matrixB=b;
                r=row;
                c=col;
                answer=ans;
        }
        /**
        * The body of the thread.
        */
        public void run() {
                answer[r][c]=ComplexMath.subtract(matrixA.getElement(r,c),matrixB.getElement(r,c));
        }
}
/**
* A thread that performs matrix operation.
* This class should not be used directly.
* @version 1.0
* @author Mark Hale
*/
final class IntegerMatrixOpThread extends Thread {
        private IntegerMatrix matrix;
        private IntegerVector vector;
        private int c,answer[];
        /**
        * Constructs a new thread.
        */
        public IntegerMatrixOpThread(IntegerMatrix m,IntegerVector v,int col,int ans[]) {
                matrix=m;
                vector=v;
                c=col;
                answer=ans;
        }
        /**
        * The body of the thread.
        */
        public void run() {
                int n=matrix.columns();
                answer[c]=0;
                for(int i=0;i<n;i++)
                        answer[c]+=matrix.getElement(c,i)*vector.getComponent(i);
        }
}
/**
* A thread that performs matrix operation.
* This class should not be used directly.
* @version 1.0
* @author Mark Hale
*/
final class DoubleMatrixOpThread extends Thread {
        private DoubleMatrix matrix;
        private DoubleVector vector;
        private double answer[];
        private int c;
        /**
        * Constructs a new thread.
        */
        public DoubleMatrixOpThread(DoubleMatrix m,DoubleVector v,int col,double ans[]) {
                matrix=m;
                vector=v;
                c=col;
                answer=ans;
        }
        /**
        * The body of the thread.
        */
        public void run() {
                int n=matrix.columns();
                answer[c]=0.0;
                for(int i=0;i<n;i++)
                        answer[c]+=matrix.getElement(c,i)*vector.getComponent(i);
        }
}
/**
* A thread that performs matrix operation.
* This class should not be used directly.
* @version 1.0
* @author Mark Hale
*/
final class ComplexMatrixOpThread extends Thread {
        private ComplexMatrix matrix;
        private ComplexVector vector;
        private Complex answer[];
        private int c;
        /**
        * Constructs a new thread.
        */
        public ComplexMatrixOpThread(ComplexMatrix m,ComplexVector v,int col,Complex ans[]) {
                matrix=m;
                vector=v;
                c=col;
                answer=ans;
        }
        /**
        * The body of the thread.
        */
        public void run() {
                int n=matrix.columns();
                answer[c]=Complex.ZERO;
                for(int i=0;i<n;i++)
                        answer[c]=ComplexMath.add(answer[c],ComplexMath.multiply(matrix.getElement(c,i),vector.getComponent(i)));
        }
}
/**
* A thread that performs matrix multiplication.
* This class should not be used directly.
* @version 1.0
* @author Mark Hale
*/
final class IntegerMatrixMultiplyThread extends Thread {
        private IntegerMatrix matrixA,matrixB;
        private int rowA,colB,answer[][];
        /**
        * Constructs a new thread.
        */
        public IntegerMatrixMultiplyThread(IntegerMatrix a,IntegerMatrix b,int row,int col,int ans[][]) {
                matrixA=a;
                matrixB=b;
                rowA=row;
                colB=col;
                answer=ans;
        }
        /**
        * The body of the thread.
        */
        public void run() {
                int n=matrixA.columns();
                answer[rowA][colB]=0;
                for(int i=0;i<n;i++)
                        answer[rowA][colB]+=matrixA.getElement(rowA,i)*matrixB.getElement(i,colB);
        }
}
/**
* A thread that performs matrix multiplication.
* This class should not be used directly.
* @version 1.0
* @author Mark Hale
*/
final class DoubleMatrixMultiplyThread extends Thread {
        private DoubleMatrix matrixA,matrixB;
        private double answer[][];
        private int rowA,colB;
        /**
        * Constructs a new thread.
        */
        public DoubleMatrixMultiplyThread(DoubleMatrix a,DoubleMatrix b,int row,int col,double ans[][]) {
                matrixA=a;
                matrixB=b;
                rowA=row;
                colB=col;
                answer=ans;
        }
        /**
        * The body of the thread.
        */
        public void run() {
                int n=matrixA.columns();
                answer[rowA][colB]=0.0;
                for(int i=0;i<n;i++)
                        answer[rowA][colB]+=matrixA.getElement(rowA,i)*matrixB.getElement(i,colB);
        }
}
/**
* A thread that performs matrix multiplication.
* This class should not be used directly.
* @version 1.0
* @author Mark Hale
*/
final class ComplexMatrixMultiplyThread extends Thread {
        private ComplexMatrix matrixA,matrixB;
        private Complex answer[][];
        private int rowA,colB;
        /**
        * Constructs a new thread.
        */
        public ComplexMatrixMultiplyThread(ComplexMatrix a,ComplexMatrix b,int row,int col,Complex ans[][]) {
                matrixA=a;
                matrixB=b;
                rowA=row;
                colB=col;
                answer=ans;
        }
        /**
        * The body of the thread.
        */
        public void run() {
                int n=matrixA.columns();
                answer[rowA][colB]=Complex.ZERO;
                for(int i=0;i<n;i++)
                        answer[rowA][colB]=ComplexMath.add(answer[rowA][colB],ComplexMath.multiply(matrixA.getElement(rowA,i),matrixB.getElement(i,colB)));
        }
}

