package JSci.maths;

/**
* The vector math library.
* This class cannot be subclassed or instantiated because all methods are static.
* @version 1.5
* @author Mark Hale
*/
public final class VectorMath extends Object {
        private VectorMath() {}

// COMPLEX CONJUGATE

        /**
        * Returns the complex conjugate of a vector.
        * @param v a complex vector
        */
        public static ComplexVector conjugate(ComplexVector v) {
                int dim=v.dimension();
                Complex array[]=new Complex[dim];
                for(int i=0;i<dim;i++)
                        array[i]=ComplexMath.conjugate(v.getComponent(i));
                return new ComplexVector(array);
        }

// ADDITION

        /**
        * Returns the addition of two vectors.
        * @param a an integer vector
        * @param b an integer vector
        * @exception VectorDimensionException If the vectors are different sizes.
        */
        public static IntegerVector add(IntegerVector a, IntegerVector b)  throws VectorDimensionException {
                int aLength=a.dimension(),bLength=b.dimension();
                if(aLength==bLength) {
                        int array[]=new int[aLength];
// Create threads
                        for(int i=0;i<aLength;i++)
                                (new IntegerVectorAddThread(a,b,i,array)).start();
                        return new IntegerVector(array);
                } else
                        throw new VectorDimensionException("Vectors are different sizes.");
        }
        /**
        * Returns the addition of two vectors.
        * @param a a double vector
        * @param b a double vector
        * @exception VectorDimensionException If the vectors are different sizes.
        */
        public static DoubleVector add(DoubleVector a, DoubleVector b)  throws VectorDimensionException {
                int aLength=a.dimension(),bLength=b.dimension();
                if(aLength==bLength) {
                        double array[]=new double[aLength];
// Create threads
                        for(int i=0;i<aLength;i++)
                                (new DoubleVectorAddThread(a,b,i,array)).start();
                        return new DoubleVector(array);
                } else
                        throw new VectorDimensionException("Vectors are different sizes.");
        }
        /**
        * Returns the addition of two vectors.
        * @param a a complex vector
        * @param b a complex vector
        * @exception VectorDimensionException If the vectors are different sizes.
        */
        public static ComplexVector add(ComplexVector a, ComplexVector b)  throws VectorDimensionException {
                int aLength=a.dimension(),bLength=b.dimension();
                if(aLength==bLength) {
                        Complex array[]=new Complex[aLength];
// Create threads
                        for(int i=0;i<aLength;i++)
                                (new ComplexVectorAddThread(a,b,i,array)).start();
                        return new ComplexVector(array);
                } else
                        throw new VectorDimensionException("Vectors are different sizes.");
        }

// SUBTRACTION

        /**
        * Returns the subtraction of two vectors.
        * @param a an integer vector
        * @param b an integer vector
        * @exception VectorDimensionException If the vectors are different sizes.
        */
        public static IntegerVector subtract(IntegerVector a, IntegerVector b)  throws VectorDimensionException {
                int aLength=a.dimension(),bLength=b.dimension();
                if(aLength==bLength) {
                        int array[]=new int[aLength];
// Create threads
                        for(int i=0;i<aLength;i++)
                                (new IntegerVectorSubtractThread(a,b,i,array)).start();
                        return new IntegerVector(array);
                } else
                        throw new VectorDimensionException("Vectors are different sizes.");
        }
        /**
        * Returns the subtraction of two vectors.
        * @param a a double vector
        * @param b a double vector
        * @exception VectorDimensionException If the vectors are different sizes.
        */
        public static DoubleVector subtract(DoubleVector a, DoubleVector b)  throws VectorDimensionException {
                int aLength=a.dimension(),bLength=b.dimension();
                if(aLength==bLength) {
                        double array[]=new double[aLength];
// Create threads
                        for(int i=0;i<aLength;i++)
                                (new DoubleVectorSubtractThread(a,b,i,array)).start();
                        return new DoubleVector(array);
                } else
                        throw new VectorDimensionException("Vectors are different sizes.");
        }
        /**
        * Returns the subtraction of two vectors.
        * @param a a complex vector
        * @param b a complex vector
        * @exception VectorDimensionException If the vectors are different sizes.
        */
        public static ComplexVector subtract(ComplexVector a, ComplexVector b)  throws VectorDimensionException {
                int aLength=a.dimension(),bLength=b.dimension();
                if(aLength==bLength) {
                        Complex array[]=new Complex[aLength];
// Create threads
                        for(int i=0;i<aLength;i++)
                                (new ComplexVectorSubtractThread(a,b,i,array)).start();
                        return new ComplexVector(array);
                } else
                        throw new VectorDimensionException("Vectors are different sizes.");
        }

// SCALAR MULTIPLICATION

        /**
        * Returns the multiplication of a vector by a scalar.
        * @param x an integer
        * @param a an integer vector
        */
        public static IntegerVector scalarMultiply(int x, IntegerVector a) {
                int aLength=a.dimension();
                int array[]=new int[aLength];
                for(int i=0;i<aLength;i++)
                        array[i]=x*a.getComponent(i);
                return new IntegerVector(array);
        }
        /**
        * Returns the multiplication of a vector by a scalar.
        * @param x a double
        * @param a a double vector
        */
        public static DoubleVector scalarMultiply(double x, DoubleVector a) {
                int aLength=a.dimension();
                double array[]=new double[aLength];
                for(int i=0;i<aLength;i++)
                        array[i]=x*a.getComponent(i);
                return new DoubleVector(array);
        }
        /**
        * Returns the multiplication of a vector by a scalar.
        * @param x a complex number
        * @param a a complex vector
        */
        public static ComplexVector scalarMultiply(Complex x, ComplexVector a) {
                int aLength=a.dimension();
                Complex array[]=new Complex[aLength];
                for(int i=0;i<aLength;i++)
                        array[i]=ComplexMath.multiply(x, a.getComponent(i));
                return new ComplexVector(array);
        }

// SCALAR DIVISION

        /**
        * Returns the division of a vector by a scalar.
        * @param a an integer vector
        * @param x an integer
        */
        public static IntegerVector scalarDivide(IntegerVector a, int x) {
                int aLength=a.dimension();
                int array[]=new int[aLength];
                for(int i=0;i<aLength;i++)
                        array[i]=a.getComponent(i)/x;
                return new IntegerVector(array);
        }
        /**
        * Returns the division of a vector by a scalar.
        * @param a a double vector
        * @param x a double
        */
        public static DoubleVector scalarDivide(DoubleVector a, double x) {
                int aLength=a.dimension();
                double array[]=new double[aLength];
                for(int i=0;i<aLength;i++)
                        array[i]=a.getComponent(i)/x;
                return new DoubleVector(array);
        }
        /**
        * Returns the division of a vector by a scalar.
        * @param a a complex vector
        * @param x a complex number
        */
        public static ComplexVector scalarDivide(ComplexVector a, Complex x) {
                int aLength=a.dimension();
                Complex array[]=new Complex[aLength];
                for(int i=0;i<aLength;i++)
                        array[i]=ComplexMath.divide(a.getComponent(i), x);
                return new ComplexVector(array);
        }

// SCALAR PRODUCT

        /**
        * Returns the scalar product of two vectors.
        * @param a an integer vector
        * @param b an integer vector
        * @exception VectorDimensionException If the vectors are different sizes.
        */
        public static int scalarProduct(IntegerVector a, IntegerVector b) throws VectorDimensionException {
                int aLength=a.dimension();
                if(aLength==b.dimension()) {
                        int answer=0;
                        for(int i=0;i<aLength;i++)
                                answer+=a.getComponent(i)*b.getComponent(i);
                        return answer;
                } else
                        throw new VectorDimensionException("Vectors are different sizes.");
        }
        /**
        * Returns the scalar product of two vectors.
        * @param a a double vector
        * @param b a double vector
        * @exception VectorDimensionException If the vectors are different sizes.
        */
        public static double scalarProduct(DoubleVector a, DoubleVector b) throws VectorDimensionException {
                int aLength=a.dimension();
                if(aLength==b.dimension()) {
                        double answer=0.0;
                        for(int i=0;i<aLength;i++)
                                answer+=a.getComponent(i)*b.getComponent(i);
                        return answer;
                } else
                        throw new VectorDimensionException("Vectors are different sizes.");
        }
        /**
        * Returns the scalar product of two vectors.
        * @param a a complex vector
        * @param b a complex vector
        * @exception VectorDimensionException If the vectors are different sizes.
        */
        public static Complex scalarProduct(ComplexVector a, ComplexVector b) throws VectorDimensionException {
                int aLength=a.dimension();
                if(aLength==b.dimension()) {
                        Complex answer=Complex.ZERO;
                        for(int i=0;i<aLength;i++)
                                answer=ComplexMath.add(answer,ComplexMath.multiply(ComplexMath.conjugate(a.getComponent(i)), b.getComponent(i)));
                        return answer;
                } else
                        throw new VectorDimensionException("Vectors are different sizes.");
        }


// CROSS PRODUCT

        /**
        * Returns the cross product of two 3D vectors.
        * @param a an integer vector
        * @param b an integer vector
        * @exception VectorDimensionException If using incompatible vectors.
        */
        public static IntegerVector crossProduct(IntegerVector a, IntegerVector b) throws VectorDimensionException {
                if(a.dimension()==3 && b.dimension()==3)
                        return new IntegerVector(
                                a.getComponent(1)*b.getComponent(2)-b.getComponent(1)*a.getComponent(2) ,
                                a.getComponent(2)*b.getComponent(0)-b.getComponent(2)*a.getComponent(0) ,
                                a.getComponent(0)*b.getComponent(1)-b.getComponent(0)*a.getComponent(1)
                        );
                else
                        throw new VectorDimensionException("Incompatible vectors.");
        }
        /**
        * Returns the cross product of two 3D vectors.
        * @param a a double vector
        * @param b a double vector
        * @exception VectorDimensionException If using incompatible vectors.
        */
        public static DoubleVector crossProduct(DoubleVector a, DoubleVector b) throws VectorDimensionException {
                if(a.dimension()==3 && b.dimension()==3)
                        return new DoubleVector(
                                a.getComponent(1)*b.getComponent(2)-b.getComponent(1)*a.getComponent(2) ,
                                a.getComponent(2)*b.getComponent(0)-b.getComponent(2)*a.getComponent(0) ,
                                a.getComponent(0)*b.getComponent(1)-b.getComponent(0)*a.getComponent(1)
                        );
                else
                        throw new VectorDimensionException("Incompatible vectors.");
        }
        /**
        * Returns the cross product of two 3D vectors.
        * @param a a complex vector
        * @param b a complex vector
        * @exception VectorDimensionException If using incompatible vectors.
        */
        public static ComplexVector crossProduct(ComplexVector a, ComplexVector b) throws VectorDimensionException {
                if(a.dimension()==3 && b.dimension()==3)
                        return new ComplexVector(
                                ComplexMath.subtract(ComplexMath.multiply(a.getComponent(1),b.getComponent(2)), ComplexMath.multiply(b.getComponent(1), a.getComponent(2))) ,
                                ComplexMath.subtract(ComplexMath.multiply(a.getComponent(2), b.getComponent(0)), ComplexMath.multiply(b.getComponent(2), a.getComponent(0))) ,
                                ComplexMath.subtract(ComplexMath.multiply(a.getComponent(0), b.getComponent(1)), ComplexMath.multiply(b.getComponent(0), a.getComponent(1)))
                        );
                else
                        throw new VectorDimensionException("Incompatible vectors.");
        }

// MAP COMPONENTS

        /**
        * Applies a function on all the vector components.
        * @param v an integer vector
        * @param f a user-defined function
        * @return a double vector
        */
        public static DoubleVector mapComponents(IntegerVector v, Mapping f) {
                int vDim=v.dimension();
                double array[]=new double[vDim];
                for(int i=0;i<vDim;i++)
                        array[i]=f.map(v.getComponent(i));
                return new DoubleVector(array);
        }
        /**
        * Applies a function on all the vector components.
        * @param v a double vector
        * @param f a user-defined function
        * @return a double vector
        */
        public static DoubleVector mapComponents(DoubleVector v, Mapping f) {
                int vDim=v.dimension();
                double array[]=new double[vDim];
                for(int i=0;i<vDim;i++)
                        array[i]=f.map(v.getComponent(i));
                return new DoubleVector(array);
        }
        /**
        * Applies a function on all the vector components.
        * @param v a complex vector
        * @param f a user-defined function
        * @return a complex vector
        */
        public static ComplexVector mapComponents(ComplexVector v, Mapping f) {
                int vDim=v.dimension();
                Complex array[]=new Complex[vDim];
                for(int i=0;i<vDim;i++)
                        array[i]=f.map(v.getComponent(i));
                return new ComplexVector(array);
        }
}

//===================================================================
//                            THREAD CLASSES FOR VectorMath
//===================================================================

/**
* A thread that performs vector addition.
* This class should not be used directly.
* @version 1.0
* @author Mark Hale
*/
final class IntegerVectorAddThread extends Thread {
        private IntegerVector vectorA,vectorB;
        private int n,answer[];
        /**
        * Constructs a new thread.
        */
        public IntegerVectorAddThread(IntegerVector a,IntegerVector b,int col,int ans[]) {
                vectorA=a;
                vectorB=b;
                n=col;
                answer=ans;
        }
        /**
        * The body of the thread.
        */
        public void run() {
                answer[n]=vectorA.getComponent(n)+vectorB.getComponent(n);
        }
}
/**
* A thread that performs matrix addition.
* This class should not be used directly.
* @version 1.0
* @author Mark Hale
*/
final class DoubleVectorAddThread extends Thread {
        private DoubleVector vectorA,vectorB;
        private double answer[];
        private int n;
        /**
        * Constructs a new thread.
        */
        public DoubleVectorAddThread(DoubleVector a,DoubleVector b,int col,double ans[]) {
                vectorA=a;
                vectorB=b;
                n=col;
                answer=ans;
        }
        /**
        * The body of the thread.
        */
        public void run() {
                answer[n]=vectorA.getComponent(n)+vectorB.getComponent(n);
        }
}
/**
* A thread that performs matrix addition.
* This class should not be used directly.
* @version 1.0
* @author Mark Hale
*/
final class ComplexVectorAddThread extends Thread {
        private ComplexVector vectorA,vectorB;
        private Complex answer[];
        private int n;
        /**
        * Constructs a new thread.
        */
        public ComplexVectorAddThread(ComplexVector a,ComplexVector b,int col,Complex ans[]) {
                vectorA=a;
                vectorB=b;
                n=col;
                answer=ans;
        }
        /**
        * The body of the thread.
        */
        public void run() {
                answer[n]=ComplexMath.add(vectorA.getComponent(n),vectorB.getComponent(n));
        }
}
/**
* A thread that performs matrix subtraction.
* This class should not be used directly.
* @version 1.0
* @author Mark Hale
*/
final class IntegerVectorSubtractThread extends Thread {
        private IntegerVector vectorA,vectorB;
        private int n,answer[];
        /**
        * Constructs a new thread.
        */
        public IntegerVectorSubtractThread(IntegerVector a,IntegerVector b,int col,int ans[]) {
                vectorA=a;
                vectorB=b;
                n=col;
                answer=ans;
        }
        /**
        * The body of the thread.
        */
        public void run() {
                answer[n]=vectorA.getComponent(n)-vectorB.getComponent(n);
        }
}
/**
* A thread that performs matrix subtraction.
* This class should not be used directly.
* @version 1.0
* @author Mark Hale
*/
final class DoubleVectorSubtractThread extends Thread {
        private DoubleVector vectorA,vectorB;
        private double answer[];
        private int n;
        /**
        * Constructs a new thread.
        */
        public DoubleVectorSubtractThread(DoubleVector a,DoubleVector b,int col,double ans[]) {
                vectorA=a;
                vectorB=b;
                n=col;
                answer=ans;
        }
        /**
        * The body of the thread.
        */
        public void run() {
                answer[n]=vectorA.getComponent(n)-vectorB.getComponent(n);
        }
}
/**
* A thread that performs matrix subtraction.
* This class should not be used directly.
* @version 1.0
* @author Mark Hale
*/
final class ComplexVectorSubtractThread extends Thread {
        private ComplexVector vectorA,vectorB;
        private Complex answer[];
        private int n;
        /**
        * Constructs a new thread.
        */
        public ComplexVectorSubtractThread(ComplexVector a,ComplexVector b,int col,Complex ans[]) {
                vectorA=a;
                vectorB=b;
                n=col;
                answer=ans;
        }
        /**
        * The body of the thread.
        */
        public void run() {
                answer[n]=ComplexMath.subtract(vectorA.getComponent(n),vectorB.getComponent(n));
        }
}

