//Program attract.java (c) 1996 by J. C. Sprott <sprott@juno.physics.wisc.edu>
//A Java applet that searches for strange attractors and displays them

import java.awt.Graphics;
import java.awt.Color;
import java.awt.Image;
import java.awt.Font;
import java.awt.FontMetrics;
import java.util.Date;
import java.util.Random;

public class attract extends java.applet.Applet
    implements Runnable {

    Thread runner;
    int w, h, nc, bc, dt;
    String s;
    Color colors[] = new Color[256];
    Image offscreenImg;
    Graphics offscreenG;
    Date theDate;
    Random rnd;

    public String getAppletInfo() {
        return "Strange Attractor Search Applet (c) 1996 J. C. Sprott";
    }

    public void init() {
        s=getParameter("width");            //image width in pixels
        if (s==null) w=320; else w=Integer.parseInt(s);
        s=getParameter("height");           //image height in pixels
        if (s==null) h=200; else h=Integer.parseInt(s);
        s=getParameter("numcolors");        //number of colors (0-256)
        if (s==null) nc=256; else nc=Integer.parseInt(s);
        if (nc>256) nc=256;
        s=getParameter("backgroundcolor");  //24-bit decimal, neg=transparent
        if (s==null) bc=-1; else bc=Integer.parseInt(s);
        s=getParameter("time");             //minimum time between cases (ms)
        if (s==null) dt=5000; else dt=Integer.parseInt(s);
        offscreenImg=createImage(this.size().width,this.size().height);
        offscreenG=offscreenImg.getGraphics();
    }

    public void start() {
        if (runner==null) {
            runner=new Thread(this);
            runner.start();
        }
    }

    public void stop() {
        if (runner!=null) {
            runner.stop();
            runner=null;
        }
    }

    public void run() {
        if (nc>0) {
            for (int i=0; i<nc; i++) {
                float c=(float)i/(float)nc;
                colors[i]=Color.getHSBColor(c,1.0f,1.0f);
            }
        }
        rnd=new Random();
        try {repaint();}
        catch(Throwable t) {}
    }

    public void paint(Graphics g) {
        double x=0, y=0, z=0, xmin=0, xmax=0, xe=0, ye=0, xnew, ynew, xenew;
        double dx=0, dh=0, dw=0, dz=0, ran=0;
        double a[]={0,0,0,0,0,0};
        int n=0, xp, yp, zp;
        long t0=0;
        boolean le=false;
        Font f = new Font("TimesRoman",Font.PLAIN,24);
        FontMetrics fm=getFontMetrics(f);
        g.setFont(f);
        s="Searching - Please wait...";
        xp=(this.size().width-fm.stringWidth(s))/2;
        yp=(this.size().height+fm.getHeight())/2;
        if (xp>0) g.drawString(s,xp,yp);
        while (true) {
            if (n==0) {
                for (int i=0; i<6; i++) {
                    ran=rnd.nextFloat();
                    a[i]=6.0*ran-3.0;
                }
                x=0;
                y=0;
                xe=x+1e-10;
                ye=y;
                xmin=1e6;
                xmax=-xmin;
                le=false;
            }
            else if (n>=1 && n<=199) {
                if (x<xmin) xmin=x;
                if (x>xmax) xmax=x;
                xenew=a[0]+xe*(a[1]+a[2]*xe+a[3]*ye)+ye*(a[4]+a[5]*ye);
                ye=xe;
                xe=xenew;
                if (Math.abs(xe-x)>.001) le=true;
            }
            else if (n==200) {
                if (!le) n=-1;
            }
            else if (n==201) {   
                dx=.1*(xmax-xmin);
                if (dx==0) dx=1;
                xmax=xmax+dx;
                xmin=xmin-dx;
                dw=w/(xmax-xmin);
                dh=h/(xmax-xmin);
                dz=nc/(xmax-xmin);
                if (bc<0) {
                    offscreenG.setColor(getBackground());
                }
                else {
                    offscreenG.setColor(new Color(bc));
                }
                offscreenG.fillRect(0,0,this.size().width,this.size().height);
                if (nc==0) offscreenG.setColor(Color.black);
                theDate=new Date();
                t0=theDate.getTime();
            }
            else {
                xp=(int)(dw*(x-xmin));
                yp=(int)(dh*(xmax-y));
                if (nc>0) {
                    zp=(int)(dz*(z-xmin))%nc;
                    if (zp<0) zp=0;
                    offscreenG.setColor(colors[zp]);
                }
                offscreenG.drawLine(xp,yp,xp,yp);
            }
            n=n+1;
            if (n%200==0) {
                theDate=new Date();
                if (n>=2000 && theDate.getTime()>t0+dt) {
                    g.drawImage(offscreenImg,0,0,this);
                    n=0;
                }
            }
            xnew=a[0]+x*(a[1]+a[2]*x+a[3]*y)+y*(a[4]+a[5]*y);
            ynew=x;
            z=y;
            y=ynew;
            x=xnew;
            if (Math.abs(x)>1e4) n=0;
        }
    }

}
