#! /usr/local/bin/perl -w

#############################################################################
#  
#   File:    database.pl
#
#   Input:   data from the HTML form database0.html
#
#   Output:  a Web page containing the search results
#
#   From the HTML page database0.html, the user may choose to list the
#   complete database or search the database for a specific search string.
#   (Note:  Regular expressions are accepted in the search string.)
#
#   A typical entry in the database file looks like:
#
#     <record>
#     name: Tom Scavo; email: trscavo@npac.syr.edu;
#     platform: PowerMac; TeXapp: Textures;
#     </record>
#
#   that is, key-value pairs are separated by a colon (:) while
#   fields are terminated by semicolons (;).
#
#############################################################################
#  
#   Initialization
#
#############################################################################

$EOL = "\015\012";
$DBFILE = "register.db";       # flat-file database
$HTMLFILE = "database0.html";  # HTML doc with form
$HOSTPATH = "http://www.npac.syr.edu/projects/tutorials/CGI/examples/basic";

#############################################################################
#  
#   Library Routines
#
#############################################################################

require "ReadParse.pl" || die "error loading ReadParse.pl\n";

#############################################################################
#  
#   Main Routines
#
#############################################################################

ReadData() &&
ProcessData();

#############################################################################
#  
#   Input
#
#############################################################################

sub ReadData {
  # ReadParse returns an associative array of key-value pairs:
  %IN = ReadParse();
  return 1;
}

#############################################################################
#  
#   Processing
#
#############################################################################

# On the HTML form, there are submit buttons called 'browse' and 'search'.
# This routine determines which button was pressed and acts accordingly.
# If the user presses the 'search' button, but forgets to type a search
# string, this routine simply reloads the HTML page.
sub ProcessData {
  if ( defined($IN{'browse'}) ) {
    BrowseDatabase();
  } elsif ( defined($IN{'search'}) ) {
    if ( defined($IN{'searchString'}) ) {
      $SEARCHSTRING = $IN{'searchString'};
      SearchDatabase();
    } else {
      print "Location: $HOSTPATH/$HTMLFILE", "$EOL$EOL";
    }
  } else {
    die "Error in database.pl\n";
  }
  return 1;
}

# Construct an HTML page that displays the entire database:
sub BrowseDatabase {
  my ($record, $name, $email, $platform, $TeXapp, %foo);
  
  # Read the database and construct the body of an ordered list:
  open (DB, "$DBFILE") || die "Unable to open $DBFILE\n";
  # Obtain a shared lock:
#   flock (DB, 1) || die "Unable to lock $DBFILE\n";
  $BODY = "";
  while ( $record = <DB> ) {
    chomp($record);
    if ( $record eq '<record>' ) {
      # pre-process this record:
      # (no pre-processing required)
    } elsif ( $record eq '</record>' ) {
      # post-process this record:
      $name = $foo{'name'};
      $email = $foo{'email'};
      $platform = $foo{'platform'};
      $TeXapp = $foo{'TeXapp'};
      $BODY .= '  <LI>';
      $BODY .= "$name  $EOL  &lt;<A HREF=\"mailto:$email\">$email</A>&gt; <BR>$EOL";
      $BODY .= "  platform = $platform; TeXapp = $TeXapp";
      $BODY .= " <P>$EOL";
    } else {
      # process this record:
      while ( $record =~ /(\w*): ([^;]*);/ ) {
        $foo{$1} = $2;
        $record = $';
      }
    }
  }
  # Release the lock:
#   flock (DB, 8) || die "Unable to unlock $DBFILE\n";
  close (DB) || die "Unable to close $DBFILE\n";

  if ( $BODY eq "" ) {
    $BODY = '  No records in database!';
  }
  
  PrintDatabase();
  return 1;
}

# Construct an HTML page that displays a portion of the database:
sub SearchDatabase {
  my ($record, $name, $email, $platform, $TeXapp, %foo);
  my ($isMatch, $noMatch);

  $noMatch = 1;

  # Read the database and construct the body of an ordered list:
  open (DB, "$DBFILE") || die "Unable to open $DBFILE\n";
  # Obtain a shared lock:
#   flock (DB, 1) || die "Unable to lock $DBFILE\n";
  $BODY = "";
  while ( $record = <DB> ) {
    chomp($record);
    if ( $record eq '<record>' ) {
      # pre-process this record
    } elsif ( $record eq '</record>' ) {
      # post-process this record:
      $isMatch = 0;
      foreach $key ( keys(%foo) ) {
        if ( $foo{$key} =~ /$SEARCHSTRING/i ) {
          $isMatch = 1;
          last;
        }
      }
      if ( $isMatch ) {
        $noMatch = 0;
        $name = $foo{'name'};
        $email = $foo{'email'};
        $platform = $foo{'platform'};
        $TeXapp = $foo{'TeXapp'};
        $BODY .= '  <LI>';
        $BODY .= "$name  $EOL  &lt;<A HREF=\"mailto:$email\">$email</A>&gt; <BR>$EOL";
        $BODY .= "  platform = $platform; TeXapp = $TeXapp";
        $BODY .= " <P>$EOL";
      }
    } else {
      # process this record:
      while ( $record =~ /(\w*): ([^;]*);/ ) {
        $foo{$1} = $2;
        $record = $';
      }
    }
  }
  # Release the lock:
#   flock (DB, 8) || die "Unable to unlock $DBFILE\n";
  close (DB) || die "Unable to close $DBFILE\n";
  
  if ( $noMatch ) {
    $BODY = '  No matches found!';
  }

  PrintSearchResults();
  return 1;
}

#############################################################################
#  
#   Print HTML pages
#
#############################################################################

sub PrintDatabase {
  print qq{Content-type: text/html$EOL$EOL
<HTML>
<HEAD>
<TITLE>Alpha LaTeX Database</TITLE>
</HEAD>
<BODY BGCOLOR="#FFFFFF">

<H1>Alpha LaTeX Registered Users</H1>

<OL>
$BODY
</OL>

<P><HR><P>

Return to
<A HREF="$HOSTPATH/$HTMLFILE">HTML form</A>

</BODY>
</HTML>
};

}  # end PrintDatabase

sub PrintSearchResults {
  print qq{Content-type: text/html$EOL$EOL
<HTML>
<HEAD>
<TITLE>Alpha LaTeX Database</TITLE>
</HEAD>
<BODY BGCOLOR="#FFFFFF">

<H1>Alpha LaTeX Registered Users</H1>

searchString = $SEARCHSTRING

<OL>
$BODY
</OL>

<P><HR><P>

Return to
<A HREF="$HOSTPATH/$HTMLFILE">HTML form</A>

</BODY>
</HTML>
};

}  # end PrintSearchResults
