#! /usr/local/bin/perl -w

#############################################################################
#  
#   File:    register.pl
#
#   Input:   data from the HTML form generated by validate.pl
#
#   Output:  depends on the input data (see below)
#
#   This script processes the HTML form generated by validate.pl.
#   If the user presses the "Yes" button, the script writes an entry
#   in the database and loads a courtesy HTML page.  (The production
#   version of this script would load a more detailed page with links
#   to download the requested software.)  If the user presses the "No"
#   button, the script reloads the original form register0.html.
#
#   A typical entry in the database file looks like:
#
#     <record>
#     name: Tom Scavo; email: trscavo@npac.syr.edu;
#     platform: PowerMac; TeXapp: Textures;
#     </record>
#
#   that is, key-value pairs are separated by a colon (:) while
#   fields are separated by semicolons (;).
#
#############################################################################
#  
#   Initialization
#
#############################################################################

$EOL = "\015\012";
$DBFILE = "register.db";
$HOSTPATH = "http://www.npac.syr.edu/projects/tutorials/CGI/examples/basic";

#############################################################################
#  
#   Library Routines
#
#############################################################################

require "ReadParse.pl" || die "error loading ReadParse.pl\n";

#############################################################################
#  
#   Main Routines
#
#############################################################################

ReadData();
if ( $INSTALL eq "Yes" ) {
  ProcessData(); 
} elsif ( $INSTALL eq "No" ) {
  ReloadForm();
} else {
  # This can't happen!
}

#############################################################################
#  
#   Input
#
#############################################################################

sub ReadData {
  my (%in);
  # ReadParse returns an associative array of key-value pairs:
  %in = ReadParse();
  $NAME = $in{'name'};
  $EMAIL = $in{'email'};
  $PLATFORM = $in{'platform'};
  $TEXAPP = $in{'TeXapp'};
  $OTHERAPP = $in{'otherApp'};
  $INSTALL = $in{'install'};
  return 1;
}

#############################################################################
#  
#   Subroutines
#
#############################################################################

# Print a dummy HTML page (the production version of this page 
# would provide links to the *requested* pieces of software):
sub ProcessData {
  
  # Update the database:
  UpdateDBFile();
  
  # Print an HTML page:
  SendConfirm();
    
  return 1;
}

# Update the database with the input data:
sub UpdateDBFile {
  open (DB, ">>$DBFILE") || die "Unable to open $DBFILE\n";
  # Obtain an exclusive lock:
  flock (DB, 2) || die "Unable to lock $DBFILE\n";
  print DB "<record>\n";
  print DB "name: $NAME; email: $EMAIL;\n";
  print DB "platform: $PLATFORM; "; 
  if ( $TEXAPP eq "other" ) {
    print DB "TeXapp: $OTHERAPP;\n";
  } else {
    print DB "TeXapp: $TEXAPP;\n";
  }
  print DB "</record>\n";
  # Release the lock:
  flock (DB, 8) || die "Unable to unlock $DBFILE\n";
  close (DB) || die "Unable to close $DBFILE\n";
    
  return 1;
}

# Reload the original HTML form:
sub ReloadForm {
  print "Location: $HOSTPATH/register0.html$EOL$EOL";    
  return 1;
}

sub SendConfirm {
  print qq{Content-type: text/html$EOL$EOL
<HTML>
<HEAD>
<TITLE>Download Alpha LaTeX</TITLE>
</HEAD>
<BODY BGCOLOR="#FFFFFF">

Thank you for registering your copy of Alpha LaTeX!

<P><HR><P>

Return to
<A HREF="$HOSTPATH/register0.html">HTML form</A>

</BODY>
</HTML>
};

}  # end SendConfirm