#! /usr/local/bin/perl -w

#############################################################################
#  
#   File:    validate.pl
#
#   Input:   data from the HTML form register0.html
#
#   Output:  a confirmation page containing the input data *or*
#            a page of error messages
#
#   From the HTML page register0.html, the user provides information
#   that registers him or her for a software package.  This script
#   validates the input data, returns a page of error messages if
#   errors are detected, or asks for confirmation of the input data.
#
#############################################################################
#  
#   Initialization
#
#############################################################################

$EOL = "\015\012";
$SCRIPT = 
  "http://osprey7.npac.syr.edu:3768/users-cgi/trscavo/tutorials/CGI/examples/basic/register.pl";
$HOSTPATH = "http://www.npac.syr.edu/projects/tutorials/CGI/examples/basic";

#############################################################################
#  
#   Library Routines
#
#############################################################################

require "ReadParse.pl" || die "error loading ReadParse.pl\n";

#############################################################################
#  
#   Main Routines
#
#############################################################################

ReadData();
if ( ValidData() ) {
  ConfirmReg();
} else {
  PrintErrors();
}

#############################################################################
#  
#   Subroutines
#
#############################################################################

sub ReadData {
  my (%in);
  # ReadParse returns an associative array of key-value pairs:
  %in = ReadParse();
  $NAME = $in{'name'};
  $EMAIL = $in{'email'};
  $PLATFORM = $in{'platform'};
  $TEXAPP = $in{'TeXapp'};
  $OTHERAPP = $in{'otherApp'};
  return 1;
}

# Validate the input data and accumulate the error messages:
sub ValidData {
  my ($isOkay);
  
  $isOkay = 1;
  $MSGS = "";
  if ( $NAME eq "" ) {
    $MSGS .= "error:  name omitted <P>$EOL";
    $isOkay = 0;
  }
  if ( $EMAIL eq "" ) {
    $MSGS .= "error:  e-mail omitted <P>$EOL";
    $isOkay = 0;
  }
  if ( $TEXAPP eq "other" && $OTHERAPP eq "" ) {
    $MSGS .= "error:  \"other\" TeX app omitted <P>$EOL";
    $isOkay = 0;
  }
  
  return $isOkay;
}

# No errors in the input data were found, so print an HTML page
# and request confirmation from the user:
sub ConfirmReg {
  my ($app);
  
  if ( $TEXAPP eq "other" ) {
    $app = $OTHERAPP;
  } else {
    $app = $TEXAPP;
  }
  
  print qq{Content-type: text/html$EOL$EOL
<HTML>
<HEAD>
<TITLE>Confirmation Request</TITLE>
</HEAD>
<BODY BGCOLOR="#FFFFFF">

<H1>Input Data</H1>

<UL>
  <LI>name = $NAME
  <LI>e-mail = $EMAIL
  <LI>platform = $PLATFORM
  <LI>TeX app = $app
</UL>

<FORM METHOD=POST ACTION="$SCRIPT">
  <INPUT TYPE=hidden NAME=name VALUE="$NAME">
  <INPUT TYPE=hidden NAME=email VALUE="$EMAIL">
  <INPUT TYPE=hidden NAME=platform VALUE="$PLATFORM">
  <INPUT TYPE=hidden NAME=TeXapp VALUE="$TEXAPP">
  <INPUT TYPE=hidden NAME=otherApp VALUE="$OTHERAPP">
  Is this correct? <P>
  <INPUT TYPE=submit NAME=install VALUE=Yes>
  <INPUT TYPE=submit NAME=install VALUE=No>
</FORM>

</BODY>
</HTML>
};
  
}  # end ConfirmReg

# Print a page of error messages and let the user reload the HTML form:
sub PrintErrors {
  print qq{Content-type: text/html$EOL$EOL
<HTML>
<HEAD>
<TITLE>Error Report</TITLE>
</HEAD>
<BODY BGCOLOR="#FFFFFF">

<H1>Input errors detected</H1>

$MSGS

<P><HR><P>

<A HREF="$HOSTPATH/register0.html"><EM>Try again!</EM></A>

</BODY>
</HTML>
};
  
}  # end PrintErrors
