#! /usr/npac/bin/perl -w

#############################################################################
#  
#   File:        fill-out-form.pl
#
#   Output:      CGI.pm version of pizza order form
#
#   Description: Creates a pizza order from the information obtained from
#                a pizza order form.  Both the order form and the order
#                are created by this script (i.e., all HTML is generated
#                on-the-fly)
#
#
#############################################################################
#  
#   Initialization
#
#############################################################################

$HOSTPATH = 'http://osprey7.npac.syr.edu:3768/users-cgi/trscavo/tutorials/CGI/examples/more-cgi';

#############################################################################
#  
#   Modules
#
#############################################################################

# Import a standard set of methods and the HTML shortcuts
# from CGI.pm (see page 187 of the Llama book for details):
use CGI qw(:standard :shortcuts);

#############################################################################
#  
#   Main Routine
#
#############################################################################

if ( param() ) {
  ReadInput() && PrintOrder();
} else {
  PrintForm();
}

##############################################################################
#
#  Input
#
##############################################################################

sub ReadInput {
  $ADDRESS = param('address');
  $PHONE = param('phone');
  $TOPPING = param('topping');
  return 1;
}

#############################################################################
#  
#   Output
#
#############################################################################

sub PrintForm {

  # Print MIME header:
  print header(-type=>'text/html');
  
  # Print first few lines of the HTML document:
  print start_html(
          -title=>'CGI Example: Fill-out Form',
          -BGCOLOR=>'white'
        ), "\n";
  
  # Print a heading:
  print h2('Internet Pizza Delivery Service'), "\n";
  
  # Print <FORM> tag (the arguments below are the defaults
  # and given only for clarity):
  print start_form(
          -method=>'POST',
          -action=>"$HOSTPATH/fill-out-form.pl"
        ), "\n";
                
  # Print a textfield for the user's street address:
  print p( "Type in your street address:\n",
           textfield(
             -name=>'address',
             -size=>36
           )
        ), "\n";
        
  # Print a textfield for the user's phone number:
  print p( "Type in your phone number:\n",
           textfield(
             -name=>'phone',
             -size=>20
           )
        ), "\n";
        
  # Prompt for pizza:
  print p( 'What kind of pizza would you like?  Check one:' ), "\n";
  
  # This form of blockquote requires the CGI shortcuts:
  print blockquote(
          radio_group(
            -name=>'topping',
            -values=>['Pepperoni','Sausage','Anchovy'],
            -linebreak=>'true'
          )
        ), "\n";
  
  # Print submit and reset buttons:
  print p( 'To place your order, click here:',
           submit('Order Pizza'),
           reset('Clear')
        ), "\n";
  
  # Print </FORM>:
  print end_form(), "\n";
  
  # Print last few lines of HTML document:
  print end_html(), "\n";

}

sub PrintOrder {
  
  # Print MIME header:
  print header(-type=>'text/html');
  
  # Print first few lines of the HTML document:
  print start_html(
          -title=>'CGI Example:  Pizza',
          -BGCOLOR=>'#ffffff'
        ), "\n";
  
  # Print some headings:
  print h1( 'Pizza Order' ), "\n";
  print h3( "$TOPPING pizza" ), "\n";
  
  # Print some paragraphs:
  print p( "Deliver to: <B>$ADDRESS</B>" ), "\n";
  print p( "Telephone: <B>$PHONE</B>" ), "\n";
  my $date = `date`; chomp($date);
  print p( "Order came in at $date" ), "\n";
  
  # Print a horizontal rule:
  print hr();
  
  # Print a link:
  print 'Return to ';
  print a({href=>"$HOSTPATH/fill-out-form.pl"},
          'HTML form'
        ), "\n";
  
  # Print last few lines of HTML document:
  print end_html(), "\n";

}
