/*
 * ImageFrame.java
 */

import java.awt.*;

/* Image Frame Object */
class ImageFrame extends Frame {
    VisibleHuman root;
    ImageFrame prev, next;
    String path;
    Panel3D controls;
    Picture picture;
    Panel3D lbScale;
    Scroller sbScale;
    Button3D Quit, Download, Apply, Org;
    int step;
    boolean canDownload;
    static int depth = Panel3D.depth;

    /** Constructor */
    public ImageFrame(VisibleHuman _root,
        String Title, String _path, Image _img,
        int x, int y, int w, int h,
        double scale, int xScale, int yScale,
        int maxp, int _step) {
        super(Title);

        /* Setup Line Structure */
        root = _root;
        next = root.frameList;
        root.frameList = this;
        if ( next != null ) next.prev = this;
        prev = null;

        /* Setup Parameters */
        path = _path;
        reshape(x, y, w, h);
        step = _step;
        canDownload = (xScale == yScale);

        /* Setup Components */
        picture = new Picture();
        picture.setScale(scale);
        picture.setScale(xScale, yScale);
        controls = new Panel3D(0);
        controls.add(Quit = new Button3D("Quit"));
        controls.add(Download = new Button3D("Download", false));
        controls.add(Apply =
          new Button3D(Button3D.CHECKBTN, "Apply", false));
        controls.add(Org =
          new Button3D(Button3D.CHECKBTN, "1:1", scale != 1.0));
        controls.add(lbScale =
          new Panel3D(String.valueOf((int)(scale*100+0.5))+"%", 0));
        controls.add(sbScale = new Scroller(Scrollbar.HORIZONTAL));
        sbScale.setValue((int)Math.round((Math.log(scale)/Math.log(2.0)*step)),
          1, -step*maxp, step*maxp+1, false);
        add("Center", picture);
        add("South", controls);

        /* Display Image Frame */
        show();
        picture.updateImage(_img);
    }

    /** Get Current Image Scale */
    public double getScale()
    {   return Math.pow(2.0, ((double)sbScale.getValue())/step);}

    /** Destroy Image Frame */
    public void dispose() {
        root.action(null, this);
        if ( prev != null ) prev.next = next;
        else root.frameList = next;
        if ( next != null ) next.prev = prev;
        super.dispose();
    }

    /** Component Event Action */
    public boolean action(Event ev, Object arg) {
        if ( arg == Quit ) {
          dispose();
          return true;
        } else if ( arg == Download ) {
          root.action(null, path);
          return true;
        } else if ( arg == Apply ) {
          picture.setScale(getScale());
          Apply.setActive(false);
          Org.setActive(picture.getScale() != 1.0);
          return true;
        } else if ( arg == Org ) {
          sbScale.setValue(0, true);
          action(ev, Apply);
          action(ev, sbScale);
          return true;
        } else if ( arg == sbScale ) {
          double v = getScale();
          lbScale.setText(String.valueOf((int)(v*100+0.5))+"%");
          Apply.setActive(picture.getScale() != v);
          return true;
        } else if ( arg == picture ) {
          if ( canDownload )
            Download.setActive(true);
          root.action(ev, this);
          return true;

        }
        return true;
    }
}
    
