/**
 * Experiment's to contrast cloning vis-a-vis Referencing 
 * @author: Kemal, Mahesh, Meryem and Shrideep.
 * @version CIS600 1.0
 * @date    2nd Oct 1998.
 */

public class customClone{

  /**
   * Creates a clone of the object. A new instance is allocated and a 
   * bitwise clone of the current object is place in the new object.
   */  
  public void cloneWidget(Widget _a) {
    try {
      Widget b = (Widget) _a.clone();
      printWidgetInfo(_a, b, "Cloned   ");
     } catch (java.lang.CloneNotSupportedException e) {
       System.out.println("Cloning not supported");
       e.printStackTrace();
    }
  }

  /**
   * A method which contrasts cloning and referencing
   */
  public void contrastCloningAndReferencing(Widget _a) {
    System.out.println();
    System.out.println("Contrasting Cloning & Referencing ");

    try {
       Widget b = (Widget) _a.clone();
       modifyWidget(b);
       System.out.println("CLONED RESULTS ===>");
       printWidgetInfo(_a, b, "Cloned   ");
    } catch (java.lang.CloneNotSupportedException e) {
      System.out.println("Cloning not supported");
      e.printStackTrace();
    }

    System.out.println("REFERENCED RESULTS ===>");
    Widget b = _a;
    modifyWidget(b);
    
    printWidgetInfo(_a, b, "Reference");
  }
  
  /**
   * A utility method which is used for printing the widgets' info.
   */
  private void printWidgetInfo(Widget a, Widget b, String style) {
    System.out.println( "Original   Widget INFO =>> Name =" + a.getName() +
			" Age = " + a.getAge());
    System.out.println( style + "  Widget INFO =>> Name =" + b.getName() +
			" Age = " + b.getAge());
  }
  
  /** 
   * As can be seen, the changes to the widget have been hardcoded to
   * make the results more clear for the clone/reference case.
   */

  private void modifyWidget(Widget x) {
    x.setAge(44);
    x.setName("Fred");
  }

  public static void main(String[] args) {
    customClone cloner = new customClone();

    /** Two Widgets with different constructors */
    Widget a, b;

    a = new Widget();
    cloner.cloneWidget(a);
    System.out.println("      ------------------------------     ");

    b = new Widget("Jane", 25);
    cloner.cloneWidget(b);
    System.out.println("      ------------------------------     ");

    cloner.contrastCloningAndReferencing(a);
  }
}

/**
  Widget needs to be a sub-class of customClone, since we are invoking a
  protected method clone() from customClone() 
  
  The Cloneable Interface is an interface with no methods, but what it does
  is announce to the Java Virtual Machine that this class is willing to be 
  cloned.
  */

class Widget extends customClone implements Cloneable {
  private String name ="Jack";
  private int age = 26;
  
  public Widget() {
    System.out.println("The NULL constructor");
  }
  
  public Widget(String _name, int _age) {
    name = _name;
    age = _age;
    System.out.println("The 2 argument promotion constructor");
  }

  public void setName(String _name) {
    name = _name;
  }
  public String getName() {
    return name;
  }
    
  public void setAge(int _age) {
    age = _age;
  }
  public int getAge() {
    return age;
  }

  protected Object clone() throws CloneNotSupportedException{
    Object a = super.clone();
    if (a ==null) {
      throw new CloneNotSupportedException();
    }
    ((Widget)a).setAge(0); 
    return (Object)a;
    //   return (Object)new Widget("Dude", 0);
  }
}










