package sunw.demo.ImageEditor;

import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.net.URL;
import java.beans.*;
import gjt.filter.*;

/**
 *  ImageEditor.java
 *  <p>
 */
public class ImageEditor extends Applet {

  private int lastXdim, lastYdim;
  private PropertyChangeSupport changes = new PropertyChangeSupport(this);
  
  private transient Image filterImage;
  private transient int indicator = 2;
  private transient String imageLocation;
  private transient String lastImageLocation;
  private String filterName="Bleach";
  
  private int  nimgw, nimgh;
  private int npixels[];
  
  /*
   * Applet methods:
   * Enables our bean to run as an applet as well.
   */
  
  public void init() {
    resize(32,32);
    filterImage = grabImage("sunw/demo/ImageEditor/faces.gif");
    lastImageLocation = imageLocation;
    lastXdim = this.getSize().width;
    lastYdim = this.getSize().height;
  }
  
  /**
   * Update simply calls Paint for better buffering
   * @see #paint
   */
  public void update (Graphics g) {
    paint(g);
  }
  
  /**
   * Check if Image is null before painting
   */
  public void paint(Graphics g) {
    if(filterImage != null)
      g.drawImage(filterImage, 0, 0, 
		  this.getSize().width, this.getSize().height, this);
  }
  
  /*
   * End Applet methods and begin Java Bean related methods
   */
  
  
  /** 
   * Returns the string associated with the current image URL
   */
  public synchronized String getImageLocation() {
    return imageLocation;
  }
  
  /** 
   * Sets a new location from which the image should be fetched
   */
  public void setImageLocation(String _location) {
    String oldImageLocation = imageLocation;
    imageLocation = _location;
    changes.firePropertyChange("New Location", oldImageLocation, 
			       imageLocation);
  }
  
  /** 
   * Returns the current new Image indicator
   */
  public synchronized int getIndicator() {
    return indicator;
  }
  
  /**
   * A change in our imageIndicator triggers an update to filterImage
   */
  public void setIndicator(int Indicator) {
    int oldIndicator = indicator;
    indicator = Indicator;
    filterImage = grabImage(imageLocation);
    repaint();
    changes.firePropertyChange("ImageIndicator", 
			       new Integer(oldIndicator),
			       new Integer(indicator));
  }
  
  /**
   * Adds a PropertyListener object to a list of PropertyChangeListeners
   */	  
  public void addPropertyChangeListener(PropertyChangeListener _listener) {
    changes.addPropertyChangeListener(_listener);
  }
  
  /** 
   * Removes a PropertyChangeListener from our listener list.
   */	  
  public void removePropertyChangeListener(PropertyChangeListener _listener) {
    changes.removePropertyChangeListener(_listener);
  }
  
  /* --------------end Java Bean-related methods --------------- */
  
  /*
   * Calculate the Image and then repaint it to the offscreen buffer
   */
  public void updateCanvas() {
    filterImage= createImage(new MemoryImageSource(nimgw, nimgh, 
						npixels, 0, nimgw)); 
    repaint();
  }

  /*
   * Gather the pixels and information regarding the width and height
   * of the Image. We employ the Mediatracker and the PixelGrabber for
   * reasons mentioned earlier
   */
   private Image grabImage(String name) {
    try {
      MediaTracker tracker = new MediaTracker(this);
      URL url = new URL(getCodeBase(),name);
      
      Image img = getImage(url);
      tracker.addImage(img, 0);
      
      try { 
	tracker.waitForID(0); 
      } catch (InterruptedException e){};
      
      imageLocation = url.toString();
      nimgw=img.getWidth(this); 
      nimgh=img.getHeight(this); 
      npixels = new int[nimgw*nimgh];
      PixelGrabber pg = new PixelGrabber(img, 0, 0, nimgw,
					 nimgh, npixels, 0, nimgw); 
      try { 
	pg.grabPixels(); 
      } catch (InterruptedException e){};     
      return img;
    } catch (Exception ex) {
      System.out.println("ImageEditor:: Failed to load default image");
      return null;
    }
   }
  
  /*
   * Image Editing - Utility Methods 
   */
  public void flipImage() {
    flipFilter flip = new flipFilter();
    npixels = flip.filter(npixels, nimgw, nimgh);
    updateCanvas();
  }
  
  public void flopImage() {
    flopFilter flop;
    flop = new flopFilter();
    npixels= flop.filter(npixels, nimgw,nimgh);
    updateCanvas();
  }
  
  public void negativeImage() {
    negFilter neg= new negFilter();
    npixels= neg.filter(npixels, nimgw,nimgh);
    updateCanvas();
  }
  
  public void grayImage() {
    grayFilter gray = new grayFilter();
    npixels= gray.filter(npixels, nimgw,nimgh);
    updateCanvas();
  }
  
  public void bleachImage() {
    bleachFilter bleach = new bleachFilter();
    npixels= bleach.filter(npixels, nimgw,nimgh);
    updateCanvas();
  }

  /*
   * The various options available and the methods they get hooked to
   */
  
  public String getFilterName() {
    return filterName;
  }
  
  public void setFilterName(String name) {
    filterName = name;
    if ("FlipFilter".equals(name)) flipImage();
    if ("FlopFilter".equals(name))  flopImage();
    if ("Negative".equals(name))  negativeImage();
    if ("Gray".equals(name))  grayImage();
    if ("Bleach".equals(name))  bleachImage();
  }
}







