/* 
 *	Copyright (c) 1997-98 
 *	NorthEast Parallel Architectures Center, Syracuse University. 
 * All Rights Reserved. Permission to use, copy, modify, and distribute this
 * software and its documentation for educational, research and non-profit
 * purposes, without fee, and without a written agreement is hereby granted,
 * provided that the above copyright notice and this paragraph appear in all
 * copies. Permission to incorporate this software into commercial products may
 * be obtained by contacting the NorthEast Parallel Architectures Center. 
 *
 * The software program and documentation are supplied "as is",
 * without any accompanying services from NPAC. NPAC does not
 * warrant that the operation of the program will be uninterrupted or
 * error-free. The end-user understands that the program was developed for
 * research purposes and is advised not to rely exclusively on the program for
 * any reason.
 *
 */

package jdce.impl.corba;

import java.io.*;
import org.omg.CosNaming.*;
import java.net.*;
import java.rmi.*;
import java.rmi.server.*;
import java.rmi.registry.*;

/**
 * Session Server 
 *
 * @version     0.5 20 April 1998
 * @author      Shrideep Pallickara
 *
 */

public class sessionServer {

  public static void main(String[] args) {
    jdce.util.message.ThreadScheduler _scheduler;
    _scheduler = new jdce.util.message.ThreadScheduler(100);

    try {
      // Initialize the ORB.
      org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init(args,null);
      // Create the sessionScheduler object.
      jdce.scheduler.sessionScheduler scheduler =
	new SessionSchedulerImpl("Scheduler");
      // Export the newly create object.
      orb.connect(scheduler);
      
      /* Get a reference to the Naming Service */
      org.omg.CORBA.Object nameServiceObj = 
	orb.resolve_initial_references("NameService");
      
      if (nameServiceObj == null) {
	System.out.println("Name Service Object = null");
	return;
      }
      
      org.omg.CosNaming.NamingContext nameService =
	org.omg.CosNaming.NamingContextHelper.narrow(nameServiceObj);
      if (nameService == null) {
	System.out.println("nameService = null");
	return;
      }
      
      /* Bind the sessionScheduler Object in the Naming Service */
      NameComponent[] schedulerName = 
      { new NameComponent("Collaboration", "Scheduler") };
      nameService.rebind(schedulerName, scheduler);
      
      ((jdce.impl.corba.SessionSchedulerImpl)scheduler).setScheduler(_scheduler);
      System.out.println(scheduler + " is ready.");
    } catch(Exception e) {
      System.out.println("Exception " + e);
    }
    
    /**
     * Update the values here for the RMI sessionScheduler hostname& port
     * get the complete hostname e.g. "jojo.npac.syr.edu"
     * and the port on which the rmiregistry runs, for e.g. 7000.
     */

    String hostname="osprey7.npac.syr.edu";
    int port = 7000;
    
    try {
      jdce.impl.rmi.SessionSchedulerImpl _session 
	= new jdce.impl.rmi.SessionSchedulerImpl("Scheduler");
      _session.setScheduler(_scheduler);
      _session.setRMIRegistryInfo(hostname,port);
      String _sessionName = "rmi://" + hostname +":" +port +"/Scheduler";
      System.out.println("rmi://" + hostname +":" +port +"/Scheduler");
      Naming.rebind(_sessionName, _session);
      System.out.println("Session Scheduler Ready to Service Requests");
     } catch (RemoteException re) {
      re.printStackTrace();
    }
    catch (MalformedURLException e) {
      e.printStackTrace();
   } 

    
    // It is the ORB's responsibility to create threads to receive requests.
    // there is no need to enter an event loop, so just block forever.
    //
    // In this example, we will join with our own thread.  However, there
    // are many different ways of doing this.  If your application uses
    // the AWT this step may not be necessary since AWT threads are not
    // daemon threads.

    try {
      Thread.currentThread().join();
    }
    catch(InterruptedException e) {
      System.out.println(e);
    }
  }
}

