/* 
 *	Copyright (c) 1997-98 
 *	NorthEast Parallel Architectures Center, Syracuse University. 
 * All Rights Reserved. Permission to use, copy, modify, and distribute this
 * software and its documentation for educational, research and non-profit
 * purposes, without fee, and without a written agreement is hereby granted,
 * provided that the above copyright notice and this paragraph appear in all
 * copies. Permission to incorporate this software into commercial products may
 * be obtained by contacting the NorthEast Parallel Architectures Center. 
 *
 * The software program and documentation are supplied "as is",
 * without any accompanying services from NPAC. NPAC does not
 * warrant that the operation of the program will be uninterrupted or
 * error-free. The end-user understands that the program was developed for
 * research purposes and is advised not to rely exclusively on the program for
 * any reason.
 *
 */

package edu.syr.npac.webflow.impl.server;

import java.io.*;
import org.omg.CosNaming.*;
import java.net.*;

/**
 * Session Server 
 * 
 * @version     0.5 20 April 1998
 * @author      Shrideep Pallickara
 *
 */

public class sessionServer {

  public static void main(String[] args) {

    try {
       for (int i=0; i < args.length; i++)
       System.out.println(args[i]);
      /* Initialize the ORB */
     org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init(args,null);
    
     org.omg.CORBA.Object nameServiceObjRef =
       orb.resolve_initial_references("NameService");
      
     org.omg.CosNaming.NamingContext rootContext =
       org.omg.CosNaming.NamingContextHelper.narrow(nameServiceObjRef);
     
     NameComponent[] projectName = {new NameComponent("WebFlow", "")};
     NamingContext projectContext=
       rootContext.bind_new_context(projectName);
     
     
     NameComponent[] applicationDomain = {new NameComponent("ApplicationDomain","")};
     
     NamingContext applicationDomainContext = 
       projectContext.bind_new_context(applicationDomain);
     
     NameComponent[] imagingApp = 
     {  new NameComponent("Imaging", "")};
     
     NamingContext imagingContext =
       applicationDomainContext.bind_new_context(imagingApp);
     
     NameComponent[] computingApp = 
     {  new NameComponent("Computing", "")};
     
     NamingContext computingContext =
      applicationDomainContext.bind_new_context(computingApp);

    
     System.out.println("Trying Convex ...");
     edu.syr.npac.webflow.manager.moduleManager _manager=
       new edu.syr.npac.webflow.impl.manager.moduleManagerImpl("Manager");
     orb.connect(_manager);
     
     edu.syr.npac.webflow.connection.connectionManager _connection =
       new edu.syr.npac.webflow.impl.connection.connectionManagerImpl("Connection Manager");
     orb.connect(_connection);

     System.out.println("Trying Convex Now...");
     edu.syr.npac.webflow.module.Module convex=
       new edu.syr.npac.webflow.impl.module.imaging.ConvexFilter("ConvexFilterModule");
     System.out.println("Created CONVEX");
     
     edu.syr.npac.webflow.module.Module bathglass=
       new edu.syr.npac.webflow.impl.module.imaging.BathGlassFilter("BathGlassFilterModule");
     
     edu.syr.npac.webflow.module.Module negative=
       new edu.syr.npac.webflow.impl.module.imaging.NegativeFilter("NegativeFilterModule");
     
     edu.syr.npac.webflow.module.Module composite=
       new edu.syr.npac.webflow.impl.module.imaging.CompositeModule("BathGlassFilter/ConvexFilter");
     
     edu.syr.npac.webflow.module.Module inputModule=
       new edu.syr.npac.webflow.impl.module.imaging.InputModule("ImageInputModule");
     
     
     orb.connect(convex); 
     orb.connect(bathglass);
     orb.connect(negative);
     orb.connect(composite);
     orb.connect(inputModule);
     
     NameComponent[] convexName = 
     { new NameComponent("ConvexFilter", "")};
     imagingContext.bind(convexName, convex);
     
     NameComponent[] bathglassName = 
     { new NameComponent("BathGlassFilter", "")};
     imagingContext.bind(bathglassName, bathglass);
     
     NameComponent[] negativeName = 
     { new NameComponent("NegativeFilter", "")};
     imagingContext.bind(negativeName, negative);
     
     NameComponent[] compositeName = 
     { new NameComponent("CompositeFilter", "")};
     imagingContext.bind(compositeName, composite);
     
     NameComponent[] inputModuleName = 
     { new NameComponent("InputModule", "")};
     imagingContext.bind(inputModuleName, inputModule);
     
     NameComponent[] junk1 = 
     {  new NameComponent("Variance", "")};
     computingContext.bind(junk1, inputModule);
     NameComponent[] junk2 = 
     {  new NameComponent("Deviation", "")};
     computingContext.bind(junk2, composite);
     
     System.out.println(convex + " is ready.");
     System.out.println(bathglass + " is ready");
     System.out.println(negative + " is ready");
     System.out.println(composite + " is ready");
     System.out.println(inputModule + " is ready");
     
    } 
    catch(Exception e) {
      e.printStackTrace();
      System.out.println("Exception " + e);
    }
    
    // It is the ORB's responsibility to create threads to receive requests.
    // there is no need to enter an event loop, so just block forever.
    //
    // In this example, we will join with our own thread.  However, there
    // are many different ways of doing this.  If your application uses
    // the AWT this step may not be necessary since AWT threads are not
    // daemon threads.
    
    try {
      Thread.currentThread().join();
    }
    catch(InterruptedException e) {
      System.out.println(e);
    }
  }
}

