// VRServer.java 
// Server Application for multiuser Virtual World System 

/**
* This is part of the source code needed for building a Collabratised
* VRML World using JSDA.
* The files needed are 
* Server-Side :: 
* VRServer.java - This is the Virtual World Server
* MuProtocol.java - This contains various defined constants used throughout
* 
* Client-side ::
* VRClient.java - 
* VRUser.java - The Java Script
* VRConsumer.java
* MuProtocol.java - This contains various defined constants used throughout
*                   the implementation of the Multi-User Virtual Worlds.
* 
* @author : Shrideep Pallickara
* @advisor: Dr. Wojtek Furmanski
* @date   : 26/Mar - 1997.
*/
import SharedData.*;

public class
VRServer implements MuProtocol {

    private static String sessionName = "VRSession";


    public static void
    main(String args[]) {
        VRClient client      = null;
	Session    VRSession = null;
	String     url         = null;
	String     sessionType = "socket";
	String     hostname    = null;
	int        hostport    = 0;

        if (VRServer_Debug) {
             System.out.println("VRServer: main.");
        }

        hostname = getHost(args);
	hostport = getPort(args);
	url = "coll://" + hostname + ":" + hostport + "/" + sessionType +
	      "/Session/" + sessionName;

	try {

/* Create a session, and publish it. */
 
            VRSession = new SharedData.socket.socketSession(sessionName);
	    Naming.rebind(url, VRSession);

/*  Get a client-side handle to the session and create a channel called
 *  "VRChannel". The clients would be using the VRChannel handles to pass
 *  information to various clients.. 
 */

            VRSession = SessionFactory.createSession(url);
            VRSession.createChannel("VRChannel", true);
            System.out.println("Setup and bound VR server.");
        } catch (SharedDataException e) {
            System.out.println("VRServer: main: shared data exception: " + e);
            if (VRServer_Debug) {
		e.printStackTrace();
	    }
        }
    }


    private static String
    getHost(String args[]) {
        String defHost = "stard";   /* Default host name for connections. */
        int length = args.length;

        if (VRServer_Debug) {
            System.out.println("VRServer: getHost.");
        }

        for (int i = 0; i < length; i++) {
            if (args[i].equals("-server")) {
                if (++i < length) {
                    return(args[i]);
                }
            }
        }
        return(defHost);
    }


    private static int
    getPort(String args[]) {
        int defPort = 4461;   /* Default port number for connections. */
        int length = args.length;

        if (VRServer_Debug) {
            System.out.println("VRServer: getPort.");
        }

        for (int i = 0; i < length; i++) {
            if (args[i].equals("-port")) {
                if (++i < length) {
                    return(Integer.parseInt(args[i]));
                }
            }    
        }
        return(defPort);
    }
}





