/* VRUser.java - This is the Java Script Node of the Virtual World File
 *               It extends Script, and also gets a handle to the VRChannel.
 *               Besides, it also establishes the Consumer on that channel.
 *               Any information that needs to be sent to any client should
 *               be sent on the VRChannel. The Consumer's on the VRChannel
 *               are responsible for calling the appropriate methods in the
 *               VRUser to meet the appropriate logical handling of the data.
 */
 

/**
* This is part of the source code needed for building a Collabratised
* VRML World using JSDA.
* The files needed are 
* Server-Side :: 
* VRServer.java - This is the Virtual World Server
* MuProtocol.java - This contains various defined constants used throughout
* 
* Client-side ::
* VRClient.java - 
* VRUser.java - The Java Script
* VRConsumer.java
* MuProtocol.java - This contains various defined constants used throughout
* 
* @author : Shrideep Pallickara
* @advisor: Dr. Wojtek Furmanski
* @date   : 26/Mar - 1997.
*/

import SharedData.*;

import vrml.*; 
import vrml.field.*; 
import vrml.node.*;

import java.util.*; 
import java.net.*; 
import java.io.*;

public class
VRUser extends Script implements MuProtocol, Runnable {

    /** The name of this Client. */
    protected String name;

    /** The VR session that this client application will join. */
    private Session session;

    /** The client that will be joining the session and channel. */
    private VRClient client;

    /** The channel that this client application will use to send data. */
    private Channel channel;
 
    /** The consumer of all data sent over the channel. */
    private VRConsumer VRConsumer;

    /** The data that will be sent over the channel. */
    private Data data;

    /** Indicates if the client is successfully connected to the server. */
    boolean connected = false;
    public String commandLine;
/* Default setup, will mostly be overriden by attributes. */

    String  hostname   = "osprey2.npac.syr.edu";
    int     hostport   = 4461;
    public final static int AVATARS = 3; 
    private Thread running = null;
    Browser b;                     // for displaying error message to user 
    SFVec3f avatarPosition[] = new SFVec3f[AVATARS];          
    SFRotation avatarRotation[] = new SFRotation[AVATARS]; 
    float[] coord = new float[3]; 
    float[] rotation; 
    int id; 

    public void initialize(){ 
      b = getBrowser(); 
      Node avatar = null; 
     

      /*The trick lies in getting the reference to the 'translation' and '
       *rotation' field of 'avatar' node and then updating these to reflect
       *positions of different avatars */
      

      for (int i=0; i < AVATARS; i++){ 
         try { 
            avatar = (Node)((SFNode)getField("avatar" + i)).getValue(); 
            avatarPosition[i] = (SFVec3f)avatar.getExposedField("translation"); 
            avatarRotation[i] = (SFRotation)avatar.getExposedField("rotation"); 
         } catch (Exception e) { 
            b.setDescription("can not get avatar"); 
            return; 
         } 
      } 
      //coord = new float[3]; 
      rotation = new float[4]; 
   } 


    private boolean
    isConnected() {
        if (VRUser_Debug) {
            System.out.println("VRUser: isConnected: " + connected);
        }

        return(connected);
    }


    private boolean
    connect() {
	String sessionType = "socket";

	if (VRUser_Debug) {
            System.out.println("VRUser: connect.");
        }

	if (name == null) {
	    throw new Error("need a non-null name to connect");
	}  

	try {
	    String sessionName = "VRSession";

/* Create a VR client. */

            System.out.println("Creating a VRMember...");
            b.setDescription("Creating a VRMember ....."); 
            client = new VRClient(name);

/* Resolve the VR session. */

	    try {
                String url = "coll://" + hostname + ":" + hostport +
			     "/" + sessionType + "/Session/" + sessionName;

		session = SessionFactory.createSession(url);
             System.out.println("Session created...");
/*  Join the session and a named channel and set the channels data consumer. */


		session.join(client);
                System.out.println("Client joined to Session....");
		channel = Channel.join(session, "VRChannel", client); 
                b.setDescription("Client joined to Channel ....."); 
                System.out.println("Client joined to Channel....");
		VRConsumer = new VRConsumer(client.getName(), this);
		channel.setConsumer(client, VRConsumer);
            b.setDescription("Session is Resolved.."); 
            System.out.println("Session is Resolved...."); 
            } catch (Exception e) {
                System.out.print("Caught exception in ");
                System.out.println("VRUser.connect: " + e); 
                b.setDescription("Screwup in Connect dude...."); 
              
		    if (VRUser_Debug) {
		       e.printStackTrace();
		    }
		    return(false);
            }
        } catch (Throwable th) {
            System.err.println("VRUser: connect caught: " + th);
            if (VRUser_Debug) {
		th.printStackTrace();
            }
	      throw new Error("SetClient.start failed : " + th);
         }
	 return(true);
     }


/*  Convert the users message to bytes and send it over the channel. */

    private void
    writeLine(String message) {
        if (VRUser_Debug) {
            System.out.println("VRUser: writeLine: " + message);
        }

	if (isConnected()) {
            try {
		data = new Data(message);
		data.setPriority(Channel.HIGH_PRIORITY);
                channel.sendToAll(client, data);
            } catch (Exception e) {
                System.out.print("Caught exception in ");
                System.out.println("VRUser.writeLine(): " + e);
                if (VRUser_Debug) {
		    e.printStackTrace();
	          }
            }
	}
    }


    private void
    disconnect() {
        try {
            if (VRUser_Debug) {
                System.out.println("VRUser: disconnect.");
            }
/* Leave the channel and leave the session. */

            channel.leave(client);
            session.leave(client);
        } catch (Exception e) {
            System.err.println("Caught exception while trying to " +
			       "disconnect from VR server: " + e);
            if (VRUser_Debug) {
		e.printStackTrace();
	    }
        }
    }


    public void
    start() {
        if (VRUser_Debug) {
            System.out.println("VRUser: start.");
        }

        if (running == null) {
            running = new Thread(this);
            running.start();
        }
    }


    public void
    stop() {
        if (VRUser_Debug) {
            System.out.println("VRUser: stop.");
        }

        if (running != null) {
            running.stop();
            running = null;
        }
    }    


    public void
    run() {
        if (VRUser_Debug) {
            System.out.println("VRUser: run.");
        }
    }


    public void
    destroy() {
        if (VRUser_Debug) {
            System.out.println("VRUser: destroy.");
        }
        running = null;
        disconnect();
    }


    private int
    rrange(int low, int high) {
        return((int)(Math.random() * (high - low + 1) + low));
    }

    public void processEvent(Event ev) { 
      if(ev.getName().equals("clicked1") == true){  
         b.setDescription("Event Name is "+ev.getName());
         ConstSFBool v = (ConstSFBool)ev.getValue(); 
 
         if(v.getValue() && (!connected)){ 
            // open network and input/output stream 
               name = "Wojtek"+ ":" +
	        (rrange(0, 1000) * rrange(0, 1000));
               connected = connect(); 
               b.setDescription("Connection to server established"); 
               try { 
               id = session.listClientNames().length -1; 
               } catch(Exception e){b.setDescription("ID'S MESSED UP"); }
         } 
         return; 
      } else if(ev.getName().equals("currentPosition")){ 
         ((ConstSFVec3f)ev.getValue()).getValue(coord);  
          //coord[0]=0f; coord[1]=0f; coord[2]=0f; 
          System.out.println("x:" + coord[0] + "y:" + coord[1] + "z:" + coord[2] );
          b.setDescription("Event Name is "+ev.getName()); 
          
         if (connected) {
         try { 
            // send my id and my current position to server 
            writeLine(id + " " + MuProtocol.MOVE + " " +  coord[0] + 
                      " " + coord[1] + " " + coord[2] ); 
         } catch (Exception e) { 
            System.err.println("Caught exception while invoke myPosition: " + e);
           /* b.setDescription("Fail to send position to server:  " + e); 
            return; */
         } }
      } else if(ev.getName().equals("myOrientation")){ 
	((ConstSFRotation)ev.getValue()).getValue(rotation); 
         try { 
            // send my id and my current position to server 
            writeLine(id + " " + MuProtocol.ROTATION + " " +  rotation[0] + 
                      " " + rotation[1] + " " + rotation[2] + " " + rotation[3]); 
         } catch (Exception e) { 
            System.err.println("Caught exception while invoke myOrient: " + e);
           /* b.setDescription("Fail to send orientation to server:  " + e); 
            return;*/ 
         } 
      }	 
   } 
 
   public void updatePosition(int id, float x, float y, float z){  
      System.out.println("Avatar" + id +"on the move ....");
      avatarPosition[id].setValue(x, y, z); 
   } 
 
   public void updateOrientation(int id, float x, float y, float z, float r){ 
      avatarRotation[id].setValue(x, y, z, r); 
   } 

   public void performUpdate() {
     StringTokenizer tok;
     float x=0f, y=0f, z=0f, r=0f;    
 
      while (commandLine != null) {
            tok = new StringTokenizer(commandLine, " ,\n\r\t");
            commandLine = null;
            int idOfMove      = Integer.parseInt(tok.nextToken());
            int command = Integer.parseInt(tok.nextToken());

            x = Float.valueOf(tok.nextToken()).floatValue();
            y = Float.valueOf(tok.nextToken()).floatValue();
            z = Float.valueOf(tok.nextToken()).floatValue();

            if(MuProtocol.MOVE == command){
	      updatePosition(idOfMove, x, y, z); // update position 
	    }else{
	       r = Float.valueOf(tok.nextToken()).floatValue();
	      updateOrientation(id, x, y, z, r); // update orientation
	    }
       }
    }

}



