/*
 *  File:  DrawableStringsTest.java
 *
 *  Given an array of strings, center the strings both 
 *  vertically and horizontally in the applet window.
 *
 *  Copyright:  Northeast Parallel Architectures Center
 *  
 */

import java.applet.Applet;
import java.awt.*;

public class DrawableStringsTest extends Applet {

   /*
    *  Private variables
    */

   private int n;            // number of strings
   private String[] string;  // an array of strings to be drawn
   
   // A reference to a DrawableStrings object:
   private DrawableStrings drawableStrings;
   
   // HTML parameters:
   int bgColor, fgColor;
   String fontName;
   int fontStyle, fontSize;
   
   /*
    *  Public methods
    *  
    */

   public void init() {
      
      // Get the parameters from the HTML document:
      getParameters();
      
      // Instantiate a DrawableString object:
      drawableStrings = new DrawableStrings( string );
      drawableStrings.setColor( new Color( fgColor ) );
      drawableStrings.setFontName( fontName );
      drawableStrings.setFontStyle( fontStyle );
      drawableStrings.setFontSize( fontSize );
      
      // Set the background color of the applet:
      this.setBackground( new Color( bgColor ) );
      
   }
   
   public void paint( Graphics g ) {
      drawableStrings.centerDraw( this );
   }
   
   /*
    *  Private methods
    *  
    */

   private void getParameters() {
      
      // Get the "numStrings" parameter from the HTML document:
      String nStr = getParameter( "numStrings" );
      // Convert string 'nStr' to an integer:
      n = ( nStr == null ) ? 0 : Integer.parseInt( nStr );
      
      // Instantiate an array of strings:
      string = new String[n];
      
      // Get the strings to be drawn:
      for ( int i = 0; i < n; i++ ) {
        string[i] = getParameter( "string" + i );
        if ( string[i] == null ) string[i] = "";
      }
      
      // Get the "fontName" parameter from the HTML document:
      fontName = getParameter( "fontName" );
      if ( fontName == null ) fontName = "SansSerif";
      
      // Get the "fontStyle" parameter from the HTML document:
      String fontStyleStr = getParameter( "fontStyle" );
      // Default font style is BOLD:
      if ( fontStyleStr == null ) fontStyleStr = "BOLD";
      // Convert font style string to integer:
      fontStyle = fontStyleStringToInt( fontStyleStr );
      
      // Get the "fontSize" parameter from the HTML document:
      String fontSizeStr = getParameter( "fontSize" );
      // Default font size is 18 points:
      if ( fontSizeStr == null ) fontSizeStr = "18";
      // Convert font size string to integer:
      fontSize = Integer.parseInt( fontSizeStr );
      
      // Get the "bgColor" parameter from the HTML document:
      String bgColorStr = getParameter( "bgColor" );
      // Default background color is white:
      if ( bgColorStr == null ) bgColorStr = "#FFFFFF";
      // Convert hexadecimal color string to an integer:
      bgColor = colorStringToInt( bgColorStr );
      
      // Get the "fgColor" parameter from the HTML document:
      String fgColorStr = getParameter( "fgColor" );
      // Default foreground color is black:
      if ( fgColorStr == null ) fgColorStr = "#000000";
      // Convert hexadecimal color string to an integer:
      fgColor = colorStringToInt( fgColorStr );
      
   }
   
   private int colorStringToInt( String colorString ) {
   
      // Strip off whitespace from color string:
      colorString = colorString.trim();
      
      // Strip off leading '#' from color string, if necessary:
      if ( colorString.charAt(0) == '#' ) {
        colorString = colorString.substring(1);
      }
      
      // Convert hexadecimal color string to an integer:
      return Integer.parseInt( colorString, 16 );
      
   }
   
   private int fontStyleStringToInt( String fontStyleString ) {
   
      int fontStyle = 0;
      
      // Strip off whitespace from font style string and
      // convert to uppercase:
      fontStyleString = fontStyleString.trim().toUpperCase();
      
      // Font styles are additive:
      if ( fontStyleString.indexOf( "BOLD" ) > -1 ) {
        fontStyle += Font.BOLD;
      }
      if ( fontStyleString.indexOf( "ITALIC" ) > -1 ) {
        fontStyle += Font.ITALIC;
      }
      
      return fontStyle;
      
   }
   
}
