/*
 *  File:  SystemColors2.java
 *
 *  Display the colors used by the system in a scrollpane
 *
 *  Copyright:  Northeast Parallel Architectures Center
 *  
 */

import java.awt.Graphics;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Color;
import java.awt.Canvas;
import java.awt.SystemColor;
import java.awt.ScrollPane;
import java.awt.Adjustable;
import java.util.Hashtable;
import java.util.Enumeration;

public class SystemColors2 extends java.applet.Applet {

   // Constants:
   private ColorCanvas canvas;

   // Initialize the applet:
   public void init() {

      canvas = new ColorCanvas();
      
      ScrollPane appletScroller = new ScrollPane();
      appletScroller.add( canvas );
      appletScroller.setSize( getSize().width, getSize().height );
      add( appletScroller );
      
      Adjustable horizontal = appletScroller.getHAdjustable();
      horizontal.setUnitIncrement( 5 );
      Adjustable vertical = appletScroller.getVAdjustable();
      vertical.setUnitIncrement( 5 );
      
   }
   
   public void paint( Graphics g ) {   
      canvas.repaint();
   }

}


class ColorCanvas extends Canvas {

   // size of each color swatch:
   private int swatchSize;
   // whitespace around each color swatch:
   private int border;
   // total size, including whitespace:
   private int unitSize;
   // number of color swatches per row:
   private int cols;
   // number of color swatches per column:
   private int rows;

   // Constructor:
   public ColorCanvas() {
   
      super();
      
      initColors();
   
      // initialize parameters:
      swatchSize = 70; border = 15;
      unitSize = swatchSize + 2*border;
      cols = 4;
      rows = ( int ) Math.ceil( ( double ) colors.size()/cols );
      
      // compute size of the canvas:
      this.setSize( cols * unitSize, rows * unitSize );
      
   }
   
   // A hashtable of colors:
   private Hashtable colors = new Hashtable();
   
   // Put the system colors in the hashtable:
   private void initColors() {
      colors.put( "desktop", SystemColor.desktop );
      colors.put( "activeCaption", SystemColor.activeCaption );
      colors.put( "activeCaptionText", SystemColor.activeCaptionText );
      colors.put( "activeCaptionBorder", SystemColor.activeCaptionBorder );
      colors.put( "inactiveCaption", SystemColor.inactiveCaption );
      colors.put( "inactiveCaptionText", SystemColor.inactiveCaptionText );
      colors.put( "inactiveCaptionBorder", SystemColor.inactiveCaptionBorder );
      colors.put( "window", SystemColor.window );
      colors.put( "windowBorder", SystemColor.windowBorder );
      colors.put( "windowText", SystemColor.windowText );
      colors.put( "menu", SystemColor.menu );
      colors.put( "menuText", SystemColor.menuText );
      colors.put( "text", SystemColor.text );
      colors.put( "textText", SystemColor.textText );
      colors.put( "textHighlight", SystemColor.textHighlight );
      colors.put( "textHighlightText", SystemColor.textHighlightText );
      colors.put( "textInactiveText", SystemColor.textInactiveText );
      colors.put( "control", SystemColor.control );
      colors.put( "controlText", SystemColor.controlText );
      colors.put( "controlHighlight", SystemColor.controlHighlight );
      colors.put( "controlLtHighlight", SystemColor.controlLtHighlight );
      colors.put( "controlShadow", SystemColor.controlShadow );
      colors.put( "controlDkShadow", SystemColor.controlDkShadow );
      colors.put( "scrollbar", SystemColor.scrollbar );
      colors.put( "info", SystemColor.info );
      colors.put( "infoText", SystemColor.infoText );
   }
   
   public void paint( Graphics g ) {
      
      // local variables:
      int x, y;
      Color color;
      String colorstring;

      g.setFont( new Font( "SansSerif", Font.PLAIN, 9 ) );
      
      // vertical adjustment for color labels:
      int descender = g.getFontMetrics().getDescent();
      
      // an enumeration of keys in the hashtable:
      Enumeration colorstrings = colors.keys();
      
      // Display rows of color boxes:
      for ( int i = 0; i < rows; i++ ) {
         y = border + unitSize*i;
         for ( int j = 0; j < cols; j++ ) {
            if ( !colorstrings.hasMoreElements() ) break;
            colorstring = ( String ) colorstrings.nextElement();
            color = ( Color ) colors.get( colorstring );
            x = border + unitSize*j;
            g.setColor( color );
            g.fillRect( x, y, swatchSize, swatchSize );
            g.setColor( Color.black );
            g.drawString( colorstring, x, y - descender );
         }
      }
      
   }

}
