/*
 *  File:  AnalogClock.java
 *
 *  An analog clock (UTC)
 *
 *  Copyright:  Northeast Parallel Architectures Center
 *  
 */

import java.applet.Applet;
import java.awt.Graphics;
import java.awt.Color;
import java.awt.Point;

public class AnalogClock extends Applet implements Runnable {

   // a thread:
   private Thread thread;
   
   // the current hour, minute, and second:
   private double h, m;
   private int s;
   
   // applet parameters:
   private int size, border;
   private int center_x, center_y;
   
   // the length of each hand:
   private int h_length, m_length, s_length;
   
   // the endpoint of each hand:
   private Point s_point, m_point, h_point;
   
   public void init() {
   
      // local constants:
      final int msPerMinute = 60 * 1000;
      final int msPerHour = 60 * msPerMinute;
      
      // number of milliseconds since midnight, January 1, 1970 UTC:
      long t = new java.util.Date().getTime();
      //long t = System.currentTimeMillis();
      System.out.println( t );  // debugging
      
      // number of milliseconds since midnight today UTC:
      t %= 24 * msPerHour;
      
      // calculate current hour, minute, and second:
      h = t / ( double ) msPerHour;  // this is wrong!
      m = t / ( double ) msPerMinute;
      s = ( int ) t % msPerHour;
      
      // calculate applet parameters:
      size = Math.min( getSize().width, getSize().height );
      border = ( int ) Math.round( 0.05 * size );
      center_x = center_y = ( int ) Math.round( size/2 );
      
      // calculate the length of each hand:
      int diff = size/2 - border;
      s_length = m_length = ( int ) Math.round( diff * 0.9 );
      h_length = ( int ) Math.round( diff * 0.7 );
      
      // calculate the endpoint of each hand:
      double theta = ( 15 - s )*Math.PI/30;          // PI/2 - s*2*PI/60
      s_point = computeEndpoint( s_length, theta );
      theta = ( 15 - m )*Math.PI/30;                 // PI/2 - m*2*PI/60
      m_point = computeEndpoint( m_length, theta );
      theta = ( 3 - h )*Math.PI/6;                   // PI/2 - h*2*PI/12
      h_point = computeEndpoint( h_length, theta );

   }
   
   // Override java.applet.Applet.start():
   public void start() {
      if ( thread == null ) {
        // Start a new thread:
        thread = new Thread( this );
        thread.start();
      }
   }
 
   // Override java.applet.Applet.stop():
   public void stop() {
      if ( thread != null ) {
        thread.stop();
        thread = null;
      }
   }
 
   // Implement java.lang.Runnable.run():
   public void run() {
      while ( thread != null ) {
         try {
            // sleep for one second:
            Thread.sleep( 1000 );
         } catch ( InterruptedException e ) {
            // Do nothing   
         };
         repaint();
      }
   }

   public void paint( Graphics g ) {
      
      // local variables:
      int width, height;
      
      // draw the clock face:
      g.setColor( Color.black );
      width = height = size - 2*border;
      g.drawOval( border, border, width, height );
      
      // erase the second hand:
      g.setColor( getBackground() );
      g.drawLine( center_x, center_y, s_point.x, s_point.y );
      
      // draw the second hand:
      g.setColor( Color.red );
      s++;
      s_point = computeEndpoint( s_length, ( 15 - s )*Math.PI/30 );
      g.drawLine( center_x, center_y, s_point.x, s_point.y );
      
      // erase the minute hand:
      g.setColor( getBackground() );
      g.drawLine( center_x, center_y, m_point.x, m_point.y );
      
      // draw the minute hand:
      g.setColor( Color.black );
      m += 1.0/60.0;
      m_point = computeEndpoint( m_length, ( 15 - m )*Math.PI/30 );
      g.drawLine( center_x, center_y, m_point.x, m_point.y );
      
      // erase the hour hand:
      g.setColor( getBackground() );
      g.drawLine( center_x, center_y, h_point.x, h_point.y );
      
      // draw the hour hand:
      g.setColor( Color.black );
      h += 1.0/3600.0;  // this is wrong!
      h_point = computeEndpoint( h_length, ( 3 - h )*Math.PI/6 );
      g.drawLine( center_x, center_y, h_point.x, h_point.y );
      
   }
  
   private Point computeEndpoint( int length, double theta ) {
     int x = center_x + ( int ) Math.round( length*Math.cos( theta ) );
     int y = center_y - ( int ) Math.round( length*Math.sin( theta ) );
     return new Point( x, y );
   }
   
}
