/*
 *  File:  MovingSquares.java
 *
 *  A moving square with boundary checking
 *
 *  Copyright:  Northeast Parallel Architectures Center
 *
 */

import java.applet.Applet;
import java.awt.*;

public class MovingSquares extends Applet implements Runnable {

  // A reference to a square:
  private Square square1, square2;
  
  // Override java.applet.Applet.init:
  public void init () {

    setBackground( Color.black );
    setForeground( Color.white );
    
    // Instantiate two squares:
    square1 = new Square( 75, 25, 50 );
    square1.setColor( Color.red );
    square2 = new Square( 25, 75, 50 );
    square2.setColor( Color.blue );
    
  }

  // A thread of execution:
  private Thread thread;
    
  // Override java.applet.Applet.start:
  public void start() {
    if ( thread == null ) {
      thread = new Thread( this );
      thread.start();
    }
  }
  
  // Override java.applet.Applet.stop:
  public void stop() {
    if ( thread != null ) {
      thread.stop();
      thread = null;
    }
  }
  
  // Implement java.lang.Runnable.run:
  public void run() {
    while ( thread != null ) {
      try {
        Thread.sleep( 20 );
      } catch ( InterruptedException e ) {
        // do nothing   
      }
      repaint();
    }
  }

  // Initially, the squares move two pixels down and to the right:
  int dx1 = 2, dy1 = 2;
  int dx2 = 2, dy2 = 2;
  
  // Override java.awt.Component.paint:
  public void paint( Graphics g ) {
  
    // Check if either square is out of bounds:
    checkBounds1( this.getBounds() );
    checkBounds2( this.getBounds() );
    
    // Move and fill square1:
    square1.translate( dx1, dy1 );
    square1.fill( g );
    
    // Move and fill square2:
    square2.translate( dx2, dy2 );
    square2.fill( g );
    
  }
  
  private void checkBounds1( java.awt.Rectangle r ) {

    // Get the bounding box of square1:
    java.awt.Rectangle bb = square1.getBounds();

    // Compute the dimensions of square1:
    int w = bb.width, h = bb.height;
    int new_x = bb.x + dx1, new_y = bb.y + dy1;
    
    // If square1 is out of bounds, reverse direction:
    if ( ( new_x < r.x ) || ( new_x + w > r.x + r.width ) )  dx1 *= -1;
    if ( ( new_y < r.y ) || ( new_y + h > r.y + r.height ) )  dy1 *= -1;

  }

  private void checkBounds2( java.awt.Rectangle r ) {

    // Get the bounding box of square2:
    java.awt.Rectangle bb = square2.getBounds();

    // Compute the dimensions of square2:
    int w = bb.width, h = bb.height;
    int new_x = bb.x + dx2, new_y = bb.y + dy2;
    
    // If square2 is out of bounds, reverse direction:
    if ( ( new_x < r.x ) || ( new_x + w > r.x + r.width ) )  dx2 *= -1;
    if ( ( new_y < r.y ) || ( new_y + h > r.y + r.height ) )  dy2 *= -1;

  }

}
