/*
 *  File:  MyMovingPolygons.java
 *
 *  Moving polygons with boundary checking and a MovablePolygon class
 *
 *  Copyright:  Northeast Parallel Architectures Center
 *
 */

import java.applet.Applet;
import java.awt.*;

public class MyMovingPolygons extends Applet implements Runnable {

  // Instance variables:
  private final int numPoly = 4;     // number of movable polygons
  private MovablePolygon polygon[];  // array of movable polygons
  private Thread thread;             // a thread
    
  // Override java.applet.Applet.init:
  public void init () {

    setBackground( Color.black );
    setForeground( Color.white );
    
    // Instantiate the polygon array:
    polygon = new MovablePolygon[ numPoly ];

    // Instantiate a square:
    polygon[0] = new MovablePolygon( 75, 75, 50, 4 );
    polygon[0] = ( MovablePolygon ) polygon[0].centerRotate( Math.PI/4 );
    polygon[0].setDelta( 2, 3 );
    polygon[0].setColor( Color.red );

    // Instantiate a regular hexagon:
    polygon[1] = new MovablePolygon( 125, 60, 50, 6 );
    polygon[1].setDelta( -3, 2 );
    polygon[1].setColor( Color.blue );

    // Instantiate an equilateral triangle:
    polygon[2] = new MovablePolygon( 60, 125, 50, 3 );
    polygon[2].setDelta( -2, 2 );
    polygon[2].setColor( Color.green );
    
    // Instantiate a regular octagon:
    polygon[3] = new MovablePolygon( 125, 125, 50, 8 );
    polygon[3].setDelta( 3, -2 );
    polygon[3].setColor( Color.yellow );

  }

  // Override java.applet.Applet.start:
  public void start() {
    if ( thread == null ) {
      thread = new Thread( this );
      thread.start();
    }
  }
  
  // Override java.applet.Applet.stop:
  public void stop() {
    if ( thread != null ) {
      thread.stop();
      thread = null;
    }
  }
  
  // Implement java.lang.Runnable.run:
  public void run() {
    while ( thread != null ) {
      try {
        Thread.sleep( 20 );
      } catch ( InterruptedException e ) {
        // do nothing   
      }
      repaint();
    }
  }

  // Override java.awt.Component.paint:
  public void paint( Graphics g ) {
  
    // Check, move, and fill each polygon:
    for ( int i = 0; i < numPoly; i++ ) {
      polygon[i].checkBounds( this.getBounds() );
      polygon[i].move();
      polygon[i].fill( g );
    }
    
  }
  
}
