/*
 *  File:  AppletDemo.java
 *
 *  Applet demonstration:  init() and destroy() are called once
 *  and only once, at the beginning and end of the applet's
 *  lifespan, respectively.  start() is called immediately after
 *  init() and every time the applet is restarted (after having
 *  been stopped).  paint( Graphics ) is always called after
 *  start() and whenever the applet window must be redrawn.  
 *  stop() is called immediately before destroy() and every 
 *  time the applet is stopped by the browser or appletviewer.  
 *  All five of these methods must be public, otherwise the 
 *  browser or appletviewer will not be able to call them.
 *  
 *  If you load this applet into a browser, inspect the Java 
 *  console for messages.  If you load this applet into 
 *  appletviewer, messages will be written to standard output.
 *
 *  Copyright:  Northeast Parallel Architectures Center
 *  
 */

import java.applet.Applet;
import java.awt.Font;
import java.awt.Graphics;

public class AppletDemo extends Applet {

  Font f;  // a font
  
  // Counts the number of method calls:
  private int nStart, nPaint, nStop;
  
  // Stores the message strings:
  private String startMsg, paintMsg, stopMsg;
  
  // Override java.applet.Applet.init():
  public void init() {
    // Initialize a font:
    f = new Font( "SansSerif", Font.BOLD, 18 );
    // Initialize counters:
    nStart = nPaint = nStop = 0;
    // Initialize message strings:
    startMsg = "start method called 0 times";
    paintMsg = "paint method called 0 times";
    stopMsg = "stop method called 0 times";
    // Print a message:
    System.out.println( "init() called" );
  }
  
  // Override java.applet.Applet.start():
  public void start() {
    // Update counter and message string:
    startMsg = "start method called " + (++nStart) + " times";
    // Print a message:
    System.out.println( startMsg );
  }
  
  // Override java.awt.Container.paint( Graphics ):
  public void paint( Graphics g ) {
    // Update counter and message string:
    paintMsg = "paint method called " + (++nPaint) + " times";
    // Print a message:
    System.out.println( paintMsg );
    // Draw the messages in the applet window:
    g.setFont( f );
    g.drawString( startMsg, 25, 25 );
    g.drawString( paintMsg, 25, 50 );
    g.drawString( stopMsg, 25, 75 );
  }

  // Override java.applet.Applet.stop():
  public void stop() {
    // Update counter and message string:
    stopMsg = "stop method called " + (++nStop) + " times";
    // Print a message:
    System.out.println( stopMsg );
  }
  
  // Override java.applet.Applet.destroy():
  public void destroy() {
    // Print a message:
    System.out.println( "destroy() called" );
  }
  
}
