/*
 *  File:  AppletDemo2.java
 *
 *  Another applet demonstration:  In addtion to init(), start(),
 *  stop(), destroy(), and paint( Graphics ), this applet also
 *  traces method update( Graphics ).  What you'll find is that
 *  update( Graphics ) isn't *ever* called automatically!  It and
 *  repaint() (which calls update) evidently exist for user 
 *  applications only.
 *  
 *  If you load this applet into a browser, inspect the Java 
 *  console for messages.  If you load this applet into 
 *  appletviewer, messages will be written to standard output.
 *
 *  Copyright:  Northeast Parallel Architectures Center
 *  
 */

import java.applet.Applet;
import java.awt.Font;
import java.awt.Graphics;

public class AppletDemo2 extends Applet {

  Font f;  // a font
  
  // Counts the number of method calls:
  private int nStart, nUpdate, nPaint, nStop;
  
  // Stores the message strings:
  private String startMsg, updateMsg, paintMsg, stopMsg;
  
  // Override java.applet.Applet.init():
  public void init() {
    // Initialize a font:
    f = new Font( "SansSerif", Font.BOLD, 18 );
    // Initialize counters:
    nStart = nUpdate = nPaint = nStop = 0;
    // Initialize message strings:
    startMsg = "start method called 0 times";
    updateMsg = "update method called 0 times";
    paintMsg = "paint method called 0 times";
    stopMsg = "stop method called 0 times";
    // Print a message:
    System.out.println( "init() called" );
  }
  
  // Override java.applet.Applet.start():
  public void start() {
    // Update counter and message string:
    startMsg = "start method called " + (++nStart) + " times";
    // Print a message:
    System.out.println( startMsg );
  }
  
  // Mimic java.awt.Container.update( Graphics ):
  public void update( Graphics g ) {
    // Update counter and message string:
    updateMsg = "update method called " + (++nUpdate) + " times";
    // Print a message:
    System.out.println( updateMsg );
    // Get the size of this applet and clear the foreground:
    int w = getSize().width;
    int h = getSize().height;
    g.setColor( getBackground() );
    g.clearRect( 0, 0, w, h );
    g.setColor( getForeground() );
    // Call the paint method:
    paint( g );
  }

  // Override java.awt.Container.paint( Graphics ):
  public void paint( Graphics g ) {
    // Update counter and message string:
    paintMsg = "paint method called " + (++nPaint) + " times";
    // Print a message:
    System.out.println( paintMsg );
    // Draw the messages in the applet window:
    g.setFont( f );
    g.drawString( startMsg, 25, 25 );
    g.drawString( updateMsg, 25, 50 );
    g.drawString( paintMsg, 25, 75 );
    g.drawString( stopMsg, 25, 100 );
  }

  // Override java.applet.Applet.stop():
  public void stop() {
    // Update counter and message string:
    stopMsg = "stop method called " + (++nStop) + " times";
    // Print a message:
    System.out.println( stopMsg );
  }
  
  // Override java.applet.Applet.destroy():
  public void destroy() {
    // Print a message:
    System.out.println( "destroy() called" );
  }
  
}
