/*
 *  File:  MyColorBoxes5.java
 *
 *  Display built-in colors with labels, a 2D array, a nested loop,
 *  some constants, and a hashtable
 *
 *  Copyright:  Northeast Parallel Architectures Center
 *  
 */

import java.applet.Applet;
import java.awt.Graphics;
import java.awt.Color;
import java.awt.FontMetrics;
import java.util.Hashtable;
import java.util.Enumeration;

public class MyColorBoxes5 extends Applet {

   // size of each color swatch:
   private final int swatchSize = 70;
   // whitespace around each color swatch:
   private final int border = 15;
   // total size, including whitespace:
   private final int unitSize = swatchSize + 2*border;
   // number of color swatches per row:
   private final int cols = 4;

   // A hashtable of colors:
   private Hashtable colors = new Hashtable();
   
   public void init() {
      colors.put( "black", Color.black );
      colors.put( "blue", Color.blue );
      colors.put( "cyan", Color.cyan );
      colors.put( "darkGray", Color.darkGray );
      colors.put( "gray", Color.gray );
      colors.put( "green", Color.green );
      colors.put( "lightGray", Color.lightGray );
      colors.put( "magenta", Color.magenta );
      colors.put( "orange", Color.orange );
      colors.put( "pink", Color.pink );
      colors.put( "red", Color.red );
      colors.put( "white", Color.white );
      colors.put( "yellow", Color.yellow );
      colors.put( "brown", new Color( 0xA5, 0x2A, 0x2A ) );
      colors.put( "turquoise", new Color( 0x40, 0xE0, 0xD0 ) );
      colors.put( "violet", new Color( 0xEE, 0x82, 0xEE ) );
      colors.put( "gold", new Color( 0xFF, 0xD7, 0x00 ) );
      colors.put( "darkBrown", new Color( 0x5C, 0x40, 0x33 ) );
      colors.put( "navy", new Color( 0x00, 0x00, 0x80 ) );
      colors.put( "darkPurple", new Color( 0x87, 0x1F, 0x78 ) );
   }
   
   public void paint( Graphics g ) {
      
      // local variables:
      int x, y;
      Color color;
      String colorstring;
      
      // vertical adjustment for color labels:
      int descender = g.getFontMetrics().getDescent();
      
      // an enumeration of keys in the hashtable:
      Enumeration colorstrings = colors.keys();
      
      // Draw rows of color boxes:
      int rows = ( int ) Math.ceil( ( double ) colors.swatchSize()/cols );
      for ( int i = 0; i < rows; i++ ) {
         y = border + unitSize*i;
         for ( int j = 0; j < cols; j++ ) {
            if ( !colorstrings.hasMoreElements() ) break;
            colorstring = ( String ) colorstrings.nextElement();
            color = ( Color ) colors.get( colorstring );
            x = border + unitSize*j;
            g.setColor( color );
            g.fillRect( x, y, swatchSize, swatchSize );
            g.setColor( Color.black );
            g.drawString( colorstring, x, y - descender );
         }
      }
      
   }

}
   
