/*
 *  File:  SystemColors.java
 *
 *  Display the colors used by the system
 *
 *  Copyright:  Northeast Parallel Architectures Center
 *  
 */

import java.applet.Applet;
import java.awt.Graphics;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Color;
import java.awt.SystemColor;
import java.util.Hashtable;
import java.util.Enumeration;

public class SystemColors extends Applet {

   // size of each color swatch:
   private final int swatchSize = 70;
   // whitespace around each color swatch:
   private final int border = 15;
   // total size, including whitespace:
   private final int unitSize = swatchSize + 2*border;
   // number of color swatches per row:
   private final int cols = 4;

   // A hashtable of colors:
   private Hashtable colors = new Hashtable();
   
   // Put the system colors in the hashtable:
   public void init() {
      colors.put( "desktop", SystemColor.desktop );
      colors.put( "activeCaption", SystemColor.activeCaption );
      colors.put( "activeCaptionText", SystemColor.activeCaptionText );
      colors.put( "activeCaptionBorder", SystemColor.activeCaptionBorder );
      colors.put( "inactiveCaption", SystemColor.inactiveCaption );
      colors.put( "inactiveCaptionText", SystemColor.inactiveCaptionText );
      colors.put( "inactiveCaptionBorder", SystemColor.inactiveCaptionBorder );
      colors.put( "window", SystemColor.window );
      colors.put( "windowBorder", SystemColor.windowBorder );
      colors.put( "windowText", SystemColor.windowText );
      colors.put( "menu", SystemColor.menu );
      colors.put( "menuText", SystemColor.menuText );
      colors.put( "text", SystemColor.text );
      colors.put( "textText", SystemColor.textText );
      colors.put( "textHighlight", SystemColor.textHighlight );
      colors.put( "textHighlightText", SystemColor.textHighlightText );
      colors.put( "textInactiveText", SystemColor.textInactiveText );
      colors.put( "control", SystemColor.control );
      colors.put( "controlText", SystemColor.controlText );
      colors.put( "controlHighlight", SystemColor.controlHighlight );
      colors.put( "controlLtHighlight", SystemColor.controlLtHighlight );
      colors.put( "controlShadow", SystemColor.controlShadow );
      colors.put( "controlDkShadow", SystemColor.controlDkShadow );
      colors.put( "scrollbar", SystemColor.scrollbar );
      colors.put( "info", SystemColor.info );
      colors.put( "infoText", SystemColor.infoText );
   }
   
   public void paint( Graphics g ) {
      
      // local variables:
      int x, y;
      Color color;
      String colorstring;

      g.setFont( new Font( "SansSerif", Font.PLAIN, 9 ) );
      
      // vertical adjustment for color labels:
      int descender = g.getFontMetrics().getDescent();
      
      // an enumeration of keys in the hashtable:
      Enumeration colorstrings = colors.keys();
      
      // Display rows of color boxes:
      int rows = ( int ) Math.ceil( ( double ) colors.size()/cols );
      for ( int i = 0; i < rows; i++ ) {
         y = border + unitSize*i;
         for ( int j = 0; j < cols; j++ ) {
            if ( !colorstrings.hasMoreElements() ) break;
            colorstring = ( String ) colorstrings.nextElement();
            color = ( Color ) colors.get( colorstring );
            x = border + unitSize*j;
            g.setColor( color );
            g.fillRect( x, y, swatchSize, swatchSize );
            g.setColor( Color.black );
            g.drawString( colorstring, x, y - descender );
         }
      }
      
   }

}
   
