/*
 *  File:  FontList.java
 *
 *  Create a scrolling list of fonts
 *
 *  Copyright:  Northeast Parallel Architectures Center
 *  
 */

import java.awt.*;
import java.awt.event.*;

public class FontList extends java.applet.Applet
                      implements ItemListener, ActionListener {

  // Allow five visible items in the list, but 
  // disallow multiple selections:
  private List fontList = new List( 5, false );
  
  private Checkbox boldCheckbox, italicCheckbox;
  
  private String fontString1, fontString2;
  private Label fontLabel1, fontLabel2;
  private Font font = new Font( "Monospaced", Font.PLAIN, 18 );

  public void init() {
    setBackground( Color.white );

    // Color the scrolling list:
    Color lightBlue = new Color( 0xB0, 0xE0, 0xE6 );
    Color  navyBlue = new Color( 0x19, 0x19, 0x70 );
    fontList.setBackground( lightBlue );
    fontList.setForeground(  navyBlue );
    fontList.setFont( new Font( "SansSerif", Font.BOLD, 18 ) );
    
    // Get list of system fonts:
    Toolkit toolkit = Toolkit.getDefaultToolkit();
    String fonts[] = toolkit.getFontList();
      
    // Add fonts to the scrolling list:
    for ( int i = 0; i < fonts.length; i++ ) {
      fontList.addItem( fonts[i] );
    }
    
    // Construct a text label:
    fontString1 = "How razorback-jumping frogs";
    fontString2 = "can level six piqued gymnasts!";
    fontLabel1 = new Label( fontString1, Label.CENTER );
    fontLabel2 = new Label( fontString2, Label.CENTER );
    fontLabel1.setFont( font );
    fontLabel2.setFont( font );
    
    // Define checkboxes:
    boldCheckbox = new Checkbox( "Bold" );
    italicCheckbox = new Checkbox( "Italic" );
    
    // Add the components to separate panels:
    Panel listPanel = new Panel();
    listPanel.add( fontList );
    Panel labelPanel = new Panel();
    labelPanel.setLayout( new GridLayout( 2, 1, 5, 5 ) );
    labelPanel.add( fontLabel1 );
    labelPanel.add( fontLabel2 );
    Panel boxPanel = new Panel();
    boxPanel.add( boldCheckbox );
    boxPanel.add( italicCheckbox );
    
    // Set layout to BorderLayout and add the panels:
    setLayout( new BorderLayout( 5, 5 ) );
    add( boxPanel, BorderLayout.NORTH );
    add( listPanel, BorderLayout.CENTER );
    add( labelPanel, BorderLayout.SOUTH );
    
    // Determine default font:
    String defaultFontName = font.getName();
    
    // Select default font in scrolling list:
    for ( int i = 0; i < fonts.length; i++ ) {
      if ( defaultFontName.equals( fonts[i] ) ) {
        fontList.select(i); break;
      }
    }
    
    // Register the applet to listen for events:
    fontList.addItemListener( this );
    fontList.addActionListener( this );
    boldCheckbox.addItemListener( this );
    italicCheckbox.addItemListener( this );
  }
  
  public void itemStateChanged( ItemEvent event ) {
    String fontName = fontList.getSelectedItem();
    int style = Font.PLAIN;
    if ( boldCheckbox.getState() ) style += Font.BOLD;
    if ( italicCheckbox.getState() ) style += Font.ITALIC;
    int size = font.getSize();
    fontLabel1.setFont( new Font( fontName, style, size ) );
    fontLabel2.setFont( new Font( fontName, style, size ) );
    fontLabel1.setText( fontString1 );
    fontLabel2.setText( fontString2 );
    showStatus( "" );
  }
  
  public void actionPerformed( ActionEvent event ) {
    String fontName = fontList.getSelectedItem();
    showStatus( "You chose " + fontName + "!" );
  }
  
}
