/*
 *  File:  FontMetricsTest3.java
 *
 *  Given two strings, center the strings both vertically and
 *  horizontally in the applet window.
 *
 *  Copyright:  Northeast Parallel Architectures Center
 *  
 */

import java.applet.Applet;
import java.awt.Graphics;
import java.awt.Font;
import java.awt.FontMetrics;

public class FontMetricsTest3 extends Applet {

   private String s0, s1;  // two strings to be drawn
   
   public void init() {
      
      // Get the "string0" parameter from the HTML document:
      s0 = getParameter( "string0" );
      if ( s0 == null ) s0 = "string0 goes here";
      
      // Get the "string1" parameter from the HTML document:
      s1 = getParameter( "string1" );
      if ( s1 == null ) s1 = "string1 goes here";
      
      // Get the "fontName" parameter from the HTML document:
      String name = getParameter( "fontName" );
      if ( name == null ) name = "Serif";
      
      // Instantiate a font object:
      Font f = new Font( name, Font.BOLD + Font.ITALIC, 24 );
      
      // Set the font:
      this.getGraphics().setFont( f );
      
       // Set the background color of the applet:
      this.setBackground( java.awt.Color.cyan );
      
   }
   
   public void paint( Graphics g ) {
      
      // Get the FontMetrics object associated with this applet:
      FontMetrics fm = g.getFontMetrics();
      
      // Determine the width of the applet:
      int appletWidth = this.getSize().width;
      
      // Determine the width of the first string:
      int stringWidth = fm.stringWidth( s0 );
      
      // Calculate the x-coordinate of the first string:
      int x = (appletWidth - stringWidth)/2;
      
      // Determine the height of the applet:
      int appletHeight = this.getSize().height;
      
      // Calculate the height of *any* string:
      int ascent = fm.getAscent();
      int descent = fm.getDescent();
      int stringHeight = ascent + descent;
      
      // Calculate the height of both strings:
      int leading = fm.getLeading();
      int height = 2 * stringHeight + leading;
      
      // Calculate the y-coordinate of the first string:
      int y = (appletHeight - height)/2 + ascent;
      
      // Draw the first string at point (x,y):
      g.drawString( s0, x, y );
      
      // Calculate the width of the second string:
      stringWidth = fm.stringWidth( s1 );
      
      // Calculate the x-coordinate of the second string:
      x = (appletWidth - stringWidth)/2;
      
      // Calculate the font height:
      int fontHeight = stringHeight + leading;
      
      // Calculate the y-coordinate of the second string:
      y += fontHeight;
            
      // Draw the second string at point (x,y):
      g.drawString( s1, x, y );
      
   }
   
}
