/*
 *  File:  FontMetricsTest4.java
 *
 *  Given  n  strings, center them both vertically and
 *  horizontally in the applet window.
 *
 *  Copyright:  Northeast Parallel Architectures Center
 *  
 */

import java.applet.Applet;
import java.awt.Graphics;
import java.awt.Font;
import java.awt.Color;
import java.awt.FontMetrics;

public class FontMetricsTest4 extends Applet {

   private int n;       // number of strings
   private String[] s;  // an array of strings to be drawn
   
   public void init() {
      
      // Get the "numStrings" parameter from the HTML document:
      String nStr = getParameter( "numStrings" );
      // Convert string 'nStr' to an integer:
      n = ( nStr == null ) ? 0 : Integer.parseInt( nStr );
      
      // Instantiate an array of strings:
      s = new String[n];
      
      // Get the strings to be drawn from the HTML document:
      for ( int i = 0; i < n; i++ ) {
        s[i] = getParameter( "string" + i );
        if ( s[i] == null ) s[i] = "";
      }
      
      // Get the "fontName" parameter from the HTML document:
      String name = getParameter( "fontName" );
      if ( name == null ) name = "Serif";
      
      // Get the "bgColor" parameter from the HTML document:
      String bgColorStr = getParameter( "bgColor" );
      // Default background color is white:
      if ( bgColorStr == null ) bgColorStr = "#FFFFFF";
      // Convert hexadecimal color string to an integer:
      int bgColor = colorStringToInt( bgColorStr );
      
      // Instantiate a font object:
      Font f = new Font( name, Font.PLAIN, 18 );
      
      // Set the font:
      this.getGraphics().setFont( f );
      
      // Set the background color of the applet:
      this.setBackground( new Color( bgColor ) );
      
   }
   
   public void paint( Graphics g ) {
      
      // If there are no strings to draw, do nothing:
      if ( n == 0 ) return;
      
      // Get the FontMetrics object associated with this applet:
      FontMetrics fm = g.getFontMetrics();
      
      // Determine the width of the applet:
      int appletWidth = this.getSize().width;
      
      // Determine the width of the first string:
      int stringWidth = fm.stringWidth( s[0] );
      
      // Calculate the x-coordinate of the first string:
      int x = (appletWidth - stringWidth)/2;
      
      // Determine the height of the applet:
      int appletHeight = this.getSize().height;
      
      // Calculate the height of *any* string:
      int ascent = fm.getAscent();
      int descent = fm.getDescent();
      int stringHeight = ascent + descent;
      
      // Calculate the height of  n  strings:
      int leading = fm.getLeading();
      int height = n * stringHeight + (n - 1) * leading;
      
      // Calculate the y-coordinate of the first string:
      int y = (appletHeight - height)/2 + ascent;
      
      // Draw the first string at point (x,y):
      g.drawString( s[0], x, y );
      
      // Calculate the font height:
      int fontHeight = stringHeight + leading;

      for ( int i = 1; i < n; i++ ) {
         
         // Calculate the width of this string:
         stringWidth = fm.stringWidth( s[i] );
         
         // Calculate the x-coordinate of this string:
         x = (appletWidth - stringWidth)/2;
         
         // Calculate the y-coordinate of this string:
         y += fontHeight;
               
         // Draw the current string at point (x,y):
         g.drawString( s[i], x, y );
         
      }
      
   }
   
   private int colorStringToInt( String colorString ) {
   
      // Strip off whitespace from color string:
      colorString = colorString.trim();
      
      // Strip off leading '#' from color string, if necessary:
      if ( colorString.charAt(0) == '#' ) {
        colorString = colorString.substring(1);
      }
      
      // Convert hexadecimal color string to an integer:
      return Integer.parseInt( colorString, 16 );
      
   }
   
}
