/*
 *  File:  FontModel.java
 *
 *  Given two strings and a point, draw the first string at the 
 *  point and the second string underneath the first.  For each 
 *  string, draw the baseline under the string and the bounding 
 *  box around the string.
 *
 *  Copyright:  Northeast Parallel Architectures Center
 *  
 */

import java.applet.Applet;
import java.awt.Graphics;
import java.awt.Font;
import java.awt.FontMetrics;

public class FontModel extends Applet {

   private String s1, s2;
   public String fontName = "Serif";
   public String fontSizeString = "24";
   
   public void init() {
      
      // The strings to be drawn:
      s1 = "How razorback-jumping frogs";
      s2 = "can level six piqued gymnasts!";
      
      // Set the background color of the applet:
      this.setBackground( java.awt.Color.cyan );
      
   }
   
   public void paint( Graphics g ) {
      
      // Save the default font:
      Font defaultFont = g.getFont();
      
      // Instantiate a font object and set the font:
      int fontSize = Integer.parseInt( fontSizeString );
      Font displayFont = new Font( fontName, Font.BOLD, fontSize );
      g.setFont( displayFont );
      
      // Calculate the height of *any* string in this font:
      int ascent = g.getFontMetrics().getAscent();
      int descent = g.getFontMetrics().getDescent();
      int stringHeight = ascent + descent;
      
      // The location of the first string:
      int x = 25, y = 75;
      
      // Draw the first string at point (x,y):
      g.drawString( s1, x, y );
      
      // Determine the width of the first string:
      int stringWidth = g.getFontMetrics().stringWidth( s1 );
      
      // Draw the baseline under the first string:
      g.drawLine( x, y, x + stringWidth, y);
      
      // Draw a dot centered at (x,y):
      int r = 3;  // radius of the dot
      g.fillOval( x - r, y - r, 2*r, 2*r );
      
      // Draw the bounding box around the first string:
      g.drawRect( x, y - ascent, stringWidth, stringHeight );
      
      // Draw labels next to the bounding box:
      g.setFont( defaultFont );
      int dy = g.getFontMetrics().getAscent();
      g.drawString( "ascent", x + stringWidth + 5, y - ascent + dy );
      g.drawString( "descent", x + stringWidth + 5, y + descent );
      g.setFont( displayFont );
      
      // Determine the font height:
      int fontHeight = g.getFontMetrics().getHeight();
      
      // Calculate the y-coordinate of the second string:
      y += fontHeight;
            
      // Draw the second string at point (x,y):
      g.drawString( s2, x, y );
      
      // Determine the width of the second string:
      stringWidth = g.getFontMetrics().stringWidth( s2 );
      
      // Draw the baseline under the second string:
      g.drawLine( x, y, x + stringWidth, y);
      
      // Draw a dot centered at (x,y):
      g.fillOval( x - r, y - r, 2*r, 2*r );
      
      // Draw the bounding box around the first string:
      g.drawRect( x, y - ascent, stringWidth, stringHeight );
      
      // Restore the default font:
      g.setFont( defaultFont );
   }
   
}
