/*
 *  MouseEvents.java
 *
 *  MouseEvents and movable objects
 */

import java.applet.*;
import java.awt.*;
import java.awt.event.*;

public class MouseEvents extends Applet 
                         implements MouseListener, MouseMotionListener {
                          
  // Instance variables:
  private final int numObj = 3;    // number of movable objects
  private MovableObject object[];  // array of movable objects
  private Image buffer;            // image object for double buffering
  private Graphics gOffScreen;     // graphics object for double buffering
  
  // Mouse handling variables:
  private int current = -1;        // index of object being dragged
  private int hit = -1;            // index of object containing cursor
  private int xoffset = 0;         // relative x-coordinate of mouse click
  private int yoffset = 0;         // relative y-coordinate of mouse click
  
  // Default object color:
  private final Color objColor = Color.cyan;
  // Selected object color:
  private final Color objColorSelect = Color.red;
  // Applet colors:
  private final Color bgColor = Color.black;
  private final Color fgColor = Color.white;

  public void init() {
  
    setBackground( bgColor );
    setForeground( fgColor );

    // Instantiate object array:
    object = new MovableObject[ numObj ];

    // Instantiate a rectangle:
    object[0] = new MovableRectangle( 10, 10, 100, 100 );
    object[0].setColor( objColor );

    // Instantiate a circle:
    object[1] = new MovableOval( 200, 10, 100, 100 );
    object[1].setColor( objColor );

    // Instantiate a triangle:
    object[2] = new MovableTriangle( 10, 200, 100, 100 );
    object[2].setColor( objColor );

    // Create an off-screen image for double buffering:
    buffer = createImage( getSize().width, getSize().height );
    // Get off-screen graphics context:
    gOffScreen = buffer.getGraphics(); 

    // Register the applet to listen for ALL mouse events:
    addMouseListener( this );
    addMouseMotionListener( this );
    
  }  // end method init()

  // Override java.awt.Component.update:
  public void update( Graphics g ) {
    // Fill background:
    gOffScreen.setColor( getBackground() );
    gOffScreen.fillRect( 1, 1, getSize().width - 2, getSize().height - 2 );
    // Draw boundary:
    gOffScreen.setColor( getForeground() );
    gOffScreen.drawRect( 0, 0, getSize().width - 1, getSize().height - 1 );
    paint( gOffScreen );
    
    // Draw the buffer in the applet window:
    g.drawImage( buffer, 0, 0, this );
  }
  
  // Override java.awt.Component.paint:
  public void paint( Graphics g ) {
    // Paint each object:
    for ( int i = 0; i < numObj; i++ ) {
      object[i].paint(g);
    }
  }

  // MouseMotionListener event handlers:
  public void mouseMoved( MouseEvent e ) {
    hit = -1;
    for ( int i = 0; i < numObj; i++ ) {
      if ( object[i].contains( e.getX(), e.getY() ) ) {
        hit = i; break;
      }
    }
    
    for ( int i = 0; i < numObj; i++ ) {
      object[i].setColor( (i == hit) ? objColorSelect : objColor );
    }
    repaint();
  }
  public void mouseDragged( MouseEvent e ) {
    if ( current != -1 ) {
      int x = e.getX() + xoffset;
      int y = e.getY() + yoffset;
      object[current].setLocation( x, y );
      repaint();
    }
  }

  // MouseListener event handlers:
  public void mousePressed( MouseEvent e ) {
    current = hit;
    if ( current != -1 ) {
      xoffset = object[current].getX() - e.getX();
      yoffset = object[current].getY() - e.getY();
    }
  }
  public void mouseReleased( MouseEvent e ) {
    current = -1;
  }
  public void mouseClicked( MouseEvent e ) { }
  public void mouseEntered( MouseEvent e ) { }
  public void mouseExited( MouseEvent e ) { }

}  // end class MouseEvents

