/*
 *  File:  Paint.java
 *
 *  A paint program
 *
 *  Copyright:  Northeast Parallel Architectures Center
 *  
 */

import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;

public class Paint extends Applet implements ActionListener {

  String localTools[] = { "Draw", "Redraw", "Clear" };
  Label toolLabel, counterLabel;
  DrawCanvas canvas;
  
  public void init() {
  
    setBackground(Color.white);
    setForeground(Color.black);
    setLayout( new BorderLayout() );

    // Create a canvas and add it to the applet:
    canvas = new DrawCanvas();
    add( canvas, BorderLayout.CENTER );
    
    // A generic button:
    Button button;
    // A panel of button tools:
    Panel toolPanel = new Panel();
    // Get a list of drawing tools:
    String tools[] = canvas.getTools();
    // The total number of tools:
    int n = tools.length + localTools.length;
    // Buttons are arranged vertically:
    toolPanel.setLayout( new GridLayout( n, 1 ) );
    // Add each button to the button panel and register the
    // applet to listen for button events (i.e., ActionEvents):
    for ( int i = 0; i < tools.length; i++ ) {
      toolPanel.add( button = new Button( tools[i] ) );
      button.addActionListener( this );
    }
    for ( int i = 0; i < localTools.length; i++ ) {
      toolPanel.add( button = new Button( localTools[i] ) );
      button.addActionListener( this );
    }
    
    // Add the tool panel to the applet:
    add( toolPanel, BorderLayout.WEST );

    // Labels for the status line:
    toolLabel = new Label( canvas.getTool(), Label.CENTER );
    counterLabel = new Label();
    counterLabel.setAlignment( Label.RIGHT );
    setCounterLabel();

    // Build the status line and add it to the applet:
    Panel statusLine = new Panel();
    statusLine.setLayout( new GridLayout( 1, 3 ) );
    statusLine.add( new Label() );  // a placeholder
    statusLine.add( toolLabel );
    statusLine.add( counterLabel );
    add( statusLine, BorderLayout.SOUTH );

    // Register the applet to listen for MouseEvents generated 
    // by the canvas:
    canvas.addMouseListener( 
      new MouseAdapter() {
        public void mouseReleased( MouseEvent e ) {
          setCounterLabel();
        }
      }
    );
    
  }  // end method init
  
  public void setCounterLabel() {
    Integer objectCounter = new Integer( canvas.getNumObj() );
    counterLabel.setText( objectCounter.toString() );
  }
  
  // This method implements the ActionListener interface:
  public void actionPerformed( ActionEvent e ) {

    Button button = (Button) e.getSource();
    String command = e.getActionCommand();
    
    if ( command.equals( "Draw" ) ) {
      button.setLabel( "Fill" );
      canvas.setFill( false );
    } else if ( command.equals( "Fill" ) ) {
      button.setLabel( "Draw" );
      canvas.setFill( true );
    } else if ( command.equals( "Redraw" ) ) {
      canvas.repaint();
    } else if ( command.equals( "Clear" ) ) {
      canvas.clear();
      setCounterLabel();
    } else {
      canvas.setMode( command );
      toolLabel.setText( command );
    }
    
  }  // end method actionPerformed
  
}  // end class Paint
