/*
 *  File:  DrawablePoint.java
 *
 *  Custom point class for Java 1.1
 *
 *  Copyright:  Northeast Parallel Architectures Center
 *  
 */

import java.awt.Point;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Component;

/*
 *  DrawablePoint inherits variables  x  and  y  from Point
 * 
 */

public class DrawablePoint extends Point implements Drawable {

   // instance variable:
   private Color color;

   // DrawablePoint constructor #1:
   public DrawablePoint() {

      // invoke the no-argument constructor of the superclass:
      super();

   }

   // DrawablePoint constructor #2:
   public DrawablePoint( int x, int y ) {
       
      // invoke a constructor of the superclass:
      super( x, y );
      
   }

   // DrawablePoint constructor #3:
   public DrawablePoint( Point p ) {
       
      // invoke a constructor of the superclass:
      super( p );
      
   }

   // implement the methods of the Drawable interface:
   public Color getColor() { return color; }
   public void setColor( Color color ) {
      this.color = color;
   }
   public void draw( Graphics g ) {
      if ( this.color != null ) g.setColor( this.color );
      g.drawLine( x, y, x, y );
   }
   public void draw( Component c ) {
      this.draw( c.getGraphics() );
   }
   public void fill( Graphics g ) {
      this.draw( g );
   }
   public void fill( Component c ) {
      this.fill( c.getGraphics() );
   }
   
   // rotate a point about the origin:
   public DrawablePoint rotate( double theta ) {
   
      final double cos_theta = Math.cos( theta );
      final double sin_theta = Math.sin( theta );
      
      int new_x = ( int ) Math.round( x * cos_theta - y * sin_theta );
      int new_y = ( int ) Math.round( x * sin_theta + y * cos_theta );
      
      this.move( new_x, new_y );

      // return the rotated point as a side effect:
      return this;
      
   }
   
}
