/*
 *  File:  MovingSquare.java
 *
 *  A moving square with boundary checking
 *
 *  Copyright:  Northeast Parallel Architectures Center
 *
 */

import java.applet.Applet;
import java.awt.*;

public class MovingSquare extends Applet implements Runnable {

  // A reference to a square:
  private Square square;
  
  // Override java.applet.Applet.init:
  public void init () {

    setBackground( Color.black );
    setForeground( Color.white );
    
    // Instantiate a square:
    square = new Square( 75, 25, 50 );
    square.setColor( Color.red );
    
  }

  // A thread of execution:
  private Thread thread;
    
  // Override java.applet.Applet.start:
  public void start() {
    if ( thread == null ) {
      thread = new Thread( this );
      thread.start();
    }
  }
  
  // Override java.applet.Applet.stop:
  public void stop() {
    if ( thread != null ) {
      thread.stop();
      thread = null;
    }
  }
  
  // Implement java.lang.Runnable.run:
  public void run() {
    while ( thread != null ) {
      try {
        Thread.sleep( 20 );
      } catch ( InterruptedException e ) {
        // do nothing   
      }
      repaint();
    }
  }

  // Initially, the square moves two pixels down and to the right:
  int dx = 2, dy = 2;
  
  // Override java.awt.Component.paint:
  public void paint( Graphics g ) {
  
    // Check if square is out of bounds:
    checkBounds( this.getBounds() );
    
    // Move and fill the square:
    square.translate( dx, dy );
    square.fill( g );
    
  }
  
  private void checkBounds( java.awt.Rectangle r ) {

    // Get the bounding box of the square:
    java.awt.Rectangle bb = square.getBounds();

    // Compute the dimensions of the square:
    int w = bb.width, h = bb.height;
    int new_x = bb.x + dx, new_y = bb.y + dy;
    
    // If the square is out of bounds, reverse direction:
    if ( ( new_x < r.x ) || ( new_x + w > r.x + r.width ) )  dx *= -1;
    if ( ( new_y < r.y ) || ( new_y + h > r.y + r.height ) )  dy *= -1;

  }

}
