/*
 *  File:  TriangleTest.java
 *
 *  Draw a Sierpinski triangle (a fractal)
 *
 *  Copyright:  Northeast Parallel Architectures Center
 *  
 */

import java.applet.Applet;
import java.awt.Graphics;
import java.awt.Color;
import java.awt.Point;

public class TriangleTest extends Applet {

   // create a reference to a triangle:
   private Triangle triangle;
   
   // create references to the vertices of a triangle:
   private Point v0, v1, v2;
   
   // maximum depth of recursion:
   private int maxDepth;      
   
   public void init() {
   
      // triangle vertices:
      v0 = new Point( 150,  25 );
      v1 = new Point( 294, 275 );
      v2 = new Point(   6, 275 );
      
      // get 'maxDepth' parameter from the HTML document:
      String maxDepthStr = getParameter( "maxDepth" );
      if ( maxDepthStr == null ) {
         maxDepth = 3;  // default
      } else {
         maxDepth = Integer.parseInt( maxDepthStr );
      }

   }
   
   public void paint( Graphics g ) {

      // draw initial triangle:
      triangle = new Triangle( v0, v1, v2 );
      triangle.setColor( Color.black );
      triangle.fill( g );
      
      // divide and fill:
      divideAndFill( g, 0 );

   }
   
   private void divideAndFill( Graphics g, int depth ) {
      
      if ( depth < maxDepth ) {
          
          // get the vertices of this triangle:
          Point[] points = triangle.getPoints();
          
          // compute the midpoint of each side:
          Point p01 = midpoint( points[0], points[1] );
          Point p02 = midpoint( points[0], points[2] );
          Point p12 = midpoint( points[1], points[2] );
          
          // fill the inner triangle:
          Triangle innerTriangle = new Triangle( p01, p02, p12 );
          innerTriangle.setColor( Color.white );
          innerTriangle.fill( g );
          
          // recurse on three subtriangles:
          triangle = new Triangle( points[0], p01, p02 );
          divideAndFill( g, depth + 1 );
          triangle = new Triangle( points[1], p01, p12 );
          divideAndFill( g, depth + 1 );
          triangle = new Triangle( points[2], p02, p12 );
          divideAndFill( g, depth + 1 );
          
      }
       
   }
   
   private Point midpoint( Point p, Point q ) {

      return new Point( (int) Math.round( (p.x + q.x)/2.0 ), 
                        (int) Math.round( (p.y + q.y)/2.0 )  );

   }
   
}
