/*
 * This application extends the previous BounceThread so that
 * ach ball is a different thread with a different time to sleep.
 * 
 */

import java.awt.*;
import java.awt.event.*;

public class BounceTimes extends Frame implements ActionListener
{
   private Canvas canvas;

   // the constructor initializes the frame to be a borderlayout 
   //    with a canvas in the center and
   //    two buttons labelled Start and Close in the south

   public BounceTimes()
   {  setLayout(new BorderLayout());
      setTitle("BounceTimes");

      canvas = new Canvas();
      add(canvas, BorderLayout.CENTER);

      Panel p = new Panel();
      Button b1 = new Button("Start");
      b1.addActionListener(this);
      p.add(b1);
      Button b2 = new Button("Close");
      b2.addActionListener(this);
      p.add(b2);
      add(p, BorderLayout.SOUTH);
   }

   // responds to the Start button by creating a new instance 
   //          of the BallTimes class and starting its thread
   // responds to the close button by exiting the application

   public void actionPerformed(ActionEvent evt)
   {  if (evt.getActionCommand().equals("Start"))
      {  BallTimes b = new BallTimes(canvas);
         b.start();
      }
      else if (evt.getActionCommand().equals("Close"))
         System.exit(0);
   }

   // the main method creates an instance of this class and sets the
   // properties of the frame 

   public static void main(String[] args)
   {  Frame f = new BounceTimes();
      f.setSize(400, 300);
      f.setVisible(true);  
   }
   
}

/* The BallTimes class is a subclass of Thread, so each instance inherits
 *   thread methods start, stop, sleep, etc.
 * The run method of the thread loops to draw a ball in a new location
 *   for a fixed number of iterations.
 * This example has an additional variable sleeptime, which is used to
 *   sleep inbetween iterations.
 */
  
class BallTimes extends Thread
{  
   private Canvas box;
   private static final int XSIZE = 10;
   private static final int YSIZE = 10;
   private int x = 0;
   private int y = 0;
   private int dx = 2;
   private int dy = 2;   
   private int sleeptime;

   // the constructor initializes the canvas to that of the frame
   // and initializes sleeptime to a random number between 4 and 44
   public BallTimes(Canvas c) 
   { box = c;
     sleeptime = ((int)(Math.random() * 40)) + 4;
   }

   // draws the ball in the current location
   public void draw()
   {  Graphics g = box.getGraphics();
      g.fillOval(x, y, XSIZE, YSIZE);
      g.dispose();
   }
   
   // increments the location of the ball, checks the boundary to see
   // if it should "bounce", and draws the ball
   public void move()
   {  Graphics g = box.getGraphics();
      g.setXORMode(box.getBackground());
      g.fillOval(x, y, XSIZE, YSIZE);
      x += dx;
      y += dy;
      Dimension d = box.getSize();
      if (x < 0) { x = 0; dx = -dx; }
      if (x + XSIZE >= d.width) { x = d.width - XSIZE; dx = -dx; }
      if (y < 0) { y = 0; dy = -dy; }
      if (y + YSIZE >= d.height) { y = d.height - YSIZE; dy = -dy; }
      g.fillOval(x, y, XSIZE, YSIZE);
      g.dispose();
   }
   
   // loops for 1000 times and moves the ball
   public void run()
   {  draw();
      for (int i = 1; i <= 1000; i++)
      {  move();
         try { Thread.sleep(sleeptime); } catch(InterruptedException e) {}
      }
   }
}



