/*
   File : CharLis.java
   Author: Stefan Robila

   Description: This is an implementation of a list of characters. It does
not differ in many things from a normal one. There are still some
particularities. First, because the Lsystems are refined by replacing all
the occurences of a character by a string, this translates in my list by a
the replacement of all the nodes containing the replacing character by a
list formed of nodes that contain the characters that form the string. So,
if I have the list:
F--F--F 

and F is replaced by F++F then my list, after expansion will be:

F++F--F++F--F++F

where every character corresponds in fact to a character in the list. I
have to say that I was not able to locate a code written in java for doing
this and I had to write my own. This is why I can affirm that this version
of the implementation is totally my work.

Second, the printList is a traversing of the list with the interpretation
of every character from the list according to the turtle graphics.

If you are asking why I chose to complicate the problem insteda of working
directly with String I am answering that I tried to get a larger size of
my pattern (if working with strings I would have had to limit myself to
256 characters) and, therefore, to get less levels of refination.

I wrote these explanations because Ifelt the need to show that my homework
is not just an application of the usage of differenr components, layouts,
frames. It is also work with list.

The structure of the CharList class is:

public class CharList{

Data:
----
     char l;                - the information field from the node
     CharList next;         - the link to the next node

Methods:
-------

public CharList(char x);        - constructor from a character

public CharList(CharList list); - copy constructor 

public void replaceList(char x,String rep);
                                - replaces every occurence of the
                                  character x in the list with a list
                                  formed of the characters from rep

public void printList(Graphics g,
                      float initialangle,
                      int angle,
                      int xinitial,
                      int yinitial,
                      float size
                      );
                               - interprets the list according to the
                                 turtle graphics
};

*/

// The code:
// --------
   
import java.awt.*;

public class CharList{
     char l;
     CharList next;

// constructor from a character
public CharList(char x){ l=x; next=null; };

// copy constructor
public CharList(CharList list){ l=list.l; next=list.next;};

// the function replaceList is replacing every occurence of x with a
// sublist formed of the characters that make up rep
public void replaceList(char x,String rep)
{
  CharList node1,node2,node3,node4;

  // node 1 is the list
  node1 = new CharList(this);

  // first, reverse the list !
  node3 = null;
  node2 = null;
  while (node1 !=null){
     node3 = new CharList(node1.l);
     node3.next = node2;
     node2 = node3;
     node1 = node1.next;
  };

  // put the reversed list in node1 again
  node1 = node3;

  // do the reverse again, this time, every time when you find x, put
  // all the nodes corresponding to the rep string
  node3 = null;
  node2 = null;
  while (node1 != null)
  {
    if (node1.l == x)
    // found x
    {
      // put all the nodes corresponding to the characters from rep
      for (int i=rep.length()-1;i>=0;i--)
      {
        node3 = new CharList(rep.charAt(i));
        node3.next = node2;
        node2 = node3;
      };
    }
    else {
      // x not found in this node, just add the node back to the list
      node3 = new CharList (node1.l);
      node3.next = node2;
      node2 = node3;
    };
     node1 = node1.next;
  };

  l = node3.l;
  next = node3.next;

  /* 
Conclusion: Initially I reversed the list. Then, while reversing it
for the second time, for every time when x was found I placed all the
characters of the rep string in the list. At the end I have the list back
but with all the x replaced by rep
 */ 
};


// the function printList() traversates the list and for every character
// does an action corresponding to the turtle graphics. Because this means
// drawing, I had to provide the start point (angle, x,y) of my turtle,
// the size of the segments to be drawn and the angle to be modified 
public void printList(Graphics g,
                      float initialangle,
                      int angle,
                      int xinitial,
                      int yinitial,
                      float size
                      )
{
    // initialize all the graphic information
    float fangle = (float)Math.PI*angle/180;
    float currentangle=initialangle; 
    int currentx = xinitial; 
    int currenty = yinitial; 
    int tempx,tempy; 
    CharList node = new CharList(this);

    // traversate the list
    while (node != null){
    switch(node.l) 
    {
      // found F, this means draw a segment, the parameters are computed
      // first
      case 'F': //move
                tempx = currentx +
                        Math.round(size*(float)Math.sin(currentangle)); 
                tempy = currenty +
                        Math.round(size*(float)Math.cos(currentangle)); 
                g.drawLine(currentx,currenty,tempx,tempy); 
                currentx = tempx; 
                currenty = tempy; 
                break; 
      // found a +, just add to the angle the standard angle
      case '+': // increase angle
                currentangle +=fangle; 
                break; 
      // similar to + but subtract
      case '-': // decrease angle
                currentangle -=fangle; 
                break; 
      default: 
                break; 
     };
     node = node.next;
   };
 };

};

