import java.awt.*;
import java.awt.event.*;
import java.lang.Integer;
import java.util.Calendar;
import java.io.File;
import javax.swing.*;
import javax.swing.event.*;

/**
 * LoginDialog - Class used to log into the talk server.
 *
 * @author Paul Kronenwetter
 * @author Jayme Manning
 *
 * <P ALIGN=CENTER><B>
 *                              Paul Kronenwetter<BR>
 * 				  Jayme Manning<BR>
 *                      Trade Secrets and/or Confidential<BR>
 *                      Commercial or Financial Information<BR>
 *                   Exempt From Disclosure Under the Freedom<BR>
 *                      of Information Act (5 USC 552(b)(3)<BR>
 *                  and 5 USC 552(b)(4)) and Under 18 USC 1905<BR>
 * </B></P>
 * <P ALIGN=CENTER>
 *                      <B>Proprietary Information Notice</B><BR>
 * <HL ALIGN=CENTER>
 * This document contains information proprietary to, and is the
 * sole property of, Paul Kronenwetter and Jayme Manning.  It shall not
 * be reproduced, used or disclosed in any manner or for any purpose 
 * not authorized in writing by both Paul Kronenwetter and 
 * Jayme Manning, and except, as retention may be so authorized, it
 * shall be returned to Paul Kronenwetter or Jayme Manning upon
 * request.</P>
 * <P ALIGN=CENTER><B>&copy 1998 by Paul Kronenwetter and Jayme
 * Manning<BR> All Rights Reserved</B></P>
 *
 * @version
 * $Revision: 1.3 $ <br>
 * $Date: 1998/12/17 16:03:38 $ <p>
 * 
 * <pre>
 * $Log: LoginDialog.java,v $
 * Revision 1.3  1998/12/17 16:03:38  cps606
 * Added isAvailable method.
 *
 * Revision 1.2  1998/12/15 22:55:57  cps606
 * * Manually edited the comments to correct formatting.
 *
 * Revision 1.1  1998/12/15 05:05:29  cps606
 * * LoginDialog - Class used to create dialog for use during login
 *   process. 
 *
 * </pre>
 */
public class LoginDialog extends JDialog {

	private Container self;
	private JLabel nameLabel;
	private JTextField nameTextField;
	private JLabel machineLabel;
	private JTextField machineTextField;
	private JLabel portLabel;
	private JTextField portTextField;

	private JButton submitButton;
	private JButton cancelButton;

	private GBC gbc;
	private GridBagLayout gbl;

	/**
	 * LoginDialog - Constructor used to build the basic dialog set.
	 * This will be attached to a dummy frame in the parent applet window.
	 *
	 * @param parent JFrame used as an anchor for the dialog box.
	 */
	public LoginDialog(JFrame parent)
	{
		super(parent,"Login:",true);
		gbc = new GBC();
		gbl = new GridBagLayout();
		self = getContentPane();

		self.setLayout(gbl);

		nameLabel = new JLabel("Name:");
		gbc.setupConstraints(0,0,1,1,100,100,GBC.WEST,GBC.NONE,0);
		gbl.setConstraints(nameLabel,gbc);
		self.add(nameLabel);

		nameTextField = new JTextField(25);
		gbc.setupConstraints(
			1,0,1,1,100,100,GBC.CENTER,GBC.HORIZONTAL,0);
		gbl.setConstraints(nameTextField,gbc);
		self.add(nameTextField);

		machineLabel = new JLabel("Machine:");
		gbc.setupConstraints(0,1,1,1,100,100,GBC.WEST,GBC.NONE,0);
		gbl.setConstraints(machineLabel,gbc);
		self.add(machineLabel);

		machineTextField = new JTextField(25);
		gbc.setupConstraints(
			1,1,1,1,100,100,GBC.CENTER,GBC.HORIZONTAL,0);
		gbl.setConstraints(machineTextField,gbc);
		self.add(machineTextField);

		portLabel = new JLabel("Port:");
		gbc.setupConstraints(0,2,1,1,100,100,GBC.WEST,GBC.NONE,0);
		gbl.setConstraints(portLabel,gbc);
		self.add(portLabel);

		portTextField = new JTextField(25);
		gbc.setupConstraints(
			1,2,1,1,100,100,GBC.CENTER,GBC.HORIZONTAL,0);
		gbl.setConstraints(portTextField,gbc);
		self.add(portTextField);

		submitButton = new JButton("Submit");
		gbc.setupConstraints(0,3,1,1,100,100,GBC.CENTER,GBC.NONE,0);
		gbl.setConstraints(submitButton,gbc);
		self.add(submitButton);
		
		cancelButton = new JButton("Cancel");
		gbc.setupConstraints(1,3,1,1,100,100,GBC.CENTER,GBC.NONE,0);
		gbl.setConstraints(cancelButton,gbc);
		self.add(cancelButton);

		pack();
	}

	/**
	 * getPreferredSize - Method used to tell the layout manager what size
	 * we want the box to be.
	 *
	 * @return Dimension - The preferred size of the dialog box.
	 */
	public Dimension getPreferredSize()
	{
		return new Dimension(250,150);
	}

	/**
	 * addActionListener - This method acts as a delegate to add an
	 * ActionListener event handler from another class to the two buttons
	 * on our dialog box.
	 *
	 * @param al ActionListener to be added to the buttons.
	 */
	public void addActionListener(ActionListener al)
	{
		submitButton.addActionListener(al);
		cancelButton.addActionListener(al);
	}

	/**
	 * resetFields - Method used to reset the entry fields for the
	 * dialog box.
	 */
	public void resetFields()
	{
		nameTextField.setText("");
		machineTextField.setText("");
		portTextField.setText("");
	}

	/**
	 * isAvaialble - Simple method used to determine if the boxes have
	 * been filled out.
	 *
	 * @return boolean - Whether or not all the data is available.
	 */
	public boolean isAvailable()
	{
		if((nameTextField.getText()).equals(""))
		{
			return false;
		}
		if((machineTextField.getText()).equals(""))
		{
			return false;
		}
		if((portTextField.getText()).equals(""))
		{
			return false;
		}

		return true;
	}

	/**
	 * getFields - Accessor method used to retrive the data from the
	 * dialog box.
	 *
	 * @return String[] - The username, machine name and port number.
	 */
	public String[] getFields()
	{
		String[] x = new String[3];
		x[0] = nameTextField.getText();
		x[1] = machineTextField.getText();
		x[2] = portTextField.getText();

		return x;
	}
}
