import java.awt.*;
import java.awt.event.*;
import java.lang.Integer;
import java.util.Calendar;
import java.io.File;
import javax.swing.*;
import javax.swing.event.*;

/**
 * UserListPane - Displays the list of users connected to the server.
 *
 * @author Paul Kronenwetter
 * @author Jayme Manning
 *
 * <P ALIGN=CENTER><B>
 *                              Paul Kronenwetter<BR>
 * 				  Jayme Manning<BR>
 *                      Trade Secrets and/or Confidential<BR>
 *                      Commercial or Financial Information<BR>
 *                   Exempt From Disclosure Under the Freedom<BR>
 *                      of Information Act (5 USC 552(b)(3)<BR>
 *                  and 5 USC 552(b)(4)) and Under 18 USC 1905<BR>
 * </B></P>
 * <P ALIGN=CENTER>
 *                      <B>Proprietary Information Notice</B><BR>
 * <HL ALIGN=CENTER>
 * This document contains information proprietary to, and is the
 * sole property of, Paul Kronenwetter and Jayme Manning.  It shall not
 * be reproduced, used or disclosed in any manner or for any purpose 
 * not authorized in writing by both Paul Kronenwetter and 
 * Jayme Manning, and except, as retention may be so authorized, it
 * shall be returned to Paul Kronenwetter or Jayme Manning upon
 * request.</P>
 * <P ALIGN=CENTER><B>&copy 1998 by Paul Kronenwetter and Jayme
 * Manning<BR> All Rights Reserved</B></P>
 *
 * @version
 * $Revision: 1.7 $ <br>
 * $Date: 1998/12/17 15:54:50 $ <p>
 * 
 * <pre>
 * $Log: UserListPane.java,v $
 * Revision 1.7  1998/12/17 15:54:50  cps606
 * Added addUser and removeUser Methods.
 * Added method to add ActionListener to button.
 *
 * Revision 1.6  1998/12/15 22:56:08  cps606
 * * Manually edited the comments to correct formatting.
 *
 * Revision 1.5  1998/12/15 00:23:18  cps606
 * * Missed a few documentation items.
 *
 * Revision 1.4  1998/12/15 00:08:05  cps606
 * * Cleaned up and added documentation.  Rearranged methods a little
 *   as seemed appropriate. 
 *
 * Revision 1.3  1998/12/13 21:43:34  cps606
 * * Added getPreferredSize method.
 * * Removed Custom List definition.
 *
 * Revision 1.2  1998/12/13 20:14:34  cps606
 * * Added a ListModel for the list and some debugging code.
 *
 * Revision 1.1  1998/12/12 03:22:12  cps606
 * * First CVS checkin.
 *
 * </pre>
 */

public class UserListPane extends JPanel {
  private final String rcsId="$Id: UserListPane.java,v 1.7 1998/12/17 15:54:50 cps606 Exp $";
  private int Rows,Columns;
  private Color bgColor=Color.white;
  private Color fgColor=Color.black;
  private JLabel paneLabel;
  private JButton signOffButton;
  private JList usersList;
  private DefaultListModel listModel;
  private JScrollPane scrollPane;
  private Component draggedFrom, draggedTo;
  private JPanel northPanel, centerPanel, southPanel;

  /**
   * Void constructor that calls realCtor.
   */
  public UserListPane() {
    realCtor();
  }

  /**
   * Replies to requests for this component's preferred size.
   *
   * @return The preferred size of this panel in a Dimension.
   */
  public Dimension getPreferredSize()
  {
	return new Dimension(250,150);
  }

  /**
   * Method to re-initialize panel.  Probably not needed, except for
   * debugging purposes. 
   */
  public void reset() {
    init();
  }

  /**
   * isAvailable - Method that returns true or false based on the availability
   * of a selection.
   *
   * @return boolean - Whether or not there are any selections available.
   */
  public boolean isAvailable()
  {
	return !(usersList.isSelectionEmpty());
  }
   
  /**
   * getSelections - Method that will return an array of Strings that represent
   * the currently selected user names from the list.
   *
   * @param String[] - The selected user names.
   */
  public String[] getSelections()
  {
	Object[] foo = usersList.getSelectedValues();
	String[] x = new String[foo.length];

	for(int i=0; i < foo.length; i++)
	{
		x[i] = ((String)foo[i]);
	}

	return x;
  }

  /**
   * addActionListener - Method used to attach an ActionListener to the 
   * signOffButton.
   *
   * @param al The ActionListener to attach.
   */
  public void addActionListener(ActionListener al)
  {
	signOffButton.addActionListener(al);
  }

// Start of private methods and classes.

  /**
   * This is the method that does the "real" work of setting up the
   * variables and other objects needed to make this panel work.
   */
  private void realCtor() {
    // Make the colors "sane"
    setForeground(fgColor);
    setBackground(bgColor);

    // Make sure this panel uses a BorderLayout.
    setLayout(new BorderLayout());

    // Label this panel
    northPanel = new JPanel();
    paneLabel = new JLabel("Users");
    northPanel.add(paneLabel);

    // Set up the button area.
    southPanel = new JPanel();
    signOffButton = new JButton("Sign Off");
    southPanel.add(signOffButton);

    // Create the user list.
    usersList = new JList();
    listModel = new DefaultListModel();
    usersList.setModel(listModel);

    // Set up the list panel for scrolling.
    scrollPane = new JScrollPane();
    scrollPane.add(usersList);
    scrollPane.getViewport().setView(usersList);
    scrollPane.setVerticalScrollBarPolicy(
	JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);

    // DEBUG - Instrumentation code & testing
    // Add stuff to the list...
    // for (int count=0;count<20;count++) {
    //   String tmpLabel=new String("Hi: " + count);
    //   listModel.addElement(tmpLabel);
    // }
    // usersList.validate();
    // DEBUG - Instrumentation code & testing

    scrollPane.getViewport().setView(usersList);

    // Add the components to the layout.
    add(northPanel,BorderLayout.NORTH);
    add(scrollPane,BorderLayout.CENTER);
    add(southPanel,BorderLayout.SOUTH);
  }

  /**
   * addUser - Method used to add a user to the List.
   *
   * @param user - String with the username.
   */
  public void addUser(String user)
  {
    listModel.addElement(new String(user));
    usersList.validate();
  }

  /**
   * removeUser - Method used to remove a user from the List.
   *
   * @param user - String with the username.
   */
  public boolean removeUser(String user)
  {
    boolean result = listModel.removeElement(new String(user));
    usersList.validate();
    return result;
  }

  /**
   * removeAllUsers - Method used to clear the list in preparation to start
   * over.
   */
  public void removeAllUsers()
  {
    listModel.removeAllElements();
    usersList.validate();
  }
   
  /**
   * Initialization method for the CalendarPane.  Also probably not
   * needed except for debugging.
   */
  private void init() {
    validate();
  }

}
