/*  Client.java applet for connecting to Oracle database.
 *    Uses file myDatabase.java
 *  Errors and informational messages can be viewed on the Java Console.
 */

import java.awt.*;
import java.net.*;
import java.io.*;
import java.applet.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

public class Client extends Applet {
  
  public myDatabase database = new myDatabase();
  public ControlPanel controlPanel;
  public Panel headerPanel;
  public DisplayPanel dp;	
  
  public void init() {
    // set background color to lightGray, add HeaderPanel, 
    //DisplayPanel and controlPanel components
    setBackground(Color.lightGray);
    headerPanel = new Panel();
    headerPanel.setFont(new Font("Helvetica", Font.PLAIN, 24));
    headerPanel.add(new Label("Database Connectivity Test"));
    add("North", headerPanel);
    add("Center", dp = new DisplayPanel());
    add("South",  controlPanel = new ControlPanel(this));
  }
  
  // search for the name specified in the database  
  public void search(String a) {
    StringBuffer asb = new StringBuffer();
    StringBuffer bsb = new StringBuffer();
    StringBuffer csb = new StringBuffer();
    // search for string a, resulting fields returned in stringbuffers
    database.find(a, asb, bsb, csb);
    System.out.println("got back from db: " + asb.toString());
    dp.showResult(asb.toString(), bsb.toString(), csb.toString());
  }

  // Important to make sure database connection is always closed
  public void destroy() {
    database.close();        
  }
}

  
/* the ControlPanel with two panels, connectionPanel with connect and 
 * close buttons, searchPanel with search and clear button and a label
 */
class ControlPanel extends Panel implements ActionListener {
  private Client parent;
  private TextField name;
  private Label lbl;
  Panel searchPanel;
  Panel connectionPanel;
  Button connectButton, closeButton, searchButton, clearButton;
  
  public ControlPanel(Client clt) {
    parent = clt;
    setLayout(new BorderLayout());
    setFont(new Font("Helvitica", Font.PLAIN, 14));
    searchPanel = new Panel();
    connectionPanel = new Panel();
    connectButton = new Button("Connect");
    closeButton = new Button("Close");
    searchButton = new Button("Search");
    clearButton = new Button("Clear");
    connectButton.addActionListener(this);
    closeButton.addActionListener(this);
    searchButton.addActionListener(this);
    clearButton.addActionListener(this);
    
    connectionPanel.add(connectButton);
    connectionPanel.add(closeButton);
    searchPanel.add(lbl = new Label("Enter name to search"));
    searchPanel.add(name = new TextField(20));
    searchPanel.add(searchButton);
    searchPanel.add(clearButton);
    add("North",connectionPanel);
    add("South",searchPanel);
  }
  
  // handle action events for the button components 
  public void actionPerformed(ActionEvent event) {
    String buttonLabel = event.getActionCommand();
    
    if(buttonLabel.equals("Search")) {
      try {
	System.out.println("search for name: " + name.getText());
	parent.search(name.getText());
      }
      catch (Exception ex) {
      }
    }
    if(buttonLabel.equals("Clear")) {
      name.setText("");
    }
    if(buttonLabel.equals("Connect")) {
      parent.database.open();
    }
    if(buttonLabel.equals("Close")) {
      parent.database.close();
    }
  }
}

/* the DisplayPanel contains a textarea to display the results of our search
 */
class DisplayPanel extends Panel {
  TextArea resultArea;
  String nodata = "Sorry! There is no such person by that name.";
  
  public DisplayPanel() {
    setLayout(new BorderLayout(10,10));
    resultArea = new TextArea(15,60);
    resultArea.setBackground(Color.white);
    add("Center",resultArea);
  }
  
  public void showResult(String Name, String Age, String Occ) {

    // if Name is empty, no string was found
    if((Name == null) || Name.equals( "" )) {
      resultArea.setFont( new Font("Times-Roman",Font.PLAIN,18));
      setForeground(Color.red);
      resultArea.setText("\n\n\n " + nodata);
    }

    // otherwise, display the results
    else {
      resultArea.setFont( new Font("Times-Roman",Font.PLAIN,16));
      setForeground(Color.black);
      resultArea.setText("\n\n Name       : " + Name + 
                     "\n Age        : " + String.valueOf(Age) + 
                     "\n Occupation : " + Occ + "\n");
    }
  }		       		
}










